#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc 2MASSDialog {} {
    global twomass

    if [winfo exists $twomass(top)] {
	raise $twomass(top)
	return
    }

    set varname twomassi
    upvar #0 $varname var
    global $varname

    set var(top) $twomass(top)
    set var(mb) $twomass(mb)
    set var(system) $twomass(system)
    set var(sky) $twomass(sky)
    set var(skyformat) $twomass(skyformat)
    set var(rformat) $twomass(rformat)
    set var(width) $twomass(width)
    set var(height) $twomass(height)
    set var(survey) $twomass(survey)
    set var(mode) $twomass(mode)
    set var(save) $twomass(save)
    set var(valid) $twomass(valid)

    set w $var(top)
    IMGSVRInit $varname "IPAC-2MASS [msgcat::mc {Server}]" 2MASSExec 2MASSAck

    # menu
    AppleMenu ${varname}(mb)

    IMGSVRFileMenu $varname
    AREditMenu $varname
    NSVRServerMenu $varname
    ARCoordMenu $varname
    ARSizeMenu $varname

    $var(mb) add cascade -label Survey -menu $var(mb).survey
    menu $var(mb).survey
    $var(mb).survey add radiobutton -label {2MASS (J Band)} \
	-variable ${varname}(survey) -value j
    $var(mb).survey add radiobutton -label {2MASS (H Band)} \
	-variable ${varname}(survey) -value h
    $var(mb).survey add radiobutton -label {2MASS (K Band)} \
	-variable ${varname}(survey) -value k

    IMGSVRPrefsMenu $varname
    IMGSVRUpdate $varname 1
}

proc 2MASSExec {varname} {
    upvar #0 $varname var
    global $varname

    if {$var(save)} {
	set var(fn) [SaveFileDialog savefitsfbox]
	if {$var(fn) == {}} {
	    return
	}

    } else {
	set var(fn) [tmpnam 2mass {.fits.gz}]
    }

    # size - convert to arcsec
    switch -- $var(rformat) {
	degrees {
	    set ww [expr $var(width)*60.*60.]
	    set hh [expr $var(height)*60.*60.]
	}
	arcmin {
	    set ww [expr $var(width)*60.]
	    set hh [expr $var(height)*60.]
	}
	arcsec {
	    set ww $var(width)
	    set hh $var(height)
	}
    }

    # now to radius
    set rr [expr sqrt($ww*$ww+$hh*$hh)/2.]
    if {$rr>1024} {
	set rr 1024
    }

    set foo "$var(x) $var(y)"

    set var(query) [http::formatQuery objstr $foo size $rr band $var(survey)]
    set var(url) \
	"http://irsa.ipac.caltech.edu/cgi-bin/Oasis/2MASSImg/nph-2massimg"

    IMGSVRLoad $varname
}

proc 2MASSAck {varname} {
    upvar #0 $varname var
    global $varname

set msg {Acknowledgments for the 2MASS 

This publication makes use of data products from the Two Micron All
Sky Survey, which is a joint project of the University of
Massachusetts and the Infrared Processing and Analysis
Center/California Institute of Technology, funded by the National
Aeronautics and Space Administration and the National Science
Foundation.
}

SimpleTextDialog "${varname}ack" [msgcat::mc {Acknowledgment}] 80 40 insert top $msg
}

# Process Cmds

proc Process2MASSCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global current

    set vvarname twomassi
    upvar #0 $vvarname vvar
    global $vvarname

    # do we have a rgb frame
    if {[$current(frame) get type] == {rgb}} {
	set base 0
    } else {
	set base 1
    }

    2MASSDialog
    set vvar(mode) current

    switch -- [string tolower [lindex $var $i]] {
	survey {
	    incr i
	    set vvar(survey) [lindex $var $i]
	}
	width {
	    incr i
	    set vvar(width) [lindex $var $i]
	}
	height {
	    incr i
	    set vvar(height) [lindex $var $i]
	}
	size {
	    incr i
	    set vvar(width) [expr [lindex $var $i]*2.]
	    set vvar(height) [expr [lindex $var $i]*2.]
	}

	coord {
	    if {$flag} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(x) [lindex $var $i]
	    incr i
	    set vvar(y) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	x {
	    incr i
	    set vvar(x) [lindex $var $i]
	}
	y {
	    incr i
	    set vvar(y) [lindex $var $i]
	}
	name {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    incr i
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
	default {
	    if {$flag && $base} {
		MultiLoad fc
	    }
	    set vvar(name) [lindex $var $i]

	    IMGSVRApply $vvarname 1
	}
    }
}
