#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ARInit {varname next} {
    upvar #0 $varname var
    global $varname

    set var(sync) 0
    set var(proc,next) $next

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(status) {}
}

proc ARApply {varname} {
    upvar #0 $varname var
    global $varname

    ARStatus $varname {}

    $var(mb).file entryconfig [msgcat::mc {Retrieve}] -state disabled
    $var(mb).file entryconfig [msgcat::mc {Cancel}] -state normal

    $var(apply) configure -state disabled
    $var(cancel) configure -state normal
}

proc ARCancel {varname} {
    upvar #0 $varname var
    global $varname

    # set state to 0 so that we don't process the finish proc
    set var(active) 0

    if [info exists var(token)] {
	http::reset $var(token)
    }
}

proc ARDestroy {varname} {
    upvar #0 $varname var
    global $varname

    ARCancel $varname

    if {[winfo exists $var(top)]} {
	destroy $var(top)
	destroy $var(mb)
    }

    unset $varname
}

proc ARReset {varname} {
    upvar #0 $varname var
    global $varname

    set var(active) 0

    if {[info exists var(token)]} {
	http::cleanup $var(token)
	unset var(token)
    }

    $var(mb).file entryconfig [msgcat::mc {Retrieve}] -state normal
    $var(mb).file entryconfig [msgcat::mc {Cancel}] -state disabled

    $var(apply) configure -state normal
    $var(cancel) configure -state disabled
}

proc ARDone {varname} {
    upvar #0 $varname var
    global $varname

    set  var(status) {Done}
    ARReset $varname
}

proc ARCancelled {varname} {
    upvar #0 $varname var
    global $varname

    set var(status) {Cancelled}
    ARReset $varname
}

proc ARError {varname message} {
    upvar #0 $varname var
    global $varname

    set var(status) $message
    ARReset $varname
}

proc ARStatus {varname message} {
    upvar #0 $varname var
    global $varname

    set var(status) $message
}

proc ARCut {varname} {
    upvar #0 $varname var
    global $varname

    tk_entryCut [focus -displayof $var(top)]
}

proc ARCopy {varname} {
    upvar #0 $varname var
    global $varname

    tk_entryCopy [focus -displayof $var(top)]
}

proc ARPaste {varname} {
    upvar #0 $varname var
    global $varname

    tk_entryPaste [focus -displayof $var(top)]
}

proc ARClear {varname} {
    upvar #0 $varname var
    global $varname

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(status) {}
}

proc ARCoord {varname} {
    upvar #0 $varname var
    global $varname

    switch -- $var(sky) {
	fk4 -
	fk5 -
	icrs {
	    $var(xname) configure -text {a} -font {symbol 12}
	    $var(yname) configure -text {d} -font {symbol 12}
	}
	galactic {
	    $var(xname) configure -text {l} -font {-family times -slant italic}
	    $var(yname) configure -text {b} -font {-family times -slant italic}
	}
	ecliptic {
	    $var(xname) configure -text {l} -font {symbol 12}
	    $var(yname) configure -text {b} -font {symbol 12}
	}
    }
}

proc AREditMenu {varname} {
    upvar #0 $varname var
    global $varname

    global ds9

    $var(mb) add cascade -label [msgcat::mc {Edit}] -menu $var(mb).edit
    menu $var(mb).edit
    $var(mb).edit add command -label [msgcat::mc {Cut}] \
	-command "ARCut $varname" -accelerator "${ds9(ctrl)}X"
    $var(mb).edit add command -label [msgcat::mc {Copy}] \
	-command "ARCopy $varname" -accelerator "${ds9(ctrl)}C"
    $var(mb).edit add command -label [msgcat::mc {Paste}] \
	-command "ARPaste $varname" -accelerator "${ds9(ctrl)}V"
    $var(mb).edit add command -label [msgcat::mc {Clear}] \
	-command "ARClear $varname"
}

proc ARCoordMenu {varname} {
    upvar #0 $varname var
    global $varname

    $var(mb) add cascade -label  [msgcat::mc {Coordinate}] -menu $var(mb).coord
    menu $var(mb).coord
    $var(mb).coord add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname}(skyformat) -value degrees
    $var(mb).coord add radiobutton -label {Sexagesimal} \
	-variable ${varname}(skyformat) -value sexagesimal
}

proc ARSizeMenu {varname} {
    upvar #0 $varname var
    global $varname

    $var(mb) add cascade -label [msgcat::mc {Size}] -menu $var(mb).size
    menu $var(mb).size
    $var(mb).size add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname}(rformat) -value degrees
    $var(mb).size add radiobutton -label [msgcat::mc {ArcMin}] \
	-variable ${varname}(rformat) -value arcmin
    $var(mb).size add radiobutton -label [msgcat::mc {ArcSec}] \
	-variable ${varname}(rformat) -value arcsec
}
