#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc ButtonsVars {} {
    ButtonsVarsFile
    ButtonsVarsEdit
    ButtonsVarsView
    ButtonsVarsFrame
    ButtonsVarsBin
    ButtonsVarsZoom
    ButtonsVarsScale
    ButtonsVarsColor
    ButtonsVarsRegion
    ButtonsVarsWCS
    ButtonsVarsHelp
}

proc CreateButtons {} {
    global ds9
    global buttons

    set ds9(buttons) [frame $ds9(main).buttons -relief ridge -bd 1]

    set buttons(majorPrev) {}
    set buttons(majorCurrent) $ds9(buttons).file

    frame $ds9(buttons).major
    pack $ds9(buttons).major -side top -fill x -expand true

    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.file \
	     [string tolower [msgcat::mc {File}]] \
	     buttons(majorCurrent) $ds9(buttons).file MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.edit \
	     [string tolower [msgcat::mc {Edit}]] \
	     buttons(majorCurrent) $ds9(buttons).edit MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.view \
	     [string tolower [msgcat::mc {View}]] \
	     buttons(majorCurrent) $ds9(buttons).view MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.frame \
	     [string tolower [msgcat::mc {Frame}]] \
	     buttons(majorCurrent) $ds9(buttons).frame MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.bin \
	     [string tolower [msgcat::mc {Bin}]] \
	     buttons(majorCurrent) $ds9(buttons).bin MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.zoom \
	     [string tolower [msgcat::mc {Zoom}]] \
	     buttons(majorCurrent) $ds9(buttons).zoom MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.scale \
	     [string tolower [msgcat::mc {Scale}]] \
	     buttons(majorCurrent) $ds9(buttons).scale MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.color \
	     [string tolower [msgcat::mc {Color}]] \
	     buttons(majorCurrent) $ds9(buttons).color MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.region \
	     [string tolower [msgcat::mc {Region}]] \
	     buttons(majorCurrent) $ds9(buttons).region MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.wcs \
	     [string tolower [msgcat::mc {WCS}]] \
	     buttons(majorCurrent) $ds9(buttons).wcs MajorButton]
    lappend buttons(major) \
	[RadioButton $ds9(buttons).major.help \
	     [string tolower [msgcat::mc {Help}]] \
	     buttons(majorCurrent) $ds9(buttons).help MajorButton]

    CreateButtonsFile
    CreateButtonsEdit
    CreateButtonsView
    CreateButtonsFrame
    CreateButtonsBin
    CreateButtonsZoom
    CreateButtonsScale
    CreateButtonsColor
    CreateButtonsRegion
    CreateButtonsWCS
    CreateButtonsHelp

    UpdateButtons buttons(file)
    UpdateButtons buttons(edit)
    UpdateButtons buttons(view)
    UpdateButtons buttons(frame)
    UpdateButtons buttons(bin)
    UpdateButtons buttons(zoom)
    UpdateButtons buttons(scale)
    UpdateButtons buttons(color)
    UpdateButtons buttons(region)
    UpdateButtons buttons(wcs)
    UpdateButtons buttons(help)

    MajorButton
}

proc MajorButton {} {
    global buttons

    pack forget $buttons(majorPrev)
    pack $buttons(majorCurrent) -side top -fill x -expand true
    set buttons(majorPrev) $buttons(majorCurrent)
}

proc ButtonButton {button text cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    button $button \
		-text $text \
		-command $cmd \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2 
	}
	aqua {
	    button $button \
		-text $text \
		-command $cmd \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	}
    }

    pack $button -side left -fill x -expand true
}

proc RadioButton {button text varname value cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    radiobutton $button \
		-text $text \
		-variable $varname \
		-value $value \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command \
		"RadioButtonAqua $button $varname \{$value\} \{$cmd\}" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2
	    
	    uplevel #0 trace variable $varname w \
		[list "RadioButtonAquaCB $button \{$value\}"]

	    upvar #0 $varname var
	    if {[$button cget -state] != {disabled}} {
		if {$var == $value} {
		    $button configure -state active
		} else {
		    $button configure -state normal
		}
	    }
	}
    }

    pack $button -side left -fill x -expand true
    return $button
}

proc RadioButtonAqua {button varname value cmd} {
    uplevel #0 [list set $varname $value]
    eval $cmd
}

proc RadioButtonAquaCB {button value varname id op} {
    upvar #0 $varname var
    global $varname

    if {[$button cget -state] != {disabled}} {
	if [array exists $varname] {
	    if {$var($id) == $value} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	} else {
	    if {$var == $value} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	}
    }
}

proc CheckButton {button text varname cmd} {
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {
	    checkbutton $button \
		-text $text \
		-variable $varname \
		-command $cmd \
		-indicatoron false \
		-selectcolor {} \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 3
	}
	aqua {
	    button $button \
		-text $text \
		-command "CheckButtonAqua $button $varname \{$cmd\}" \
		-takefocus 0 \
		-bd 1 \
		-height 1 \
		-padx 2 -pady 2

	    uplevel #0 trace variable $varname w \
		[list "CheckButtonAquaCB $button"]

	    upvar #0 $varname var
	    if {[$button cget -state] != {disabled}} {
		if {$var} {
		    $button configure -state active
		} else {
		    $button configure -state normal
		}
	    }
	}
    }

    pack $button -side left -fill x -expand true
}

proc CheckButtonAqua {button varname cmd} {
    upvar #0 $varname var

    uplevel #0 [list set $varname [expr !$var]]
    eval $cmd
}

proc CheckButtonAquaCB {button varname id op} {
    upvar #0 $varname var
    global $varname

    if {[$button cget -state] != {disabled}} {
	if [array exists $varname] {
	    if {$var($id)} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	} else {
	    if {$var} {
		$button configure -state active
	    } else {
		$button configure -state normal
	    }
	}
    }
}

proc UpdateButtons {varname} {
    upvar #0 $varname var
    global buttons

    foreach {which what} $var {
	pack forget $which
    }
    foreach {which what} $var {
	ShowButton $which $what
    }
}

proc ShowButton {which varname} {
    upvar #0 $varname var
    global buttons

    if {$var} {
	pack $which -side left -fill x -expand true
    }
}

proc PrefsDialogButtonbar {} {
    global prefs

    set w $prefs(tabs)

    frame $w.buttonbar
    $prefs(tabs) tab configure buttonbar -window $w.buttonbar -fill both

    frame $w.buttonbar.menu -relief ridge -bd 1
    pack $w.buttonbar.menu -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.buttonbar

    PrefsDialogButtonbarFile $w.buttonbar.menu.file
    PrefsDialogButtonbarEdit $w.buttonbar.menu.edit
    PrefsDialogButtonbarView $w.buttonbar.menu.view
    PrefsDialogButtonbarFrame $w.buttonbar.menu.frame
    PrefsDialogButtonbarBin $w.buttonbar.menu.bin
    PrefsDialogButtonbarZoom $w.buttonbar.menu.zoom
    PrefsDialogButtonbarScale $w.buttonbar.menu.scale
    PrefsDialogButtonbarColor $w.buttonbar.menu.color
    PrefsDialogButtonbarRegion $w.buttonbar.menu.region
    PrefsDialogButtonbarWCS $w.buttonbar.menu.wcs
    PrefsDialogButtonbarHelp $w.buttonbar.menu.help

    grid \
	$w.buttonbar.menu.file \
	$w.buttonbar.menu.edit \
	$w.buttonbar.menu.view \
	$w.buttonbar.menu.frame \
	$w.buttonbar.menu.bin \
	$w.buttonbar.menu.zoom \
	-padx 4 -sticky w

    grid \
	$w.buttonbar.menu.scale \
	$w.buttonbar.menu.color \
	$w.buttonbar.menu.region \
	$w.buttonbar.menu.wcs \
	$w.buttonbar.menu.help \
	-padx 4 -sticky w
}
