#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

global cat
set cat(def) { \
	   {{SIMBAD} catsimbad simbad simbad} \
	   {{NED} catned ned ned} \
	   {- - -} \
	   {{SDSS Data Release 6} catsdss6 sdss 6} \
	   {{SDSS Data Release 5} catsdss5 sdss 5} \
	   {{SDSS Data Release 4} catsdss4 cds {II/267/sdss4}} \
	   {{SDSS Data Release 3} catsdss3 cds {II/259/sdss3}} \
	   {{SAO Star Catalog J2000} catsao cds {I/131A}} \
	   {{HST Guide Star Catalog} cathst cds {I/254}} \
	   {{GSC 2.2 Catalog} catgsc cds {I/271}} \
	   {{Tycho-2 Catalog} cattycho cds {I/259}} \
	   {{Hipparcos AC 2000.2 Catalog} catac cds {I/275}} \
	   {{NOMAD Catalog} catnomad cds {I/297/out}} \
	   {{USNO-A2.0 Catalog} catua2 cds {I/252}} \
	   {{USNO-B1.0 Catalog} catub1 cds {I/284}} \
	   {{USNO UCAC2 Catalog} catucac2 cds {I/289}} \
	   {- - -} \
	   {{2MASS All-Sky Catalog of Point Sources} cat2mass cds {II/246}}\
	   {{IRAS Point Sources} catiras cds {II/125/main}}\
	   {{DENIS Database} catdenis cds {B/denis}} \
	   {- - -} \
	   {{Second ROSAT PSPC Catalog} catrosat cds {IX/30}} \
	   {{ROSAT All-Sky Survey Faint Source Catalog} catrosf cds {IX/29}}\
	   {- - -} \
	   {{FIRST Survey Catalog} catfirst cds {VIII/71}} \
	   {- - -} \
	   {{HST Archive Exposures Catalog} cathst cds {B/hst/hstlog}} \
	   {{ESO Science Archive Catalog} cateso cds {B/eso/safcat}} \
	   {{Log of CFHT Exposures} catcfht cds {B/cfht/cfht}} \
	   {{The Chandra Archive Log} catchandra cds {B/chandra/chandra}} \
	   {{XMM-Newton Observation Log} catxmm cds {B/xmm/xmmlog}} \
}

proc CATDialog {varname format catalog title action} {
    global cat
   
    # first determine if aready in use, then increment
    if {[lsearch $cat(cats) $varname] >= 0} {
	incr cat(id)
	append varname $cat(id)
    }

    upvar #0 $varname var
    global $varname

    global ds9

    global debug
    if {$debug(tcl,cat)} {
	puts "CATDialog $varname :$format:$catalog:$title:$action"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    global current
    if {$current(frame) == {}} {
	return
    }

    # AR variables
    ARInit $varname CATServer

    # CAT variables
    lappend cat(cats) $varname

    set var(catdb) ${varname}catdb
    set var(tbldb) ${varname}tbldb
    set var(symdb) ${varname}symdb
    set var(symdl) ${varname}symdl
    set var(fltdl) ${varname}fltdl

    set var(frame) $current(frame)
    set var(system) $cat(system)
    set var(sky) $cat(sky)
    set var(skyformat) $cat(skyformat)
    set var(rformat) $cat(rformat)
    set var(width) $cat(width)
    set var(height) $cat(height)
    set var(max) $cat(max)
    set var(allrows) $cat(allrows)
    set var(allcols) $cat(allcols)
    set var(show) $cat(show)
    set var(edit) $cat(edit)
    set var(panto) $cat(panto)

    set var(blink) 0
    set var(blink,count) 0
    set var(blink,marker) {}

    CATSet $varname $format $catalog $title
    CATSymDBInit $varname

    # top
    set w $var(top)
    set title [msgcat::mc {Catalog Tool}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CATDestroy $varname"
    wm minsize $w 500 600

    # menu
    $w configure -menu $var(mb)
    menu $var(mb)

    # apple
    AppleMenu ${varname}(mb)

    # file
    $var(mb) add cascade -label [msgcat::mc {File}] -menu $var(mb).file
    menu $var(mb).file
    $var(mb).file add command -label "[msgcat::mc {Load}]..." \
	-command "CATLoadFile $varname"
    $var(mb).file add command -label "[msgcat::mc {Load Other}]..." \
	-command "CATLoadCSVFile $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "[msgcat::mc {Save}]..." \
	-command "CATSaveFile $varname"
    $var(mb).file add separator
    $var(mb).file add command \
	-label "[msgcat::mc {Display Catalog Header}]..." \
	-command "CATHeader $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Filter}] \
	-command "CATTable $varname"
    $var(mb).file add command -label [msgcat::mc {Clear}] \
	-command "CATOff $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Retrieve}] \
	-command "CATApply $varname 0"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Acknowledgment}] \
	-command "CATAck $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Update from Current Frame}] \
	-command "CATUpdate $varname"
    $var(mb).file add command \
	-label [msgcat::mc {Update from Current Crosshair}] \
	-command "CATCrosshair $varname"
    $var(mb).file add separator
	$var(mb).file add command -label [msgcat::mc {Copy to Regions}] \
	-command "CATPlotRegions $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
	-command "CATPageSetup $varname"
    $var(mb).file add command -label "[msgcat::mc {Print}]..." \
	-command "CATPrint $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "CATDestroy $varname"

#    menu $var(mb).file.other
#    $var(mb).file.other add command -label {FITS} 
#    $var(mb).file.other add command -label {CSV} -command "CATLoadCSVFile $varname"

    switch $ds9(wm) {
	x11 {
	    $var(mb).file entryconfig "[msgcat::mc {Page Setup}]..." \
		-state disabled
	}
	win32 {}
	aqua {}
    }

    # edit
    AREditMenu $varname

    # catalog server
    CATServerMenu $varname

    # name server
    NSVRServerMenu $varname
    ARCoordMenu $varname
    ARSizeMenu $varname

    # symbol
    global $var(symdb)
    set flt $var(symdb)
    set sn [starbase_colnum $var(symdb) shape]
    set cn [starbase_colnum $var(symdb) color]

    $var(mb) add cascade -label [msgcat::mc {Symbol}] -menu $var(mb).symbol

    menu $var(mb).symbol
    $var(mb).symbol add cascade -label [msgcat::mc {Shape}] \
	-menu $var(mb).symbol.shape
    $var(mb).symbol add cascade -label [msgcat::mc {Color}] \
	-menu $var(mb).symbol.color
    $var(mb).symbol add separator
    $var(mb).symbol add command -label "[msgcat::mc {Advanced}]..." \
	-command "CATSymDialog $varname"

    menu $var(mb).symbol.shape
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Circle Point}] \
	-variable ${flt}(1,$sn) -value {circle point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Box Point}] \
	-variable ${flt}(1,$sn) -value {box point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Diamond Point}] \
	-variable ${flt}(1,$sn) -value {diamond point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Cross Point}] \
	-variable ${flt}(1,$sn) -value {cross point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {X Point}] \
	-variable ${flt}(1,$sn) -value {x point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Arrow Point}] \
	-variable ${flt}(1,$sn) -value {arrow point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {BoxCircle Point}]\
	-variable ${flt}(1,$sn) -value {boxcircle point} \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add separator
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Circle}] \
	-variable ${flt}(1,$sn) -value circle \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Ellipse}] \
	-variable ${flt}(1,$sn) -value ellipse \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Box}] \
	-variable ${flt}(1,$sn) -value box \
	-command "CATPlot $varname"
    $var(mb).symbol.shape add radiobutton -label [msgcat::mc {Text}] \
	-variable ${flt}(1,$sn) -value text \
	-command "CATPlot $varname"

    DefaultColorMenu $var(mb).symbol.color ${flt}(1,$cn) "CATPlot $varname"

    $var(mb) add cascade -label [msgcat::mc {Preferences}] -menu $var(mb).prefs
    menu $var(mb).prefs
    $var(mb).prefs add checkbutton -label [msgcat::mc {Show Regions}] \
	-variable ${varname}(show) -command "CATPlot $varname"
    $var(mb).prefs add checkbutton -label [msgcat::mc {Edit}] \
	-variable ${varname}(edit) -command "CATEdit $varname"
    $var(mb).prefs add checkbutton -label [msgcat::mc {Pan To}] \
	-variable ${varname}(panto)
    $var(mb).prefs add separator
    $var(mb).prefs add checkbutton -label [msgcat::mc {All Rows}] \
	-variable ${varname}(allrows)
    $var(mb).prefs add checkbutton -label [msgcat::mc {All Columns}] \
	-variable ${varname}(allcols)

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.tbl -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.f
    pack $w.param.f -side left

    label $w.param.f.ttitle -text [msgcat::mc {Catalog}]
    label $w.param.f.title -textvariable ${varname}(title) \
	-relief groove -width 60 -anchor w

    label $w.param.f.tcat -text [msgcat::mc {Identification}]
    label $w.param.f.cat -textvariable ${varname}(catalog) \
	-relief groove -width 60 -anchor w

    label $w.param.f.nametitle -text [msgcat::mc {Name}]
    entry $w.param.f.name -textvariable ${varname}(name) -width 60

    label $w.param.f.sky -textvariable ${varname}(sky) -anchor w
    set var(xname) [label $w.param.f.xtitle -text {} \
			-font {symbol 12} -width 1]
    entry $w.param.f.x -textvariable ${varname}(x) -width 14
    set var(yname) [label $w.param.f.ytitle -text {} \
			-font {symbol 12} -width 1]
    entry $w.param.f.y -textvariable ${varname}(y) -width 14
    label $w.param.f.skyformat -textvariable ${varname}(skyformat)

    label $w.param.f.wtitle -text [msgcat::mc {Width}]
    entry $w.param.f.w -textvariable ${varname}(width) -width 14
    label $w.param.f.htitle -text [msgcat::mc {Height}]
    entry $w.param.f.h -textvariable ${varname}(height) -width 14
    label $w.param.f.format -textvariable ${varname}(rformat)

    label $w.param.f.mfilter -text [msgcat::mc {Filter}]
    entry $w.param.f.filter -textvariable ${varname}(filter) -width 40
    button $w.param.f.bfilter -text [msgcat::mc {Edit}] \
	-command "CATEditDialog $varname filter $var(catdb)"

    label $w.param.f.msort -text [msgcat::mc {Sort}]
    menubutton $w.param.f.sort -textvariable ${varname}(sort) \
	-menu $w.param.f.sort.m -relief raised -width 14
    radiobutton $w.param.f.isort -text [msgcat::mc {Increase}] \
	-variable ${varname}(sort,dir) -value "-increasing" \
	-command "CATTable $varname"
    radiobutton $w.param.f.dsort -text [msgcat::mc {Decrease}] \
	-variable ${varname}(sort,dir) -value "-decreasing" \
	-command "CATTable $varname"

    label $w.param.f.mtitle -text [msgcat::mc {Max Rows}]
    entry $w.param.f.max -textvariable ${varname}(max) -width 14

    checkbutton $w.param.f.allrows -text [msgcat::mc {All Rows}] \
	-variable ${varname}(allrows)

    checkbutton $w.param.f.allcols -text [msgcat::mc {All Columns}] \
	-variable ${varname}(allcols)

    label $w.param.f.ftitle -text [msgcat::mc {Found}] 
    set var(found) [label $w.param.f.found -relief groove -width 14 -anchor w]

    label $w.param.f.mra -text [msgcat::mc {RA}]
    menubutton $w.param.f.ra -textvariable ${varname}(ra) \
	-menu $w.param.f.ra.m -relief raised -width 14
    label $w.param.f.mdec -text [msgcat::mc {DEC}]
    menubutton $w.param.f.dec -textvariable ${varname}(dec) \
	-menu $w.param.f.dec.m -relief raised -width 14

    grid rowconfigure $w.param.f 0 -pad 4
    grid rowconfigure $w.param.f 1 -pad 4
    grid rowconfigure $w.param.f 2 -pad 4
    grid rowconfigure $w.param.f 3 -pad 4
    grid rowconfigure $w.param.f 4 -pad 4
    grid rowconfigure $w.param.f 5 -pad 4
    grid rowconfigure $w.param.f 6 -pad 4
    grid rowconfigure $w.param.f 7 -pad 4

    grid $w.param.f.ttitle x $w.param.f.title - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.tcat x $w.param.f.cat - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.nametitle x $w.param.f.name - - - - \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.sky $w.param.f.xtitle $w.param.f.x $w.param.f.ytitle \
	$w.param.f.y $w.param.f.skyformat -padx 4 -pady 1 -sticky w
    grid $w.param.f.wtitle x $w.param.f.w $w.param.f.htitle \
	$w.param.f.h $w.param.f.format -padx 4 -pady 1 -sticky w
    grid $w.param.f.mfilter x $w.param.f.filter - - $w.param.f.bfilter \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.msort x $w.param.f.sort $w.param.f.isort $w.param.f.dsort \
	-padx 4 -pady 1 -sticky w
    grid $w.param.f.mtitle x $w.param.f.max $w.param.f.ftitle $w.param.f.found \
	$w.param.f.allrows -padx 4 -pady 1 -sticky w
    grid $w.param.f.mra x $w.param.f.ra $w.param.f.mdec $w.param.f.dec \
	$w.param.f.allcols -padx 4 -pady 1 -sticky w

    # table
    set var(tbl) [table $var(top).tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(tbldb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $cat(mincols) \
		      -rows $cat(minrows) \
		      -width -1 \
		      -height -1 \
		      -maxwidth 300 \
		      -maxheight 300 \
		      -titlerows 1 \
		      -xscrollcommand [list $var(top).tbl.xscroll set]\
		      -yscrollcommand [list $var(top).tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -anchor w \
		      -browsecommand [list CATSelectCmd $varname %s] \
		     ]

    scrollbar $var(top).tbl.yscroll -command [list $var(tbl) yview] \
	-orient vertical
    scrollbar $var(top).tbl.xscroll -command [list $var(tbl) xview] \
	-orient horizontal

    grid $var(tbl) $var(top).tbl.yscroll -sticky news
    grid $var(top).tbl.xscroll -stick news
    grid rowconfigure $var(top).tbl 0 -weight 1
    grid columnconfigure $var(top).tbl 0 -weight 1

    # status
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.item -anchor w -pady 4

    # buttons
    button $w.buttons.filter -text [msgcat::mc {Filter}] \
	-command "CATTable $varname"
    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "CATApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command "CATOff $varname"
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "CATDestroy $varname"
    pack $w.buttons.filter $w.buttons.apply $w.buttons.cancel \
	$w.buttons.clear $w.buttons.close -side left -padx 10 -expand true

    # needs to go after sort menu button is defined
    CATSortMenu $varname
    CATRADECMenu $varname
    switch $var(format) {
	cds {$var(mb) entryconfig [msgcat::mc {Catalog Server}] -state normal}
	sdss -
	simbad -
	ned {$var(mb) entryconfig [msgcat::mc {Catalog Server}] -state disabled}
    }

    ARCoord $varname
    CATUpdate $varname

    ARStatus $varname {}

    switch -- $action {
	apply {CATApply $varname 0}
	sync {CATApply $varname 1}
	db {CATDB $varname}
	none {}
    }
}

proc CATApply {varname sync} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATApply $varname $sync"
    }

    if {$var(catalog) == {}} {
	Error [msgcat::mc {No Catalog specified}]
	return
    }

    set var(sync) $sync
    ARApply $varname
    if {$var(name) != {}} {
	set var(system) wcs
	set var(sky) fk5
	ARCoord $varname

	NSVRServer $varname
    } else {
	CATServer $varname
    }
}

proc CATDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global $var(tbldb)
    global $var(symdb)
    global $var(symdl)
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATDestroy $varname"
    }

    upvar #0 $var(symdl) svar
    if [info exists svar(top)] {
	if [winfo exists $svar(top)] {
	    CATSymDestroy $var(symdl)
	}
    }

    catch {unset $var(symdb)}
    catch {unset $var(tbldb)}
    catch {unset $var(catdb)}

    set i [lsearch $cat(cats) $varname]
    if {$i>=0} {
	set cat(cats) [lreplace $cat(cats) $i $i]
    }

    ARDestroy $varname
}

# Dialog Commands

proc CATLoadFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn starbase_read
    }
}

proc CATLoadCSVFile {varname} {
    upvar #0 $varname var
    global $varname

    set fn [OpenFileDialog catcsvfbox]
    if {$fn != {}} {
	CATLoadFn $varname $fn CATCSVRead
    }
}

proc CATLoadFn {varname fn reader} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoadFn $varname $fn $reader"
    }

    CATOff $varname
    CATSet $varname {} {} $fn

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(width) {}
    set var(height) {}

    if {[file exists $fn]} {
	$reader $var(catdb) $fn
    } else {
	Error "[msgcat::mc {Unable to open file}] $fn"
	return
    }

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

proc CATDB {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    set var(name) {}
    set var(x) {}
    set var(y) {}
    set var(width) {}
    set var(height) {}

    CATSortMenu $varname
    CATConfigRADEC $varname
    CATRADECMenu $varname
    CATTable $varname
}

proc CATSaveFile {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    set fn [SaveFileDialog catfbox]
    if {$fn != {}} {
	starbase_write $var(tbldb) $fn
    }
}

proc CATOff {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global $var(tbldb)
    global current

    catch {unset $var(catdb)}
    catch {unset $var(tbldb)}
    set db $var(tbldb)
    set ${db}(Nrows) {}

    $var(tbl) selection clear all

    $var(frame) marker catalog $varname delete

    CATSortMenu $varname
    CATRADECMenu $varname
    set var(filter) {}
    set var(sort) {}
    set var(ra) {}
    set var(dec) {}
}

proc CATUpdate {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATUpdate $varname"
    }

    if {[$var(frame) has wcs equatorial $var(system)]} {
	set coord [$var(frame) get fits center \
		       $var(system) $var(sky) $var(skyformat)]
	set var(x) [lindex $coord 0]
	set var(y) [lindex $coord 1]

	set size [$var(frame) get fits size \
		      $var(system) $var(sky) $var(rformat)]
	set var(width) [lindex $size 0]
	set var(height) [lindex $size 1]
	set var(name) {}
    }
}

proc CATKey {which key} {
    global ds9
    global cat

    set cat(key) $key
    set cat(key,update) {}

    $which marker catalog key
    foreach rr $cat(key,update) {
	eval "CATPlotUpdate [lindex $rr 0] [lindex $rr 1]"
    }

    unset cat(key)
    unset cat(key,update)
}

proc CATCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    if {[$var(frame) has wcs equatorial $var(system)]} {
	set coord [$var(frame) get crosshair \
		       $var(system) $var(sky) sexagesimal]
	set var(x) [lindex $coord 0]
	set var(y) [lindex $coord 1]
	set var(name) {}
    }
}

proc CATAck {varname} {
    upvar #0 $varname var
    global $varname

    switch $var(format) {
	cds {CATCDSAck $varname}
	sdss {CATSDSSAck $varname}
	simbad {CATSIMBADAck $varname}
	ned {CATNEDAck $varname}
    }
}

proc CATPrint {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global ds9
    switch $ds9(wm) {
	x11 {CATPSPrint $varname}
	win32 {win32 pm print text [::textutil::tabify::untabify2 [starbase_write_ $var(tbldb)] 12]}
	aqua {macosx pm print text [::textutil::tabify::untabify2 [starbase_write_ $var(tbldb)] 12]}
    }	
}

proc CATPSPrint {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global ps

    if {[PSPrintDialog txt]} { 
	if {$ps(dest) == "file"} {
	    catch {set ch [open "| cat > $ps(filename)" w]}
	} else {
	    catch {set ch [open "| $ps(cmd)" w]}
	}

	if {$ch != {}} {
	    starbase_writefp $var(tbldb) $ch
	    close $ch
	} else {
	    Error [msgcat::mc {An error has occurred while printing}]
	    return
	}
    }
}

proc CATPageSetup {varname} {
    upvar #0 $varname var
    global $varname

    global ds9
    switch $ds9(wm) {
	x11 {}
	win32 {win32 pm pagesetup}
	aqua {macosx pm pagesetup}
    }
}

proc CATHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    SimpleTextDialog ${varname}hdr "$var(title) [msgcat::mc {Header}]" \
	80 20 insert top [CATGetHeader $varname]
}

proc CATGetHeader {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    if [CATValidDB $var(tbldb)] {
	set hdr {}
	set nl [expr $${var(tbldb)}(HLines) - 2]
	for {set ll 1} {$ll <= $nl} {incr ll} {
	    append hdr "[expr $${var(tbldb)}(H_$ll)]\n"
	}
	return $hdr
    }
    return {}
}

proc CATEdit {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global current

    $var(frame) marker catalog unselect all
    $var(frame) marker catalog unhighlite all

    CATPlot $varname

    if {$var(edit)} {
	$var(tbl) configure \
	    -state normal \
	    -selectmode single
    } else {
	$var(tbl) configure \
	    -state disabled \
	    -selectmode extended
    }
}

# Other procedures

proc CATServer {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATServer $varname"
    }

    if {($var(x) != {}) && 
	($var(y) != {}) && 
	($var(width) != {}) && 
	($var(height) != {})} {

	ARStatus $varname "Searching [string range $var(title) 0 50]"

	switch $var(format) {
	    cds {CATCDS $varname}
	    sdss {CATSDSS $varname}
	    simbad {CATSIMBAD $varname}
	    ned {CATNED $varname}
	}
    } else {
	Info [msgcat::mc {Please specify width, height, and either name or (ra,dec)}]

	$var(mb).file entryconfig [msgcat::mc {Retrieve}] -state normal
	$var(mb).file entryconfig [msgcat::mc {Cancel}] -state disabled

	$var(apply) configure -state normal
	$var(cancel) configure -state disabled
    }
}

proc CATLoad {varname} {
    upvar #0 $varname var
    global $varname

    # clear previous db
    global $var(catdb)
    catch {unset $var(catdb)}

    global http

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoad $varname $var(url)?$var(query)"
    }

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(reader) $var(catdb)] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    CATLoadFinish $varname $var(token)
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list $var(reader) $var(catdb)] \
					 -command \
					 [list CATLoadFinish $varname]\
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    }
}

proc CATLoadFinish {varname token} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATLoadFinish $varname"
    }

    if {!($var(active))} {
	ARCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	{} -
	200 -
	203 -
	404 -
	503 {
	    ARDone $varname

	    CATSortMenu $varname
	    CATConfigRADEC $varname
	    CATRADECMenu $varname
	    CATTable $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    global debug
		    if {$debug(tcl,cat)} {
			puts "CATLoadFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    CATLoad $varname
		}
	    }
	}

	default {ARError $varname "[msgcat::mc {Error code was returned}] $code"}
    }
}

proc CATTable {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATTable $varname"
    }

    if {![CATValidDB $var(catdb)]} {
	Error [msgcat::mc {No catalog data loaded}]
	return
    }

    # delete any previous tbldb
    set db ${varname}tbldb
    global $db
    catch {unset $db}

    # clear the selection
    $var(tbl) selection clear all

    # clear regions
    $var(frame) marker catalog $varname delete

    if {$var(filter) == {} && $var(sort) == {}} {
	set var(tbldb) $var(catdb)
    } else {
	set var(tbldb) ${varname}tbldb
	global $var(tbldb)
	if {![CATFltSort $varname]} {
	    Error "[msgcat::mc {Unable to evaluate filter}] $var(filter)"
	    catch {unset $var(tbldb)}
	    set var(tbldb) $var(catdb)
	}
    }

    global $var(tbldb)
    $var(tbl) configure -variable $var(tbldb)
    $var(found) configure -textvariable ${var(tbldb)}(Nrows)

#    starbase_writefp $var(catdb) stdout
#    starbase_writefp $var(tbldb) stdout

    if {[starbase_nrows $var(tbldb)] == 0} {
	Info [msgcat::mc {No Items Found}]
	return
    }

    set nc [starbase_ncols $var(tbldb)]
    if { $nc > $cat(mincols)} {
	$var(tbl) configure -cols $nc
    } else {
	$var(tbl) configure -cols $cat(mincols)
    }

    # add header
    set nr [expr [starbase_nrows $var(tbldb)]+1]
    if {$nr > $cat(minrows)} {
	$var(tbl) configure -rows $nr
    } else {
	$var(tbl) configure -rows $cat(minrows)
    }

    CATPlot $varname

    set nr [starbase_nrows $var(tbldb)]
    if {$nr >= $var(max) && !$var(allrows)} {
	Info "$nr [msgcat::mc {rows of data have been downloaded. More may be available. You may wish to adjust the maximum allowed}]"
    }
}

proc CATPlot {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATPlot $varname"
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname [msgcat::mc {Plotting Regions}]

    if {[$var(frame) has wcs equatorial wcs]} {
	# delete any previous 
	$var(frame) marker catalog $varname delete

	if {$var(show)} {
	    global reg
	    set reg [CATReg $varname {} 1]
	    if {$reg != {}} {
		$var(frame) marker catalog command ds9 var reg
	    }
	    unset reg
	}
    }

    ARStatus $varname {Done}
}

proc CATPlotRegions {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATRegions $varname"
    }

    # do we have a db?
    if {![CATValidDB $var(tbldb)]} {
	return
    }

    ARStatus $varname [msgcat::mc {Generating Regions}]

    if {[$var(frame) has wcs equatorial wcs]} {
	global reg
	set reg [CATReg $varname {} 0]
	if {$reg != {}} {
	    $var(frame) marker command ds9 var reg
	}
	unset reg
    }

    ARStatus $varname {Done}
}

proc CATPlotUpdate {varname row} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATPlotUpdate $varname $row"
    }

    set id [$var(frame) get marker catalog "\{${varname}.${row}\}" id]
    set sel [$var(frame) get marker catalog $id select]
    set hh [$var(frame) get marker catalog $id highlite]

    $var(frame) marker catalog "\{${varname}.${row}\}" delete
    global reg
    set reg [CATReg $varname $row 1]
    if {$reg != {}} {
	$var(frame) marker catalog command ds9 var reg

	set id [$var(frame) get marker catalog "\{${varname}.${row}\}" id]
	if {$sel} {
	    $var(frame) marker catalog "\{${varname}.${row}\}" select
	}
	if {$hh} {
	    $var(frame) marker catalog "\{${varname}.${row}\}" highlite
	}
    }
    unset reg
}

proc CATSet {varname format catalog title} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATSet $varname :$format:$catalog:$title:"
    }

    set var(format) $format
    set var(catalog) $catalog
    set var(title) $title
    set var(filter) {}
    set var(ra) {}
    set var(dec) {}
    set var(sort) {}
    set var(sort,dir) "-increasing"
}

proc CATValidDB {varname} {
    upvar #0 $varname var
    global $varname

    if {[info exists var(Nrows)] && 
	[info exists var(Ncols)] &&
	[info exists var(Header)]} {
	return 1
    } else {
	return 0
    }
}

proc CATCatalogMenu {} {
    global cat
    global ds9

    foreach mm $cat(def) {
	set ll [lindex $mm 0]
	set ww [lindex $mm 1]
	set ss [lindex $mm 2]
	set cc [lindex $mm 3]

	if {$ll != "-"} {
	    $ds9(mb).analysis.cat add command -label $ll \
		-command [list CATDialog $ww $ss $cc $ll apply]
	} else {
	    $ds9(mb).analysis.cat add separator
	}
    }
}

proc CATServerMenu {varname} {
    upvar #0 $varname var
    global $varname

    global cat

    $var(mb) add cascade -label [msgcat::mc {Catalog Server}] \
	-menu $var(mb).server
    menu $var(mb).server
    $var(mb).server add radiobutton -label {CDS, Strasbourg France} \
	-variable cat(server) -value cds
    $var(mb).server add radiobutton -label {CFA, Boston USA} \
	-variable cat(server) -value sao
    $var(mb).server add radiobutton -label {CADC, Canada} \
	-variable cat(server) -value cadc
    $var(mb).server add radiobutton -label {ADAC, Tokyo Japan} \
	-variable cat(server) -value adac
    $var(mb).server add radiobutton -label {IUCAA, Pune India} \
	-variable cat(server) -value iucaa
    $var(mb).server add radiobutton -label {INASAN, Russia} \
	-variable cat(server) -value inasan
    $var(mb).server add radiobutton -label {BEJING, China} \
	-variable cat(server) -value bejing
    $var(mb).server add radiobutton -label {CAMBRIDGE, UK} \
	-variable cat(server) -value cambridge
    $var(mb).server add radiobutton -label {JAC, Hawaii USA} \
	-variable cat(server) -value jac
}

proc CATSortMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global ds9

    set m $var(top).param.f.sort.m
    catch {destroy $m}

    menu $m -tearoff 0
    $m add command -label {} -command "CATSortCmd $varname {}"
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATSortCmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }
}

proc CATSortCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(sort) $val
    CATTable $varname
}

proc CATRADECMenu {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    global ds9

    set m $var(top).param.f.ra.m
    catch {destroy $m}

    menu $m -tearoff 0
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATRACmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }

    set m $var(top).param.f.dec.m
    catch {destroy $m}

    menu $m -tearoff 0
    if [CATValidDB $var(catdb)] {
	set cnt -1
	foreach col [starbase_columns $var(catdb)] {
	    $m add command -label $col -command "CATDECCmd $varname \{$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$m entryconfig $col -columnbreak 1
	    }
	}
    }
}

proc CATRACmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(ra) $val
    CATPlot $varname
}

proc CATDECCmd {varname val} {
    upvar #0 $varname var
    global $varname

    set ${varname}(dec) $val
    CATPlot $varname
}

proc CATConfigRADEC {varname} {
    upvar #0 $varname var
    global $varname
    global $var(catdb)

    set var(ra) {}
    set var(dec) {}

    if {![CATValidDB $var(catdb)]} {
	return
    }

    if {[starbase_ncols $var(catdb)] < 2} {
	return
    }

    set var(ra) "_RAJ2000"
    set var(dec) "_DEJ2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "_RAB2000"
    set var(dec) "_DEB2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "RAJ2000"
    set var(dec) "DEJ2000"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "RA"
    set var(dec) "DEC"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "RA(deg)"
    set var(dec) "DEC(deg)"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    set var(ra) "ra"
    set var(dec) "dec"
    if {!([lsearch [starbase_columns $var(catdb)] $var(ra)] == -1) &&
	!([lsearch [starbase_columns $var(catdb)] $var(dec)] == -1)} {
	return
    }

    # default
    set var(ra) [starbase_colname $var(catdb) 1]
    set var(dec) [starbase_colname $var(catdb) 2]

    return
}

# Other interface

proc CATOther {} {
    CATDialog catoth {} {} {} none
}

proc CATClearCurrentFrame {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) marker catalog delete all
    }
}

proc CATEditDialog {varname which db} {
    upvar #0 $varname var
    global $varname
    global ed
    global ds9

    set w ".${varname}edit"
    set mb ".${varname}editmb"

    set ed(ok) 0
    set ed(text) $w.param.txt

    DialogCreate $w [msgcat::mc {Edit}]

    $w configure -menu $mb
    menu $mb

    # apple
    AppleMenu mb

    # file
    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] -command {set ed(ok) 1}
    $mb.file add separator
    $mb.file add command -label "[msgcat::mc {Load}]..." -command "CATEditLoad"
    $mb.file add command -label "[msgcat::mc {Save}]..." -command "CATEditSave"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Cancel}] -command {set ed(ok) 0}

    # edit
    $mb add cascade -label [msgcat::mc {Edit}] -menu $mb.edit
    menu $mb.edit
    $mb.edit add command -label [msgcat::mc {Undo}] \
	-command "$ed(text) edit undo" -accelerator "${ds9(ctrl)}Z"
    $mb.edit add command -label [msgcat::mc {Redo}] \
	-command "$ed(text) edit redo" -accelerator "${ds9(shiftctrl)}Z"
    $mb.edit add separator
    $mb.edit add command -label [msgcat::mc {Cut}] \
	-command "tk_textCut $ed(text)" -accelerator "${ds9(ctrl)}X"
    $mb.edit add command -label [msgcat::mc {Copy}] \
	-command "tk_textCopy $ed(text)" -accelerator "${ds9(ctrl)}C"
    $mb.edit add command -label [msgcat::mc {Paste}] \
	-command "tk_textPaste $ed(text)" -accelerator "${ds9(ctrl)}V"

    global $db
    # column
    $mb add cascade -label [msgcat::mc {Column}] -menu $mb.col
    catch {destroy $mb.col}
    menu $mb.col
    if [CATValidDB $db] {
	set cnt -1
	foreach col [starbase_columns $db] {
	    $mb.col add command -label "$col" \
		-command "$ed(text) insert insert \{\$$col\}"

	    # wrap if needed
	    incr cnt
	    if {$cnt>=$ds9(menu,size,wrap)} {
		set cnt 0
		$mb.col entryconfig $col -columnbreak 1
	    }
	}
    }

    # operator
    $mb add cascade -label [msgcat::mc {Operator}] -menu $mb.op
    menu $mb.op
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add command -label {!} \
	-command "$ed(text) insert insert {!}"
    $mb.op add command -label {(} \
				   -command "$ed(text) insert insert {(}"
	$mb.op add command -label {)} \
	-command "$ed(text) insert insert {)}"
    $mb.op add separator
    $mb.op add command -label {*} \
	-command "$ed(text) insert insert {*}"
    $mb.op add command -label {/} \
	-command "$ed(text) insert insert {/}"
    $mb.op add command -label {%} \
	-command "$ed(text) insert insert {%}"
    $mb.op add command -label {+} \
	-command "$ed(text) insert insert {+}"
    $mb.op add command -label {-} \
	-command "$ed(text) insert insert {-}"
    $mb.op add separator
    $mb.op add command -label {<} \
	-command "$ed(text) insert insert {<}"
    $mb.op add command -label {>} \
	-command "$ed(text) insert insert {>}"
    $mb.op add command -label {<=} \
	-command "$ed(text) insert insert {<=}"
    $mb.op add command -label {>=} \
	-command "$ed(text) insert insert {>=}"
    $mb.op add command -label {==} \
	-command "$ed(text) insert insert {==}"
    $mb.op add command -label {!=} \
	-command "$ed(text) insert insert {!=}"
    $mb.op add separator
    $mb.op add command -label {&&} \
	-command "$ed(text) insert insert {&&}"
    $mb.op add command -label {||} \
	-command "$ed(text) insert insert {||}"

    # operator
    $mb add cascade -label [msgcat::mc {Math Function}] -menu $mb.math
    menu $mb.math
    $mb.math add command -label {acos} \
	-command "$ed(text) insert insert {acos()}"
    $mb.math add command -label {asin} \
	-command "$ed(text) insert insert {asin()}"
    $mb.math add command -label {atan} \
	-command "$ed(text) insert insert {atan()}"
    $mb.math add command -label {atan2} \
	-command "$ed(text) insert insert {atan2(,)}"
    $mb.math add command -label {ceil} \
	-command "$ed(text) insert insert {ceil()}"
    $mb.math add command -label {cos} \
	-command "$ed(text) insert insert {cos()}"
    $mb.math add command -label {cosh} \
	-command "$ed(text) insert insert {cosh()}"
    $mb.math add command -label {exp} \
	-command "$ed(text) insert insert {exp()}"
    $mb.math add command -label {floor} \
	-command "$ed(text) insert insert {floor()}"
    $mb.math add command -label {fmod} \
	-command "$ed(text) insert insert {fmod(,)}"
    $mb.math add command -label {hypot} \
	-command "$ed(text) insert insert {hypot(,)}"
    $mb.math add command -label {log} \
	-command "$ed(text) insert insert {log()}"
    $mb.math add command -label {log10} \
	-command "$ed(text) insert insert {log10()}"
    $mb.math add command -label {pow} \
	-command "$ed(text) insert insert {pow(,)}"
    $mb.math add command -label {sin} \
	-command "$ed(text) insert insert {sin()}"
    $mb.math add command -label {sinh} \
	-command "$ed(text) insert insert {sinh()}"
    $mb.math add command -label {sqrt} \
	-command "$ed(text) insert insert {sqrt()}"
    $mb.math add command -label {tan} \
	-command "$ed(text) insert insert {tan()}"
    $mb.math add command -label {tanh} \
	-command "$ed(text) insert insert {tanh()}"
    $mb.math add command -label {abs} \
	-command "$ed(text) insert insert {abs()}"
    $mb.math add command -label {double} \
	-command "$ed(text) insert insert {double()}"
    $mb.math add command -label {int} \
	-command "$ed(text) insert insert {int()}"
    $mb.math add command -label {round} \
	-command "$ed(text) insert insert {round()}"

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    text $w.param.txt \
	-height 10 \
	-width 60 \
	-font {courier 12} \
	-yscrollcommand "$w.param.yscroll set" \
	-xscrollcommand "$w.param.xscroll set" \
	-undo true \
	-wrap none 

    scrollbar $w.param.yscroll -command [list $ed(text) yview] \
	-orient vertical
    scrollbar $w.param.xscroll -command [list $ed(text) xview] \
	-orient horizontal

    grid $ed(text) $w.param.yscroll -sticky news
    grid $w.param.xscroll -stick news
    grid rowconfigure $w.param 0 -weight 1
    grid columnconfigure $w.param 0 -weight 1

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok $w.buttons.cancel -side left -padx 10 -expand true

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    $ed(text) insert end $var($which)
    $ed(text) see end

    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok

    if {$ed(ok)} {
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	set var($which) [string trim $flt]
    }

    DialogDismiss $w
    catch {destroy $mb}
    unset ed
}

proc CATEditSave {} {
    global ed

    set fn [SaveFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn w} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	set flt [$ed(text) get 1.0 end]
	catch {regsub {\n} $flt " " $flt}
	puts $fp [string trim $flt]
	catch {close $fp}
    }
}

proc CATEditLoad {} {
    global ed

    set fn [OpenFileDialog catfltfbox]
    if {$fn != {}} {
	if [catch {open $fn r} fp] {
	    Error "[msgcat::mc {Unable to open file}] $fn: $fp"
	    return
	}
	$ed(text) delete 1.0 end
	$ed(text) insert end [read -nonewline $fp]
	$ed(text) see end
	catch {close $fp}
    }
}

# Process Cmds

proc ProcessCatalogCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global cat

    # we need to be realized
    RealizeDS9

    set cvarname {}

    # determine catalog name
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	cds -
	sdss {
	    # see if its from our list of cats
	    incr i
	    set which [string tolower [lindex $var $i]]
	    foreach mm $cat(def) {
		set ll [lindex $mm 0]
		set ww [lindex $mm 1]
		set ss [lindex $mm 2]
		set cc [lindex $mm 3]
		
		if {"cat${which}" == $ww} {
		    CATDialog $ww $ss $cc $ll sync
		    return
		}
	    }

	    # not a default, assume other name
	    switch -- $item {
		cds {CATDialog catoth $item $which $which sync}
		sdss {CATDialog catsdss6 sdss 6 {SDSS Data Release 6} sync}
	    }

	    return
	}

	file -
	load {
	    incr i
	    set reader starbase_read
	    if {[lindex $var $i] == "csv"} {
		set reader CATCSVRead
		incr i
	    }

	    set fn [lindex $var $i]
	    if {$fn != {}} {
		CATDialog catoth {} {} {} none
		CATLoadFn [lindex $cat(cats) end] $fn $reader
		FileLastFull catfbox $fn
	    }
	    return
	}

	name -
	coordinate -
	size -
	save -
	header -
	filter -
	clear -
	retrieve -
	cancel -
	print -
	close -
	server -
	symbol -
	sort -
	maxrows -
	allcols -
	allrows -
	ra -
	dec -
	show -
	hide -
	edit -
	panto {set cvarname [lindex $cat(cats) end]}

	default {
	    # is it an existing cat?
	    if {[lsearch $cat(cats) $item] == -1} {
		set which $item
		foreach mm $cat(def) {
		    set ll [lindex $mm 0]
		    set ww [lindex $mm 1]
		    set ss [lindex $mm 2]
		    set cc [lindex $mm 3]
		    
		    if {"cat${which}" == $ww} {
			CATDialog $ww $ss $cc $ll sync
			return
		    }
		}

		# not a default, assume other name
		switch -- $item {
		    cds {CATDialog catoth $item $which $which sync}
		    sdss {CATDialog catsdss6 sdss 6 {SDSS Data Release 6} sync}
		}

		return
	    } else {
		set cvarname $item
		incr i
	    }
	}
    }

    # we should have a catalog now
    global $cvarname
    upvar #0 $cvarname cvar

    if {![info exists cvar(top)]} {
	Error "[msgcat::mc {Unable to find catalog window}] $cvarname"
	return
    }
    if {![winfo exists $cvar(top)]} {
	Error "[msgcat:: mc {Unable to find catalog window}] $cvarname"
	return
    }

    # now, process it
    set item [string tolower [lindex $var $i]]
    switch -- $item {
	name {
	    incr i
	    set cvar(name) [lindex $var $i]
	}
	coordinate {
	    incr i
	    set cvar(x) [lindex $var $i]
	    incr i
	    set cvar(y) [lindex $var $i]
	    incr i
	    set cvar(sky) [lindex $var $i]
	}
	size {
	    incr i
	    set cvar(width) [lindex $var $i]
	    incr i
	    set cvar(height) [lindex $var $i]
	    incr i
	    set cvar(rformat) [lindex $var $i]
	}
	save {
	    global $cvar(tbldb)
	    incr i
	    starbase_write $cvar(tbldb) [lindex $var $i]
	}
	header {}
	filter {
	    incr i
	    set item [lindex $var $i]
	    switch -- $item {
		load {
		    incr i
		    set fn [lindex $var $i]
		    if [catch {open $fn r} fp] {
			Error "[msgcat::mc {Unable to open file}] $fn: $fp"
			return
		    }
		    set flt [read -nonewline $fp]
		    catch {regsub {\n} $flt " " $flt}
		    set cvar(filter) [string trim $flt]
		    catch {close $fp}
		}
		default {
		    set cvar(filter) $item
		}
	    }
	    CATTable $cvarname
	}
	clear {CATOff $cvarname}
	retrieve -
	retreive {CATApply $cvarname 1}
	cancel {ARCancel $cvarname}
	print {CATPrint $cvarname}
	close {CATDestroy $cvarname}
	server {
	    incr i
	    set cat(server) [lindex $var $i]
	}
	symbol {
	    global $cvar(symdb)
	    set row 1
	    incr i
	    if {[string is integer [lindex $var $i]]} {
		set row [lindex $var $i]
		incr i
	    }

	    switch -- [lindex $var $i] {
		condition {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) condition] \
			[lindex $var $i]
		    CATPlot $cvarname
		}
		shape {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) shape] [lindex $var $i]
		    CATPlot $cvarname
		}
		color {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) color] [lindex $var $i]
		    CATPlot $cvarname
		}
		text {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) text] [lindex $var $i]
		    CATPlot $cvarname
		}
		size {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) size] [lindex $var $i]
		    CATPlot $cvarname
		}
		size2 {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) size2] [lindex $var $i]
		    CATPlot $cvarname
		}
		units {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) units] [lindex $var $i]
		    CATPlot $cvarname
		}
		angle {
		    incr i
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) angle] [lindex $var $i]
		    CATPlot $cvarname
		}

		add {
		    set row [expr [starbase_nrows $cvar(symdb)]+1]
		    starbase_rowins $cvar(symdb) $row
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) shape] $cat(sym,shape)
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) color] $cat(sym,color)
		    starbase_set $cvar(symdb) $row \
			[starbase_colnum $cvar(symdb) units] $cat(sym,units)
		    CATPlot $cvarname
		}
		remove {
		    starbase_rowdel $cvar(symdb) $row
		    CATPlot $cvarname
		}
		save {
		    incr i
		    starbase_write $cvar(symdb) [lindex $var $i]
		}
		load {
		    incr i
		    set fn [lindex $var $i]
		    if {[file exists $fn]} {
			starbase_read $cvar(symdb) $fn
			CATPlot $cvarname
		    } else {
			Error "[msgcat::mc {Unable to open file}] $fn"
			return
		    }
		}
	    }
	}
	sort {
	    incr i
	    set cvar(sort) [lindex $var $i]
	    incr i
	    switch -- [lindex $var $i] {
		incr {
		    set cvar(sort,dir) "-increasing"
		}
		decr {
		    set cvar(sort,dir) "-decreasing"
		}
	    }
	    CATTable $cvarname
	}
	maxrows {
	    incr i
	    set cvar(max) [lindex $var $i]
	}
	allrows {
	    set cvar(allrows) 1
	}
	allcols {
	    set cvar(allcols) 1
	}
	show {
	    set cvar(show) 1
	    CATPlot $cvarname
	}
	hide {
	    set cvar(show) 0
	    CATPlot $cvarname
	}
	panto {
	    incr i
	    set cvar(panto) [FromYesNo [lindex $var $i]]
	}
	edit {
	    incr i
	    set cvar(edit) [FromYesNo [lindex $var $i]]
	    CATEdit $cvarname
	}
	ra {
	    incr i
	    set cvar(ra) [lindex $var $i]
	    CATPlot $cvarname
	}
	dec {
	    incr i
	    set cvar(dec) [lindex $var $i]
	    CATPlot $cvarname
	}
    }
}
