#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

global catcdssrch
    set catcdssrch(list,wave) [list none Radio IR optical UV EUV X-ray Gamma-ray]
    set catcdssrch(list,mission) [list none ANS ASCA BeppoSAX CGRO COBE Chandra Copernicus EUVE EXOSAT Einstein FAUST FUSE GINGA GOODS GRANAT HEAO HST HUT Hipparcos INTEGRAL IRAS ISO IUE MSX OAO-2 ORFEUS ROSAT RXTE SAS-2 SIRTF SMM SOHO Spitzer TD1 UIT ULYSSES WUPPE XMM]
    set catcdssrch(list,astro) [list none AGN Abundances Ages Associations Atomic_Data BL_Lac_objects Binaries:cataclysmic Binaries:eclipsing Binaries:spectroscopic Blue_objects Clusters_of_galaxies Constellations Diameters Earth Ephemerides Equivalent_widths Extinction Galaxies Galaxies:Markarian Galaxies:spectra Globular_Clusters Gravitational_lensing HII_regions Interstellar_Medium Magnetic_fields Masers Masses Models Multiple_Stars Nebulae Nonstellar Novae Obs_Log Open_Clusters Orbits Parallaxes Photometry Photometry:intermediate-band Photometry:narrow-band Photometry:surface Photometry:wide-band Planetary_Nebulae Planets+Asteroids Polarization Positional_Data Proper_Motions Pulsars QSOs Redshifts Rotational_Velocities Seyfert_Galaxies Spectral_Classification Spectrophotometry Spectroscopy Stars Stars:Emission Stars:WR Stars:early-type Stars:late-type Stars:peculiar Stars:variable Stars:white_dwarf Sun SuperNovae SuperNovae_Remnants Velocities YSOs _META_]

proc CATCDSSrchDialog {varname action} {
    upvar #0 $varname var
    global $varname

    global ds9
    global cat
    global catcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchDialog $varname"
    }

    # main dialog
    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    if [winfo exists $var(top)] {
	raise $var(top)
	return
    }

    # AR variables
    set var(status) {}
    set var(sync) 0

    # CATCDSSrch variables
    set var(tbldb) ${varname}tbldb

    set var(source) {}
    set var(words) {}
    set var(wave) {}
    set var(mission) {}
    set var(astro) {}

    # top
    set w $var(top)
    set title [msgcat::mc {Search for Catalogs}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CATCDSSrchDestroy $varname"

    # menu
    $w configure -menu $var(mb)
    menu $var(mb)

    # apple
    AppleMenu ${varname}(mb)

    # file
    $var(mb) add cascade -label [msgcat::mc {File}] \
	-menu $var(mb).file
    menu $var(mb).file
    $var(mb).file add command -label [msgcat::mc {Retrieve}] \
	-command "CATCDSSrchApply $varname 0"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $var(mb).file add command -label [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    $var(mb).file add separator
    $var(mb).file add command -label "[msgcat::mc {Load}]..." \
	-command "CATCDSSrchLoadFile $varname"
    $var(mb).file add command -label "[msgcat::mc {Save}]..." \
	-command "CATCDSSrchSaveFile $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"

    # edit
    AREditMenu $varname

    # catalog server
    CATServerMenu $varname

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.tbl -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    pack $w.buttons $w.status -side bottom -fill x -ipadx 4 -ipady 4
    pack $w.param -side top -fill x -ipadx 4 -ipady 4
    pack $w.tbl -side top -fill both -expand true -ipadx 4 -ipady 4

    # param
    frame $w.param.name 
    label $w.param.or -text [msgcat::mc {Or}]
    frame $w.param.words
    frame $w.param.srch 
    pack $w.param.name $w.param.or $w.param.words -side top -anchor w -pady 4
    pack $w.param.srch -side bottom -fill both -expand true -anchor w -pady 4

    # param name
    label $w.param.name.title -text [msgcat::mc {Name or Designation}]
    entry $w.param.name.source -textvariable ${varname}(source) -width 45
    pack $w.param.name.title $w.param.name.source -side top -anchor w

    # param keywords
    label $w.param.words.title \
	-text [msgcat::mc {Words matching title, description}]
    entry $w.param.words.key -textvariable ${varname}(words) -width 45
    pack $w.param.words.title $w.param.words.key -side top -anchor w

    # param search
    frame $w.param.srch.wave
    frame $w.param.srch.mission
    frame $w.param.srch.astro
    pack $w.param.srch.wave $w.param.srch.mission $w.param.srch.astro \
	-side left -fill both -expand true -padx 4

    # param search wave
    frame $w.param.srch.wave.f
    label $w.param.srch.wave.title -text [msgcat::mc {Wavelength}]
    pack $w.param.srch.wave.title -side top -anchor w
    pack $w.param.srch.wave.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.wave.f.scroll \
	-command "$w.param.srch.wave.f.list yview"
    set ${varname}(listbox,wave) [listbox $w.param.srch.wave.f.list \
			      -yscroll "$w.param.srch.wave.f.scroll set" \
			      -setgrid 1 \
			      -width 15 \
			      -height 10 \
			      -selectmode browse \
			      -exportselection 0 \
			      -listvariable catcdssrch(list,wave)]
    grid $w.param.srch.wave.f.list $w.param.srch.wave.f.scroll -sticky news
    grid rowconfigure $w.param.srch.wave.f 0 -weight 1
    grid columnconfigure $w.param.srch.wave.f 0 -weight 1

    # param search mission
    frame $w.param.srch.mission.f
    label $w.param.srch.mission.title -text [msgcat::mc {Mission}]
    pack $w.param.srch.mission.title -side top -anchor w
    pack $w.param.srch.mission.f -side bottom -fill both -expand true  -anchor w

    scrollbar $w.param.srch.mission.f.scroll \
	-command "$w.param.srch.mission.f.list yview"
    set ${varname}(listbox,mission) [listbox $w.param.srch.mission.f.list \
				 -yscroll "$w.param.srch.mission.f.scroll set"\
				 -setgrid 1 \
				 -width 15 \
				 -height 10 \
				 -selectmode browse \
				 -exportselection 0 \
				 -listvariable catcdssrch(list,mission)]
    grid $w.param.srch.mission.f.list $w.param.srch.mission.f.scroll \
	-sticky news
    grid rowconfigure $w.param.srch.mission.f 0 -weight 1
    grid columnconfigure $w.param.srch.mission.f 0 -weight 1

    # param search astro
    frame $w.param.srch.astro.f
    label $w.param.srch.astro.title -text [msgcat::mc {Astronomical}]
    pack $w.param.srch.astro.title -side top -anchor w
    pack $w.param.srch.astro.f -side bottom -fill both -expand true -anchor w

    scrollbar $w.param.srch.astro.f.scroll \
	-command "$w.param.srch.astro.f.list yview"
    set ${varname}(listbox,astro) [listbox $w.param.srch.astro.f.list \
			       -yscroll "$w.param.srch.astro.f.scroll set" \
			       -setgrid 1 \
			       -width 15 \
			       -height 10 \
			       -selectmode browse \
			       -exportselection 0 \
			       -listvariable catcdssrch(list,astro)]
    grid $w.param.srch.astro.f.list $w.param.srch.astro.f.scroll -sticky news
    grid rowconfigure $w.param.srch.astro.f 0 -weight 1
    grid columnconfigure $w.param.srch.astro.f 0 -weight 1

    # tbl
    set var(tbl) [table $var(top).tbl.t \
		      -state disabled \
		      -usecommand 0 \
		      -variable $var(tbldb) \
		      -colorigin 1 \
		      -roworigin 0 \
		      -cols $catcdssrch(mincols) \
		      -rows $catcdssrch(minrows) \
		      -colstretchmode all \
		      -width -1 \
		      -height -1 \
		      -maxwidth 400 \
		      -maxheight 190 \
		      -titlerows 1 \
		      -xscrollcommand [list $var(top).tbl.xscroll set]\
		      -yscrollcommand [list $var(top).tbl.yscroll set]\
		      -selecttype row \
		      -selectmode extended \
		      -anchor w
		     ]
    #-browsecommand [list CATCDSSrchSelectCB $varname]

    scrollbar $var(top).tbl.yscroll -command [list $var(tbl) yview] \
	-orient vertical
    scrollbar $var(top).tbl.xscroll -command [list $var(tbl) xview] \
	-orient horizontal

    grid $var(tbl) $var(top).tbl.yscroll -sticky news
    grid $var(top).tbl.xscroll -stick news
    grid rowconfigure $var(top).tbl 0 -weight 1
    grid columnconfigure $var(top).tbl 0 -weight 1

    # status
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.item -anchor w -pady 4

    # buttons
    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "CATCDSSrchApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel $varname" -state disabled]
    button $w.buttons.catalog -text [msgcat::mc {Catalog}] \
	-command "CATCDSSrchCatalog $varname"
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command "CATCDSSrchClear $varname"
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "CATCDSSrchDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.catalog \
	$w.buttons.clear $w.buttons.close -side left -padx 10 -expand true

    ARStatus $varname {}

    # initialize
    $var(listbox,wave) selection set 0
    $var(listbox,mission) selection set 0
    $var(listbox,astro) selection set 0
    $w.param.name.source select range 0 end

    switch -- $action {
	apply {CATCDSSrchApply $varname 0}
	sync {CATCDSSrchApply $varname 1}
	none {}
    }
}

proc CATCDSSrchApply {varname sync} {
    upvar #0 $varname var
    global $varname

    global catcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchApply $varname $sync"
    }

    set id [$var(listbox,wave) curselection]
    if {$id > 0} {
	set var(wave) [lindex $catcdssrch(list,wave) $id]
    } else {
	set var(wave) {}
    }
    set id [$var(listbox,mission) curselection]
    if {$id > 0} {
	set var(mission) [lindex $catcdssrch(list,mission) $id]
    } else {
	set var(mission) {}
    }
    set id [$var(listbox,astro) curselection]
    if {$id > 0} {
	set var(astro) [lindex $catcdssrch(list,astro) $id]
    } else {
	set var(astro) {}
    }

    set var(sync) $sync
    ARApply $varname
    ARStatus $varname [msgcat::mc {Searching for catalogs}]

    CATCDSSrch $varname
}

proc CATCDSSrchDestroy {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchDestroy $varname"
    }

    catch {unset $var(tbldb)}

    ARDestroy $varname
}

proc CATCDSSrchLoadFile {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchLoad $varname"
    }

    catch {unset $var(tbldb)}

    set fn [OpenFileDialog catcdssrchfbox]
    if {$fn != {}} {
	if {[file exists $fn]} {
	    starbase_read $var(tbldb) $fn
	} else {
	    Error "[msgcat::mc {Unable to open file}] $fn"
	    return
	}

	CATCDSSrchTable $varname
    }
}

proc CATCDSSrchSaveFile {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    set fn [SaveFileDialog catcdssrchfbox]
    if {$fn != {}} {
	starbase_write $var(tbldb) $fn
    }
}

proc CATCDSSrchClear {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    catch {unset $var(tbldb)}
}

proc CATCDSSrchCatalog {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchCatalog $varname"
    }

    set row 0
    foreach ss [$var(tbl) curselection] {
	set rr [lindex [split $ss ,] 0]
	if {$rr != $row} {
	    set id [starbase_get $var(tbldb) $rr 1]
	    set title [starbase_get $var(tbldb) $rr 2]
	    if {$id != {}} {
		CATDialog "cat$id" {cds} $id $title apply
	    }
	    set row $rr
	}
    }
}

proc CATCDSSrch {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrch $varname"
    }

    global cat
    global http

    switch -- $cat(server) {
	cds {set site {vizier.u-strasbg.fr}}
	sao {set site {vizier.cfa.harvard.edu}}
	cadc {set site {vizier.hia.nrc.ca}}
	adac {set site {vizier.nao.ac.jp}}
	iucaa {set site {urania.iucaa.ernet.in}}
	moscow {set site {www.inasan.rssi.ru}}
	bejing {set site {data.bao.ac.cn}}
	cambridge {set site {archive.ast.cam.ac.uk}}
	ukirt {set site {www.ukirt.jach.hawaii.edu}}
    }

    set cgidir {viz-bin}
    set script {asu-tsv}

    set var(url) "http://$site/$cgidir/$script"
    
    # defaults
    set query {}
    append query {-meta&}
    append query {-out.max=1000&}
    if {$var(source) != {}} {
	append query "-source=$var(source)&"
    }
    if {$var(words) !={}} {
	append query "-words=$var(words)&"
    }
    if {$var(wave) !={}} {
	append query "-kw.Wavelength=$var(wave)&"
    }
    if {$var(mission) !={}} {
	append query "-kw.Mission=$var(mission)&"
    }
    if {$var(astro) !={}} {
	append query "-kw.Astronomical=$var(astro)&"
    }

    set var(query) $query
    CATCDSSrchLoad $varname
}

proc CATCDSSrchLoad {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchLoad $varname"
    }

    catch {unset $var(tbldb)}

    global http

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list CATCDSSrchReader $var(tbldb)] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    CATCDSSrchLoadFinish $varname $var(token)
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -query $var(query) \
					 -timeout $http(timeout) \
					 -handler \
					 [list CATCDSSrchReader $var(tbldb)] \
					 -command \
					 [list CATCDSSrchLoadFinish $varname] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    ARError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    }
}

proc CATCDSSrchLoadFinish {varname token} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchLoadFinish $varname"
    }

    if {!($var(active))} {
	ARCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	200 -
	203 -
	404 -
	503 {
	    ARDone $varname
	    CATCDSSrchTable $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    global debug
		    if {$debug(tcl,hv)} {
			puts "CATCDSSrchLoadFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    CATCDSSrchLoad $varname
		}
	    }
	}

	default {ARError $varname "[msgcat::mc {Error code was returned}] $code"}
    }
}

proc CATCDSSrchReader {t sock token} {
    upvar #0 $t T
    global $t

    set result 0

    if { ![info exists ${t}(state)]  } {
	set T(state) 0
    }

    switch -- $T(state) {
	0 {
	    # init db
	    fconfigure $sock -blocking 1
	    set T(Nrows) 0
	    set T(Ncols) 0
	    set T(Header) {}
	    set T(HLines) 0

	    set T(state) 1
	}

	1 {
	    # process header
	    incr ${t}(HLines)
	    set n $T(HLines)
	    
	    if { [gets $sock line] == -1 } {
		set T(state) -1
		set T(HLines) [expr $T(HLines) - 1]
		set T(Nrows) 0
		set T(Ncols) 0
		return
	    }

	    set result [string length "$line"]
	    set T(H_$n) $line
	    if {[string range $line 0 4] == "\#RESO"} {
		# remove units line
		unset T(H_$n)
		incr ${t}(HLines) -1
		
		# create header
		set T(H_$n) "Resource\tDescription\tDetail"
		set T(Header) [split $T(H_$n) "\t"]
		incr ${t}(HLines)
		set T(H_[expr $n+1]) "--------\t-----------\t------"
		set T(Dashes) [split $T(H_$n) "\t"]
		set T(Ndshs) [llength $T(Dashes)]
		
		starbase_colmap $t

		# process $line
		incr ${t}(Nrows)
		set r $T(Nrows)
		set T($r,1) [string trim [lindex [split $line {=}] 1]]
		set T($r,2) {}
		set T($r,3) {}

		set T(state) 3
	    }
	}

	2 {
	    # process RESOURCE
	    if {[gets $sock line] == -1} {
		set T(state) 0
		return
	    }

	    set result [string length "$line"]
	    set line [string trim $line]
	    if {$line != {}} {
		switch -- [string range $line 0 4] {
		    "#RESO" {
			incr ${t}(Nrows)
			set r $T(Nrows)
			set T($r,1) [string trim [lindex [split $line {=}] 1]]
			set T($r,2) {}
			set T($r,3) {}

			set T(state) 3
		    }
		}
	    }
	}

	3 {
	    # process Description
	    if {[gets $sock line] == -1} {
		set T(state) 0
		return
	    }

	    set result [string length "$line"]
	    set line [string trim $line]
	    set r $T(Nrows)
	    if {$line != {}} {
		switch -- [string range $line 0 4] {
		    "#INFO" {
			# eat it
		    }
		    "#Name" {
			# new style
			set T($r,1) [string trim [lindex [split $line {:}] 1]]
			set T($r,2) {}
			set T($r,3) {}

			set T(state) 5
		    }
		    default {
			if {$r>0} {
			    set val [string trim [string range $line 5 end]]
			    catch {set T($r,2) "$val"}
			    set T(state) 4
			}
		    }
		}
	    }
	}

	4 {
	    # process Detail
	    if {[gets $sock line] == -1} {
		set T(state) 0
		return
	    }

	    set r $T(Nrows)
	    if {$r>0} {
		set val [string trim [string range $line 5 end]]
		catch {set T($r,3) "$val"}
		set T(state) 4
	    }
	    set T(state) 2
	}

	5 {
	    # new style process description
	    if {[gets $sock line] == -1} {
		set T(state) 0
		return
	    }

	    set result [string length "$line"]
	    set line [string trim $line]
	    if {$line != {}} {
		switch -- [string range $line 0 4] {
		    "#Titl" {
			# eat it
		    }
		    default {
			set r $T(Nrows)
			if {$r>0} {
			    set val [string trim [string range $line 5 end]]
			    catch {set T($r,2) "$val"}
			    set T(state) 4
			}
		    }
		}
	    }
	}
    }

    return $result
}

proc CATCDSSrchTable {varname} {
    upvar #0 $varname var
    global $varname
    global $var(tbldb)

    global catcdssrch

    global debug
    if {$debug(tcl,cat)} {
	puts "CATCDSSrchTable $varname"
    }

#    starbase_writefp $var(tbldb) stdout

    global $var(tbldb)
    $var(tbl) configure -variable $var(tbldb)

    if {[starbase_nrows $var(tbldb)] == 0} {
	ARStatus $varname [msgcat::mc {No Items Found}]
	return
    }

    set nc [starbase_ncols $var(tbldb)]
    $var(tbl) configure -cols $nc

    # add header row
    set nr [expr [starbase_nrows $var(tbldb)]+1]
    if {$nr > $catcdssrch(minrows)} {
	$var(tbl) configure -rows $nr
    } else {
	$var(tbl) configure -rows $catcdssrch(minrows)
    }

    ARStatus $varname "[starbase_nrows $var(tbldb)] [msgcat::mc {Items Found}]"
}

