#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATNED {varname} {
    upvar #0 $varname var
    global $varname
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATNED $varname"
    }

    global http

    set var(reader) CATNEDReader

    set site {nedwww.ipac.caltech.edu}
    set cgidir {cgi-bin}
    set script {nph-objsearch}

    set var(url) "http://$site/$cgidir/$script"
    
    # defaults
    set query {}

    # skyframe
    switch -- $var(sky) {
	fk4 {
	    append query {in_csys=Equatorial}
	    append query {&in_equinox=}
	    append query [http::formatQuery {B1950.0}]
	}
	fk5 -
	icrs {
	    append query {in_csys=Equatorial}
	    append query {&in_equinox=}
	    append query [http::formatQuery {J2000.0}]
	}
	galactic {
	    append query {in_csys=Galactic}
	}
    }

    # coordinate
    switch $var(skyformat) {
	degrees {
	    set xx [uformat d h: $var(x)]
	    set yy [uformat d d: $var(y)]
	}
	sexagesimal {
	    set xx $var(x)
	    set yy $var(y)
	}
    }
    append query {&lon=}
    append query [http::formatQuery $xx]
    append query {&lat=}
    append query [http::formatQuery $yy]

    # size - convert to arcmin
    switch -- $var(rformat) {
	degrees {
	    set ww [expr $var(width)/60.]
	    set hh [expr $var(height)/60.]
	}
	arcmin {
	    set ww $var(width)
	    set hh $var(height)
	}
	arcsec {
	    set ww [expr $var(width)*60.]
	    set hh [expr $var(height)*60.]
	}
    }
    # now to radius
    set rr [expr sqrt($ww*$ww+$hh*$hh)/2.]
    append query "&radius="
    append query [http::formatQuery $rr]

    # finish
    append query {&out_csys=Equatorial&out_equinox=J2000.0&obj_sort=Distance+to+search+center&of=ascii_tab&zv_breaker=30000.0&list_limit=5&img_stamp=NO&search_type=Near+Position+Search&z_constraint=Unconstrained&z_value1=&z_value2=&z_unit=z&ot_include=ANY&nmp_op=ANY}

    set var(query) $query

    CATLoad $varname
}

proc CATNEDReader {t sock token} {
    upvar #0 $t T
    global $t

    global debug
    if {$debug(tcl,cat)} {
#	puts "CATNEDReader"
    }

    set result 0

    if { ![info exists ${t}(state)]  } {
	set T(state) 0
    }

    switch -- $T(state) {
	0 {
	    # init db
	    fconfigure $sock -blocking 1
	    set T(Nrows) 0
	    set T(Ncols) 0
	    set T(Header) {}
	    set T(HLines) 0

	    set T(state) 1
	}

	1 {
	    # process header
	    if {[gets $sock line] == -1} {
		set T(Nrows) 0
		set T(Ncols) 0
		set T(Header) {}
		set T(HLines) 0

		set T(state) -1
		return $result
	    }

	    set result [string length "$line"]

	    # start of data?
	    if {[string range $line 0 2] == {No.}} {
		# cols
		incr ${t}(HLines)
		set n $T(HLines)
		set T(H_$n) $line
		set T(Header) [split $T(H_$n) "\t"]

		# dashes
		set T(Dashes) [regsub -all {[A-Za-z0-9]} $T(H_$n) {-}]
		set T(Ndshs) [llength $T(Dashes)]
		starbase_colmap $t

		set T(state) 2
	    }
	}

	2 { 
	    # process table
	    if {[gets $sock line] == -1} {
		set T(state) 0
	    } else {
		set result [string length "$line"]
		set line [string trim $line]

		if {$line != {}} {
		    # ok, save it
		    incr ${t}(Nrows)
		    set r $T(Nrows)

		    set NCols [starbase_ncols $t]
		    set c 1
		    foreach val [split $line "\t"] {
			set T($r,$c) $val
			incr c
		    }
		    for {} {$c <= $NCols} {incr c} {
			set T($r,$c) {}
		    }
		}
	    }
	}
    }

    return $result
}

proc CATNEDAck {varname} {
    upvar #0 $varname var
    global $varname

    set msg {Acknowledgments for NED

This research has made use of the NED database,
operated at CDS, Strasbourg, France.
}
    SimpleTextDialog ${varname}ack [msgcat::mc {Acknowledgment}] 80 10 insert top $msg
}
