#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATSIMBAD {varname} {
    upvar #0 $varname var
    global $varname
    global cat

    global debug
    if {$debug(tcl,cat)} {
	puts "CATSIMBAD $varname"
    }

    global http

    set var(reader) CATSIMBADReader

    set site {simbad.u-strasbg.fr}
    set cgidir {simbad}
    set script {sim-script}

    set var(url) "http://$site/$cgidir/$script"
    
    # defaults
    set query {submit=submit+script&script=format+object+f1+}

    # format
    append query [http::formatQuery {"%COO(d;A)\t%COO(d;D)\t%IDLIST(1)\t%OTYPE(S)\t%PM(A)\t%PM(D)\t%PLX(V)\t%RV(Z)\t%FLUXLIST(B;F)\t%FLUXLIST(V;F)\t%SP(S)\n"}]

    # eol
    append query "%0D%0A"

    # coordinate
    # coord (degrees)
    switch $var(skyformat) {
	degrees {
	    set xx $var(x)
	    set yy $var(y)
	}
	sexagesimal {
	    set xx [h2d [Sex2H $var(x)]]
	    set yy [Sex2D $var(y)]
	}
    }
    append query "query+coo+"
    append query [http::formatQuery $xx]
    append query "+"
    if {$yy>0} {
	append query [http::formatQuery "+$yy"]
    } else {
	append query [http::formatQuery $yy]
    }

    # radius
    set ww $var(width)
    set hh $var(height)
    set rr [expr sqrt($ww*$ww+$hh*$hh)/2.]

    append query "+radius"
    append query [http::formatQuery "=$rr"]
    switch -- $var(rformat) {
	degrees {append query "d"}
	arcmin {append query "m"}
	arcsec {append query "s"}
    }

    # skyframe
    append query "+frame"
    switch -- $var(sky) {
	fk4 {append query [http::formatQuery "=FK4"]}
	fk5 {append query [http::formatQuery "=FK5"]}
	icrs {append query [http::formatQuery "=ICRS"]}
	galactic {append query [http::formatQuery "=GAL"]}
    }

    # eol
    append query "%0D%0A"

    set var(query) $query

    CATLoad $varname
}

proc CATSIMBADReader {t sock token} {
    upvar #0 $t T
    global $t

    global debug
    if {$debug(tcl,cat)} {
#	puts "CATSIMBADReader"
    }

    set result 0

    if { ![info exists ${t}(state)]  } {
	set T(state) 0
    }

    switch -- $T(state) {
	0 {
	    # init db
	    fconfigure $sock -blocking 1
	    set T(Nrows) 0
	    set T(Ncols) 0
	    set T(Header) {}
	    set T(HLines) 0

	    set T(state) 1
	}

	1 {
	    # process header
	    if {[gets $sock line] == -1} {
		set T(Nrows) 0
		set T(Ncols) 0
		set T(Header) {}
		set T(HLines) 0

		set T(state) -1
		return $result
	    }

	    set result [string length "$line"]

	    # error?
	    if {[string range $line 0 8] == {::error::}} {
		set T(Nrows) 0
		set T(Ncols) 0
		set T(Header) {}
		set T(HLines) 0

		set T(state) -1
		return $result
	    }

	    # start of data?
	    if {[string range $line 0 7] == {::data::}} {
		# cols
		set line "RA\tDEC\tIdentifier\tObject\tPMRA\tPMDEC\tPX\tRV(z)\tB\tV\tSpectralType"

		incr ${t}(HLines)
		set n $T(HLines)
		set T(H_$n) $line
		set T(Header) [split $T(H_$n) "\t"]

		# dashes
		set T(Dashes) [regsub -all {[A-Za-z0-9]} $T(H_$n) {-}]
		set T(Ndshs) [llength $T(Dashes)]
		starbase_colmap $t

		set T(state) 2
	    }
	}

	2 { 
	    # process table
	    if {[gets $sock line] == -1} {
		set T(state) 0
	    } else {
		set result [string length "$line"]
		set line [string trim $line]

		if {$line != {}} {
		    # ok, save it
		    incr ${t}(Nrows)
		    set r $T(Nrows)

		    set NCols [starbase_ncols $t]
		    set c 1
		    foreach val [split $line "\t"] {
			set T($r,$c) $val
			incr c
		    }
		    for {} {$c <= $NCols} {incr c} {
			set T($r,$c) {}
		    }
		}
	    }
	}
    }

    return $result
}

proc CATSIMBADAck {varname} {
    upvar #0 $varname var
    global $varname

    set msg {Acknowledgments for SIMBAD

This research has made use of the SIMBAD database,
operated at CDS, Strasbourg, France.
}
    SimpleTextDialog ${varname}ack [msgcat::mc {Acknowledgment}] 80 10 insert top $msg
}
