#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CompositeDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exists $w] {
	raise $w
	return
    }

    # variables
    set marker($frame,$id,global) [$frame get marker $id composite global]

    # base
    MarkerBaseCenterDialog $frame $id 275 145

    # init
    MarkerBaseCenterRotateCB $frame $id

   # callbacks
    $frame marker $id callback rotate CompositeRotateCB $frame

    # Angle
    label $w.ref.f.angleTitle -text [msgcat::mc {Angle}]
    entry $w.ref.f.angle -textvariable marker($frame,$id,angle) -width 13
    label $w.ref.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]
    label $w.ref.f.compTitle -text [msgcat::mc {Angle Complement}]
    label $w.ref.f.comp -relief groove -textvariable marker($frame,$id,comp) \
	-width 13 -anchor w
    label $w.ref.f.compUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $w.ref.f.angleTitle $w.ref.f.angle x $w.ref.f.angleUnit \
	-padx 4 -sticky w
    grid $w.ref.f.compTitle $w.ref.f.comp  x $w.ref.f.compUnit \
	-padx 4 -sticky w

    CompositeCompAngle $frame $id

    # Global
    label $w.ref.f.globaltitle -text [msgcat::mc {Global Properties}]
    checkbutton $w.ref.f.global -variable marker($frame,$id,global) \
	-command "CompositeGlobal $frame $id"
    grid $w.ref.f.globaltitle $w.ref.f.global -padx 4 -sticky w
}

# actions

proc CompositeClose {frame id} {
    global marker
    set which $marker($frame,$id,which)

    $frame marker $id delete callback rotate CompositeRotateCB
    MarkerBaseCenterClose $frame $id
}

proc CompositeApply {frame id} {
    CompositeRotate $frame $id
    MarkerBaseCenterApply $frame $id
}

proc CompositeRotate {frame id} {
    global marker

    $frame marker $id angle $marker($frame,$id,angle) \
	$marker($frame,$id,system) $marker($frame,$id,sky)

    CompositeCompAngle $frame $id
}

proc CompositeCompAngle {frame id} {
    global marker

    set comp [expr 360-$marker($frame,$id,angle)]

    if {$comp >= 360} {
	set comp [expr $comp - 360]
    }
    if {$comp < 0} {
	set comp [expr $comp + 360]
    }

    set marker($frame,$id,comp) $comp
}

proc CompositeGlobal {frame id} {
    global marker
    
    $frame marker $id composite global $marker($frame,$id,global)
}

# callbacks

proc CompositeCoordCB {frame id} {
    global debug
    if {$debug(tcl,marker)} {
	puts "CompositeCoordCB"
    }
    MarkerBaseCoordCB $frame $id
    MarkerBaseCenterMoveCB $frame $id
    MarkerBaseCenterRotateCB $frame $id
}

proc CompositeRotateCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "CompositeRotateCB"
    }

    set marker($frame,$id,angle) \
	[$frame get marker $id angle $marker($frame,$id,system) \
	     $marker($frame,$id,sky)]

    CompositeCompAngle $frame $id
}
