#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# special kludge to work around a infinit loop caused by cube.tcl and idletasks

proc ContourUpdate {} {
    SetWatchCursor
    ContourUpdateNow
    UnsetWatchCursor
}

proc ContourUpdateNow {} {
    global contour
    global contourdlg
    global current

    if {$current(frame) != {}} {
	if {$contour(view) && [$current(frame) has fits]} {
	    ContourCheckParams
	    if {[winfo exists $contour(top)]} {
		set levels {}
		# remove endl
		regsub -all "\n" "[$contourdlg(txt) get 1.0 end]" " " levels
		# and trim any trailing spaces
		set levels [string trimright $levels " "]

		$current(frame) contour level "\"$levels\""
		if {$levels != {}} {
		    $current(frame) contour create \
			$contour(color) $contour(width) \
			$contour(smooth) $contour(method) "\"$levels\""
		}
		UpdateContourDialogMenu
	    } else {
		$current(frame) contour create \
		    $contour(color) $contour(width) \
		    $contour(levels) $contour(smooth) $contour(method)
	    }
	} else {
	    $current(frame) contour delete
	}
    }
}

proc ContourCheckParams {} {
    global contour

    if {$contour(smooth) < 1} {
	set contour(smooth) 1
    }
    if {$contour(levels) < 1} {
	set contour(levels) 1
    }
}

proc ContourDialog {} {
    global contour
    global contourdlg
    global current
    global ds9

    # see if we already have a ctr window visible
    if [winfo exists $contour(top)] {
	raise $contour(top)
	return
    }

    set w $contour(top)
    set title [msgcat::mc {Contour Parameters}]

    # create the contour window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ContourDestroyDialog
    wm minsize $w 375 200

    # local variables
    set contourdlg(txt) $w.param.right.level.value.text
    set txtscr $w.param.right.level.value.yscroll

    $w configure -menu $contour(mb)

    menu $contour(mb)

    AppleMenu contour(mb)

    $contour(mb) add cascade -label [msgcat::mc {File}] \
	-menu $contour(mb).file
    $contour(mb) add cascade -label [msgcat::mc {Edit}] \
	-menu $contour(mb).edit
    $contour(mb) add cascade -label [msgcat::mc {Color}] \
	-menu $contour(mb).color
    $contour(mb) add cascade -label [msgcat::mc {Width}] \
	-menu $contour(mb).width
    $contour(mb) add cascade -label [msgcat::mc {Scale}] \
	-menu $contour(mb).scale
    $contour(mb) add cascade -label [msgcat::mc {Limits}] \
	-menu $contour(mb).limit
    $contour(mb) add cascade -label [msgcat::mc {Method}] \
	-menu $contour(mb).method

    menu $contour(mb).file
    $contour(mb).file add command -label [msgcat::mc {Apply}] \
	-command ContourApplyDialog
    $contour(mb).file add command -label [msgcat::mc {Generate}] \
	-command ContourGenerateDialog
    $contour(mb).file add command -label [msgcat::mc {Clear}] \
	-command ContourOffDialog
    $contour(mb).file add separator
    $contour(mb).file add command -label [msgcat::mc {Copy Contours}] \
	-command ContourCCopyDialog
    $contour(mb).file add command -label "[msgcat::mc {Paste Contours}]..." \
	-command ContourCPasteDialog
    $contour(mb).file add separator
    $contour(mb).file add command -label "[msgcat::mc {Load Contours}]..." \
	-command ContourLoadDialog
    $contour(mb).file add command -label "[msgcat::mc {Save Contours}]..." \
	-command ContourSaveDialog
    $contour(mb).file add separator
    $contour(mb).file add command \
	-label "[msgcat::mc {Load Contour Levels}]..." \
	-command ContourLoadLevels
    $contour(mb).file add command \
	-label "[msgcat::mc {Save Contour Levels}]..." \
	-command ContourSaveLevels
    $contour(mb).file add separator
    $contour(mb).file add command -label [msgcat::mc {Convert to Polygons}] \
	-command Contour2Polygons
    $contour(mb).file add separator
    $contour(mb).file add command -label [msgcat::mc {Close}] \
	-command ContourDestroyDialog

    menu $contour(mb).edit
    $contour(mb).edit add command -label [msgcat::mc {Cut}] \
	-command ContourCutDialog -accelerator "${ds9(ctrl)}X"
    $contour(mb).edit add command -label [msgcat::mc {Copy}] \
	-command ContourCopyDialog -accelerator "${ds9(ctrl)}C"
    $contour(mb).edit add command -label [msgcat::mc {Paste}] \
	-command ContourPasteDialog -accelerator "${ds9(ctrl)}V"

    DefaultColorMenu $contour(mb).color contour(color) ContourColorDialog
    DefaultWidthMenu $contour(mb).width contour(width) ContourWidthDialog

    menu $contour(mb).scale
    $contour(mb).scale add radiobutton -label [msgcat::mc {Linear}] \
	-variable contour(scale) -value linear
    $contour(mb).scale add radiobutton -label [msgcat::mc {Log}] \
	-variable contour(scale) -value log
    $contour(mb).scale add radiobutton -label [msgcat::mc {Power}] \
	-variable contour(scale) -value pow
    $contour(mb).scale add radiobutton -label [msgcat::mc {Square Root}] \
	-variable contour(scale) -value sqrt
    $contour(mb).scale add radiobutton -label [msgcat::mc {Squared}] \
	-variable contour(scale) -value squared
    $contour(mb).scale add radiobutton \
	-label [msgcat::mc {Histogram Equalization}] \
	-variable contour(scale) -value histequ

    menu $contour(mb).limit
    $contour(mb).limit add radiobutton -label [msgcat::mc {Min Max}] \
	-variable contour(mode) -value minmax -command ContourModeDialog
    $contour(mb).limit add separator
    $contour(mb).limit add radiobutton -label {99.5%} \
	-variable contour(mode) -value 99.5 -command ContourModeDialog
    $contour(mb).limit add radiobutton -label {99%} \
	-variable contour(mode) -value 99 -command ContourModeDialog
    $contour(mb).limit add radiobutton -label {98%} \
	-variable contour(mode) -value 98 -command ContourModeDialog
    $contour(mb).limit add radiobutton -label {95%} \
	-variable contour(mode) -value 95 -command ContourModeDialog
    $contour(mb).limit add radiobutton -label {90%} \
	-variable contour(mode) -value 90 -command ContourModeDialog
    $contour(mb).limit add separator
    $contour(mb).limit add radiobutton -label {ZScale} \
	-variable contour(mode) -value zscale -command ContourModeDialog 
    $contour(mb).limit add radiobutton -label {ZMax} \
	-variable contour(mode) -value zmax -command ContourModeDialog 
    $contour(mb).limit add radiobutton -label [msgcat::mc {User}] \
	-variable contour(mode) -value user -command ContourModeDialog 

    menu $contour(mb).method
    $contour(mb).method add radiobutton -label [msgcat::mc {Block}] \
	-variable contour(method) -value block
    $contour(mb).method add radiobutton -label [msgcat::mc {Smooth}] \
	-variable contour(method) -value smooth

    frame $w.param
    frame $w.buttons -relief ridge -bd 1
    pack $w.param -fill x -expand true
    pack $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    frame $w.param.left
    frame $w.param.left.slide -relief ridge -bd 1
    frame $w.param.left.clip -relief ridge -bd 1
    frame $w.param.right
    frame $w.param.right.level -relief ridge -bd 1

    pack $w.param.left.slide $w.param.left.clip \
	-fill x -expand true -ipadx 4 -ipady 4
    pack $w.param.right.level -fill both -expand true
    pack $w.param.left $w.param.right -side left -fill both -expand true

    scale $w.param.left.slide.num -from 0 -to 50 -length 300 \
	-variable contour(levels) -orient horizontal \
	-label [msgcat::mc {Contour Levels}] \
	-tickinterval 10 -showvalue true -resolution 1
    scale $w.param.left.slide.res -from 0 -to 32 -length 300 \
	-variable contour(smooth) -orient horizontal \
	-label [msgcat::mc {Contour Smoothness}] \
	-tickinterval 4 -showvalue true -resolution 1
    pack $w.param.left.slide.num $w.param.left.slide.res \
	-side top -ipadx 4 -ipady 4 -anchor w

    label $w.param.left.clip.title -text [msgcat::mc {Limits}]

    label $w.param.left.clip.lowtitle -text [msgcat::mc {Low}]
    entry $w.param.left.clip.low -textvariable contour(min) -width 10

    label $w.param.left.clip.hightitle -text [msgcat::mc {High}]
    entry $w.param.left.clip.high -textvariable contour(max) -width 10

    grid $w.param.left.clip.title -row 0 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.param.left.clip.lowtitle -row 0 -column 1 -padx 4 -pady 1 -sticky w
    grid $w.param.left.clip.low -row 0 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.param.left.clip.hightitle -row 0 -column 3 \
	-padx 4 -pady 1 -sticky w
    grid $w.param.left.clip.high -row 0 -column 4 -padx 4 -pady 1 -sticky w

    label $w.param.right.level.title -text [msgcat::mc {Levels}]
    frame $w.param.right.level.value
    text $contourdlg(txt) -height 10 -width 15 -wrap none -bd 2 \
	-font {courier 12} -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $contourdlg(txt) yview] -orient vertical

    grid $w.param.right.level.value.text \
	$w.param.right.level.value.yscroll -sticky news
    grid $w.param.right.level.title -row 0 -column 0 -padx 4 -sticky w
    grid $w.param.right.level.value -row 1 -column 0 -padx 4 -sticky w

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command ContourApplyDialog
    button $w.buttons.generate -text [msgcat::mc {Generate}] \
	-command ContourGenerateDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command ContourOffDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command ContourDestroyDialog
    pack $w.buttons.apply $w.buttons.generate $w.buttons.clear \
	$w.buttons.close -side left -padx 10 -expand true

    UpdateContourMenu
    UpdateContourDialog
}

proc ContourApplyDialog {} {
    global contour

    set contour(view) 1
    ContourUpdate
}

proc ContourDestroyDialog {} {
    global contour

    if {[winfo exists $contour(top)]} {
	destroy $contour(top)
	destroy $contour(mb)
    }

    if {[info exists contourdlg]} {
	unset contourdlg
    }
}

proc ContourGenerateDialog {} {
    global contour
    global contourdlg
    global current

    ContourCheckParams

    $contourdlg(txt) delete 1.0 end
    if {$current(frame) != {}} {
	$current(frame) contour level {""}
	if {([$current(frame) has fits]) && 
	    ($contour(min) != {}) && 
	    ($contour(max) != {})} {
	    set ll [$current(frame) get colorscale level $contour(levels) \
			$contour(min) $contour(max) $contour(scale)]
	    regsub -all " " "$ll" "\n" ll
	    $contourdlg(txt) insert end "$ll"
	}
    }
}

proc ContourOffDialog {} {
    global contour
    global current

    set contour(view) 0
    if {$current(frame) != {}} {
	$current(frame) contour delete all
    }
    UpdateContourDialog
}

proc ContourCutDialog {} {
    global contour
    global contourdlg

    set w [focus -displayof $contour(top)]
    if {$w == $contourdlg(txt)} {
	tk_textCut $w
    } else {
	tk_entryCut $w
    }
}

proc ContourCopyDialog {} {
    global contour
    global contourdlg

    set w [focus -displayof $contour(top)]
    if {$w == $contourdlg(txt)} {
	tk_textCopy $w
    } else {
	tk_entryCopy $w
    }
}

proc ContourPasteDialog {} {
    global contour
    global contourdlg

    set w [focus -displayof $contour(top)]
    if {$w == $contourdlg(txt)} {
	tk_textPaste $w
    } else {
	tk_entryPaste $w
    }
}

proc ContourCCopyDialog {} {
    global contour
    global current

    set contour(copy) $current(frame)
    UpdateContourDialog
}

proc ContourCPasteDialog {} {
    global contour
    global current

    if {$current(frame) != {} && $contour(copy) != {}} {
	ContourParamsDialog paste {}
    }
}

proc ContourColorDialog {} {
    global contour
    global current
    
    if {$current(frame) != {}} {
	$current(frame) contour color $contour(color)
    }
}

proc ContourWidthDialog {} {
    global contour
    global current
    
    if {$current(frame) != {}} {
	$current(frame) contour width $contour(width)
    }
}

proc ContourModeDialog {} {
    global current
    global contour

    if {($current(frame) != {}) && [$current(frame) has fits]} {
	set limits [$current(frame) get clip $contour(mode)]
	set contour(min) [lindex $limits 0]
	set contour(max) [lindex $limits 1]
    } else {
	set contour(min) {}
	set contour(max) {}
    }
}

proc ContourLoadLevels {} {
    set fn [OpenFileDialog contourlevfbox]
    ContourLoadLevelsNow $fn
}

proc ContourLoadLevelsNow {fn} {
    global contourdlg

    if {$fn != {}} {
	set id [open $fn r]
	$contourdlg(txt) delete 1.0 end
	$contourdlg(txt) insert end [read $id]
	close $id
    }
}

proc ContourSaveLevels {} {
    set fn [SaveFileDialog contourlevfbox]
    ContourSaveLevelsNow $fn
}

proc ContourSaveLevelsNow {fn} {
    global contourdlg

    if {$fn != {}} {
	set id [open $fn w]
	puts -nonewline $id "[$contourdlg(txt) get 1.0 end]"
	close $id
    }
}

proc ContourSaveDialog {} {
    global current

    set fn [SaveFileDialog contourfbox]

    if {$fn != {}} {
	if {$current(frame) != {}} {
	    ContourParamsDialog save $fn
	}
    }
}

proc ContourLoadDialog {} {
    global current

    set fn [OpenFileDialog contourfbox]

    if {$fn != {}} {
	if {$current(frame) != {}} {
	    ContourParamsDialog load $fn
	}
    }
}

proc Contour2Polygons {} {
    global current
    global contour

    if {$current(frame) != {}} {
	$current(frame) contour create polygon \
	    color = $contour(color) width = $contour(width)
	$current(frame) contour delete
    }    
}

proc UpdateContourMenu {} {
    global contour
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateContourMenu"
    }

    if {($current(frame) != {}) && [$current(frame) has fits]} {
	set contour(view) [$current(frame) has contour]

	set contour(scale) [$current(frame) get colorscale]
	set contour(mode) [$current(frame) get clip mode]
	set limits [$current(frame) get clip $contour(mode)]
	set contour(min) [lindex $limits 0]
	set contour(max) [lindex $limits 1]
    } else {
	set contour(min) {}
	set contour(max) {}
    }
}

proc UpdateContourDialog {} {
    global contour
    global contourdlg
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateContourDialog"
    }

    if {[winfo exists $contour(top)] && $current(frame) != {}} {
	set levels [$current(frame) get contour level]
	regsub -all "\n" "$levels" " " levels
	set levels [join $levels "\n"]
	if {$levels != {}} {
	    $contourdlg(txt) delete 1.0 end
	    $contourdlg(txt) insert end $levels
	    UpdateContourDialogMenu
	} else {
	    ContourGenerateDialog
	}
    }
}

proc UpdateContourDialogMenu {} {
    global current
    global contour

    if [winfo exists $contour(top)] {
	if {$current(frame) != {}} {
	    if {[$current(frame) has contour]} {
		set contour(method) [$current(frame) get contour method]
		set contour(color) [$current(frame) get contour color]
		set contour(width) [$current(frame) get contour width]
	    }
	}
    }
}

proc ContourParamsDialog {action fn} {
    global ctld
    global current
    global contour
    global ds9

    set w {.ctld}

    set ctld(ok) 0
    set ctld(system) wcs
    set ctld(sky) fk5
    set ctld(color) green
    set ctld(width) 1

    DialogCreate $w [msgcat::mc {Contour Parameters}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons  -side top -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.dummy -text {} -width 40
    label $w.param.coordtitle -text [msgcat::mc {Coordinate System}]
    menubutton $w.param.coordbutton -relief raised \
	-menu $w.param.coordbutton.menu -textvariable ctld(system)

    DefaultCoordMenu2 $w.param.coordbutton.menu ctld(system) ctld(sky) {}

    label $w.param.colortitle -text [msgcat::mc {Color}]
    menubutton $w.param.colorbutton -relief raised \
	-textvariable ctld(color) -menu $w.param.colorbutton.menu
    DefaultColorMenu $w.param.colorbutton.menu ctld(color) {}

    label $w.param.widthtitle -text [msgcat::mc {Width}]
    tk_optionMenu $w.param.widthbutton ctld(width) \
	Thin 1 2 3 4
    $w.param.widthbutton.menu configure

    grid rowconfigure $w.param 0 -pad 0
    grid rowconfigure $w.param 1 -pad 2
    grid rowconfigure $w.param 2 -pad 2
    grid rowconfigure $w.param 3 -pad 2

    grid $w.param.dummy -columnspan 2
    grid $w.param.coordtitle $w.param.coordbutton -padx 4 -sticky w
    if {$action != "save"} {
	grid $w.param.colortitle $w.param.colorbutton -padx 4 -sticky w
	grid $w.param.widthtitle $w.param.widthbutton -padx 4 -sticky w
    }

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ctld(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ctld(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ctld(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ctld(ok)
    DialogDismiss $w

    if {$ctld(ok)} {
	set ctld(color) [string tolower $ctld(color)]
	switch -- $ctld(width) {
	    "Thin" {set ctld(width) 0}
	}

	SetWatchCursor
	switch -- $action {
	    paste {
		set ptr [$contour(copy) contour copy $ctld(system) $ctld(sky)]
		if {$ptr != {}} {
		    $current(frame) contour paste \
			$ctld(color) $ctld(width) $ptr $ctld(system) $ctld(sky)
		}
	    }
	    load {
		$current(frame) contour load \
		    $ctld(color) $ctld(width) "\{$fn\}" \
		    $ctld(system) $ctld(sky)
	    }
	    save {
		$current(frame) contour save "\{$fn\}" $ctld(system) $ctld(sky)
	    }
	}
	UnsetWatchCursor
	UpdateContourDialog
    }

    # we want to destroy this window

    destroy $w 

    unset ctld
}

proc ProcessContourCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global contour
    global current

    switch -- [string tolower [lindex $var $i]] {
	clear {
	    ContourOffDialog
	}
	load {
	    incr i
	    set fn [lindex $var $i]
	    incr i
	    set sys [lindex $var $i]
	    incr i
	    set sky [lindex $var $i]
	    incr i
	    set color [lindex $var $i]
	    incr i
	    set width [lindex $var $i]

	    RealizeDS9
	    ProcessContourFix sys sky color width
	    if {$fn != {}} {
		$current(frame) contour load $color $width "\{$fn\}" $sys $sky
	    }
	    FileLastFull contourfbox $fn
	    UpdateContourDialog
	}
	save {
	    incr i
	    set fn [lindex $var $i]
	    incr i
	    set sys [lindex $var $i]
	    incr i
	    set sky [lindex $var $i]
	    set color green
	    set width 1

	    ProcessContourFix sys sky color width
	    if {$fn != {}} {
		$current(frame) contour save "\{$fn\}" $sys $sky
	    }
	    FileLastFull contourfbox $fn
	}
	convert {
	    Contour2Polygons
	}
	loadlevels {
	    ContourDialog

	    incr i
	    ContourLoadLevelsNow [lindex $var $i]

	    ContourUpdate
	}
	savelevels {
	    ContourDialog

	    incr i
	    ContourSaveLevelsNow [lindex $var $i]
	}

	copy {
	    ContourCCopyDialog
	}
	paste {
	    incr i
	    if {$current(frame) != {} && $contour(copy) != {}} {
		set sys [lindex $var $i]
		incr i
		set sky [lindex $var $i]
		incr i
		set color [lindex $var $i]
		incr i
		set width [lindex $var $i]

		ProcessContourFix sys sky color width
		set ptr [$contour(copy) contour copy $sys $sky]
		if {$ptr != {}} {
		    $current(frame) contour paste $color $width $ptr $sys $sky
		}
	    }
	}

	color {
	    incr i
	    set contour(color) [lindex $var $i]
	    ContourColorDialog
	}
	width {
	    incr i
	    set contour(width) [lindex $var $i]
	    ContourWidthDialog
	}

	smooth {
	    incr i
	    set contour(smooth) [lindex $var $i]
	    ContourUpdate
	}
	method {
	    incr i
	    set contour(method) [lindex $var $i]
	    ContourUpdate
	}

	nlevels {
	    ContourDialog

	    incr i
	    set contour(levels) [lindex $var $i]
	    ContourGenerateDialog

	    ContourUpdate
	}
	scale {
	    ContourDialog

	    incr i
	    set contour(scale) [lindex $var $i]
	    ContourGenerateDialog

	    ContourUpdate
	}
	mode {
	    ContourDialog

	    incr i
	    set contour(mode) [lindex $var $i]
	    ContourModeDialog
	    ContourGenerateDialog

	    ContourUpdate
	}
	limits {
	    ContourDialog

	    incr i
	    set contour(min) [lindex $var $i]
	    incr i
	    set contour(max) [lindex $var $i]
	    ContourGenerateDialog

	    ContourUpdate
	}

	levels {
	    ContourDialog

	    global contourdlg
	    $contourdlg(txt) delete 1.0 end
	    incr i
	    $contourdlg(txt) insert end [lindex $var $i]

	    ContourUpdate
	}

	generate {
	    ContourDialog
	    ContourGenerateDialog
	    ContourUpdate
	}

	yes -
	true -
	1 -
	no -
	false -
	0 {
	    set contour(view) [FromYesNo [lindex $var $i]]
	    ContourUpdate
	}

	default {
	    set contour(view) 1
	    ContourUpdate
	    incr i -1
	}
    }
}

proc ProcessContourFix {sysname skyname colorname widthname} {
    upvar $sysname sys
    upvar $skyname sky
    upvar $colorname color
    upvar $widthname width

    switch -- $sys {
	{} {set sys image}
	fk4 -
	fk5 -
	icrs -
	galactic -
	ecliptic {
	    set width $color;
	    set color $sky;
	    set sky $sys;
	    set sys wcs
	}
    }

    switch -- $sky {
	{} {set sky fk5}
	fk4 -
	fk5 -
	icrs -
	galactic -
	ecliptic {}
	default {
	    set width $color
	    set color $sky
	    set sky fk5
	}
    }

    switch -- $color {
	{} {set color green}
	white -
	black -
	red -
	green -
	blue -
	cyan -
	magenta -
	yellow {}
	default {
	    set width $color
	    set color green
	}
    }

    if {$width == {}} {
	set width 1
    }
}
