#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

#!/bin/sh
# the next line restarts using wish
# exec wish8.4 "$0" "$@"

# who are we?
set ds9(wm) [tk windowingsystem]

# options
switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {
	option add {*Frame*Background} lightgrey
	option add {*Label*Background} lightgrey
	option add {*Entry*HighlightBackground} lightgrey
	option add {*Button*HighlightBackground} lightgrey
	option add {*Menubutton*Background} lightgrey
	option add {*Checkbutton*Background} lightgrey
	option add {*Text*HighlightBackground} lightgrey
	option add {*Text*BorderWidth} 2
	option add {*Text*Relief} sunken
    }
}
option add {*Entry*BorderWidth} 1
option add {*Scale*BorderWidth} 1
option add {*Radiobutton*BorderWidth} 1
option add {*Checkbutton*BorderWidth} 1
option add {*Scrollbar*BorderWidth} 1
option add {*Label*BorderWidth} 2

# debug level
set debug(tcl,events) 0
set debug(tcl,update) 0
set debug(tcl,idletasks) 0
set debug(tcl,layout) 0
set debug(tcl,info) 0
set debug(tcl,marker) 0
set debug(tcl,watch) 0
set debug(tcl,hv) 0
set debug(tcl,cat) 0
set debug(tcl,grid) 0
set debug(tcl,http) 0
set debug(tcl,ftp) 0
set debug(tcl,image) 0

set debug(saotk,ast) 0
set debug(saotk,mosaic) 0
set debug(saotk,parser) 0
set debug(saotk,perf) 0
set debug(saotk,wcs) 0
set debug(saotk,bin) 0
set debug(saotk,compress) 0
set debug(saotk,gz) 0
set debug(saotk,rgb) 0

set debug(iis) 0

# environment vars
# we don't want to see any error messages if xpa is not available
set env(XPA_VERBOSITY) 0
# set filter ptype to contained (default is process)
set env(FILTER_PTYPE) c

# initialize, depending on how we are running: choices are zvfs, or wish
# for now, its not possible to change the current work directory since
# all of the paths are relative
if {![catch {package present checkdns}]} {
    set ds9(root) "/usr/share/ds9"

    package require msgcat
    package require http

    

    package require base64
    package require log
    package require ftp
    package require textutil::repeat
    package require textutil::tabify

    package require tkcon

    package require BLT
    package require Tktable
    package require tclxpa
    package require img::base
    package require img::jpeg
    package require img::tiff
    package require img::png
    package require img::gif
    package require img::ppm
    package require img::bmp
    package require img::xbm
    package require img::window

    source $ds9(root)/src/source.tcl

    # fix ::tk and msgcat
    rename ::tk::msgcat::mc {}
    rename ::tk::msgcat::mcmax {}

    namespace import ::msgcat::mc
    namespace import ::msgcat::mcmax
    ::msgcat::mcload [file join $::tk_library msgs]

} else {
    set ds9(root) ".."

    lappend auto_path .

    package require msgcat
    package require http

    package require base64
    package require log
    package require ftp
    package require repeat
    package require tabify

    package require tkcon

    package require BLT
    package require Tktable
    package require checkdns
    package require saotk
    package require Tkhtml
    package require tclxpa
    package require iis
    package require tkmpeg
    package require img::base
    package require img::jpeg
    package require img::tiff
    package require img::png
    package require img::gif
    package require img::ppm
    package require img::bmp
    package require img::xbm
    package require img::window

    package require DS9

    # we want our version of the standard dialogs
    source $ds9(root)/src/tkfbox.tcl
    source $ds9(root)/src/xmfbox.tcl
}

# let's start
set ds9(init) 1

# Set Application Name
set ds9(app) [file rootname [file tail $argv0]]
tk appname $ds9(app)
set ds9(title) "$ds9(app)"
set ds9(version) {5.2}

# Visual
set ds9(visual) {}
set ds9(depth) 8

# Language
set ds9(language) locale
set ds9(language,name) [LanguageToName $ds9(language)]
set ds9(language,dir) {}

# Menu Font
switch $ds9(wm) {
    x11 {
	set ds9(font) default
	set ds9(font,size) 10
	set ds9(font,style) normal
    }
    win32 {
	set ds9(font) default
	set ds9(font,size) 10
	set ds9(font,style) normal
    }
    aqua {
	set ds9(font) default
	set ds9(font,size) 10
	set ds9(font,style) normal
    }
}

# Menu Shortcut
switch $ds9(wm) {
    x11 {
	set ds9(ctrl) "Control-"
	set ds9(shiftctrl) "Shift-Control-"
    }
    win32 {
	set ds9(ctrl) "Control-"
	set ds9(shiftctrl) "Control-"
    } 
    aqua {
	set ds9(ctrl) "Command-"
	set ds9(shiftctrl) "Shift-Command-"

	event add <<Redo>> <Command-Key-Z>
    }
}
event add <<Open>> <${ds9(ctrl)}o>
event add <<PageSetup>> <${ds9(ctrl)}P>
event add <<Print>> <${ds9(ctrl)}p>
event add <<SelectAll>> <${ds9(ctrl)}a>
event add <<Find>> <${ds9(ctrl)}f>
event add <<FindNext>> <${ds9(ctrl)}g>

# ds9 Variables
# General Note:
# ds9(frames) contains list of all frames
# ds9(active) contains list of active (show) frames
# active(<framename>) contains status show/hide for framename
# slave(<framename>) contains list of frames that are slaves to framename
# master(<framename>) contains the name of the master frame of framename

# main component names
set ds9(top) .
set ds9(mb) .mb
set ds9(main) {}
set ds9(image) {}
set ds9(canvas) {}
set ds9(panel) {}
set ds9(info) {}
set ds9(panner) {}
set ds9(magnifier) {}
set ds9(buttons) {}
set ds9(graph,sp) {}
set ds9(graph,horz) {}
set ds9(graph,vert) {}

set ds9(frames) {}
set ds9(active) {}
set ds9(num) 1
set ds9(lock) 0
set ds9(next) {}
set ds9(mode) pointer

set ds9(preload) {}
set ds9(preprint) {}

# set default standard dialog
switch $ds9(wm) {
    x11 {set ds9(dialog) motif}
    win32 {set ds9(dialog) native}
    aqua {set ds9(dialog) native}
}
set ds9(dialog,dir) {}

# Menu Variables
switch $ds9(wm) {
    x11 -
    win32 {
	set ds9(menu,size,frame,goto) 1
	set ds9(menu,size,frame,active) 4
	set ds9(menu,size,analysis) 30
    }
    aqua {
	set ds9(menu,size,frame,goto) 0
	set ds9(menu,size,frame,active) 3
	set ds9(menu,size,analysis) 29
    }
}
set ds9(menu,size,wrap) 20

set ds9(display,mode) single
set ds9(display,user) single
set ds9(automarker) 1
set ds9(confirm) 1

set ds9(bg,color) white
set ds9(nan,color) white

set ds9(xpa) 1
set ds9(xpa,tcl) 0
set ds9(xpa,ptr) 0
set ds9(xpa,idletasks) 1

set ds9(internalpad) 2
set ds9(freeze) 0

# imexam vars
set ds9(imexam) 0
set ds9(imexam,key) 0
set ds9(imexam,frame) {}
set ds9(imexam,x) {}
set ds9(imexam,y) {}
set ds9(imexam,event) {}
set ds9(imexam,mode) {}

# Button event control
set ds9(b1) 0
set ds9(b2) 0
set ds9(b3) 0

set ds9(sb1) 0
set ds9(sb2) 0
set ds9(sb3) 0

set ds9(cb1) 0
set ds9(cb2) 0
set ds9(cb3) 0

set ds9(csb1) 0
set ds9(csb2) 0
set ds9(csb3) 0

set ds9(meta) 0

# external analysis and file formats
set ds9(ext,file) ".$ds9(app).fil"
set ds9(ext,alt) ".$ds9(app).file"
set ds9(analysis,file) ".$ds9(app).ans"
set ds9(analysis,alt) ".$ds9(app).analysis"
set ds9(analysis,user) {}
set ds9(analysis,user2) {}
set ds9(analysis,user3) {}
set ds9(analysis,user4) {}

# temp dir
InitTempDir

# help
InitHelp

# View Variables
set view(layout) horizontal
set view(info) 1
set view(panner) 1
set view(magnifier) 1
set view(buttons) 1
set view(colorbar) 1
set view(graph,horz) 0
set view(graph,vert) 0

set view(info,filename) 1
set view(info,object) 1
set view(info,minmax) 0
set view(info,lowhigh) 0
set view(info,wcs) 1
foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
    set "view(info,wcs$l)" 0
}
set view(info,detector) 0
set view(info,amplifier) 0
set view(info,physical) 1
set view(info,image) 1
set view(info,frame) 1

set view(colorbar,numerics) 1

# Info Variables
set info(row) -1

# Internal file formats
set ds9(fileProcs) {Stdin ExternalFits GzipFits BZip2Fits CompressFits PackFits FTZFits}

# Pixel Table Variables
set pixel(top) .pixel
set pixel(mb) .pixelmb
set pixel(size) 5
set pixel(maxsize) 9
set pixel(copy) {}

# Cursor Variables
set cursor(save) {}
set cursor(count) 0
set cursor(id) 0
set cursor(timer) 0
set cursor(timer,abort) 0

# Crosshair Variables
set crosshair(lock) none

# Frame Variables
set current(frame) {}
set current(ext) {}
set current(colorbar) {}
set current(cursor) {}

set current(zoom) { 1 1 }
set current(rotate) 0
set current(orient) none

set current(rgb) red

blt::vector create histX histY

# RGB Variables
set rgb(red) 1
set rgb(green) 1
set rgb(blue) 1
set rgb(system) wcs
set rgb(lock,scale) 0
set rgb(lock,bin) 0
set rgb(lock,colorbar) 0
set rgb(lock,smooth) 0

set rgb(top) .rgb
set rgb(mb) .rgbmb

# Scale Variables
set scale(top) .scale
set scale(mb) .scalemb
set scale(min) 1
set scale(max) 100
set scale(xaxis) full
set scale(yaxis) log
set scale(preserve) 0

set scale(type) linear
set scale(mode) minmax
set scale(scope) local
set scale(datasec) 1

# MinMax Variables
set minmax(mode) auto
set minmax(sample) 25

# ZScale Variables
set zscale(contrast) .25
set zscale(sample) 600
set zscale(line) 120

# PanZoom Variables
set panzoom(top) .pz
set panzoom(mb) .pzmb
set panzoom(system) image
set panzoom(sky) fk5
set panzoom(skyformat) sexagesimal
set panzoom(mode) click
set panzoom(preserve) 0
set panzoom(state) 1
set panzoom(last) {0 0}
set panzoom(speed) 512
set panzoom(x) 0

# Group Variablables
set group(top) .grp
set group(mb) .grpmb

# Marker Variables
set marker(copy) {}
set marker(show) 1
set marker(show,text) 1
set marker(shape) circle
set marker(color) green
set marker(width) 1
set marker(edit) 1
set marker(move) 1
set marker(rotate) 1
set marker(delete) 1
set marker(fixed) 0
set marker(include) 1
set marker(source) 1
set marker(font) helvetica
set marker(font,size) 10
set marker(font,style) normal
set marker(preserve) 0
set marker(format) ds9
set marker(system) physical
set marker(sky) fk5
set marker(skyformat) degrees
set marker(strip) 0
set marker(wcs) 0
set marker(maxdialog) 48

# Default Dialog
set marker(dialog,system) physical
set marker(dialog,sky) fk5
set marker(dialog,skyformat) degrees
set marker(dialog,dist,system) physical
set marker(dialog,dist,format) degrees

# Default Marker Variables
set marker(circle,radius) 20

set marker(annulus,inner) 15
set marker(annulus,outer) 30
set marker(annulus,annuli) 1

set marker(panda,inner) 15
set marker(panda,outer) 30
set marker(panda,annuli) 1
set marker(panda,ang1) 0
set marker(panda,ang2) 360
set marker(panda,angnum) 4

set marker(ellipse,radius1) 40
set marker(ellipse,radius2) 20

set marker(ellipseannulus,radius1) 40
set marker(ellipseannulus,radius2) 20
set marker(ellipseannulus,radius3) 60
set marker(ellipseannulus,annuli) 1

set marker(epanda,radius1) 40
set marker(epanda,radius2) 20
set marker(epanda,radius3) 60
set marker(epanda,annuli) 1
set marker(epanda,ang1) 0
set marker(epanda,ang2) 360
set marker(epanda,angnum) 4

set marker(box,radius1) 80
set marker(box,radius2) 40

set marker(boxannulus,radius1) 80
set marker(boxannulus,radius2) 40
set marker(boxannulus,radius3) 120
set marker(boxannulus,annuli) 1

set marker(bpanda,radius1) 80
set marker(bpanda,radius2) 40
set marker(bpanda,radius3) 120
set marker(bpanda,annuli) 1
set marker(bpanda,ang1) 0
set marker(bpanda,ang2) 360
set marker(bpanda,angnum) 4

set marker(polygon,width) 20
set marker(polygon,height) 20
set marker(projection,thick) 0
set marker(compass,radius) 40

set marker(point,size) 11

InitTemplates

# WCS Variables
set wcs(system) wcs
set wcs(sky) fk5
set wcs(skyformat) sexagesimal
set wcs(align) 0
set wcs(align,system) wcs
set wcs(align,sky) fk5

# Colorbar Variables
set colorbar(top) .clrbar
set colorbar(mb) .clrbarmb
set colorbar(width) 512
set colorbar(height) 40
set colorbar(height,num) 20
set colorbar(gap) 4
set colorbar(num) 64
switch $ds9(wm) {
    x11 -
    win32 {
	set colorbar(start) 1
	set colorbar(count) 1
    }
    aqua {
	set colorbar(start) 0
	set colorbar(count) 0
    }
}
set colorbar(cmaps) 0
set colorbar(lock) 0
set colorbar(private) false
set colorbar(lock) 0
set colorbar(frame) {}
set colorbar(area) 0
set colorbar(mode) center

set colorbar(map) Grey
set colorbar(invert) 0

# Display Coords Variables
set coord(value) 1
set coord(image) 1
set coord(physical) 0
set coord(amplifier) 0
set coord(detector) 0
set coord(wcs) 1
foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
    set "coord(wcs$l)" 0
}
set coord(sky) fk5
set coord(skyformat) sexagesimal

# Canvas Variables
set canvas(bg) white
set canvas(font) "*helvetica-medium-r-*-100-*"

# Panner Variables
set panner(size) 128

set panner(compass,image) 1
set panner(compass,wcs) 1
set panner(compass,wcs,system) wcs
set panner(compass,wcs,sky) fk5

# Magnifier Variables
set magnifier(size) 128
set magnifier(region) 1
set magnifier(cursor) 1
set magnifier(zoom) 4

# Graph Variables
set graph(horz,id) 0
set graph(vert,id) 0

set graph(size) 150
set graph(gap,x) 50
set graph(gap,y) 25

set graph(x,min) 0
set graph(x,max) 10
set graph(y,min) 1
set graph(y,max) 100

set graph(horz,grid) 1
set graph(horz,log) false
blt::vector create graphHorzX graphHorzY

set graph(vert,grid) 1
set graph(vert,log) false
blt::vector create graphVertX graphVertY

# Printing Variables
switch $ds9(wm) {
    x11 {set ps(dest) printer}
    win32 {set ps(dest) file}
    aqua {set ps(dest) printer}
}

set ps(cmd) {lp}
set ps(filename) {ds9.ps}
set ps(color) rgb
set ps(color2) color
set ps(level) 2
set ps(resolution) 150

set ps(orient) portrait
set ps(scale) scaled
set ps(size) letter
set ps(width) 8.5
set ps(height) 11

# Tile Variables
set tile(top) .tile
set tile(mb) .tilemb
set tile(mode) grid
set tile(grid,row) 10
set tile(grid,col) 10
set tile(grid,mode) automatic
set tile(grid,gap) 4

# Blink Variables
set blink(interval) 500
set blink(index) -1
set blink(id) 0

# FITS Bin Table Support
set bin(top) .bl
set bin(mb) .blmb
set bin(function) sum
set bin(factor) { 1 1 }
set bin(depth) 1
set bin(buffersize) 1024

# Data Cube Variables
set cube(top) .cube
set cube(mb) .cubemb
set cube(slice) 1
set cube(depth) 1
set cube(id) 0

# Smooth Variables
set smooth(top) .sm
set smooth(mb) .smmb
set smooth(view) 0

set smooth(function) gaussian
set smooth(radius) 3

# Mask Variables
set mask(view) 1
set mask(color) red

# Contour Variables
set contour(top) .ct
set contour(mb) .ctmb
set contour(view) 0
set contour(smooth) 4
set contour(levels) 5
set contour(scale) linear
set contour(mode) minmax
set contour(min) {}
set contour(max) {}
set contour(copy) {}

set contour(method) block
set contour(color) green
set contour(width) 1

# Coordinate Grid
set grid(top) .grid
set grid(mb) .gridmb
set grid(view) 0

set grid(system) wcs
set grid(sky) fk5
set grid(skyformat) sexagesimal

set grid(type) analysis
set grid(type,numlab) interior

set grid(grid) 1
set grid(grid,color) 4
set grid(grid,width) 1
set grid(grid,style) 0

set grid(axes) 1
set grid(axes,color) 2
set grid(axes,width) 1
set grid(axes,style) 0

set grid(tick) 1
set grid(tick,color) 1
set grid(tick,width) 1
set grid(tick,style) 0

set grid(border) 1
set grid(border,color) 4
set grid(border,width) 1
set grid(border,style) 0

set grid(labelling) interior
set grid(labelup) 0
set grid(edge,1) bottom
set grid(edge,2) left

set grid(numlab) 1
set grid(numlab,font) helvetica
set grid(numlab,style) bold
set grid(numlab,size) 10
set grid(numlab,color) 3
set grid(numlab,gap1) {}
set grid(numlab,gap2) {}

set grid(title) 1
set grid(title,text) {}
set grid(title,def) 1
set grid(title,gap) {}
set grid(title,font) helvetica
set grid(title,style) bold
set grid(title,size) 12
set grid(title,color) 0

set grid(textlab) 1
set grid(label1) {}
set grid(textlab,def1) 1
set grid(gap1) {}
set grid(gapunit1) {}
set grid(label2) {}
set grid(textlab,def2) 1
set grid(gap2) {}
set grid(gapunit2) {}
set grid(textlab,font) helvetica
set grid(textlab,style) bold
set grid(textlab,size) 10
set grid(textlab,color) 0
set grid(textlab,gap1) {}
set grid(textlab,gap2) {}

# Examine Variables
set examine(mode) new
set examine(zoom) 4

# IIS Variables
set iis(state) 0
set iis(ififo) "/dev/imt1i"
set iis(ofifo) "/dev/imt1o"
set iis(port) 5137
set iis(unix) "/tmp/.IMT%d"
set iis(width) 512
set iis(height) 512
set iis(x) -1
set iis(y) -1
set iis(frame) {}

# Name Resolution Variables
set nres(top) .nres
set nres(mb) .nresmb
set nres(server) simbad-sao
set nres(system) wcs
set nres(sky) fk5
set nres(skyformat) degrees

# SkyView Variables
set skyview(top) .skyview
set skyview(mb) .skyviewmb
set skyview(system) wcs
set skyview(sky) fk5
set skyview(skyformat) sexagesimal
set skyview(rformat) arcsec
set skyview(width) 300
set skyview(height) 300
set skyview(mode) new
set skyview(save) 0
set skyview(valid) 0
set skyview(survey) SDSSi

# SAO Variables
set sao(top) .sao
set sao(mb) .saomb
set sao(system) wcs
set sao(sky) fk5
set sao(skyformat) sexagesimal
set sao(rformat) arcmin
set sao(width) 15
set sao(height) 15
set sao(mode) new
set sao(save) 0
set sao(valid) 0

# ESO Variables
set eso(top) .eso
set eso(mb) .esomb
set eso(system) wcs
set eso(sky) fk5
set eso(skyformat) sexagesimal
set eso(rformat) arcmin
set eso(width) 15
set eso(height) 15
set eso(mode) new
set eso(save) 0
set eso(valid) 0
set eso(survey) {DSS1}

# STSCI Variables
set stsci(top) .stsci
set stsci(mb) .stscimb
set stsci(system) wcs
set stsci(sky) fk5
set stsci(skyformat) sexagesimal
set stsci(rformat) arcmin
set stsci(width) 15
set stsci(height) 15
set stsci(mode) new
set stsci(save) 0
set stsci(valid) 0
set stsci(survey) {all}

# 2MASS Variables
set twomass(top) .twomass
set twomass(mb) .twomassmb
set twomass(system) wcs
set twomass(sky) fk5
set twomass(skyformat) sexagesimal
set twomass(rformat) arcmin
set twomass(width) 15
set twomass(height) 15
set twomass(survey) j
set twomass(mode) new
set twomass(save) 0
set twomass(valid) 0

# FIRST Variables
set first(top) .first
set first(mb) .firstmb
set first(system) wcs
set first(sky) fk5
set first(skyformat) sexagesimal
set first(rformat) arcmin
set first(width) 15
set first(height) 15
set first(mode) new
set first(save) 0
set first(valid) 1

# Catalog Variables
set cat(id) 0
set cat(cats) {}
set cat(server) sao
set cat(system) wcs
set cat(sky) fk5
set cat(skyformat) sexagesimal
set cat(rformat) arcmin

set cat(width) 15
set cat(height) 15
set cat(max) 5000
set cat(allrows) 1
set cat(allcols) 0
set cat(show) 1
set cat(edit) 0
set cat(panto) 1

set cat(minrows) 20
set cat(mincols) 10

set cat(source) {}
set cat(words) {}
set cat(wave) {}
set cat(mission) {}
set cat(astro) {}

set cat(sym,shape) {circle point}
set cat(sym,color) green
set cat(sym,units) physical

# catalog sym
set catsym(minrows) 6
set catsym(mincols) 8

# catalog search
set catcdssrch(minrows) 11
set catcdssrch(mincols) 3

# VO Variables
set vo(top) .vo
set vo(mb) .vomb
set vo(server) {http://cxc.harvard.edu/chandraed/list.txt}
set vo(server,title) {}
set vo(server,host) {}
set vo(server,url) {}
set vo(server,button) {}
set vo(hv) 1
#set vo(method) mime
set vo(method) xpa

# Analysis Variables
set analysis(menu,count) 0
set analysis(menu,hmenu,count) 0
set analysis(menu,hmenu) {}
set analysis(bind,count) 0
set analysis(param,count) 0
set analysis(param,seq) 0
set analysis(log) 0

# HTTP Variables
set http(timeout) 240000
set http(last) {}
set http(proxy) 0
set http(proxy,host) {}
set http(proxy,port) {}
set http(auth) 0
set http(auth,user) {}
set http(auth,passwd) {}

# HV Variables
set hv(incr) 0
foreach n {1 2 3 4} {
    set hv(archive,menu,$n) {}
    set hv(archive,url,$n) {}
}

# SaveImage Variables
set saveimage(format) jpeg
set saveimage(base) "ds9"
set saveimage(jpeg,quality) 75
set saveimage(tiff,compression) none
set saveimage(mpeg,quality) 2

# SaveMPEG Variables
set savempeg(quality) 2

# Plot Tool Variables
set ap(unique,id) 0
set ap(xpa) {}
set ap(tt) "ap"

# Preference Variables
array set pbin [array get bin]
array set pcontour [array get contour]
array set pmask [array get mask]
array set pminmax [array get minmax]
array set pps [array get ps]
array set pscale [array get scale]
array set psmooth [array get smooth]
array set pview [array get view]
array set pwcs [array get wcs]
array set pzscale [array get zscale]

set prefs(top) .pf
set prefs(version) [lindex $ds9(version) 0]
set prefs(ds9,mode) $ds9(mode)
set prefs(blink,interval) $blink(interval)
set prefs(tile,mode) $tile(mode)
set prefs(zoom) $current(zoom)
set prefs(orient) $current(orient)
set prefs(rotate) $current(rotate)

set prefs(colorbar,map) $colorbar(map)
set prefs(colorbar,invert) $colorbar(invert)
set prefs(analysis,log) $analysis(log)

# Marker Prefs
set prefs(marker,shape) $marker(shape)
set prefs(marker,color) $marker(color)
set prefs(marker,width) $marker(width)
set prefs(marker,edit) $marker(edit)
set prefs(marker,move) $marker(move)
set prefs(marker,rotate) $marker(rotate)
set prefs(marker,delete) $marker(delete)
set prefs(marker,fixed) $marker(fixed)
set prefs(marker,include) $marker(include)
set prefs(marker,source) $marker(source)
set prefs(marker,font) $marker(font)
set prefs(marker,font,size) $marker(font,size)
set prefs(marker,font,style) $marker(font,style)
set prefs(marker,format) $marker(format)
set prefs(marker,strip) $marker(strip)
set prefs(marker,wcs) $marker(wcs)
set prefs(marker,system) $marker(system)
set prefs(marker,sky) $marker(sky)
set prefs(marker,skyformat) $marker(skyformat)

# Analysis Plot prefs
set prefs(ap,grid) 1
set prefs(ap,grid,log) linearlinear

set prefs(ap,discrete) 0
set prefs(ap,discrete,symbol) circle
set prefs(ap,discrete,color) red

set prefs(ap,linear) 1
set prefs(ap,linear,width) 1
set prefs(ap,linear,color) black
set prefs(ap,linear,dash) no

set prefs(ap,step) 0
set prefs(ap,step,width) 1
set prefs(ap,step,color) black
set prefs(ap,step,dash) no

set prefs(ap,quadratic) 0
set prefs(ap,quadratic,width) 1
set prefs(ap,quadratic,color) black
set prefs(ap,quadratic,dash) no

set prefs(ap,error) 1
set prefs(ap,error,width) 1
set prefs(ap,error,color) red
set prefs(ap,error,style) 1

set prefs(ap,titleFont) helvetica
set prefs(ap,titleSize) 12
set prefs(ap,titleStyle) normal

set prefs(ap,textlabFont) helvetica
set prefs(ap,textlabSize) 10
set prefs(ap,textlabStyle) normal

set prefs(ap,numlabFont) helvetica
set prefs(ap,numlabSize) 10
set prefs(ap,numlabStyle) normal

# Initialize button vars before prefs
ButtonsVars

# Load any preferences here, before we do any real work
LoadPrefs

# adjust default canvas size based on layout
switch $view(layout) {
    horizontal {
	set canvas(width) 680
	set canvas(height) 450
    }
    vertical {
	set canvas(width) 540
	set canvas(height) 646
    }
}

# set default font
if {$ds9(font) != {default}} {
    option add *font "$ds9(font) $ds9(font,size) $ds9(font,style)"
}

# Toplevel-- Start here.
# We want to withdraw the window til everything is ready to go

wm withdraw $ds9(top)
switch $ds9(wm) {
    x11 -
    win32 {}
    aqua {::tk::unsupported::MacWindowStyle style $ds9(top) document "closeBox fullZoom collapseBox resizable"}
}

set title "SAOImage $ds9(title)"
wm title $ds9(top) "$title"
wm iconname $ds9(top) "$title"
wm protocol $ds9(top) WM_DELETE_WINDOW QuitDS9
unset title

$ds9(top) configure -menu $ds9(mb)

# we need to set certain variables before anything else
# such as color, title, language
ProcessCommandLineFirst

# initialize language
switch $ds9(language) {
    locale {
	switch $ds9(wm) {
	    aqua {
		foreach ll [MacOSXGetLocale] {
		    if {[SetLanguage $ll]} {
			break;
		    }
		}
	    }
	    default {}
	}
    }
    default {SetLanguage $ds9(language)}
}

# set the visual
set ds9(visual) [winfo visual .]
set ds9(depth) [winfo depth .]

switch $ds9(wm) {
    x11 {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    } 
    win32 {
	if {$ds9(depth)==32} {
	    set ds9(depth) 24
	}
    }
    aqua {
	if {$ds9(depth)==15} {
	    set ds9(depth) 16
	}
    }
}

switch -- $ds9(visual)$ds9(depth) {
    pseudocolor8 {}
    truecolor8 {}
    truecolor16 {}
    truecolor24 {}
    default {BadVisualError}
}

# create our main frame
set ds9(main) [frame ${ds9(top)}ds9]
pack $ds9(main) -fill both -expand true

# Create image canvas
CreateCanvas

# Create Colorbar-- Create this first, so in case of a private colormap,
# gui colors will be allocated in the new colormap, not the default colormap
CreateColorbar

# Create other parts of the display
CreateMenuBar
CreateInfoPanel
CreateButtons
CreateGraphs

# Make sure that the wm knows when to swap in the colormap (if needed)
wm colormapwindows . "$ds9(main) $ds9(canvas)"

# Initialize the display
InitColorbar
InitPanner
InitMagnifier
InitDialogBox

# Set our current state of things
ChangeMode

# force a update, then layout
update
LayoutView

# our first frame
CreateFrame

# do this last so we don't get an ConfigureView event
InitCanvas

# ok, ready to show the window
wm deiconify $ds9(top)
update

# Init external File Formats
# we want this before processing the command line
InitExternalFile

# Init analysis file formats
InitAnalysisFile

# Configure HTTP
ConfigHTTP

# and process any command line items
# we want to see something before any fits files are loaded
ProcessCommandLine

# Initialize IIS
# after command line options to set port/fifo/unix...
catch {IISInit}

# Initialize XPA
InitXPA

# and finally, process any command line options that are dependant on xpa
ProcessCommandLineXPA

# any preload/preprint commands received?
ProcessPreLoad
ProcessPrePrint

# Load any initalization tcl code
SourceInitFile {.ini}

# do we have the correct prefs file?
CheckPrefs

# ok, we're done
set ds9(init) 0
