#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Open

proc OpenFits {layer} {
    global current

    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    # just in case (could be invoked via a menu keyshortcut)
    if {$current(frame) == {}} {
	CreateFrame
    }

    StartLoad
    LoadFits $fn $layer
    FinishLoad
}

proc OpenDataCubeFits {layer} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadDataCubeFits $fn $layer
    FinishLoad
}

proc OpenMosaicImageIRAFFits {layer} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadMosaicImageIRAFFits $fn $layer
    FinishLoad
}

proc OpenMosaicIRAFFits {layer} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadMosaicIRAFFits $fn $layer
    FinishLoad
}

proc OpenMosaicImageWCSFits {layer} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    set sys [MosaicWCSDialog]
    if {$sys != {}} {
	StartLoad
	LoadMosaicImageWCSFits $sys $fn $layer
	FinishLoad
    }
}

proc OpenMosaicWCSFits {layer} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    set sys [MosaicWCSDialog]
    if {$sys != {}} {
	StartLoad
	LoadMosaicWCSFits $sys $fn $layer
	FinishLoad
    }
}

proc OpenMosaicImageWFPC2Fits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadMosaicImageWFPC2Fits $fn
    FinishLoad
}

proc OpenRGBImageFits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadRGBImageFits $fn
    FinishLoad
}

proc OpenRGBCubeFits {} {
    set fn [OpenFileDialog fitsfbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadRGBCubeFits $fn
    FinishLoad
}

proc OpenArray {layer} {
    set fn [OpenFileDialog arrayfbox]
    if {[string length $fn] == 0} {
	return
    }

    # do we have an array spec tag'd on
    if {![regexp -nocase {(.*)(\[.*\])} $fn foo base ext]} {
	set ext [ArrayDialog 1]
	if {$ext != {}} {
	    append fn "$ext"
	} else {
	    return
	}
    }

    StartLoad
    LoadArray $fn $layer
    FinishLoad
}

proc OpenRGBArray {} {
    set fn [OpenFileDialog arrayfbox]
    if {[string length $fn] == 0} {
	return
    }

    # do we have an array spec tag'd on
    if {![regexp -nocase {(.*)(\[.*\])} $fn foo base ext]} {
	set ext [ArrayDialog 3]
	if {$ext != {}} {
	    append fn "$ext"
	} else {
	    return
	}
    }

    StartLoad
    LoadRGBArray $fn
    FinishLoad
}

proc OpenURL {} {
    global http

    set fn [URLDialog http(last)]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadURL $fn
    FinishLoad
}

proc OpenVar {} {
    set fn [OpenFileDialog fitsvarfbox]
    if {[string length $fn] == 0} {
	return
    }

    if {![catch {set ch [open "$fn"]}]} {
	fconfigure $ch -translation binary -encoding binary
	global vardata
	set vardata [read -nonewline $ch]
	close $ch

	StartLoad
	LoadVar vardata $fn {}
	FinishLoad
    }
}

proc OpenImage {} {
    set fn [OpenFileDialog imagefbox]
    if {[string length $fn] == 0} {
	return
    }

    StartLoad
    LoadImage $fn
    FinishLoad
}

# Load

proc LoadFits {fn layer} {
    global current
    global ds9
    global loadParam
    global marker

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    ConvertFile $fn
    # save load type, since ProcessLoad will clear loadParam
    if {$loadParam(load,type) == "mmapincr"} {
	set mmap 1
    } else {
	set mmap 0
    }
    ProcessLoad

    # now autoload markers
    if {$ds9(automarker) && $mmap} {
	# now, load fits[REGION] if present
	set id [string first "\[" $fn]
	if {$id > 0} {
	    set base [string range $fn 0 [expr $id-1]]
	} else {
	    set base $fn
	}

	set reg "${base}\[REGION\]"
	if {[$current(frame) fitsy has ext "\"$reg\""]} {
	    RealizeDS9
	    catch {
		$current(frame) marker load fits "\"$reg\"" \
		    $marker(color) $marker(width) \
		    "\{$marker(font) $marker(font,size) $marker(font,style)\}"
	    }
	}
    }
}

proc LoadSFits {hdr fn layer} {
    global current
    global ds9
    global loadParam
    global marker

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadDataCubeFits {fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {data cube}
    ConvertFile $fn
    ProcessLoad
}

proc LoadMosaicImageIRAFFits {fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic image iraf}
    ConvertFile $fn
    ProcessLoad
}

proc LoadMosaicIRAFFits {fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic iraf}
    ConvertFile $fn
    ProcessLoad
}

proc LoadMosaicIRAFSFits {hdr fn layer} {
    global loadParam

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic iraf}
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadMosaicImageWCSFits {sys fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic image $sys]
    ConvertFile $fn
    ProcessLoad
}

proc LoadMosaicWCSFits {sys fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic $sys]
    ConvertFile $fn
    ProcessLoad
}

proc LoadMosaicWCSSFits {sys hdr fn layer} {
    global loadParam

    set loadParam(load,type) smmap
    set loadParam(load,layer) $layer
    set loadParam(file,type) fits
    set loadParam(file,mode) [list mosaic $sys]
    set loadParam(file,name) $fn
    set loadParam(file,header) $hdr
    ProcessLoad
}

proc LoadMosaicImageWFPC2Fits {fn} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) {}
    set loadParam(file,type) fits
    set loadParam(file,mode) {mosaic image wfpc2}
    ConvertFile $fn
    ProcessLoad
}

proc LoadRGBImageFits {fn} {
    global loadParam
    global current

    if {[$current(frame) get type] == {rgb}} {
	set loadParam(load,type) mmapincr
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {rgb image}
	ConvertFile $fn
	ProcessLoad
    } else {
	Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
    }
}

proc LoadRGBCubeFits {fn} {
    global loadParam
    global current

    if {[$current(frame) get type] == {rgb}} {
	set loadParam(load,type) mmapincr
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {rgb cube}
	ConvertFile $fn
	ProcessLoad
    } else {
	Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
    }
}

proc LoadRGBCubeSFits {hdr fn} {
    global loadParam
    global current

    if {[$current(frame) get type] == {rgb}} {
	set loadParam(load,type) smmap
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {rgb cube}
	set loadParam(file,name) $fn
	set loadParam(file,header) $hdr
	ProcessLoad
    } else {
	Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
    }
}

proc LoadRGBArray {fn} {
    global loadParam
    global current

    # if no zdim is present, insert one
    set exp {.*\[.*zdim[ ]*=[ ]*[0-9]+}
    if {![regexp $exp $fn]} {
	set i [string last "\]" $fn]
        set fn "[string range $fn 0 [expr $i-1]],zdim=3\]"
    }

    if {[$current(frame) get type] == {rgb}} {
	set loadParam(load,type) mmapincr
	set loadParam(load,layer) {}
	set loadParam(file,type) array
	set loadParam(file,mode) {rgb cube}
	ConvertFile $fn
	ProcessLoad
    } else {
	Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
    }
}

proc LoadVar {varname fn mode} {
    global loadParam

    set loadParam(load,type) var
    set loadParam(load,layer) {}
    set loadParam(var,name) $varname
    set loadParam(file,type) fits
    set loadParam(file,name) "$fn"
    set loadParam(file,mode) $mode
    ProcessLoad
}

proc LoadArray {fn layer} {
    global loadParam

    set loadParam(load,type) mmapincr
    set loadParam(load,layer) $layer
    set loadParam(file,type) array
    set loadParam(file,mode) {}
    ConvertFile $fn
    ProcessLoad
}

proc LoadImage {fn} {
    global loadParam

    switch -- [file extension "$fn"] {
	.gif {set fm "gif"}
	.jpg {set fm "jpeg"}
	.tif {set fm "tiff"}
	.png {set fm "png"}
	.ppm {set fm "ppm"}
	
	default {
	    Error [msgcat::mc {Unsupported Image format}]
	    return
	}
    }

    set rr [catch {image create photo -format $fm -file $fn} ph]
    if {$rr != 0} {
	Error "[msgcat::mc {Unable read image file}] $fn"
	return
    }

    set ww [image width $ph]
    set hh [image height $ph]
    foreach tt [$ph data] {

    }

    image delete $ph
}

proc LoadShared {filetype mode idtype id fn} {
    global loadParam

    set loadParam(load,type) shared
    set loadParam(load,layer) {}
    set loadParam(shared,idtype) $idtype
    set loadParam(shared,id) $id
    set loadParam(file,type) $filetype
    set loadParam(file,name) $fn
    set loadParam(file,mode) $mode
    ProcessLoad
}

proc LoadURL {url} {
    if {[string length $url] == 0} {
	return
    }

    ParseURL $url r
    switch -- $r(scheme) {
	ftp {LoadFTP $r(authority) $r(path)}
	file {LoadFits $r(path) {}}
	http -
	default {LoadHTTP $url}
    }
}

proc LoadFTP {host path} {
    global loadParam
    global ds9
    global debug

    set ftp [ftp::Open $host "ftp" "-ds9@" -mode passive]
    if {$ftp > -1} {
	set fn "$ds9(tmpdir)/[file tail $path]"
	set ftp::VERBOSE $debug(tcl,ftp)
	set "ftp::ftp${ftp}(Output)" FTPLog
	ftp::Type $ftp binary
	if [ftp::Get $ftp $path $fn] {
	    # alloc it because we are going to delete it after load
	    set loadParam(load,type) allocgz
	    set loadParam(load,layer) {}
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {}
	    ConvertFile $fn
	    ProcessLoad
	}

	ftp::Close $ftp

	if [file exists $fn] {
	    catch {file delete -force $fn}
	}
    }
}

proc LoadHTTP {url} {
    global loadParam
    global ds9
    global http

    ParseURL $url r
    set fn "$ds9(tmpdir)/[file tail $r(path)]"

    set code 200
    set meta {}
    set mime "application/fits"
    set encoding {}

    set ch [open $fn w]
    set token [http::geturl $url -timeout $http(timeout) -channel $ch -binary 1 -headers "[ProxyHTTP]"]
    catch {close $ch}

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Mime-type
    # we want to strip and extra info after ';'
    regexp -nocase {([^;])*} $t(type) mime

    # Content-Encoding
    foreach {name value} $meta {
	if {[regexp -nocase ^content-encoding $name]} {
	    switch -- $value {
		gzip -
		x-gzip {set encoding gzip}
		compress -
		bzip2 {set encoding bzip2}
		Z {set encoding compress}
		pack -
		z {set encoding pack}
		default {}
	    }
	}
    }

    HTTPLog $token
    http::cleanup $token

    global debug
    if {$debug(tcl,hv)} {
	puts "Load HTTP: fn $fn : code $code : meta $meta : mime $mime : encoding $encoding"
    }

    switch -- "$mime" {
	"image/fits" -
	"application/fits" {}

	"application/fits-image" -
	"application/fits-table" -
	"application/fits-group" {}

	"image/x-fits" -
	"binary/x-fits" -
	"application/x-fits" {}

	"image/x-gfits" -
	"binary/x-gfits" -
	"image/gz-fits" -
	"display/gz-fits" {set encoding gzip}

	"image/bz2-fits" -
	"display/bz2-fits" {set encoding bzip2}

	"image/x-cfits" -
	"binary/x-cfits" {set encoding compress}

	"image/x-zfits" -
	"binary/x-zfits" {set encoding pack}

	default {
	    Error "[msgcat::mc {File not Found or Unable to load FITS data MIME type}] $mime"
	    return
	}
    }

    # alloc it because we are going to delete it after load
    set loadParam(load,type) allocgz
    set loadParam(load,layer) {}
    set loadParam(file,type) fits
    set loadParam(file,mode) {}
    set loadParam(file,name) $fn
    set loadParam(file,fn) $fn

    # may have to convert the file, based on content-encoding
    switch -- "$encoding" {
	bzip2 {
	    catch {set ch [open "| bunzip2 < $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
	compress {
	    catch {set ch [open "| uncompress < $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
	pack {
	    catch {set ch [open "| pcat $fn " r]}
	    set loadParam(load,type) channel
	    set loadParam(channel,name) $ch
	}
    }

    ProcessLoad

    if {[file exists $fn]} {
	catch {file delete -force $fn}
    }
}

# Support

proc MultiLoad {fc} {
    global ds9
    upvar $fc frameCount

    global debug
    if {$debug(tcl,layout)} {
	puts "MultiLoad"
    }

    # create a new frame
    if {$frameCount != 0} {
	CreateFrame  
    }

    # go into tile mode if more than one
    if {$frameCount >= 1 && $ds9(display,user) != "tile"} {
	set ds9(display,user) tile
	DisplayMode
    }
    incr frameCount
}

proc MultiRGBLoad {fc} {
    global ds9
    upvar $fc frameCount

    global debug
    if {$debug(tcl,layout)} {
	puts "MultiRGBLoad"
    }

    # create a new frame
    CreateRGBFrame  

    # go into tile mode if more than one
    if {$frameCount >= 1 && $ds9(display,user) != "tile"} {
	set ds9(display,user) tile
	DisplayMode
    }
    incr frameCount
}

proc URLDialog {prevURL} {
    upvar $prevURL result

    if {[EntryDialog [msgcat::mc {URL}] [msgcat::mc {Enter World Wide Web Location (URL)}] 80 result]} {
	return $result
    } else {
	return {}
    }
}

proc ProcessLoad {} {
    global current
    global loadParam

    switch -- $loadParam(load,type) {
	alloc -
	allocgz {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
		           $loadParam(load,type) \
			   \"$loadParam(file,fn)\" \
		           $loadParam(load,layer) \
	    }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
	channel {
	    fconfigure $loadParam(channel,name) -translation binary \
		-encoding binary
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(channel,name) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
	mmap -
	mmapincr {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(load,layer) \
	    }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
	smmap {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,header)\" \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
	shared {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(shared,idtype) \
			   $loadParam(shared,id) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) image $loadParam(file,name)"
	    }
	}
	sshared {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(shared,idtype) \
			   $loadParam(shared,hdr) \
			   $loadParam(shared,id) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) image $loadParam(file,name)"
	    }
	}
	socket -
	socketgz {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(socket,id) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
	var {
	    if [catch {$current(frame) load $loadParam(file,type) \
			   $loadParam(file,mode) \
			   \"$loadParam(file,name)\" \
			   $loadParam(load,type) \
			   $loadParam(var,name) \
		           $loadParam(load,layer) \
		       }] {
		Error "[msgcat::mc {Unable to load}] $loadParam(file,type) $loadParam(file,mode) $loadParam(file,name)"
	    }
	}
    }

    unset loadParam
}

proc StartLoad {} {
    SetWatchCursor
}

proc FinishLoad {} {
    global loadParam
    global current

    GridUpdate
    ContourUpdate

    # do after updategrid and updatecontour
    UpdateDS9
    UnsetWatchCursor

    # if header(s) were open, remove them
    DestroyHeader $current(frame)

    # do we have a data cube?
    if [$current(frame) has fits cube] {
	CubeDialog
    }
}

proc ConvertFile {filename} {
    global ds9
    global loadParam

    # default
    set loadParam(file,name) $filename
    set loadParam(file,fn) $filename

    foreach t $ds9(fileProcs) {
	if [$t $filename] {
	    return
	}
    }
}

# File Types

proc Stdin {filename} {
    global loadParam

    if {[string index $filename 0] == "-"} {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "stdin[string range $filename 1 end]"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    } elseif {[string range $filename 0 4] == "stdin" || 
	      [string range $filename 0 4] == "STDIN"} {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "stdin[string range $filename 5 end]"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc BZip2Fits {filename} {
    global loadParam

    if { [regexp {(.*)\.bz2($|\[)} $filename matched root] } {
	if [catch {set ch [open "| bunzip2 < $root.bz2 " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(file,name) "$filename"
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc CompressFits {filename} {
    global loadParam

    if { [regexp {(.*)\.Z($|\[)} $filename matched root] } {
	if [catch {set ch [open "| uncompress < $root.Z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(file,name) "$filename"
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc PackFits {filename} {
    global loadParam

    if { [regexp {(.*)\.z($|\[)} $filename matched root] } {
	if [catch {set ch [open "| pcat $root.z " r]}] {
	    return 0
	}
	set loadParam(load,type) channel
	set loadParam(file,name) "$filename"
	set loadParam(channel,name) $ch
	return 1
    }

    return 0
}

proc GzipFits {filename} {
    global loadParam

    if { [regexp {(.*)\.gz($|\[)} $filename matched root] } {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "$filename"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc FTZFits {filename} {
    global loadParam

    if { [regexp -nocase {(.*)\.(ftz)($|\[)} $filename matched root ext] } {
	set loadParam(load,type) allocgz
	set loadParam(file,name) "$filename"
	set loadParam(file,fn) $loadParam(file,name)
	return 1
    }

    return 0
}

proc ExternalFits {fn} {
    global extFits
    global loadParam

    foreach id [array names extFits] {
	if {[string match $id "$fn"]} {
	    regsub -all {\$filename} $extFits($id) "$fn" result
	    set cmd "| $result"
	    if [catch {set ch [open "$cmd" r]}] {
		return 0
	    }
	    set loadParam(load,type) channel
	    set loadParam(file,name) "$fn"
	    set loadParam(channel,name) $ch
	    return 1
	}
    }
    return 0
}

proc ArrayDialog {depth} {
    global env
    global ed

    set w {.arr}
    set length 300

    set ed(ok) 0
    set ed(x) 512
    set ed(y) 512
    set ed(z) $depth
    set ed(bitpix) -32
    set ed(skip) 0
    set ed(arch) bigendian

    if {[info exists env(DS9_ARRAY)]} {
	if {[regexp {.*(dims.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(dim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	    set ed(y) $item
	}
	if {[regexp {.*(xdim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(x) $item
	}
	if {[regexp {.*(ydim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(y) $item
	}
	if {[regexp {.*(zdim.?=)([0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(z) $item
	}
	if {[regexp {.*(bitpix.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(bitpix) $item
	}
	if {[regexp {.*(skip.?=)(-?[0-9]+)} $env(DS9_ARRAY) foo f1 item]} {
	    set ed(skip) $item
	}
	if {[regexp {.*arch.?=bigendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
	if {[regexp {.*arch.?=littleendian} $env(DS9_ARRAY) foo item]} {
	    set ed(arch) $item
	}
    }

    DialogCreate $w [msgcat::mc {Load Array}]

    frame $w.arr
    frame $w.arr.dim -relief ridge -bd 1
    frame $w.arr.bitpix -relief ridge -bd 1
    frame $w.arr.skip -relief ridge -bd 1
    frame $w.arr.arch -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.arr.dim $w.arr.bitpix $w.arr.skip $w.arr.arch -side top \
	-ipadx 4 -ipady 4 -fill x -expand true
    pack $w.arr -fill x -expand true
    pack $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    label $w.arr.dim.title -text [msgcat::mc {Dimension}]
    label $w.arr.dim.dummy -width 4
    entry $w.arr.dim.x -textvariable ed(x) -width 6
    entry $w.arr.dim.y -textvariable ed(y) -width 6
    entry $w.arr.dim.z -textvariable ed(z) -width 6
    grid $w.arr.dim.title $w.arr.dim.x $w.arr.dim.y $w.arr.dim.z \
	$w.arr.dim.dummy -padx 4 -sticky w

    label $w.arr.bitpix.title -text [msgcat::mc {Pixel Size}]
    label $w.arr.bitpix.dummy -width 9
    radiobutton $w.arr.bitpix.char -text {Char} \
	-variable ed(bitpix) -value 8
    radiobutton $w.arr.bitpix.short -text {Short} \
	-variable ed(bitpix) -value 16
    radiobutton $w.arr.bitpix.ushort -text {UShort} \
	-variable ed(bitpix) -value -16
    radiobutton $w.arr.bitpix.long -text {Long} \
	-variable ed(bitpix) -value 32
    radiobutton $w.arr.bitpix.float -text {Float} \
	-variable ed(bitpix) -value -32
    radiobutton $w.arr.bitpix.double -text {Double} \
	-variable ed(bitpix) -value -64
    grid $w.arr.bitpix.title $w.arr.bitpix.char \
	$w.arr.bitpix.dummy -padx 4 -sticky w
    grid x $w.arr.bitpix.short -padx 4 -sticky w
    grid x $w.arr.bitpix.ushort -padx 4 -sticky w
    grid x $w.arr.bitpix.long -padx 4 -sticky w
    grid x $w.arr.bitpix.float -padx 4 -sticky w
    grid x $w.arr.bitpix.double -padx 4 -sticky w

    label $w.arr.skip.t1 -text [msgcat::mc {Skip First}]
    label $w.arr.skip.t2 -text [msgcat::mc {Bytes}]
    label $w.arr.skip.dummy -width 4
    entry $w.arr.skip.skip -textvariable ed(skip) -width 6
    grid $w.arr.skip.t1 $w.arr.skip.skip $w.arr.skip.t2 \
	$w.arr.skip.dummy -padx 4 -sticky w

    label $w.arr.arch.title -text [msgcat::mc {Architecture}]
    label $w.arr.arch.dummy -width 0
    radiobutton $w.arr.arch.big -text {Big-Endian} \
	-variable ed(arch) -value bigendian
    radiobutton $w.arr.arch.little -text {Little-Endian} \
	-variable ed(arch) -value littleendian
    grid $w.arr.arch.title $w.arr.arch.big $w.arr.arch.dummy -padx 4 -sticky w
    grid x $w.arr.arch.little -padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    set r {}
    if {$ed(ok)} {
	set r "\[xdim=$ed(x),ydim=$ed(y),zdim=$ed(z),bitpix=$ed(bitpix),skip=$ed(skip),arch=$ed(arch)\]"
    }

    # we want to destroy this window

    destroy $w 

    unset ed
    return $r
}

proc MosaicWCSDialog {} {
    global env
    global ed

    set w {.wcs}
    set length 300

    set ed(ok) 0
    set ed(sys) wcs
    set ed(label) WCS

    DialogCreate $w [msgcat::mc {Load Mosaic}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.title -text [msgcat::mc {Select Coordinate System }]
    menubutton $w.param.sys -textvariable ed(label) \
	-menu $w.param.sys.m -relief raised -width 10
    grid $w.param.title $w.param.sys

    menu $w.param.sys.m
    $w.param.sys.m add radiobutton -label [msgcat::mc {WCS}] \
	-variable ed(sys) -value "wcs" -command [list set ed(label) WCS]
    $w.param.sys.m add separator
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$w.param.sys.m add radiobutton -variable ed(sys) \
	    -label "[msgcat::mc {WCS}] $l" \
	    -value "wcs$l" \
	    -command [list set ed(label) "[msgcat::mc {WCS}] $l"]
    }

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    set r {}
    if {$ed(ok)} {
	set r $ed(sys)
    }

    # we want to destroy this window
    destroy $w 

    unset ed
    return $r
}

# Header

proc DisplayHeaderMenu {} {
    global current

    set last {}
    set prim {}
    set cnt [$current(frame) get fits count]

    if {$cnt > 0} {
	global slb
	set slb(count) 0

	for {set ii 1} {$ii <= $cnt} {incr ii} {
	    set fn [$current(frame) get fits file name $ii]
	    set simple [$current(frame) get fits header $ii keyword {SIMPLE}]
	    set xten [$current(frame) get fits header $ii keyword {XTENSION}]

	    if {$xten != {}} {
		set b [string first {[} $fn]
		set pn [string range $fn 0 [expr $b-1]]
		if {$b > 0 && $pn != $prim} {
		    incr slb(count)
		    set slb($slb(count),item) $pn
		    set slb($slb(count),value) "-$ii"
		    set prim $pn
		}
	    }
	    
	    if {$fn != $last && $fn != $prim} {
		incr slb(count)
		set slb($slb(count),item) $fn
		set slb($slb(count),value) $ii
		set last $fn
	    }
	}

	if {$slb(count) == 1} {
	    DisplayHeader $current(frame) 1 $fn
	} else {
	    SLBDialog slb {Select Header} 40

	    if {$slb(value) != {}} {
		DisplayHeader $current(frame) $slb(value) $slb(item)
	    }
	}
	unset slb
    }
}

proc DisplayHeader {frame which title} {
    global current

    set varname "hd[string range $frame end end]-$which"

    upvar #0 $varname var
    global $varname

    SimpleTextDialog $varname $title 80 40 insert top [$current(frame) get fits header $which]

    # create a special text tag for keywords
    $var(text) tag configure keyword -font {courier 12} -foreground blue
    
    # color tag keywords
    set stop [$var(text) index end]
    for {set ii 1.0} {$ii<$stop} {set ii [expr $ii+1]} {
	$var(text) tag add keyword $ii "$ii +8 chars"
    }
}

proc DestroyHeader {frame} {
    global st

    set ttt "hd[string range $frame end end]"

    foreach x [array names st] {
	set f [split $x ,]
	if {[lindex $f 1] == "top"} {
	    set varname [lindex $f 0]

	    upvar #0 $varname var
	    global $varname

	    set fff [split $tt :]
	    if {[lindex $fff 0] == $ttt} {
		if {[info exists $varname] && [winfo exists ${varname}(top)]} {
		    SimpleTextDestroy $varname
		}
	    }
	}
    }
}

proc ProcessHeaderCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current

    if {$current(frame) != {}} {
	set item [lindex $var $i]
	if {$item != {} && [string is integer [lindex $var $i]]} {
	    set ii [lindex $var $i]
	} else {
	    set ii 1
	    incr i -1
	}

	catch {DisplayHeader $current(frame) $ii [$current(frame) get fits file name $ii]}
    }
}

proc ProcessShmCmd {varname iname flag fcName} {
    upvar $varname var
    upvar $iname i
    upvar $fcName fc

    global loadParam
    global current
    global ds9

    StartLoad

    set done 0
    while {!$done} {

	# defaults
	set loadParam(load,type) shared
	set loadParam(load,layer) {}
	set loadParam(file,type) fits
	set loadParam(file,mode) {}

	set nn [lindex $var [expr $i+4]]
	if {$nn == {} || [string range $nn 0 0] == "-"} {
	    set def 1
	} else {
	    set def 0
	}

	# do we have a rgb frame
	if {[$current(frame) get type] == {rgb}} {
	    set base 0
	} else {
	    set base 1
	}

	switch -- [lindex $var $i] {
	    key -
	    shmid {
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(shared,idtype) [lindex $var $i]
		set loadParam(shared,id) [lindex $var [expr $i+1]]
		set loadParam(file,name) [lindex $var [expr $i+2]]
		incr i 2
	    }

	    fits {
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    sfits {
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(load,type) sshared
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,hdr) [lindex $var [expr $i+2]]
		set loadParam(shared,id) [lindex $var [expr $i+3]]
		set loadParam(file,name) [lindex $var [expr $i+4]]
		incr i 4
	    }

	    mosaicimage {
		if {$flag && $base} {
		    MultiLoad fc
		}

		if {$def} {
		    set loadParam(file,mode) {mosaic image iraf}
		    set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		    set loadParam(shared,id) [lindex $var [expr $i+2]]
		    set loadParam(file,name) [lindex $var [expr $i+3]]
		    incr i 3
		} else {
		    set loadParam(file,mode) \
			[list mosaic image [lindex $var [expr $i+1]]]
		    set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		    set loadParam(shared,id) [lindex $var [expr $i+3]]
		    set loadParam(file,name) [lindex $var [expr $i+4]]
		    incr i 4
		}
	    }
	    mosaic {
		if {$def} {
		    set loadParam(file,mode) {mosaic iraf}
		    set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		    set loadParam(shared,id) [lindex $var [expr $i+2]]
		    set loadParam(file,name) [lindex $var [expr $i+3]]
		    incr i 3
		} else {
		    set loadParam(file,mode) \
			[list mosaic [lindex $var [expr $i+1]]]
		    set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		    set loadParam(shared,id) [lindex $var [expr $i+3]]
		    set loadParam(file,name) [lindex $var [expr $i+4]]
		    incr i 4
		}
	    }
	    smosaic {
		set loadParam(load,type) sshared
		set loadParam(file,mode) \
		    [list mosaic [lindex $var [expr $i+1]]]
		set loadParam(shared,idtype) [lindex $var [expr $i+2]]
		set loadParam(shared,hdr) [lindex $var [expr $i+3]]
		set loadParam(shared,id) [lindex $var [expr $i+4]]
		set loadParam(file,name) [lindex $var [expr $i+5]]
		incr i 5
	    }

	    mosaicimageiraf {
		# backward compatibility
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(file,mode) {mosaic image iraf}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaiciraf {
		# backward compatibility
		set loadParam(file,mode) {mosaic iraf}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicimagewcs {
		# backward compatibility
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(file,mode) {mosaic image wcs}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicwcs {
		# backward compatibility
		set loadParam(file,mode) {mosaic wcs}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    mosaicimagewfpc2 {
		# backward compatibility
		if {$flag && $base} {
		    MultiLoad fc
		}
		set loadParam(file,mode) {mosaic image wfpc2}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }

	    rgbcube {
		if {$flag} {
		    MultiRGBLoad fc
		}
		if {[$current(frame) get type] == {rgb}} {
		    set loadParam(file,mode) {rgb cube}
		} else {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    srgbcube {
		if {$flag} {
		    MultiRGBLoad fc
		}

		set loadParam(load,type) sshared
		if {[$current(frame) get type] == {rgb}} {
		    set loadParam(file,mode) {rgb cube}
		} else {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,hdr) [lindex $var [expr $i+2]]
		set loadParam(shared,id) [lindex $var [expr $i+3]]
		set loadParam(file,name) [lindex $var [expr $i+4]]
		incr i 4
	    }
	    rgbimage {
		if {$flag} {
		    MultiRGBLoad fc
		}

		if {[$current(frame) get type] == {rgb}} {
		    set loadParam(file,mode) {rgb image}
		} else {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    rgbarray {
		if {$flag} {
		    MultiRGBLoad fc
		}

		if {[$current(frame) get type] == {rgb}} {
		    set loadParam(file,type) array
		    set loadParam(file,mode) {rgb cube}
		} else {
		    Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
		}
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }
	    array {
		if {$flag && $base} {
		    MultiLoad fc
		}

		set loadParam(file,type) array
		set loadParam(shared,idtype) [lindex $var [expr $i+1]]
		set loadParam(shared,id) [lindex $var [expr $i+2]]
		set loadParam(file,name) [lindex $var [expr $i+3]]
		incr i 3
	    }

	    default {
		if {$flag && $base} {
		    MultiLoad fc
		}

		set loadParam(shared,idtype) key
		set loadParam(shared,id) [lindex $var $i]
		set loadParam(file,name) [lindex $var [expr $i+1]]
		incr i 1
	    }
	}

	ProcessLoad

	# more to come?
	incr i
	if {([lindex $var $i] == "-shm") || 
	    ([lindex $var $i] == "shm")} {
	    set done 0
	    incr i
	} else {
	    set done 1
	    incr i -1
	}
    }
    FinishLoad
}

proc ProcessPreserveCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global scale
    global panzoom
    global marker

    switch -- [string tolower [lindex $var $i]] {
	scale {
	    incr i
	    set scale(preserve) [FromYesNo [lindex $var $i]]
	    PreserveScale
	}
	pan {
	    incr i
	    set panzoom(preserve) [FromYesNo [lindex $var $i]]
	    PreservePan
	}
	marker -
	regions {
	    incr i
	    set marker(preserve) [FromYesNo [lindex $var $i]]
	    MarkerPreserve
	}
    }
}

