#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Public Procedures

proc CreateFrame {} {
    CreateNamedFrame base
}

proc CreateRGBFrame {} {
    CreateNamedFrame rgb
    RGBDialog
}

proc CreateNamedFrame {type} {
    global ds9

    # find the first open slot
    set num $ds9(num)
    while {1} {
	set which "Frame$num"
 	if {[lsearch $ds9(frames) $which]==-1} {
 	    CreateNameNumberFrame $which $type
	    set ds9(num) [expr $num+1]
 	    return
 	}
 	incr num
    }
}

proc CreateGotoFrame {num type} {
    global ds9
    global active

    set which "Frame$num"
    if {[lsearch $ds9(frames) $which]==-1} {
	CreateNameNumberFrame $which $type
	set ds9(num) [expr $num+1]
    } else {
	if {$active($which)==0} {
	    set active($which) 1
	    UpdateActiveFrames
	}
	set ds9(next) $which
	GotoFrame
    }
}

proc CreateNameNumberFrame {which type} {
    global ds9
    global active
    global current
    global panner
    global magnifier
    global bin
    global wcs
    global colorbar
    global scale
    global minmax
    global zscale
    global marker
    global panzoom
    global mask

    set ds9(next) $which

    # update frame lists
    lappend ds9(frames) $ds9(next)
    lappend ds9(active) $ds9(next)
    set active($ds9(next)) 1

    # and update frame list menus
    UpdateFrameMenuItems

    # and create the frame
    switch -- $ds9(visual) {
	pseudocolor {
	    if {$type == {rgb}} {
		Error [msgcat::mc {Unable to create RGB frame with pseudocolor visual}]
	    }
	    $ds9(canvas) create frame$ds9(visual)$ds9(depth) \
		-x 0 -y 0 -anchor nw -command $ds9(next) -tag $ds9(next)
	    $ds9(next) colormap [colorbar get colormap]
	}
    	truecolor {
	    switch -- $type {
		base {
		    $ds9(canvas) create frame$ds9(visual)$ds9(depth) \
			-x 0 -y 0 -anchor nw -command $ds9(next) \
			-tag $ds9(next)
		    $ds9(next) colormap [colorbar get colormap]
		}
		rgb {
		    $ds9(canvas) create framergb$ds9(visual)$ds9(depth) \
			-x 0 -y 0 -anchor nw -command $ds9(next) \
			-tag $ds9(next)
		    $ds9(next) colormap [colorbarrgb get colormap]
		}
	    }
	}
    }
    $ds9(next) colormap area $colorbar(area) $colorbar(mode)

    $ds9(next) panner 'panner' $panner(size) $panner(size)
    $ds9(next) panner wcs $panner(compass,wcs,system) $panner(compass,wcs,sky)
    $ds9(next) magnifier 'magnifier' $magnifier(size) $magnifier(size)
    $ds9(next) magnifier zoom $magnifier(zoom)
    $ds9(next) magnifier graphics $magnifier(region)
    $ds9(next) magnifier cursor $magnifier(cursor)

    $ds9(next) zoom to $current(zoom)
    $ds9(next) rotate to $current(rotate)
    $ds9(next) orient $current(orient)
    $ds9(next) pan preserve $panzoom(preserve)

    $ds9(next) wcs align $wcs(align) $wcs(align,system) $wcs(align,sky)

    $ds9(next) datasec $scale(datasec)

    $ds9(next) bg color $ds9(bg,color)
    $ds9(next) nan color $ds9(nan,color)

    $ds9(next) marker show $marker(show)
    $ds9(next) marker show text $marker(show,text)
    $ds9(next) marker preserve $marker(preserve)

    $ds9(next) mask color $mask(color)

    # channel dependent items
    if {[$ds9(next) get type] == {rgb}} {
	foreach c {red green blue} {
	    $ds9(next) rgb channel $c

	    $ds9(next) colorscale $scale(type)

	    $ds9(next) clip scope $scale(scope)
	    $ds9(next) clip mode $scale(mode)
	    $ds9(next) clip user $scale(min) $scale(max)
	    $ds9(next) clip preserve $scale(preserve)

	    $ds9(next) clip minmax mode $minmax(mode)
	    $ds9(next) clip minmax sample $minmax(sample)

	    $ds9(next) clip zscale contrast $zscale(contrast) 
	    $ds9(next) clip zscale sample $zscale(sample) 
	    $ds9(next) clip zscale line $zscale(line)

	    $ds9(next) bin function $bin(function)
	    $ds9(next) bin factor to $bin(factor)
	    $ds9(next) bin depth $bin(depth)
	    $ds9(next) bin buffer size $bin(buffersize)
	}
	$ds9(next) rgb channel red
	
    } else {
	$ds9(next) colorscale $scale(type)

	$ds9(next) clip scope $scale(scope)
	$ds9(next) clip mode $scale(mode)
	$ds9(next) clip user $scale(min) $scale(max)
	$ds9(next) clip preserve $scale(preserve)

	$ds9(next) clip minmax mode $minmax(mode)
	$ds9(next) clip minmax sample $minmax(sample)

	$ds9(next) clip zscale contrast $zscale(contrast) 
	$ds9(next) clip zscale sample $zscale(sample) 
	$ds9(next) clip zscale line $zscale(line)

	$ds9(next) bin function $bin(function)
	$ds9(next) bin factor to $bin(factor)
	$ds9(next) bin depth $bin(depth)
	$ds9(next) bin buffer size $bin(buffersize)
    }

    if {$ds9(mode) == "crosshair"} {
	$ds9(next) crosshair on
    }

    set current(frame) $ds9(next)
    DisplayMode
}

proc DeleteAllFramesMenu {} {
    global ds9

    if {$ds9(confirm)} {
	if {[tk_messageBox -type okcancel -icon question -message [msgcat::mc {Delete All Frames?}]] != {ok}} {
	    return
	}
    }
    DeleteAllFrames
}

proc DeleteCurrentFrame {} {
    global current

    if {$current(frame) != {}} {
	DeleteSingleFrame $current(frame)
    }
}

proc DeleteAllFrames {} {
    global ds9

    # we can not just cycle thru all frames, since some frames maybe slaves
    # foreach f $ds9(frames)
    set f [lindex $ds9(frames) 0]
    while {$f != {}} {
	DeleteFrame $f
	set f [lindex $ds9(frames) 0]
    }

    UpdateActiveFrames

    ClearInfoBox
    PixelTableClearDialog
    ClearGraphData
}

proc DeleteSingleFrame {which} {
    DeleteFrame $which

    UpdateActiveFrames

    ClearInfoBox
    PixelTableClearDialog
    ClearGraphData
}

proc DeleteFrame {which} {
    global ds9
    global active
    global header
    global current
    global contour
    global marker

    # contour copy
    if {$contour(copy) == $which} {
	set contour(copy) {}
    }
    # marker copy
    if {$marker(copy) == $which} {
	set marker(copy) {}
    }
    
    # kill the header dialog (if needed)
    DestroyHeader $which

    # delete any slave frames
    DeleteExamine $which

    # delete canvas widget
    $ds9(canvas) delete $which

    # delete it from the frame list
    set i [lsearch $ds9(frames) $which]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    unset active($which)

    # delete menu entries
    UpdateFrameMenuItems
}

proc UpdateCurrentFrame {} {
    global current

    UpdateFrame $current(frame)
}

proc UpdateAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	UpdateFrame $f
    }
}

proc UpdateFrame {which} {
    SetWatchCursor
    if {$which != {}} {
	$which update
    }
    UnsetWatchCursor
}

# Event Processing

proc BindEventsFrame {which} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "BindEventsFrame $which"
    }

    $ds9(canvas) bind $which <Motion> [list MotionFrame $which %x %y]
    $ds9(canvas) bind $which <Shift-Motion> \
	[list ShiftMotionFrame $which %x %y]
    $ds9(canvas) bind $which <Control-Motion> \
	[list ControlMotionFrame $which %x %y]

    $ds9(canvas) bind $which <Enter> [list EnterFrame $which %x %y]
    $ds9(canvas) bind $which <Leave> [list LeaveFrame $which]

    BindEventsFrameButton $which
    if {![$which has iis]} {
	BindEventsFrameKey $which
    }
}

proc BindEventsFrameButton {which} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "BindEventsFrameButton $which"
    }

    $ds9(canvas) bind $which <Button-1> [list Button1Frame $which %x %y]
    $ds9(canvas) bind $which <Shift-Button-1> \
	[list ShiftButton1Frame $which %x %y]
    $ds9(canvas) bind $which <Control-Button-1> \
	[list ControlButton1Frame $which %x %y]
    $ds9(canvas) bind $which <Control-Shift-Button-1> \
	[list ControlShiftButton1Frame $which %x %y]
    $ds9(canvas) bind $which <B1-Motion> [list Motion1Frame $which %x %y]
    $ds9(canvas) bind $which <ButtonRelease-1> \
	[list Release1Frame $which %x %y]

    $ds9(canvas) bind $which <Double-1> [list Double1Frame $which %x %y]
    $ds9(canvas) bind $which <Double-ButtonRelease-1> \
	[list DoubleRelease1Frame $which %x %y]

    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(canvas) bind $which <Button-2> \
		[list Button2Frame $which %x %y]
	    $ds9(canvas) bind $which <Shift-Button-2> \
		[list ShiftButton2Frame $which %x %y]
	    $ds9(canvas) bind $which <B2-Motion> \
		[list Motion2Frame $which %x %y]
	    $ds9(canvas) bind $which <ButtonRelease-2> \
		[list Release2Frame $which %x %y]
	} 
	aqua {
	    # swap button-2 and button-3 on the mighty mouse
	    $ds9(canvas) bind $which <Button-3> \
		[list Button2Frame $which %x %y]
	    $ds9(canvas) bind $which <Shift-Button-3> \
		[list ShiftButton2Frame $which %x %y]
	    $ds9(canvas) bind $which <B3-Motion> \
		[list Motion2Frame $which %x %y]
	    $ds9(canvas) bind $which <ButtonRelease-3> \
		[list Release2Frame $which %x %y]

	    # x11 option key emulation
	    $ds9(canvas) bind $which <Option-Button-1> \
		[list Button2Frame $which %x %y]
	    $ds9(canvas) bind $which <Option-B1-Motion> \
		[list Motion2Frame $which %x %y]
	    $ds9(canvas) bind $which <Option-ButtonRelease-1> \
		[list Release2Frame $which %x %y]

	    # x11 command key emulation
	    # we need this to eat the Button-1 events so it passes to the canvas
	    $ds9(canvas) bind $which <Command-Button-1> {set foo bar}
	    $ds9(canvas) bind $which <Command-B1-Motion> {set foo bar}
	    $ds9(canvas) bind $which <Command-ButtonRelease-1> {set foo bar}
	}
    }
}

proc BindEventsFrameKey {which} {
    global ds9
    global analysis

    global debug
    if {$debug(tcl,events)} {
	puts "BindEventsFrameKey $which"
    }

    $ds9(canvas) bind $which <Key> [list KeyFrame $which %K %A %x %y]

    for {set i 0} {$i<$analysis(bind,count)} {incr i} {
	$ds9(canvas) bind $which "$analysis(bind,$i,item)" \
	    "AnalysisTask $i bind $which %x %y"
    }
}

proc UnBindEventsFrame {which} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "UnBindEventsFrame $which"
    }

    $ds9(canvas) bind $which <Motion> {}
    $ds9(canvas) bind $which <Shift-Motion> {}
    $ds9(canvas) bind $which <Control-Motion> {}

    $ds9(canvas) bind $which <Enter> {}
    $ds9(canvas) bind $which <Leave> {}

    UnBindEventsFrameButton $which
    if {![$which has iis]} {
	UnBindEventsFrameKey $which
    }
}

proc UnBindEventsFrameButton {which} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "UnBindEventsFrameButton $which"
    }

    $ds9(canvas) bind $which <Button-1> {}
    $ds9(canvas) bind $which <Shift-Button-1> {}
    $ds9(canvas) bind $which <Control-Button-1> {}
    $ds9(canvas) bind $which <Control-Shift-Button-1> {}
    $ds9(canvas) bind $which <B1-Motion> {}
    $ds9(canvas) bind $which <ButtonRelease-1> {}

    $ds9(canvas) bind $which <Double-1> {}
    $ds9(canvas) bind $which <Double-ButtonRelease-1> {}

    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(canvas) bind $which <Button-2> {}
	    $ds9(canvas) bind $which <Shift-Button-2> {}
	    $ds9(canvas) bind $which <B2-Motion> {}
	    $ds9(canvas) bind $which <ButtonRelease-2> {}
	} 
	aqua {
	    $ds9(canvas) bind $which <Command-Button-1> {}
	    $ds9(canvas) bind $which <Command-B1-Motion> {}
	    $ds9(canvas) bind $which <Command-ButtonRelease-1> {}

	    $ds9(canvas) bind $which <Button-3> {}
	    $ds9(canvas) bind $which <Shift-Button-3> {}
	    $ds9(canvas) bind $which <B3-Motion> {}
	    $ds9(canvas) bind $which <ButtonRelease-3> {}

	    $ds9(canvas) bind $which <Option-Button-1> {}
	    $ds9(canvas) bind $which <Option-B1-Motion> {}
	    $ds9(canvas) bind $which <Option-ButtonRelease-1> {}
	}
    }
}

proc UnBindEventsFrameKey {which} {
    global ds9
    global analysis

    global debug
    if {$debug(tcl,events)} {
	puts "UnBindEventsFrameKey $which"
    }

    $ds9(canvas) bind $which <Key> {}

    for {set i 0} {$i<$analysis(bind,count)} {incr i} {
	$ds9(canvas) bind $which "$analysis(bind,$i,item)" {}
    }
}

proc EnterFrame {which x y} {
    global ds9
    global view

    global debug
    if {$debug(tcl,events)} {
	puts "EnterFrame"
    }

    # check to see if this event was generated while processing other events
    if {$ds9(b1) || $ds9(sb1) || $ds9(cb1) || $ds9(csb1) || $ds9(b2) || $ds9(b3)} {
	return
    }

    $ds9(canvas) focus $which

    switch -- $ds9(mode) {
	crosshair {
	    set coord [$which get crosshair canvas]
	    set x [lindex $coord 0]
	    set y [lindex $coord 1]
	}
	none -
	pointer -
	catalog -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {}
    }

    EnterInfoBox $which $x $y canvas
    UpdatePixelTableDialog $which $x $y canvas
    UpdateGraph $which $x $y canvas
    UpdateMagnifier $which $x $y

    if {$view(magnifier)} {
	$which magnifier on
    }

    UpdateEditMenu
}

proc LeaveFrame {which} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "LeaveFrame"
    }

    # check to see if this event was generated while processing other events
    if {$ds9(b1) || $ds9(sb1) || $ds9(cb1) || $ds9(csb1) || $ds9(b2) || $ds9(b3)} {
	return
    }

    $ds9(canvas) focus {}

    switch -- $ds9(mode) {
	crosshair {}
	none -
	pointer -
	catalog -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {
	    LeaveInfoBox
	    PixelTableClearDialog
	    ClearGraphData
	}
    }

    $which magnifier off
    magnifier clear
}

proc MotionFrame {which x y} {
    global debug
    if {$debug(tcl,events)} {
	puts "MotionFrame"
    }

    DoMotion $which $x $y sizing fleur
}

proc ShiftMotionFrame {which x y} {
    global debug
    if {$debug(tcl,events)} {
	puts "ShiftMotionFrame"
    }

    DoMotion $which $x $y exchange fleur
}

proc ControlMotionFrame {which x y} {
    global debug
    if {$debug(tcl,events)} {
	puts "ControlMotionFrame"
    }

    DoMotion $which $x $y sizing draped_box
}

proc DoMotion {which x y cursor1 cursor2} {
    global ds9
    global current

    # if button 3 is down, ignore this event, we are doing something already

    if {$ds9(b3) || $ds9(b2)} {
	return
    }

    switch -- $ds9(mode) {
	crosshair {}
	pointer -
	catalog {
	    if {$which == $current(frame)} {
		MarkerCursor $which $x $y $cursor1 $cursor2
	    }
	    UpdateColormapLevelMosaic $which $x $y canvas
	    UpdateInfoBox $which $x $y canvas
	    UpdatePixelTableDialog $which $x $y canvas
	    UpdateGraph $which $x $y canvas
	}
	none -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {
	    UpdateColormapLevelMosaic $which $x $y canvas
	    UpdateInfoBox $which $x $y canvas
	    UpdatePixelTableDialog $which $x $y canvas
	    UpdateGraph $which $x $y canvas
	}
    }

    UpdateMagnifier $which $x $y
}

proc Button1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "Button1Frame"
    }

    # let others know that the mouse is down

    set ds9(b1) 1

    switch -- $ds9(mode) {
	none {
	    if {$which == $current(frame)} {
	    } else {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1

		set ds9(next) $which
		GotoFrame
	    }
	    UpdateMagnifier $which $x $y
	}
	pointer {
	    if {$which == $current(frame)} {
		MarkerButton $which $x $y
	    } else {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1

		set ds9(next) $which
		GotoFrame
	    }
	    UpdateMagnifier $which $x $y
	}
	catalog {
	    if {$which == $current(frame)} {
		CATButton $which $x $y
	    } else {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1

		set ds9(next) $which
		GotoFrame
	    }
	    UpdateMagnifier $which $x $y
	}
	crosshair {
	    ButtonCrosshair $which $x $y

	    UpdateColormapLevelMosaic $which $x $y canvas
	    UpdateInfoBox $which $x $y canvas
	    UpdatePixelTableDialog $which $x $y canvas
	    UpdateGraph $which $x $y canvas
	    UpdateMagnifier $which $x $y
	}
	colorbar {}
	pan {
	    ButtonPan $which $x $y
	    UpdateMagnifier $which $x $y
	}
	zoom {
	    ButtonZoom $which $x $y
	    UpdateMagnifier $which $x $y
	}
	rotate {ButtonRotate $which $x $y}
	examine {ButtonExamine $which $x $y}
	illustrate {}
	imexam {
	    if {$ds9(imexam)} {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1

		set ds9(imexam,frame) $which
		set ds9(imexam,x) $x
		set ds9(imexam,y) $y

		set ds9(imexam) 0
	    }
	}
    }
}

proc ShiftButton1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "ShiftButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(sb1) 1

    switch -- $ds9(mode) {
	none {}
	pointer {
	    if {$which == $current(frame)} {
		MarkerShift $which $x $y
	    }
	    UpdateMagnifier $which $x $y
	}
	catalog {
	    if {$which == $current(frame)} {
		CATShift $which $x $y
	    }
	    UpdateMagnifier $which $x $y
	}
	crosshair {}
	colorbar {}
	pan {}
	zoom {ShiftZoom $which}
	rotate {}
	examine {}
	illustrate {}
	imexam {}
    }
}

proc ControlButton1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "ControlButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(cb1) 1

    switch -- $ds9(mode) {
	none {}
	pointer -
	catalog {
	    if {$which == $current(frame)} {
		MarkerControl $which $x $y
	    } else {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1
	    }
	    UpdateMagnifier $which $x $y
	}
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {}
    }
}

proc ControlShiftButton1Frame {which x y} {
    global ds9
    global current
    global marker

    global debug
    if {$debug(tcl,events)} {
	puts "ControlShiftButton1Frame"
    }

    # let others know that the mouse is down

    set ds9(csb1) 1

    switch -- $ds9(mode) {
	none {}
	pointer -
	catalog {
	    if {$which == $current(frame)} {
		MarkerControlShift $which $x $y
	    } else {
		# we need this cause MarkerMotion maybe called, 
		# and we don't want it
		set marker(motion) none
		set marker(handle) -1
	    }
	    UpdateMagnifier $which $x $y
	}
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {}
    }
}

proc Motion1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "Motion1Frame"
    }

    # abort if we are here by accident (such as a double click)

    if {($ds9(b1) == 0) && ($ds9(sb1) == 0) && \
	    ($ds9(cb1) == 0) && ($ds9(csb1) == 0)} {
	return
    }

    switch -- $ds9(mode) {
	none {UpdateMagnifier $which $x $y}
	pointer {
	    if {$which == $current(frame)} {
		MarkerMotion $which $x $y
	    }
	    UpdateMagnifier $which $x $y
	}
	catalog {
	    if {$which == $current(frame)} {
		CATMotion $which $x $y
	    }
	    UpdateMagnifier $which $x $y
	}
	crosshair {
	    if {$ds9(b1)} {
		MotionCrosshair $which $x $y

		UpdateColormapLevelMosaic $which $x $y canvas
		UpdateInfoBox $which $x $y canvas
		UpdatePixelTableDialog $which $x $y canvas
		UpdateGraph $which $x $y canvas
		UpdateMagnifier $which $x $y
	    }
	}
	colorbar {}
	pan {
	    if {$ds9(b1)} {
		MotionPan $which $x $y
		UpdateMagnifier $which $x $y
	    }
	}
	zoom {UpdateMagnifier $which $x $y}
	rotate {
	    if {$ds9(b1)} {
		MotionRotate $which $x $y
	    }
	}
	examine {}
	illustrate {}
	imexam {}
    }
}

proc Release1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "Release1Frame"
    }

    # abort if we are here by accident (such as a double click)
    if {($ds9(b1) == 0) && ($ds9(sb1) == 0) && \
	    ($ds9(cb1) == 0) && ($ds9(csb1) == 0)} {
	return
    }

    switch -- $ds9(mode) {
	none {}
	pointer {
	    if {$which == $current(frame)} {
		MarkerRelease $which $x $y
	    }
	}
	catalog {
	    if {$which == $current(frame)} {
		CATRelease $which $x $y
	    }
	}
	crosshair {
	    if {$ds9(b1)} {
		ReleaseCrosshair $which $x $y

		UpdateColormapLevelMosaic $which $x $y canvas
		UpdateInfoBox $which $x $y canvas
		UpdatePixelTableDialog $which $x $y canvas
		UpdateGraph $which $x $y canvas
	    }
	}
	colorbar {}
	pan {
	    if {$ds9(b1)} {
		ReleasePan $which $x $y
	    }
	}
	zoom {}
	rotate {
	    if {$ds9(b1)} {
		ReleaseRotate $which $x $y
	    }
	}
	examine {}
	illustrate {}
	imexam {}
    }

    # let others know that the mouse is up
    set ds9(b1) 0
    set ds9(sb1) 0
    set ds9(cb1) 0
    set ds9(csb1) 0

    UpdateEditMenu
    UpdateMagnifier $which $x $y
}

proc Double1Frame {which x y} {
    global ds9
    global current

    global debug
    if {$debug(tcl,events)} {
	puts "Double1Frame"
    }

    switch -- $ds9(mode) {
	none {}
	pointer {
	    if {$which == $current(frame)} {
		MarkerDouble $which $x $y
		UpdateMagnifier $which $x $y
	    }
	}
	catalog -
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {}
    }
}

proc DoubleRelease1Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "DoubleRelease1Frame"
    }

    switch -- $ds9(mode) {
	none -
	pointer -
	catalog -
	crosshair -
	colorbar -
	pan -
	zoom -
	rotate -
	examine -
	illustrate -
	imexam {}
    }

    UpdateEditMenu
    UpdateMagnifier $which $x $y
}

proc Button2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "Button2Frame"
    }

    set ds9(b2) 1
    ButtonPan $which $x $y
}

proc ShiftButton2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "ShiftButton2Frame"
    }

    set ds9(sb2) 1
}

proc Motion2Frame {which x y} {
    global debug
    if {$debug(tcl,events)} {
	puts "Motion2Frame"
    }

    MotionPan $which $x $y
}

proc Release2Frame {which x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "Release2Frame"
    }

    ReleasePan $which $x $y

    # let others know that the mouse is up

    set ds9(b2) 0
    set ds9(sb2) 0
}

proc KeyFrame {which K A x y} {
    global ds9

    global debug
    if {$debug(tcl,events)} {
	puts "KeyFrame $which $K $A $x $y"
    }

    switch $K {
	Meta_L -
	Meta_R {
	    set ds9(meta) 1
	    return
	}
    }

    # modal bindings
    switch -- $ds9(mode) {
	none {
	    switch -- $K {
		c {
		    if {!$ds9(meta)} {
			DisplayCoordDialog $which $x $y canvas
		    }
		}
		plus {CubeNext}
		minus {CubePrev}
 
		Up -
		k {$which warp 0 -1}
		Down -
		j {$which warp 0 1}
		Left -
		h {$which warp -1 0}
		Right -
		l {$which warp 1 0}
	    }
	}
	pointer {
	    switch -- $K {
		c {
		    if {!$ds9(meta)} {
			DisplayCoordDialog $which $x $y canvas
		    }
		}
		plus {CubeNext}
		minus {CubePrev}

		Delete -
		BackSpace {MarkerDeleteKey $which $x $y}

		Up -
		k {MarkerArrowKey $which 0 -1}
		Down -
		j {MarkerArrowKey $which 0 1}
		Left -
		h {MarkerArrowKey $which -1 0}
		Right -
		l {MarkerArrowKey $which 1 0}

		i {$which marker property include 1 $x $y}
		e {$which marker property include 0 $x $y}
		s {$which marker property source 1 $x $y}
		b {$which marker property source 0 $x $y}
		g {GroupCreate}
		G {GroupCreateSilent}
	    }    
	}
	catalog {
	    switch -- $K {
		Up -
		k {MarkerArrowKey $which 0 -1}
		Down -
		j {MarkerArrowKey $which 0 1}
		Left -
		h {MarkerArrowKey $which -1 0}
		Right -
		l {MarkerArrowKey $which 1 0}
	    }	    
	    CATKey $which $K
	}
	crosshair {
	    switch -- $K {
		c {
		    if {!$ds9(meta)} {
			DisplayCoordDialog $which $x $y canvas
		    }
		}
		plus {CubeNext}
		minus {CubePrev}

		Up -
		k {ArrowKeyCrosshair $which 0 -1}
		Down -
		j {ArrowKeyCrosshair $which 0 1}
		Left -
		h {ArrowKeyCrosshair $which -1 0}
		Right -
		l {ArrowKeyCrosshair $which 1 0}
	    }
	}
	colorbar {}
	pan {
	    switch -- $K {
		Up -
		k {Pan 0 1 canvas {}}
		Down -
		j {Pan 0 -1 canvas {}}
		Left -
		h {Pan 1 0 canvas {}}
		Right -
		l {Pan -1 0 canvas {}}
	    }
	}
	zoom {}
	rotate {}
	examine {}
	illustrate {}
	imexam {
	    if {$ds9(imexam,key)} {
		set ds9(imexam,frame) $which
		set ds9(imexam,x) $x
		set ds9(imexam,y) $y
		set ds9(imexam,event) $K

		set ds9(imexam,key) 0
	    }
	}
    }

    # reset meta key
    set ds9(meta) 0

    UpdateEditMenu
}

# Other Public Routines

proc CopyFrame {} {
    global ds9

    switch -- $ds9(mode) {
	pointer {MarkerCopy}
    }
}

proc CutFrame {} {
    global ds9

    switch -- $ds9(mode) {
	pointer {MarkerCut}
    }
}

proc PasteFrame {} {
    global ds9
    global marker

    switch -- $ds9(mode) {
	pointer {
	    set marker(paste,system) {}
	    set marker(paste,sky) {}
	    MarkerPaste
	}
    }
}

proc UndoFrame {} {
    global ds9

    switch -- $ds9(mode) {
	pointer {
	    MarkerUndo
	}
    }
}

proc FirstFrame {} {
    global ds9
    set ds9(next) [lindex $ds9(active) 0]
    GotoFrame
}

proc PrevFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(active) $current(frame)]
    if {$i>0} {
	set ds9(next) [lindex $ds9(active) [expr $i-1]]
    } else {
	set last [expr [llength $ds9(active)] -1]
	set ds9(next) [lindex $ds9(active) $last]
    }
    GotoFrame
}

proc NextFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(active) $current(frame)]
    set last [expr [llength $ds9(active)] -1]
    if {$i<$last} {
	set ds9(next) [lindex $ds9(active) [expr $i+1]]
    } else {
	set ds9(next) [lindex $ds9(active) 0]
    }
    GotoFrame
}

proc LastFrame {} {
    global ds9

    set ds9(next) [lindex $ds9(active) [expr [llength $ds9(active)]-1]]
    GotoFrame
}

proc MoveFirstFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(frames) $current(frame)]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    set ds9(frames) [linsert $ds9(frames) 0 $current(frame)]

    UpdateFrameMenuItems
    UpdateActiveFrames
}

proc MovePrevFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(frames) $current(frame)]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    if {$i>0} {
	set ds9(frames) [linsert $ds9(frames) [expr $i-1] $current(frame)]
    } else {
	set ds9(frames) [linsert $ds9(frames) end $current(frame)]
    }

    UpdateFrameMenuItems
    UpdateActiveFrames
}

proc MoveNextFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(frames) $current(frame)]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    set last [llength $ds9(frames)]
    if {$i<$last} {
	set ds9(frames) [linsert $ds9(frames) [expr $i+1] $current(frame)]
    } else {
	set ds9(frames) [linsert $ds9(frames) 0 $current(frame)]
    }

    UpdateFrameMenuItems
    UpdateActiveFrames
}

proc MoveLastFrame {} {
    global ds9
    global current

    set i [lsearch $ds9(frames) $current(frame)]
    set ds9(frames) [lreplace $ds9(frames) $i $i]
    set ds9(frames) [linsert $ds9(frames) end $current(frame)]

    UpdateFrameMenuItems
    UpdateActiveFrames
}

proc UpdateActiveFrames {} {
    global ds9
    global active
    global current

    # reset active list
    set ds9(active) {}
    foreach f $ds9(frames) {
	if {$active($f)} {
	    lappend ds9(active) $f
	    $ds9(mb).frame.goto entryconfig \
		"[msgcat::mc {Frame}] [string range $f 5 end]" -state normal
	} else {
	    $ds9(mb).frame.goto entryconfig \
		"[msgcat::mc {Frame}] [string range $f 5 end]" -state disabled
	}

    }

    # New layout if needed
    if {[llength $ds9(active)] > 0} {
	if {[lsearch $ds9(active) $current(frame)] == -1} {
	    set current(frame) [lindex $ds9(active) 0]
	    set ds9(next) $current(frame)
	}
    }

    SetWatchCursor
    DisplayMode
    LayoutFrames
    UnsetWatchCursor
}

proc ActiveFrameAll {} {
    global ds9
    global active

    foreach f $ds9(frames) {
	set active($f) 1
    }
    UpdateActiveFrames
}

proc ActiveFrameNone {} {
    global ds9
    global active

    foreach f $ds9(frames) {
	set active($f) 0
    }
    UpdateActiveFrames
}

proc GotoFrame {} {
    global ds9
    global current
    global active

    if {$current(frame) != {} && $current(frame) != $ds9(next)} {
	$current(frame) highlite off
	$current(frame) panner off

	switch -- $ds9(display,mode) {
	    blink -
	    single {
		$current(frame) hide
		UnBindEventsFrame $current(frame)
	    }
	    tile {}
	}
    }

    if {$current(frame) != $ds9(next)} {
	set current(frame) $ds9(next)
	FrameToFront $current(frame)
    }
}

proc DisplayMode {} {
    global ds9
    global blink

    switch -- $ds9(display,user) {
	single {set ds9(display,mode) $ds9(display,user)}
	tile -
	blink {
	    if {[llength $ds9(active)] > 1} {
		set ds9(display,mode) $ds9(display,user)
	    } else {
		set ds9(display,mode) single
	    }
	}
    }

    switch -- $ds9(display,mode) {
	single -
	tile {
	    # turn off blink if on
	    if {$blink(id)>0} {
		after cancel $blink(id)
		set blink(id) 0
		set blink(index) -1
	    }

	    SetWatchCursor
	    LayoutFrames
	    UnsetWatchCursor
	}
	blink {
	    # ignore if we are already blinking
	    if {$blink(id)==0} {
		SetWatchCursor
		LayoutFrames
		UnsetWatchCursor

		BlinkTimer
	    }
	}
    }
}

proc BlinkTimer {} {
    global blink
    global ds9

    if {$ds9(active) != {}} {
	incr blink(index)
	if {$blink(index) >= [llength $ds9(active)]} {
	    set blink(index) 0
	}
	set ds9(next) [lindex $ds9(active) $blink(index)]
	GotoFrame
    }

    set blink(id) [after $blink(interval) BlinkTimer]
}

proc ResetCurrentFrame {} {
    global current

    ResetFrame $current(frame)
}

proc ResetAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	ResetFrame $f
    }
}

proc ResetFrame {which} {
    if {$which != {}} {
	SetWatchCursor
	$which reset

	RefreshInfoBox $which
	PixelTableClearDialog
	ClearGraphData

	UnsetWatchCursor

	GridUpdateZoom
	UpdateZoomMenu
	UpdateZoomDialog
	UpdateScaleMenu
	UpdateScaleDialog

	UpdateGraphYAxis
	UpdateGraphXAxis
    }
}

proc ClearCurrentFrame {} {
    global current

    ClearFrame $current(frame)
}

proc ClearAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	ClearFrame $f
    }
}

proc ClearFrame {which} {
    global current
    global header

    if {$which != {}} {

	# delete any slave frames
	DeleteExamine $which

	DestroyHeader $which

	SetWatchCursor
	$which clear
	UnsetWatchCursor

	ClearInfoBox
	PixelTableClearDialog
	ClearGraphData

	UpdateDS9
    }
}

proc MatchFrames {which} {
    global ds9
    global current
    global wcs

    if {$current(frame) != {}} {
	SetWatchCursor

	set wcs(align) 0
	$current(frame) wcs align $wcs(align) $wcs(align,system) $wcs(align,sky)
	set pan [$current(frame) get pan $which scientific]

	foreach f $ds9(frames) {
	    if {$f != $current(frame)} {
		$f pan to $which $pan
		$f zoom to $current(zoom)
		$f rotate to $current(rotate)
		$f orient $current(orient)

		$f wcs align $wcs(align) $wcs(align,system) $wcs(align,sky)
	    }
	}

	UnsetWatchCursor
    }
}

proc MatchFramesWCS {} {
    global ds9
    global current
    global wcs

    if {$current(frame) != {}} {
	if  {[$current(frame) has wcs $wcs(align,system)]} {
	    SetWatchCursor

	    set wcs(align) 1
	    $current(frame) wcs align $wcs(align) $wcs(align,system) $wcs(align,sky)
	    set pan [$current(frame) get pan $wcs(align,system) FK5 scientific]
	    set align [$current(frame) get wcs align]

	    foreach f $ds9(frames) {
		if {$f != $current(frame) && [$f has wcs $wcs(align,system)]} {
		    $f pan to $wcs(align,system) FK5 $pan
		    $f zoom to $current(zoom)
		    $f rotate to $current(rotate)
		    $f orient $current(orient)

		    $f wcs align $align
		}
	    }
	    UnsetWatchCursor
	}
    }
}

# Private Procedures

proc FrameToFront {which} {
    global ds9
    global current
    global view
    global colorbar
    global blink

    global debug
    if {$debug(tcl,layout)} {
	puts "FrameToFront"
    }

    # process proper colorbar
    switch -- $ds9(visual) {
	pseudocolor {
	    if {$view(colorbar)} {
		colorbar show
	    } else {
		colorbar hide
	    }
	    if {$view(colorbar,numerics)} {
		colorbar numerics show
	    } else {
		colorbar numerics hide
	    }
	    set current(colorbar) colorbar

	    colorbar set colormap [$which get colormap]
	    set colorbar(map) [colorbar get name]
	}
	truecolor {
	    switch -- [$which get type] {
		base {
		    if {$view(colorbar)} {
			colorbar show
		    } else {
			colorbar hide
		    }
		    if {$view(colorbar,numerics)} {
			colorbar numerics show
		    } else {
			colorbar numerics hide
		    }
		    colorbarrgb hide
		    set current(colorbar) colorbar

		    colorbar set colormap [$which get colormap]
		    set colorbar(map) [colorbar get name]
		}
		rgb {
		    colorbar hide
		    if {$view(colorbar)} {
			colorbarrgb show
		    } else {
			colorbarrgb hide
		    }
		    if {$view(colorbar,numerics)} {
			colorbarrgb numerics show
		    } else {
			colorbarrgb numerics hide
		    }
		    set current(colorbar) colorbarrgb

		    colorbarrgb set colormap rgb [$which get colormap]
		    colorbarrgb rgb channel [$which get rgb channel]
		}
	    }
	}
    }
    set colorbar(invert) [$current(colorbar) get invert]

    $ds9(canvas) raise $which

    $which show
    switch -- $ds9(display,mode) {
	single -
	blink {
	    RefreshInfoBox $which
	    if {!$ds9(freeze)} {
		BindEventsFrame $current(frame)
	    }
	}
	tile {$which highlite on}
    }

    if {$view(panner)} {
	$which panner on
    }

    UpdateDS9
}

proc TileDialog {} {
    global tile
    global tiledlg
    global ds9

    # see if we already have a window visible
    if [winfo exists $tile(top)] {
	raise $tile(top)
	return
    }

    set w $tile(top)
    set title [msgcat::mc {Tile Parameters}]

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW TileDestroyDialog

    $w configure -menu $tile(mb)

    set tiledlg(mode) $tile(grid,mode)
    set tiledlg(row) $tile(grid,row)
    set tiledlg(col) $tile(grid,col)
    set tiledlg(gap) $tile(grid,gap)

    menu $tile(mb)

    AppleMenu tile(mb)

    $tile(mb) add cascade -label [msgcat::mc {File}] -menu $tile(mb).file

    menu $tile(mb).file
    $tile(mb).file add command -label [msgcat::mc {Close}] \
	-command TileDestroyDialog

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -side top -ipadx 4 -ipady 4 -fill x -expand true

    label $w.param.t1 -text [msgcat::mc {Grid}]
    radiobutton $w.param.auto -text [msgcat::mc {Automatic}] \
	-variable tiledlg(mode) -value automatic
    radiobutton $w.param.manual -text [msgcat::mc {Manual}] \
	-variable tiledlg(mode) -value manual

    label $w.param.t2 -text [msgcat::mc {Frame Layout}]
    entry $w.param.row -textvariable tiledlg(row) -width 6
    label $w.param.t2x -text {x}
    entry $w.param.col -textvariable tiledlg(col) -width 6

    label $w.param.t3 -text [msgcat::mc {Frame Gap}]
    entry $w.param.gap -textvariable tiledlg(gap) -width 6
    label $w.param.t4 -text [msgcat::mc {Pixels}]

    grid $w.param.t1 $w.param.auto - - $w.param.manual \
	-padx 4 -pady 2 -sticky w
    grid $w.param.t2 $w.param.row $w.param.t2x $w.param.col \
	-padx 4 -pady 2 -sticky w
    grid $w.param.t3 $w.param.gap - $w.param.t4 -padx 4 -pady 2 -sticky w

    # Buttons

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command TileApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command TileDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true
}

proc TileDestroyDialog {} {
    global tile
    global tiledlg

    if {[winfo exists $tile(top)]} {
	destroy $tile(top)
	destroy $tile(mb)
    }

    if {[info exists tiledlg)]} {
	unset tiledlg
    }
}

proc TileApplyDialog {} {
    global tile
    global tiledlg

    set tile(mode) grid
    set tile(grid,mode) $tiledlg(mode)
    set tile(grid,row) $tiledlg(row)
    set tile(grid,col) $tiledlg(col)
    set tile(grid,gap) $tiledlg(gap)
    
    DisplayMode
}

# Prefs Cmds

proc PrefsBkgrColor {} {
    global ds9

    foreach f $ds9(frames) {
	$f bg color $ds9(bg,color)
    }
}

proc PrefsNaNColor {} {
    global ds9

    foreach f $ds9(frames) {
	$f nan color $ds9(nan,color)
    }
}

# Process Cmds

proc ProcessFrameCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    global ds9
    global active
    global rgb

    catch {
	switch -- [string tolower [lindex $var $i]] {
	    center {
		incr i
		switch -- [lindex $var $i] {
		    all {CenterAllFrame}
		    {} {CenterCurrentFrame; incr i -1}
		    default {
			set f "Frame[lindex $var $i]"
			CenterFrame $f
		    }
		}
	    }
	    clear {
		incr i
		switch -- [lindex $var $i] {
		    all {ClearAllFrame}
		    {} {ClearCurrentFrame; incr i -1}
		    default {
			set f "Frame[lindex $var $i]"
			ClearFrame $f
		    }
		}
	    }
	    delete {
		incr i
		switch -- [lindex $var $i] {
		    all {DeleteAllFrames}
		    {} {DeleteCurrentFrame; incr i -1}
		    default {
			set f "Frame[lindex $var $i]"
			DeleteSingleFrame $f
		    }
		}
	    }
	    new {
		incr i
		switch -- [lindex $var $i] {
		    rgb {CreateRGBFrame}
		    default {CreateFrame; incr i -1}
		}
	    }
	    reset {
		incr i
		switch -- [lindex $var $i] {
		    all {ResetAllFrame}
		    {} {ResetCurrentFrame; incr i -1}
		    default {
			set f "Frame[lindex $var $i]"
			ResetFrame $f
		    }
		}
	    }
	    refresh {
		incr i
		switch -- [lindex $var $i] {
		    all {UpdateAllFrame}
		    {} {UpdateCurrentFrame; incr i -1}
		    default {
			set f "Frame[lindex $var $i]"
			UpdateFrame $f
		    }
		}
	    }
	    hide {
		incr i
		switch -- [lindex $var $i] {
		    all {ActiveFrameNone}
		    {} {
			set active($current(frame)) 0
			UpdateActiveFrames
			incr i -1
		    }
		    default {
			set f "Frame[lindex $var $i]"
			set active($f) 0
			UpdateActiveFrames
		    }
		}
	    }
	    show {
		incr i
		switch -- [lindex $var $i] {
		    all {ActiveFrameAll}
		    default {
			set f "Frame[lindex $var $i]"
			set active($f) 1
			UpdateActiveFrames
		    }
		}
	    }
	    move {
		incr i
		switch -- [lindex $var $i] {
		    first {MoveFirstFrame}
		    back {MovePrevFrame}
		    forward {MoveNextFrame}
		    last {MoveLastFrame}
		}
	    }
	    first {FirstFrame}
	    prev {PrevFrame}
	    next {NextFrame}
	    last {LastFrame}
	    frameno {incr i; CreateGotoFrame [lindex $var $i] base}
	    default {CreateGotoFrame [lindex $var $i] base}
	}
    }
}

proc ProcessSingleCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    # we need to realized now in case of multiple loads then -single
    # followed by Match Frame
    RealizeDS9

    global ds9
    set ds9(display,user) single
    DisplayMode
}

proc ProcessTileCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global tile

    switch -- [string tolower [lindex $var $i]] {
	mode {
	    incr i
	    set tile(mode) [lindex $var $i]
	}
	grid {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		mode {
		    incr i
		    set tile(grid,mode) [lindex $var $i]
		}
		layout {
		    incr i
		    set tile(grid,row) [lindex $var $i]
		    incr i
		    set tile(grid,col) [lindex $var $i]
		}
		gap {
		    incr i
		    set tile(grid,gap) [lindex $var $i]
		}
		default {
		    set tile(mode) grid
		}
	    }
	}
	column {
	    set tile(mode) column
	}
	row {
	    set tile(mode) row
	}

	yes -
	true -
	1 -
	no -
	false -
	0 {
	    if {[FromYesNo [lindex $var $i]]} {
		set ds9(display,user) tile
	    } else {
		set ds9(display,user) single
	    }
	}
	default {
	    set ds9(display,user) tile
	    incr i -1
	}
    }
    DisplayMode
}

proc ProcessBlinkCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    set ds9(display,user) blink
    DisplayMode
}

proc ProcessMatchCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    RealizeDS9
    switch -- [string tolower [lindex $var $i]] {
	frames {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		image -
		physical {MatchFrames [lindex $var $i]}
		wcs {MatchFramesWCS}
	    }
	}
	colorbars {MatchColorbars}
	scales {MatchScales}
    }
}

proc ProcessLockCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global crosshair

    RealizeDS9
    switch -- [string tolower [lindex $var $i]] {
	crosshairs {
	    incr i
	    switch -- [string range [string tolower [lindex $var $i]] 0 2] {
		ima -
		phy -
		amp -
		det -
		wcs {set crosshair(lock) [lindex $var $i]}
	    }
	}
    }
}
