#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc GroupCreate {} {
    global current
    global group

    if {$current(frame) != {}} {
	set name [$current(frame) get marker tag default name]
	if [EntryDialog [msgcat::mc {New Group}] [msgcat::mc {Enter Group Name}] 30 name] {
	    $current(frame) marker tag "\{$name\}"
	    UpdateGroupDialog
	}
    }
}

proc GroupCreateSilent {} {
    global current
    global group

    if {$current(frame) != {}} {
	set name [$current(frame) get marker tag default name]
	$current(frame) marker tag "\{$name\}"
	UpdateGroupDialog
    }
}

proc GroupDialog {} {
    global ds9
    global group

    # see if we already have a window visible
    if [winfo exists $group(top)] {
	raise $group(top)
	return
    }

    set w $group(top)
    set title [msgcat::mc {Groups}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW GroupDestroyDialog

    $w configure -menu $group(mb)

    menu $group(mb)

    AppleMenu group(mb)

    $group(mb) add cascade -label [msgcat::mc {File}] -menu $group(mb).file

    menu $group(mb).file
    $group(mb).file add command -label [msgcat::mc {Update Group}] \
	-command GroupUpdateDialog
    $group(mb).file add separator
    $group(mb).file add command -label [msgcat::mc {New Group}] \
	-command GroupCreate
    $group(mb).file add command -label [msgcat::mc {Edit Group Name}] \
	-command GroupEditDialog
    $group(mb).file add separator
    $group(mb).file add command -label [msgcat::mc {Delete Group}] \
	-command GroupDeleteDialog
    $group(mb).file add command -label [msgcat::mc {Delete All Groups}] \
	-command GroupDeleteAllDialog
    $group(mb).file add separator
    $group(mb).file add command -label [msgcat::mc {Close}] \
	-command GroupDestroyDialog

    frame $w.frame -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.frame $w.buttons -side top -fill both -expand 1 -ipadx 4 -ipady 4

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" \
	-width 25 -height 15 -setgrid 1 -selectmode single
    pack $w.frame.list -side left -fill both -expand 1
    pack $w.frame.scroll -side left -fill y -expand 1

    button $w.buttons.update -text [msgcat::mc {Update}] \
	-command GroupUpdateDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command GroupDestroyDialog
    pack $w.buttons.update $w.buttons.close -side left -expand true -padx 10

    bind $w.frame.list <ButtonRelease-1> GroupButtonDialog

    UpdateGroupDialog
}

proc GroupButtonDialog {} {
    global group
    global current

    set w $group(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
#	    $current(frame) marker tag select "\{[$w.frame.list get $i]\}"
	    $current(frame) marker "\{[$w.frame.list get $i]\}" select only
	}
    }
}

proc GroupDestroyDialog {} {
    global group

    if {[winfo exists $group(top)]} {
	destroy $group(top)
	destroy $group(mb)
    }
}

proc GroupUpdateDialog {} {
    global group
    global current

    set w $group(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    $current(frame) marker tag update "\{[$w.frame.list get $i]\}"
	}
    }
}

proc GroupEditDialog {} {
    global group
    global current
    
    set w $group(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    set which [$w.frame.list get $i]
	    if [EntryDialog [msgcat::mc {Group Name}] [msgcat::mc {Enter Group Name}] 40 which] {
		$current(frame) marker tag edit \
		    "\{[$w.frame.list get $i]\}" "\{$which\}"
		UpdateGroupDialog
	    }
	}
    }
}

proc GroupDeleteDialog {} {
    global group
    global current
    global ds9

    set w $group(top)

    if {$current(frame) != {}} {
	set i [$w.frame.list curselection]
	if {[string length $i] != 0} {
	    set which [$w.frame.list get $i]
	    $current(frame) marker tag delete "\{$which\}"
	    UpdateGroupDialog
	}
    }
}

proc GroupDeleteAllDialog {} {
    global current
    global ds9

    if {$current(frame) != {}} {

	if {$ds9(confirm)} {
	    if {[tk_messageBox -type okcancel -icon question -message [msgcat::mc {Delete All Groups?}]] != {ok}} {
		return
	    }
	}
	$current(frame) marker tag delete all
	UpdateGroupDialog
    }
}

proc UpdateGroupDialog {} {
    global current
    global group

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateGroupDialog"
    }

    if {[winfo exists $group(top)]} {
	set w $group(top)

	# clear the list
	$w.frame.list delete 0 end

	if {$current(frame) != {}} {
	    set grps [lsort [$current(frame) get marker tag all]]
	    foreach f $grps {
		$w.frame.list insert end $f
	    }
	}
    }
}
