#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Public Proceduress

proc HV {varname title url font init} {
    upvar #0 $varname var
    global $varname

    global ds9

    global debug
    if {$debug(tcl,hv)} {
	puts "HV $varname $title $url"
    }

    set var(top) ".${varname}"
    set var(mb) ".${varname}mb"

    set w $var(top)

    # see if we already have a window visible
    
    if {[winfo exists $w]} {
	raise $w
    } else {
	set var(widget) {}
	set var(status) {}
	set var(frame) new
	set var(file,mode) { }
	set var(save) 0
	set var(title) "$title"
	set var(copy) {}

	set var(active) 0
	set var(index) 0
	set var(font,size) $font
	set var(init) $init

	set var(images,forward) ${varname}forward
	set var(images,back) ${varname}back
	set var(images,reload) ${varname}reload
	set var(images,stop) ${varname}stop
	set var(images,gray) ${varname}gray

	# init some vars
	HVClearAll $varname
	set var(delete) 0

	# create window
	toplevel $w
	switch $ds9(wm) {
	    x11 -
	    win32 {}
	    aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox fullZoom collapseBox resizable"}
	}

	wm title $w $title
	wm iconname $w $title
	wm group $w $ds9(top)
	wm protocol $w WM_DELETE_WINDOW "HVDestroy $varname"

	$w configure -menu $var(mb)

	menu $var(mb)
	AppleMenu ${varname}(mb)

	$var(mb) add cascade -label [msgcat::mc {File}] -menu $var(mb).file
	$var(mb) add cascade -label [msgcat::mc {Edit}] -menu $var(mb).edit
	$var(mb) add cascade -label [msgcat::mc {View}] -menu $var(mb).view
	$var(mb) add cascade -label [msgcat::mc {Frame}] -menu $var(mb).frame
	$var(mb) add cascade -label [msgcat::mc {Open Fits As}] \
	    -menu $var(mb).open

	menu $var(mb).file
	$var(mb).file add command -label [msgcat::mc {Open URL}] \
	    -command "HVURLDialogCmd $varname"
	$var(mb).file add command -label [msgcat::mc {Open File}] \
	    -command "HVFileDialogCmd $varname"
	$var(mb).file add separator
	$var(mb).file add command -label [msgcat::mc {Close}] \
	    -command "HVDestroy $varname"

	menu $var(mb).edit
	$var(mb).edit add command -label [msgcat::mc {Cut}] \
	    -state disabled -accelerator "${ds9(ctrl)}X"
	$var(mb).edit add command -label [msgcat::mc {Copy}] \
	    -command "HVCopyCmd $varname" -accelerator "${ds9(ctrl)}C"
	$var(mb).edit add command -label [msgcat::mc {Paste}] \
	    -state disabled -accelerator "${ds9(ctrl)}X"
	$var(mb).edit add command -label [msgcat::mc {Clear}] \
	    -state disabled
	$var(mb).edit add separator
	$var(mb).edit add command -label [msgcat::mc {Clear Cache}] \
	    -command "HVClearCache $varname"

	menu $var(mb).view
	$var(mb).view add command -label [msgcat::mc {Back}] \
	    -command "HVBackCmd $varname"
	$var(mb).view add command -label [msgcat::mc {Forward}] \
	    -command "HVForwardCmd $varname"
	$var(mb).view add separator
	$var(mb).view add command -label [msgcat::mc {Stop}] \
	    -command "HVStopCmd $varname"
	$var(mb).view add command -label [msgcat::mc {Reload}] \
	    -command "HVReloadCmd $varname"
	$var(mb).view add separator
	$var(mb).view add cascade -label [msgcat::mc {Text Size}] \
	    -menu $var(mb).view.font
	$var(mb).view add separator
	$var(mb).view add command -label [msgcat::mc {Page Source}] \
	    -command "HVPageSourceCmd $varname"

	menu $var(mb).view.font 
	$var(mb).view.font add radiobutton -label [msgcat::mc {Smaller}] \
	    -variable ${varname}(font,size) -value -2 \
	    -command "HVFontCmd $varname"
	$var(mb).view.font add radiobutton -label [msgcat::mc {Normal}] \
	    -variable ${varname}(font,size) -value 0 \
	    -command "HVFontCmd $varname"
	$var(mb).view.font add radiobutton -label [msgcat::mc {Larger}] \
	    -variable ${varname}(font,size) -value 2 \
	    -command "HVFontCmd $varname"
	$var(mb).view.font add radiobutton -label [msgcat::mc {Largest}] \
	    -variable ${varname}(font,size) -value 4 \
	    -command "HVFontCmd $varname"

	menu $var(mb).frame 
	$var(mb).frame add checkbutton \
	    -label [msgcat::mc {Save FITS on download}] \
	    -variable ${varname}(save)
	$var(mb).frame add separator
	$var(mb).frame add radiobutton \
	    -label [msgcat::mc {Create New Frame on Download}] \
	    -variable ${varname}(frame) -value new
	$var(mb).frame add radiobutton \
	    -label [msgcat::mc {Use Current Frame on Download}] \
	    -variable ${varname}(frame) -value current

	menu $var(mb).open
	$var(mb).open add radiobutton -label {Fits} \
	    -variable ${varname}(file,mode) -value { }
	$var(mb).open add separator
	$var(mb).open add radiobutton -labe {RGB Fits Image} \
	    -variable ${varname}(file,mode) -value {rbg image}
	$var(mb).open add radiobutton -labe {RGB Fits Cube} \
	    -variable ${varname}(file,mode) -value {rbg cube}
	$var(mb).open add separator
	$var(mb).open add radiobutton -labe {Multi Ext Data Cube} \
	    -variable ${varname}(file,mode) -value {data cube}
	$var(mb).open add separator
	$var(mb).open add radiobutton -label {Mosaic IRAF} \
	    -variable ${varname}(file,mode) -value {mosaic image iraf}
	$var(mb).open add radiobutton -label {Mosaic IRAF Segment} \
	    -variable ${varname}(file,mode) -value {mosaic iraf}
	$var(mb).open add separator
	$var(mb).open add radiobutton -label {Mosaic WCS} \
	    -variable ${varname}(file,mode) -value {mosaic image wcs}
	$var(mb).open add radiobutton -label {Mosaic WCS Segment} \
	    -variable ${varname}(file,mode) -value {mosaic wcs}
	$var(mb).open add separator
	$var(mb).open add radiobutton -label {Mosaic Image WFPC2} \
	    -variable ${varname}(file,mode) -value {mosaic image wfpc2}

	image create photo $var(images,back) -data {R0lGODlhDwANAKL/AM///8DAwJD//y/I/y+X/y9n/wAAAAAAACH5BAEAAAEALAAAAAAPAA0AAAM0GLq2/qE0+AqYVFmB6eZFKEoRIAyCaaYCYWxDLM9uYBAxoe/7dA8ug3AoZOg6mRsyuUxmEgA7}
	image create photo $var(images,forward) -data {R0lGODlhDwANAKL/AM///8DAwJD//y/I/y+X/y9n/wAAAAAAACH5BAEAAAEALAAAAAAPAA0AAAM3GLpa/K8YSMuYlBVwV/kgCAhdsAFoig7ktA1wLA9SQdw4DkuB4f8/Ag2TMRB4GYUBmewRm09FAgA7}
	image create photo $var(images,stop) -data {R0lGODlhDQANALP/AP///1Lq81I5Of+EhCEAAHsAAMYAAP+UQv9zCHuMjP8AMf8AKf+MnK1CSv8QIQAAACH5BAEAAAEALAAAAAANAA0AAARWMMjUTC1J6ubOQYdiCBuIIMuiiCT1OWu6Ys05AMPC4ItBGB8dYMdI+RoHR4qY6v1CwlvRcEQ4brndwFAgJAwIRdPIzVTEYiqXJBEU1FQCW5Mg2O0ZSQQAOw==}
	image create photo $var(images,reload) -data {R0lGODlhDAANALP/AP///zk5OVJSUoSEhKWlpcDAwP//1v//xr3erZTOezGcEFKtSimce3NzezkxOQAAACH5BAEAAAUALAAAAAAMAA0AAARRcJBJyRilEMC5AcjQaB1wHMYkCFuXLKDQONsBLIuynEBAGAcJAnYy0AyGBOLENPg4qGUISTMdEIoEg4A6ohK6BND4YyqBqCdyve453vB44BEBADs=}

	image create photo $var(images,gray) -data {R0lGODdhPAA+APAAALi4uAAAACwAAAAAPAA+AAACQISPqcvtD6OctNqLs968+w+G4kiW5omm6sq27gvH8kzX9o3n+s73/g8MCofEovGITCqXzKbzCY1Kp9Sq9YrNFgsAO}

	frame $w.b -relief ridge -bd 1
	pack $w.b -side top -fill x

	button $w.b.back -image $var(images,back) -width 15 -height 13 \
	    -command "HVBackCmd $varname"
	button $w.b.forward -image $var(images,forward) -width 15 -height 13 \
	    -command "HVForwardCmd $varname"
	button $w.b.stop -image $var(images,stop) -width 15 -height 13 \
	    -command "HVStopCmd $varname"
	button $w.b.reload -image $var(images,reload) -width 15 -height 13 \
	    -command "HVReloadCmd $varname"
	pack $w.b.back $w.b.forward $w.b.stop $w.b.reload -side left

	frame $w.h
	grid rowconfigure $w.h 0 -weight 1
	grid columnconfigure $w.h 0 -weight 1
	pack $w.h -side top -fill both -expand true

	set var(widget) [html $w.h.html \
				-yscrollcommand "$w.h.yscroll set" \
				-xscrollcommand "$w.h.xscroll set" \
				-padx 5 \
				-pady 9 \
				-formcommand "HVFormCB $varname" \
				-imagecommand "HVImageCB $varname" \
				-scriptcommand "HVScriptCB $varname"\
				-appletcommand "HVAppletCB $varname" \
				-framecommand "HVFrameCB $varname" \
				-underlinehyperlinks 1 \
				-bg white \
				-width 640 \
				-height 512 \
				-fontcommand "HVFontCB $varname" \
				-tablerelief raised]

	$var(widget) token handler {NOSCRIPT} "HVNoScriptCB $varname"
	$var(widget) token handler {/NOSCRIPT} "HVNoScriptCB $varname"

	scrollbar $w.h.yscroll -orient vertical -command "$w.h.html yview"
	scrollbar $w.h.xscroll -orient horizontal -command "$w.h.html xview"

	grid $w.h.html $w.h.yscroll -sticky news
	grid $w.h.xscroll -stick news

	frame $w.s -relief ridge -bd 1
	pack $w.s -side bottom -fill x

	label $w.s.status -textvariable ${varname}(status) \
	    -font {helvetica 10} -width 120 -anchor w
	pack $w.s.status -side left

	bind $var(widget).x <Motion> "HVMotion $varname %x %y" 
	bind $var(widget).x <Button-1> "HVButton1 $varname %x %y"
	bind $var(widget).x <B1-Motion> "HVMotion1 $varname %x %y" 
	bind $var(widget).x <ButtonRelease-1> "HVRelease1 $varname %x %y"

	# we have a problem with the html widget. first time thur, some
	# structures are not allocated/initialized. if we first display
	# a blank page, all seems ok
	$var(widget) clear
	$var(widget) parse "<html>\n<body>\n<form method=\"get\" action=\"foo\">\n</form>\n</body>\n</html>"

	global debug
	if {$debug(tcl,idletasks)} {
	    puts "HV"
	}
	update idletasks
    }

    selection handle $var(top) [list HVExportSelection $varname]

    if {$url != {}} {
	# no need to resolve
	HVLoadURL $varname $url {}
    }

    # windows: we're on top of the image window, so set focus to this window
    switch $ds9(wm) {
	x11 {}
	win32 {focus $w}
	aqua {}
    }
}

# Bindings

proc HVMotion {varname x y} {
    upvar #0 $varname var
    global $varname

    global ds9

    set url [$var(widget) href $x $y] 

    if {[string length $url] > 0} {
	switch $ds9(wm)  {
	    x11 -
	    win32 {$var(widget) configure -cursor hand2}
	    aqua {$var(widget) configure -cursor pointinghand}
	}
    } else {
	$var(widget) configure -cursor {}
    }

    HVStatus $varname $url
}

proc HVButton1 {varname x y} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVButton1"
    }

    global xpa
    global vo
    global ds9

    $var(widget) selection clear
    set var(sel,x) -1
    set var(sel,y) -1

    HVClearIndex $varname $var(index)

    set url [$var(widget) href $x $y]
    if {[string length $url] != 0} {

	# sub xpa method
	set exp {%40%40XPA_METHOD%40%40|@@XPA_METHOD@@}
	if {[regexp $exp $url]} {
	    regsub -all $exp $url [XPAMethod] url
	    if {$debug(tcl,hv)} {
		puts "HVLinkBind XPA_METHOD $url"
	    }
	}

	# sub vo method
	set exp {%40%40VO_METHOD%40%40|@@VO_METHOD@@}
	if {[regexp $exp $url]} {
	    regsub -all $exp $url $vo(method) url
	    if {$debug(tcl,hv)} {
		puts "HVLinkBind VO_METHOD $url"
	    }
	}

	# already resolved
	HVLoadURL $varname $url {}
    } else {
	set var(sel,x) $x
	set var(sel,y) $y
    }
}

proc HVMotion1 {varname x y} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVMotion1"
    }

    if {$var(sel,x) != -1 && $var(sel,y) != -1} {
	$var(widget) selection set @$var(sel,x),$var(sel,y) @$x,$y
    }
}

proc HVRelease1 {varname x y} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVRelease1"
    }

    if {$var(sel,x) != -1 && 
	$var(sel,y) != -1} {
	set var(copy) [$var(widget) text ascii @$var(sel,x),$var(sel,y) @$x,$y]
	selection own -command [list HVLostSelection $varname] $var(top)
    }
}

# Commands

proc HVCopyCmd {varname} {
    upvar #0 $varname var
    global $varname

    clipboard clear -displayof $var(top)
    clipboard append -displayof $var(top) $var(copy)
}

proc HVExportSelection {varname offset bytes} {
    upvar #0 $varname var
    global $varname

    if {$var(copy) != {}} {
	return [string range $var(copy) $offset [expr $offset+$bytes]]
    }
}

proc HVLostSelection {varname} {
    upvar #0 $varname var
    global $varname

    $var(widget) selection clear
    set var(copy) {}
}

proc HVURLDialogCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug

    set url "$var(url)"
    if {[EntryDialog [msgcat::mc {URL}] [msgcat::mc {Enter World Wide Web Location (URL)}] 80 url]} {
	if {[string length $url] == 0} {
	    return
	}

	ParseURL $url r
	switch -- $r(scheme) {
	    {} {
		# append 'http://' if needed
		if {[string range $r(path) 0 0] == "/"} {
		    set url "http:/$url"
		} else {
		    set url "http://$url"
		}
		
		if {$debug(tcl,hv)} {
		    puts "HVURLDialogCmd new $url"
		}
	    }
	}

	# clear the base
	$var(widget) config -base {}

	HVClearIndex $varname 0
	HVClearAll $varname
	# no need to resolve
	HVLoadURL $varname $url {}
    }
}

proc HVFileDialogCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug

    set fn [OpenFileDialog hvhtmlfbox]
    if {"$fn" != {}} {
	# clear the base
	$var(widget) config -base {}

	HVClearIndex $varname 0
	HVClearAll $varname
	# no need to resolve
	HVLoadURL $varname "$fn" {}
    }
}

proc HVBackCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVBackCmd index $var(index)"
    }

    incr ${varname}(index) -1
    if {[info exists ${varname}(index,$var(index))]} {
	set url [lindex $var(index,$var(index)) 0]
	set query [lindex $var(index,$var(index)) 1]
	if {$debug(tcl,hv)} {
	    puts "HVBackCmd $var(index) $url $query"
	}
	# clear the base
	$var(widget) config -base {}

	# HVGotoHTML will incr the index again
	incr ${varname}(index) -1
	# no need to resolve
	HVLoadURL $varname $url $query
    } else {
	incr ${varname}(index)
    }
}

proc HVForwardCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVForwardCmd $var(index)"
    }

    incr ${varname}(index)
    if {[info exists ${varname}(index,$var(index))]} {
	set url [lindex $var(index,$var(index)) 0]
	set query [lindex $var(index,$var(index)) 1]
	if {$debug(tcl,hv)} {
	    puts "HVForwardCmd $var(index) $url $query"
	}
	# clear the base
	$var(widget) config -base {}

	# HVGotoHTML will incr the index again
	incr ${varname}(index) -1
	# no need to resolve
	HVLoadURL $varname $url $query
    } else {
	incr ${varname}(index) -1
    }
}

proc HVGotoCmd {varname nn} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVGotoCmd $nn"
    }

    set var(index) $nn
    if {[info exists ${varname}(index,$var(index))]} {
	set url [lindex $var(index,$var(index)) 0]
	set query [lindex $var(index,$var(index)) 1]
	if {$debug(tcl,hv)} {
	    puts "HVGotoCmd $var(index) $url $query"
	}
	# clear the base
	$var(widget) config -base {}

	# HVGotoHTML will incr the index again
	incr ${varname}(index) -1
	# no need to resolve
	HVLoadURL $varname $url $query
    } else {
	incr ${varname}(index)
    }
}

proc HVReloadCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVReloadCmd"
    }

    # clear the base
    $var(widget) config -base {}

    # HVGotoHTML will incr the index again
    incr ${varname}(index) -1
    # no need to resolve
    HVLoadURL $varname $var(url) $var(query)
}

proc HVStopCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "\n*** HVStopCmd ***\n"
    }

    HVCancel $varname
}

proc HVPageSourceCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVPageSourceCmd"
    }

    SimpleTextDialog ${varname}txt $var(url) 80 20 insert top $var(data)
}

proc HVFontCmd {varname} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVFontCmd"
    }

    HVRefresh $varname
}

proc HVArchUserCmd {varname title url} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVArchUserCmd"
    }

    if {[string length $url] == 0} {
	return
    }

    ParseURL $url r
    switch -- $r(scheme) {
	{} {
	    # append 'http://' if needed
	    if {[string range $r(path) 0 0] == "/"} {
		set url "http:/$url"
	    } else {
		set url "http://$url"
	    }
	    
	    if {$debug(tcl,hv)} {
		puts "HVArchUserCmd new $url"
	    }
	}
    }
    HV $varname $title $url 2 {}
}

proc HVAnalysisCmd {varname title url} {
    upvar #0 $varname var
    global $varname

    global debug
    if {$debug(tcl,hv)} {
	puts "HVAnalysisCmd $varname $title $url"
    }

    if {[string length $url] == 0} {
	HV $varname "$title" {} 2 {}
    } else {
	ParseURL $url r
	switch -- $r(scheme) {
	    {} {
		# append 'http://' if needed
		if {[string range $r(path) 0 0] == "/"} {
		    set url "http:/$url"
		} else {
		    set url "http://$url"
		}
		
		if {$debug(tcl,hv)} {
		    puts "HVURLDialogCmd new $url"
		}
	    }
	}
	HV $varname "$title" $url 2 {} 
    }
}

proc HVAnalysisURL {which i url} {
    set varname "at${which}${i}"
    global $varname

    HVClearAll $varname
    HVSetAnalysis $varname 1 $which $i
    HVLoadURL $varname $url {}
}

proc HVAnalysisCancel {which i} {
    set varname "at${which}${i}"
    global $varname

    HVCancel $varname
}

# Archive Servers
# Optical

# Infrared

# High Energy

proc HVArchChandraChaser {} {
    global current

    set coord {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 degrees]
	    set size \
		[expr [lindex [$current(frame) get fits size wcs fk5 arcmin] 0]/2.]
	}
    }

    set l {}
    if {[string length $coord] != 0} {
	lappend l "1 lon [lindex $coord 0]"
	lappend l "1 lat [lindex $coord 1]"
	lappend l "1 radius $size"
    }

    global hvchandrachaser
    HV hvchandrachaser {Chandra Chaser} http://cda.harvard.edu/chaser/mainEntry.do 2 $l
}

proc HVArchChandraPop {} {
    global current

    set coord {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 degrees]
	    set size \
		[expr [lindex [$current(frame) get fits size wcs fk5 arcmin] 0]/2.]
	}
    }

    set l {}
    if {[string length $coord] != 0} {
	lappend l "1 lon [lindex $coord 0]"
	lappend l "1 lat [lindex $coord 1]"
	lappend l "1 radius $size"
	lappend l "1 searchBy position"
    }

    global hvchandrapop
    HV hvchandrapop {Chandra Popular} http://cda.harvard.edu/pop/mainEntry.do 2 $l
}

proc HVArchChandraFTP {} {
    global current

    set ra {}
    set dec {}
    set wid {}

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    set ra [lindex $coord 0]
	    set dec [lindex $coord 1]

	    set wid [lindex [$current(frame) get fits size wcs fk5 degrees] 0]
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 ra \{$ra\}"
	lappend l "1 dec \{$dec\}"
	lappend l "1 wid \{$wid\}"
    }

    global hvchandraftp
    HV hvchandraftp {Chandra FTP} \
	http://www.cfa.harvard.edu/archive/chandra/search 2 $l
}

proc HVArchRosat {} {
    global current

    set ra {}
    set dec {}
    set cprd {}

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    set ra [split [lindex $coord 0] :]
	    set dec [split [lindex $coord 1] :]
	    set raa "[lindex $ra 0]h[lindex $ra 1]m[lindex $ra 2]s"
	    set decc "[lindex $dec 0]d[lindex $dec 1]m[lindex $dec 2]s"
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 lon \{$raa\}"
	lappend l "1 lat \{$decc\}"
    }
    lappend l "1 cprd im1 \{photon image 0.1-2.4 keV (fits)\}"

    global hvrosat
    HV hvrosat {Rosat All-Sky} \
	http://www.xray.mpe.mpg.de/cgi-bin/rosat/rosat-survey 2 $l
}

proc HVArchSkyView {} {
    global current

    set value {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 degrees]
	    set value "[lindex $coord 0], [lindex $coord 1]"
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "1 VCOORD \{$value\}"
    }

    global hvskyview
    HV hvskyview SkyView http://skyview.gsfc.nasa.gov/cgi-bin/basicform.pl 2 $l
}

proc HVArchW3Browse {} {
    global current

    set value {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 degrees]
	    set value "[lindex $coord 0], [lindex $coord 1]"
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "3 Entry \{$value\}"
    }

    global hvw3browse
    HV hvw3browse W3Browse \
	http://heasarc.gsfc.nasa.gov/db-perl/W3Browse/w3browse.pl 2 $l
}

# Radio

proc HVArchNVSS {} {
    global current

    set ra {}
    set dec {}
    set sra 1
    set sdec 1

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    regsub -all {:} [lindex $coord 0] { } ra
	    regsub -all {:} [lindex $coord 1] { } dec

	    set s [$current(frame) get fits size wcs fk5 degrees]
	    set sra [lindex $s 0]
	    set sdec [lindex $s 1]
	    if {$sra > 2} {
		set sra 2
	    }
	    if {$sdec > 2} {
		set sdec 2
	    }
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 RA \{$ra\}"
	lappend l "1 Dec \{$dec\}"
	lappend l "1 Size \{$sra $sdec\}"
    }
    lappend l "1 Type \{image/x-fits\} \{FITS Image\}"

    global hvnvss
    HV hvnvss NVSS http://www.cv.nrao.edu/nvss/postage.shtml 2 $l
}

proc HVArch4MASS {} {
    global current

    set ra {}
    set dec {}

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    regsub -all {:} [lindex $coord 0] { } ra
	    regsub -all {:} [lindex $coord 1] { } dec
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 RA \{$ra\}"
	lappend l "1 Dec \{$dec\}"
    }

    global hvmass4
    HV hvmass4 4MASS http://www.cv.nrao.edu/4mass/findFITS.shtml 2 $l
}

proc HVArchSIRTF {} {
    global current

    set ra {}
    set dec {}
    set sra 1
    set sdec 1

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    regsub -all {:} [lindex $coord 0] { } ra
	    regsub -all {:} [lindex $coord 1] { } dec

	    set s [$current(frame) get fits size wcs fk5 arcmin]
	    set sra [lindex $s 0]
	    set sdec [lindex $s 1]
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 RA \{$ra\}"
	lappend l "1 Dec \{$dec\}"
	lappend l "1 Size \{$sra $sdec\}"
    }
    lappend l "1 Type \{image/x-fits\} \{FITS Image\}"

    global hvsirtf
    HV hvsirtf SIRTF http://www.cv.nrao.edu/sirtf_fls/SFpostage.shtml 2 $l
}

proc HVArchFirst {} {
    global current

    set value {}
    set size {4.5}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set value [$current(frame) get fits center wcs fk5 sexagesimal]
	    set size [lindex [$current(frame) get fits size wcs fk5 arcmin] 0]
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "1 RA \{$value\}"
	lappend l "1 ImageSize \{$size\}"
	lappend l "1 ImageType \{FITS Image\}"
    }

    global hvfirst
    HV hvfirst First http://third.ucllnl.org/cgi-bin/firstcutout 2 $l
}

# Other

proc HVArchDataScope {} {
    global current

    set value {}
    set size {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 degrees]
	    set value "[lindex $coord 0], [lindex $coord 1]"
	    set size [lindex [$current(frame) get fits size wcs fk5 degrees] 0]
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "1 position \{$value\}"
	lappend l "1 delta \{$size\}"
    }

    global hvnvo
    HV hvnvo {NVO DataScope}  http://heasarc.gsfc.nasa.gov/cgi-bin/vo/datascope_old/init.pl 2 $l
}

proc HVArchNED {} {
    global current

    set ra {}
    set dec {}

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    regsub -all {:} [lindex $coord 0] { } ra
	    regsub -all {:} [lindex $coord 1] { } dec
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 lon \{$ra\}"
	lappend l "1 lat \{$dec\}"
    }

    global hvned
    HV hvned {NED} http://nedwww.ipac.caltech.edu/ 2 $l
}

proc HVArchSIMBADSAO {} {
    global hvsimbadsao
    HVArchSIMBAD hvsimbadsao {SIMBAD@SAO} http://simbad.harvard.edu/Simbad
}

proc HVArchSIMBADCDS {} {
    global hvsimbadcds
    HVArchSIMBAD hvsimbadcds {SIMBAD@CDS} http://simbad.u-strasbg.fr/Simbad
}

proc HVArchSIMBAD {varname title url} {
    upvar #0 $varname var
    global $varname

    global current

    set value {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set value [$current(frame) get fits center wcs fk5 sexagesimal]
	    regsub -all {:} $value { } value
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "1 Coord \{$value\}"
    }

    HV $varname $title $url 2 $l
}

proc HVArchADSSAO {} {
    global hvadssao
    HVArchADS hvadssao {ADS@SAO} http://adswww.harvard.edu/
}

proc HVArchADSCDS {} {
    global hvadscds
    HVArchADS hvadscds {ADS@CDS} http://cdsads.u-strasbg.fr/
}

proc HVArchADS {varname title url} {
    upvar #0 $varname var
    global $varname

    global current

    set value {}
    set size {}
    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set value [$current(frame) get fits center wcs fk5 sexagesimal]
	    set size [lindex [$current(frame) get fits size wcs fk5 arcmin] 0]
	    regsub -all {:} $value { } value
	    # limit size to 1 arcmin, otherwise you get too many responces
	    if {$size < .1} {
		append value " : $size"
	    } else {
		append value " : .1"
	    }
	}
    }

    set l {}
    if {[string length $value] != 0} {
	lappend l "1 object \{$value\}"
    }

    HV $varname $title $url 2 $l
}

proc HVArchSAOTDC {} {
    global current

    set ra {}
    set dec {}
    set rad {}

    if {$current(frame) != {}} {
	if {[$current(frame) has wcs equatorial wcs]} {
	    set coord [$current(frame) get fits center wcs fk5 sexagesimal]
	    set ra [lindex $coord 0]
	    set dec [lindex $coord 1]

	    set s [lindex [$current(frame) get fits size wcs fk5 arcsec] 0]
	    set rad [expr $s/2.]
	}
    }

    set l {}
    if {[string length $ra] != 0} {
	lappend l "1 ra \{$ra\}"
	lappend l "1 dec \{$dec\}"
	lappend l "1 rad \{$rad\}"
    }

    global hvsaotdc
    HV hvsaotdc {SAO Telescope Data Center} http://tdc-www.harvard.edu/archive/ 2 $l
}

# Process Cmds

proc ProcessWebCmd {varname iname} {
    global hvweb

    set w {hvweb}

    upvar $varname var
    upvar $iname i

    set url [lindex $var $i]
    if {[string length $url] == 0} {
	HV $w Web {} 2 {}
    } else {
	ParseURL $url r
	switch -- $r(scheme) {
	    {} {
		# append 'http://' if needed
		if {[string range $r(path) 0 0] == "/"} {
		    set url "http:/$url"
		} else {
		    set url "http://$url"
		}
	    }
	}
	HV $w Web $url 2 {} 
    }
}

