#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc LineDialog {frame id} {
    global marker

    set w ".marker$id"

    # see if we already have a header window visible
    if [winfo exists $w] {
	raise $w
	return
    }

    # variables
    set arrows [$frame get marker $id line arrow]
    set marker($frame,$id,p1arrow) [lindex $arrows 0]
    set marker($frame,$id,p2arrow) [lindex $arrows 1]

    # base
    MarkerBaseLineDialog $frame $id 375 150

    # Arrows
    checkbutton $w.ref.f.p1arrow -variable marker($frame,$id,p1arrow) \
	-text [msgcat::mc {Arrow}] -command "LineArrow $frame $id"
    checkbutton $w.ref.f.p2arrow -variable marker($frame,$id,p2arrow) \
	-text [msgcat::mc {Arrow}] -command "LineArrow $frame $id"
    grid $w.ref.f.p1arrow -row 0 -column 4 -padx 4 -sticky w
    grid $w.ref.f.p2arrow -row 1 -column 4 -padx 4 -sticky w
}

# actions

proc LineClose {frame id} {
    MarkerBaseLineClose $frame $id
}

proc LineApply {frame id} {
    global marker

    $frame marker $id line point \
	$marker($frame,$id,system) $marker($frame,$id,sky) \
	$marker($frame,$id,x) $marker($frame,$id,y) \
	$marker($frame,$id,x2) $marker($frame,$id,y2)

    MarkerBaseLineApply $frame $id
}

proc LineArrow {frame id} {
    global marker
    
    $frame marker $id line arrow \
	$marker($frame,$id,p1arrow) $marker($frame,$id,p2arrow)
}

# callbacks

proc LineCoordCB {frame id} {
    global debug
    if {$debug(tcl,marker)} {
	puts "LineCoordCB"
    }
    MarkerBaseCoordCB $frame $id
    LineEditCB $frame $id
}

proc LineEditCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "LineEditCB"
    }

    MarkerBaseLineEditCB $frame $id

    set marker($frame,$id,dist) \
	[$frame get marker $id line length \
	     $marker($frame,$id,dcoord) $marker($frame,$id,dformat)]

    set marker($frame,$id,angle) \
	[$frame get marker $id angle \
	     $marker($frame,$id,system) $marker($frame,$id,sky)]
}

proc LineDistCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "LineDistCB"
    }

    MarkerBaseDistCB $frame $id

    set marker($frame,$id,dist) \
	[$frame get marker $id line length \
	     $marker($frame,$id,dcoord) $marker($frame,$id,dformat)]
}


