#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBaseDialog {frame id width height} {
    global marker
    global ds9

    set w ".marker$id"
    set mb ".mb$id"

    set tt [$frame get marker $id type]
    switch -- [lindex $tt 1] {
	point {
	    set type "[string totitle [lindex $tt 0]] [string totitle [lindex $tt 1]]"
	    set which Point
	}
	{} {
	    set type [string totitle [lindex $tt 0]]
	    set which $type
	}
    }

    # variables - some may already be initialized
    set marker($frame,$id,which) $which
    if {![info exists marker($frame,$id,system)]} {
	set marker($frame,$id,system) $marker(dialog,system)
    }
    set marker($frame,$id,clabel) $marker($frame,$id,system)
    if {![info exists marker($frame,$id,sky)]} {
	set marker($frame,$id,sky) $marker(dialog,sky)
    }
    if {![info exists marker($frame,$id,skyformat)]} {
	set marker($frame,$id,skyformat) $marker(dialog,skyformat)
    }

    # init
    MarkerBaseTextCB $frame $id
    MarkerBaseColorCB $frame $id
    MarkerBaseLineWidthCB $frame $id
    MarkerBasePropertyCB $frame $id
    MarkerBaseFontCB $frame $id
    ${which}CoordCB $frame $id

    # callbacks
    $frame marker $id callback delete MarkerBaseDeleteCB $frame
    $frame marker $id callback text MarkerBaseTextCB $frame
    $frame marker $id callback color MarkerBaseColorCB $frame
    $frame marker $id callback width MarkerBaseLineWidthCB $frame
    $frame marker $id callback property MarkerBasePropertyCB $frame
    $frame marker $id callback font MarkerBaseFontCB $frame

    # menus
    MarkerBaseMenu $frame $id
    MarkerBaseFileMenu $frame $id
    DefaultColorMenu $mb.color marker($frame,$id,color) \
	"MarkerBaseColor $frame $id"
    DefaultWidthMenu $mb.width marker($frame,$id,linewidth) \
	"MarkerBaseLineWidth $frame $id"
    MarkerBasePropertyMenu $frame $id
    DefaultFontMenu $mb.font \
	marker($frame,$id,font) marker($frame,$id,font,size) \
	marker($frame,$id,font,style) "MarkerBaseFont $frame $id"
    MarkerBaseCoordMenu $frame $id

    # window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w [msgcat::mc "$type"]
    wm iconname $w [msgcat::mc "$type"]
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "${which}Close $frame $id"
    $w configure -menu $mb
    wm minsize $w $width $height

    # dialog
    frame $w.basic -relief ridge -bd 1
    frame $w.basic.f
    frame $w.ref -relief ridge -bd 1
    frame $w.ref.f
    frame $w.buttons -relief ridge -bd 1

    pack $w.basic.f $w.ref.f -anchor w -padx 4 -pady 4
    pack $w.basic $w.ref -fill x 
    pack $w.buttons -side bottom -fill x -ipadx 4 -ipady 4

    # ID
    label $w.basic.f.idTitle -text [msgcat::mc {Number}]
    label $w.basic.f.idValue -text "$id"

    # Text
    label $w.basic.f.textTitle -text [msgcat::mc {Text}]
    entry $w.basic.f.textValue -textvariable marker($frame,$id,text) -width 45

    grid $w.basic.f.idTitle $w.basic.f.idValue -padx 4 -sticky w
    grid $w.basic.f.textTitle $w.basic.f.textValue -padx 4 -sticky w

    # Buttons
    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command "${which}Apply $frame $id"
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "${which}Close $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "${which}Apply $frame $id"

    # some window managers need a hint
    raise $w
}

# actions

proc MarkerBaseClose {frame id} {
    $frame marker $id delete callback delete MarkerBaseDeleteCB
    $frame marker $id delete callback text MarkerBaseTextCB
    $frame marker $id delete callback color MarkerBaseColorCB
    $frame marker $id delete callback width MarkerBaseLineWidthCB
    $frame marker $id delete callback property MarkerBasePropertyCB
    $frame marker $id delete callback font MarkerBaseFontCB

    MarkerBaseDeleteCB $frame $id
}

proc MarkerBaseApply {frame id} {
    global marker

    $frame marker $id text \{$marker($frame,$id,text)\}
    UpdateRegionMenu
}

proc MarkerBaseColor {frame id} {
    global marker

    $frame marker $id color $marker($frame,$id,color)
}

proc MarkerBaseLineWidth {frame id} {
    global marker

    $frame marker $id width $marker($frame,$id,linewidth)
}

proc MarkerBaseProperty {frame id prop} {
    global marker

    $frame marker $id property $prop $marker($frame,$id,$prop)
}

proc MarkerBaseFont {frame id} {
    global marker

    $frame marker $id font \
	\"$marker($frame,$id,font) $marker($frame,$id,font,size) \
	$marker($frame,$id,font,style)\"
}

# callbacks

proc MarkerBaseDeleteCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseDeleteCB"
    }

    set w ".marker$id"
    set mb ".mb$id"

    destroy $w
    destroy $mb

    # variables
    foreach m [array names marker] {
	set mm [split $m ,]
	if {[lindex $mm 0] == $frame && [lindex $mm 1] == $id} {
	    unset marker($m)
	}
    }
}

proc MarkerBaseTextCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseTextCB"
    }

    set marker($frame,$id,text) [$frame get marker $id text]
}

proc MarkerBaseColorCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseColorCB"
    }

    set marker($frame,$id,color) [$frame get marker $id color]
}

proc MarkerBaseLineWidthCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseLineWidthCB"
    }

    set marker($frame,$id,linewidth) [$frame get marker $id width]
}

proc MarkerBasePropertyCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBasePropertyCB"
    }

    set marker($frame,$id,edit) [$frame get marker $id property edit]
    set marker($frame,$id,move) [$frame get marker $id property move]
    set marker($frame,$id,rotate) [$frame get marker $id property rotate]
    set marker($frame,$id,delete) [$frame get marker $id property delete]
    set marker($frame,$id,fixed) [$frame get marker $id property fixed]
    set marker($frame,$id,include) [$frame get marker $id property include]
    set marker($frame,$id,source) [$frame get marker $id property source]
}

proc MarkerBaseFontCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseFontCB"
    }

    set f [$frame get marker $id font]

    set marker($frame,$id,font) [lindex $f 0]
    set marker($frame,$id,font,size) [lindex $f 1]
    set marker($frame,$id,font,style) [lindex $f 2]
}

proc MarkerBaseCoordCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseCoordCB"
    }

    AdjustCoord $frame marker($frame,$id,system)

    set marker($frame,$id,clabel) $marker($frame,$id,system)
    switch -- $marker($frame,$id,system) {
	image -
	physical -
	amplifier -
	detector {}
	default {
	    if [$frame has wcs $marker($frame,$id,system)] {
		if [$frame has wcs equatorial $marker($frame,$id,system)] {
		    set marker($frame,$id,clabel) $marker($frame,$id,sky)
		} else {
		    set name [$frame get wcs name $marker($frame,$id,system)]
		    if {$name != {}} {
			set marker($frame,$id,clabel) $name
		    }
		}
	    }
	}
    }
}

proc MarkerBaseDistCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseDistCB"
    }

    AdjustDist $frame marker($frame,$id,dcoord)

    set marker($frame,$id,dlabel) $marker($frame,$id,dcoord)
    switch -- $marker($frame,$id,dcoord) {
	image -
	physical -
	amplifier -
	detector {}
	default {
	    if [$frame has wcs $marker($frame,$id,dcoord)] {
		if [$frame has wcs equatorial $marker($frame,$id,dcoord)] {
		    set marker($frame,$id,dlabel) $marker($frame,$id,dformat)
		} else {
		    set name [$frame get wcs name $marker($frame,$id,dcoord)]
		    if {$name != {}} {
			set marker($frame,$id,dlabel) $name
		    }
		}
	    }
	}
    }
}

# menus

proc MarkerBaseMenu {frame id} {
    global marker

    set mb ".mb$id"

    menu $mb
    AppleMenu mb

    $mb add cascade -label [msgcat::mc {File}] -menu $mb.file
    $mb add cascade -label [msgcat::mc {Color}] -menu $mb.color
    $mb add cascade -label [msgcat::mc {Width}] -menu $mb.width
    $mb add cascade -label [msgcat::mc {Property}] -menu $mb.properties
    $mb add cascade -label [msgcat::mc {Font}] -menu $mb.font
    $mb add cascade -label [msgcat::mc {Coordinate}] -menu $mb.coord
}

proc MarkerBaseFileMenu {frame id} {
    global marker

    set mb ".mb$id"
    set which $marker($frame,$id,which)

    menu $mb.file
    $mb.file add command -label [msgcat::mc {Apply}] \
	-command "${which}Apply $frame $id"
    $mb.file add separator
    $mb.file add command -label [msgcat::mc {Close}] \
	-command "${which}Close $frame $id"
}

proc MarkerBasePropertyMenu {frame id} {
    global marker

    set mb ".mb$id"

    menu $mb.properties
    $mb.properties add checkbutton -label [msgcat::mc {Can Edit}] \
	-variable marker($frame,$id,edit) \
	-command "MarkerBaseProperty $frame $id edit"
    $mb.properties add checkbutton -label [msgcat::mc {Can Move}] \
	-variable marker($frame,$id,move) \
	-command "MarkerBaseProperty $frame $id move"
    $mb.properties add checkbutton -label [msgcat::mc {Can Rotate}] \
	-variable marker($frame,$id,rotate) \
	-command "MarkerBaseProperty $frame $id rotate"
    $mb.properties add checkbutton -label [msgcat::mc {Can Delete}] \
	-variable marker($frame,$id,delete) \
	-command "MarkerBaseProperty $frame $id delete"
    $mb.properties add checkbutton -label [msgcat::mc {Fixed in Size}] \
	-variable marker($frame,$id,fixed) \
	-command "MarkerBaseProperty $frame $id fixed"
    $mb.properties add separator
    $mb.properties add radiobutton -label [msgcat::mc {Include}] \
	-variable marker($frame,$id,include) -value 1 \
	-command "MarkerBaseProperty $frame $id include"
    $mb.properties add radiobutton -label [msgcat::mc {Exclude}] \
	-variable marker($frame,$id,include) -value 0 \
	-command "MarkerBaseProperty $frame $id include"
    $mb.properties add separator
    $mb.properties add radiobutton -label [msgcat::mc {Source}] \
	-variable marker($frame,$id,source) -value 1 \
	-command "MarkerBaseProperty $frame $id source"
    $mb.properties add radiobutton -label [msgcat::mc {Background}] \
	-variable marker($frame,$id,source) -value 0 \
	-command "MarkerBaseProperty $frame $id source"
}

proc MarkerBaseCoordMenu {frame id} {
    global marker
    global ds9

    set mb ".mb$id"
    set which $marker($frame,$id,which)
    set cb ${which}CoordCB

    DefaultCoordMenu $mb.coord marker($frame,$id,system) \
	marker($frame,$id,sky) marker($frame,$id,skyformat) "$cb $frame $id"

    UpdateCoordMenu $frame $mb.coord
}

proc MarkerBaseDistMenu {frame id name label cb coord format} {
    global marker
    global ds9

    set mb ".mb$id"
    $mb add cascade -label $label -menu $mb.$name

    DefaultDistMenu $mb.$name marker($frame,$id,$coord) \
	marker($frame,$id,$format) "$cb $frame $id"

    UpdateDistMenu $frame $mb.$name 0 0
}


