#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBasePandaDialog {frame id width height} {
    global marker
    global ds9

    set w ".marker$id"
    set mb ".mb$id"

    # variables
    set marker($frame,$id,dcoord) $marker(dialog,dist,system)
    set marker($frame,$id,dlabel) $marker(dialog,dist,system)
    set marker($frame,$id,dformat) $marker(dialog,dist,format)

    set marker($frame,$id,method) dist
    set marker($frame,$id,init) 0

    # base
    MarkerBaseAnnulusDialog $frame $id $width $height
    set which $marker($frame,$id,which)

    # menus
    MarkerBaseAnnulusMethodMenu $frame $id
    MarkerBaseDistMenu $frame $id radius Radius ${which}DistCB dcoord dformat

    # callbacks
    $frame marker $id callback move ${which}EditCB $frame
    $frame marker $id callback edit ${which}EditCB $frame
    $frame marker $id callback end edit ${which}EditCB $frame

    # Angles
    label $w.ref.left.bottom.f.ang1Title -text [msgcat::mc {Start}]
    label $w.ref.left.bottom.f.ang2Title -text [msgcat::mc {End}]
    label $w.ref.left.bottom.f.anglesTitle -text [msgcat::mc {Angles}]
    entry $w.ref.left.bottom.f.ang1 -textvariable marker($frame,$id,ang1) \
	-width 13
    entry $w.ref.left.bottom.f.ang2 -textvariable marker($frame,$id,ang2) \
	-width 13
    label $w.ref.left.bottom.f.anglesformat -text [msgcat::mc {Degrees}] -padx 4
    label $w.ref.left.bottom.f.angnumTitle -text [msgcat::mc {Number}]
    entry $w.ref.left.bottom.f.angnum -textvariable marker($frame,$id,angnum) \
	-width 13

    grid x $w.ref.left.bottom.f.ang1Title $w.ref.left.bottom.f.ang2Title \
	-padx 4 -sticky w
    grid $w.ref.left.bottom.f.anglesTitle $w.ref.left.bottom.f.ang1 \
	$w.ref.left.bottom.f.ang2 $w.ref.left.bottom.f.anglesformat \
	-padx 4 -sticky w
    grid $w.ref.left.bottom.f.angnumTitle $w.ref.left.bottom.f.angnum \
	-padx 4 -sticky w

    # Right
    frame $w.ref.right.radius -relief ridge -bd 1
    frame $w.ref.right.angles -relief ridge -bd 1
    pack $w.ref.right.radius $w.ref.right.angles -side left

    # Radius
    frame $w.ref.right.radius.title
    frame $w.ref.right.radius.value
    pack $w.ref.right.radius.title $w.ref.right.radius.value \
	-side top -padx 4 -pady 4

    label $w.ref.right.radius.title.title -text [msgcat::mc {Radius}]
    label $w.ref.right.radius.title.radiusformat \
	-textvariable marker($frame,$id,dlabel) -padx 4

    grid $w.ref.right.radius.title.title \
	$w.ref.right.radius.title.radiusformat \
	-padx 4 -sticky news

    set txtscr $w.ref.right.radius.value.yscroll
    set marker($frame,$id,annulitxt) $w.ref.right.radius.value.txt
    text $marker($frame,$id,annulitxt) -height 15 -width 15 -wrap none \
	-font {courier 12} -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $marker($frame,$id,annulitxt) yview] \
	-orient vertical

    grid $marker($frame,$id,annulitxt) $txtscr -sticky news

    # Angles
    frame $w.ref.right.angles.title
    frame $w.ref.right.angles.value
    pack $w.ref.right.angles.title $w.ref.right.angles.value \
	-side top -padx 4 -pady 4

    label $w.ref.right.angles.title.title -text [msgcat::mc {Angles}]
    label $w.ref.right.angles.title.anglesformat -text [msgcat::mc {Degrees}] \
	-padx 4

    grid $w.ref.right.angles.title.title \
	$w.ref.right.angles.title.anglesformat \
	-padx 4 -sticky news

    set txtscr $w.ref.right.angles.value.yscroll
    set marker($frame,$id,angtxt) $w.ref.right.angles.value.txt
    text $marker($frame,$id,angtxt) -height 15 -width 15 -wrap none \
	-font {courier 12} -yscrollcommand "$txtscr set"
    scrollbar $txtscr -command [list $marker($frame,$id,angtxt) yview] \
	-orient vertical

    grid $marker($frame,$id,angtxt) $txtscr -sticky news

    set marker($frame,$id,init) 1
}

# actions

proc MarkerBasePandaClose {frame id} {
    global marker

    set which $marker($frame,$id,which)
    $frame marker $id delete callback move ${which}EditCB
    $frame marker $id delete callback edit ${which}EditCB
    $frame marker $id delete callback end edit ${which}EditCB
    MarkerBaseCenterClose $frame $id
}

proc MarkerBasePandaApply {frame id} {
    global marker

    set which $marker($frame,$id,which)
    set levels {}
    regsub -all "\n" "[$marker($frame,$id,annulitxt) get 1.0 end]" " " levels
    # and trim any trailing spaces
    set levels [string trimright $levels " "]

    set angles {}
    regsub -all "\n" "[$marker($frame,$id,angtxt) get 1.0 end]" " " angles
    # and trim any trailing spaces
    set angles [string trimright $angles " "]

    if {($levels != {}) && ($angles != {})} {
	$frame marker $id $which edit "\{$angles\}" "\{$levels\}" \
	    $marker($frame,$id,system) $marker($frame,$id,sky) \
	    $marker($frame,$id,dcoord) $marker($frame,$id,dformat)
    }

    MarkerBaseCenterApply $frame $id
}

proc MarkerBasePandaGenerateAngles {frame id} {
    global marker

    $marker($frame,$id,angtxt) delete 1.0 end

    set ang1 $marker($frame,$id,ang1)
    set ang2 $marker($frame,$id,ang2)
    set angnum $marker($frame,$id,angnum)

    if {($ang1 != {}) && ($ang2 != {}) && ($angnum != {})} {
	# normalize between 0 <= ang < 360
	if {$ang1 > 0} {
	    while {$ang1 >= 360} {
		set ang1 [expr $ang1-360]
	    }
	} else {
	    while {$ang1 < 0} {
		set ang1 [expr $ang1+360]
	    }
	}

	if {$ang2 > 0} {
	    while {$ang2 >= 360} {
		set ang2 [expr $ang2-360]
	    }
	} else {
	    while {$ang2 < 0} {
		set ang2 [expr $ang2+360]
	    }
	}

	# with ang2 > ang1
	while {$ang1 >= $ang2} {
	    set ang2 [expr $ang2+360]
	}

	for {set i 0} {$i<=$angnum} {incr i} {
	    set v [expr ((($ang2-$ang1)/double($angnum))*$i)+$ang1]
	    $marker($frame,$id,angtxt) insert end "$v\n"
	}
    }
}

# callbacks


proc MarkerBasePandaCoordCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBasePandaCoordCB"
    }

    set which $marker($frame,$id,which)
    MarkerBaseCoordCB $frame $id
    MarkerBaseCenterMoveCB $frame $id
    if {$marker($frame,$id,init)} {
	${which}EditCB $frame $id
    }
}

proc MarkerBasePandaDistCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBasePandaDistCB"
    }

    set which $marker($frame,$id,which)
    MarkerBaseDistCB $frame $id
    ${which}EditCB $frame $id
}

