#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MarkerBaseRectDialog {frame id width height unit} {
    global marker
    global ds9

    set w ".marker$id"
    set unit2 [string totitle $unit]

    # variables
    set marker($frame,$id,dcoord) $marker(dialog,dist,system)
    set marker($frame,$id,dlabel) $marker(dialog,dist,system)
    set marker($frame,$id,dformat) $marker(dialog,dist,format)

    # base
    MarkerBaseCenterDialog $frame $id $width $height
    set which $marker($frame,$id,which)

    # menus
    MarkerBaseDistMenu $frame $id $unit $unit2 ${which}DistCB dcoord dformat

    # init
    ${which}DistCB $frame $id
    MarkerBaseCenterRotateCB $frame $id

    # callbacks
    $frame marker $id callback move ${which}EditCB $frame
    $frame marker $id callback edit ${which}EditCB $frame
    $frame marker $id callback rotate MarkerBaseCenterRotateCB $frame

    # Radius
    label $w.ref.f.radiusTitle -text $unit2
    entry $w.ref.f.radius1 -textvariable marker($frame,$id,radius1) -width 13 
    entry $w.ref.f.radius2 -textvariable marker($frame,$id,radius2) -width 13 
    label $w.ref.f.radiusCoord -textvariable marker($frame,$id,dlabel) -padx 4

    grid $w.ref.f.radiusTitle $w.ref.f.radius1 $w.ref.f.radius2 \
	$w.ref.f.radiusCoord -padx 4 -sticky w

    label $w.ref.f.angleTitle -text [msgcat::mc {Angle}]
    entry $w.ref.f.angle -textvariable marker($frame,$id,angle) -width 13 
    label $w.ref.f.angleUnit -padx 4 -text [msgcat::mc {Degrees}]

    grid $w.ref.f.angleTitle $w.ref.f.angle x $w.ref.f.angleUnit \
	-padx 4 -sticky w
}

# actions

proc MarkerBaseRectClose {frame id} {
    global marker
    set which $marker($frame,$id,which)

    $frame marker $id delete callback move ${which}EditCB
    $frame marker $id delete callback edit ${which}EditCB
    $frame marker $id delete callback rotate MarkerBaseCenterRotateCB
    MarkerBaseCenterClose $frame $id
}

proc MarkerBaseRectApply {frame id} {
    global marker
    set which $marker($frame,$id,which)

    if {$marker($frame,$id,radius1) != {} &&
	$marker($frame,$id,radius2) !={}} {
	$frame marker $id $which radius \
	    $marker($frame,$id,radius1) $marker($frame,$id,radius2) \
	    $marker($frame,$id,dcoord) $marker($frame,$id,dformat)
    }

    MarkerBaseCenterRotate $frame $id
    MarkerBaseCenterApply $frame $id
}

# callbacks

proc MarkerBaseRectCoordCB {frame id} {
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseRectCoordCB"
    }
    MarkerBaseCoordCB $frame $id
    MarkerBaseCenterMoveCB $frame $id
    MarkerBaseCenterRotateCB $frame $id
}

proc MarkerBaseRectEditCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseRectEditCB"
    }
    set which $marker($frame,$id,which)

    set r [$frame get marker $id $which radius \
	       $marker($frame,$id,dcoord) $marker($frame,$id,dformat)]

    set marker($frame,$id,radius1) [lindex $r 0]
    set marker($frame,$id,radius2) [lindex $r 1]
}

proc MarkerBaseRectDistCB {frame id} {
    global marker
    global debug
    if {$debug(tcl,marker)} {
	puts "MarkerBaseRectDistCB"
    }
    set which $marker($frame,$id,which)

    MarkerBaseDistCB $frame $id
    ${which}EditCB $frame $id
}
