#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc MaskUpdate {} {
    global current
    global mask

    if {$current(frame) != {}} {
	$current(frame) mask $mask(view)
    }
}

proc MaskClear {} {
    global current

    if {$current(frame) != {}} {
	$current(frame) mask clear
    }
}

proc MaskColor {} {
    global current
    global mask

    if {$current(frame) != {}} {
	$current(frame) mask color $mask(color)
    }
}

proc UpdateMaskMenu {} {
    global current
    global mask
    global ds9

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateMaskMenu"
    }

    if {$current(frame) == {}} {
	return
    }

    set mask(view) [$current(frame) get mask]
    set mask(color) [$current(frame) get mask color]

    if {[$current(frame) get type] == {base}} {
	$ds9(mb).analysis entryconfig [msgcat::mc {Mask}] -state normal
	$ds9(mb).analysis entryconfig [msgcat::mc {Mask Parameters}] \
	    -state normal

	if {[$current(frame) has fits]} {
	    $ds9(mb).analysis.mask entryconfig \
		"[msgcat::mc {Open}]..." -state normal
	    $ds9(mb).analysis.mask entryconfig \
		[msgcat::mc {Open Other}] -state normal

	    if {[$current(frame) has fits cube]} {
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Multi Ext as Data Cube}]..." \
		    -state normal
	    } else {
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Multi Ext as Data Cube}]..." \
		    -state disabled
	    }

	    if {[$current(frame) has fits mosaic]} {
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic IRAF}]..." -state normal
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic IRAF Segment}]..." -state normal
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic WCS}]..." -state normal
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic WCS Segment}]..." -state normal
	    } else {
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic IRAF}]..." -state disabled
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
		    -state disabled
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic WCS}]..." -state disabled
		$ds9(mb).analysis.mask.other entryconfig \
		    "[msgcat::mc {Open Mosaic WCS Segment}]..." -state disabled
	    }

	} else {
	    $ds9(mb).analysis.mask entryconfig "[msgcat::mc {Open}]..." \
		-state disabled
	    $ds9(mb).analysis.mask entryconfig [msgcat::mc {Open Other}] \
		-state disabled
	}

    } else {
	$ds9(mb).analysis entryconfig [msgcat::mc {Mask}] -state disabled
	$ds9(mb).analysis entryconfig [msgcat::mc {Mask Parameters}] \
	    -state disabled
    }
}

proc ProcessMaskCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global mask

    set rr {}

    switch -- [string tolower [lindex $var $i]] {
	color {
	    incr i
	    set mask(color) [lindex $var $i]
	    MaskColor
	}
	clear {
	    MaskClear
	}

	yes -
	true -
	1 -
	no -
	false -
	0 {
	    set mask(view) [FromYesNo [lindex $var $i]]
	    MaskUpdate
	}

	default {
	    # this is devation from standard API
	    # set mask(view) 1
	    # MaskUpdate

	    set rr mask
	    incr i -1
	}
    }

    return $rr
}
