#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Menus

# Default colormap names
# [msgcat::mc {Grey}]
# [msgcat::mc {Red}]
# [msgcat::mc {Green}]
# [msgcat::mc {Blue}]
# [msgcat::mc {Heat}]
# [msgcat::mc {Cool}]
# [msgcat::mc {Rainbow}]
# [msgcat::mc {Standard}]
# [msgcat::mc {Staircase}]
# [msgcat::mc {Color}]

proc ColorMenu {} {
    global colorbar
    global ds9

    menu $ds9(mb).color

    # we don't have to worry about user added cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	$ds9(mb).color add radiobutton -label [msgcat::mc $title] \
	    -variable colorbar(map) -command "ChangeColormapID $ii"
	incr colorbar(count)
	incr colorbar(cmaps)
    }

    $ds9(mb).color add separator
    $ds9(mb).color add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable colorbar(invert) -command InvertColorbar
    $ds9(mb).color add command -label [msgcat::mc {Reset Colormap}] \
	-command ResetColormap
    $ds9(mb).color add separator
    $ds9(mb).color add command -label "[msgcat::mc {Colormap Parameters}]..." \
	-command ColormapDialog
}

proc PrefsDialogColor {} {
    global prefs
    global colorbar

    set w $prefs(tabs)

    frame $w.color
    $prefs(tabs) tab configure color -window $w.color -fill both

    frame $w.color.menu -relief ridge -bd 1
    frame $w.color.truecolor -relief ridge -bd 1
    pack $w.color.menu $w.color.truecolor -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.color

    # Default Menu
    set f $w.color.menu.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Default Menu}]
    menubutton $f.color -relief raised -text [msgcat::mc {Color}] \
	-menu $f.color.menu

    grid $f.title $f.color -padx 4 -sticky w

    set m $f.color.menu
    menu $m

    # we don't wany any user added cmaps
    set id [colorbar list id]
    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	set jj [lindex $id $ii]
	set title [colorbar get name $jj]
	$m add radiobutton -label [msgcat::mc $title] \
	    -variable prefs(colorbar,map)
    }
    $m add separator
    $m add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable prefs(colorbar,invert)

    # Truecolor
    set f $w.color.truecolor.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Truecolor Colorbar}]
    radiobutton $f.full \
	-text [msgcat::mc {Update Full Screen}] \
	-variable colorbar(area) -value 0 -command PrefsColorbarArea
    radiobutton $f.x128 -text {128x128} \
	-variable colorbar(area) -value 128 -command PrefsColorbarArea
    radiobutton $f.x256 -text {256x256} \
	-variable colorbar(area) -value 256 -command PrefsColorbarArea
    radiobutton $f.x512 -text {512x512} \
	-variable colorbar(area) -value 512 -command PrefsColorbarArea
    radiobutton $f.center -text [msgcat::mc {About Center}] \
	-variable colorbar(mode) -value center -command PrefsColorbarArea
    radiobutton $f.click \
	-text [msgcat::mc {About Mouse Click}] \
	-variable colorbar(mode) -value click -command PrefsColorbarArea

    grid $f.title -padx 4 -sticky w
    grid $f.full $f.x128 $f.x256 $f.x512 -padx 4 -sticky w
    grid $f.center $f.click - -padx 4 -sticky w
}

# Buttons

proc ButtonsVarsColor {} {
    global buttons

    # we have a chicken or the egg problem
    # the colorbar has not been defined yet, but we must define vars 
    # before prefs are processed, so hard code all default cmaps
    array set buttons {
	color,grey 1
	color,red 0
	color,green 0
	color,blue 0
	color,a 1
	color,b 1
	color,bb 1
	color,he 1
	color,i8 1
	color,aips0 1
	color,sls 0
	color,hsv 0
	color,heat 1
	color,cool 1
	color,rainbow 1
	color,standard 0
	color,staircase 0
	color,color 0
	color,invert 0
	color,reset 0
	color,params 0
    }
}

proc CreateButtonsColor {} {
    global buttons
    global ds9
    global colorbar

    frame $ds9(buttons).color

    # don't worry about user created cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	set name [string tolower $title]
	RadioButton $ds9(buttons).color.$name \
	    [string tolower [msgcat::mc $title]] \
	    colorbar(map) $title "ChangeColormapID $ii"
    }

    CheckButton $ds9(buttons).color.invert \
	[string tolower [msgcat::mc {Invert}]] colorbar(invert) InvertColorbar
    ButtonButton $ds9(buttons).color.reset \
	[string tolower [msgcat::mc {Reset}]] ResetColormap
    ButtonButton $ds9(buttons).color.params \
	[string tolower [msgcat::mc {Parameters}]] ColormapDialog

    set buttons(color) {}
    # don't worry about user created cmaps yet
    set id [colorbar list id]
    foreach ii $id {
	set title [colorbar get name $ii]
	set name [string tolower $title]
	append buttons(color) "$ds9(buttons).color.$name buttons(color,$name) "
    }
    append buttons(color) "$ds9(buttons).color.invert buttons(color,invert) "
    append buttons(color) "$ds9(buttons).color.reset buttons(color,reset) "
    append buttons(color) "$ds9(buttons).color.params buttons(color,params) "
}

proc PrefsDialogButtonbarColor {f} {
    global buttons
    global colorbar

    menubutton $f -relief raised -text [msgcat::mc {Color}] -menu $f.menu
    
    set m $f.menu
    menu $m

    # we don't wany any user added cmaps
    set id [colorbar list id]
    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	set jj [lindex $id $ii]
	set title [colorbar get name $jj]
	set name [string tolower $title]
	$m add checkbutton -label [msgcat::mc $title] \
	    -variable buttons(color,$name) \
	    -command {UpdateButtons buttons(color)}
    }
    $m add separator
    $m add checkbutton -label [msgcat::mc {Invert Colormap}] \
	-variable buttons(color,invert) \
	-command {UpdateButtons buttons(color)}
    $m add checkbutton -label [msgcat::mc {Reset Colormap}] \
	-variable buttons(color,reset) \
	-command {UpdateButtons buttons(color)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Colormap Parameters}]..." \
	-variable buttons(color,params) \
	-command {UpdateButtons buttons(color)}
}

# Support

proc UpdateColorMenu {} {
    global ds9
    global current
    global colorbar
    global buttons

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateColorMenu"
    }

    if {$current(frame) != {}} {
	if {[$current(frame) get type] == {rgb}} {
	    # menus
	    for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
		$ds9(mb).color entryconfig $ii -state disabled
	    }

	    # buttons
	    # we don't wany any user added cmaps
	    set id [colorbar list id]
	    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
		set jj [lindex $id $ii]
		set title [colorbar get name $jj]
		set name [string tolower $title]
		$ds9(buttons).color.$name configure -state disabled
	    }
	} else {
	    # menus
	    for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
		$ds9(mb).color entryconfig $ii -state normal
	    }

	    # buttons
	    # we don't wany any user added cmaps
	    set id [colorbar list id]
	    for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
		set jj [lindex $id $ii]
		set title [colorbar get name $jj]
		set name [string tolower $title]
		$ds9(buttons).color.$name configure -state normal
	    }
	}
    } else {
	# menus
	for {set ii $colorbar(start)} {$ii<$colorbar(count)} {incr ii} {
	    $ds9(mb).color entryconfig $ii -state normal
	}

	# buttons
	# we don't wany any user added cmaps
	set id [colorbar list id]
	for {set ii 0} {$ii<$colorbar(cmaps)} {incr ii} {
	    set jj [lindex $id $ii]
	    set title [colorbar get name $jj]
	    set name [string tolower $title]
	    $ds9(buttons).color.$name configure -state normal
	}
    }

    # for aqua
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {set colorbar(map) $colorbar(map)}
    }
}

