#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CreateMenuBar {} {
    global ds9

    menu $ds9(mb) 
    $ds9(mb) add cascade -label [msgcat::mc {File}] -menu $ds9(mb).file
    $ds9(mb) add cascade -label [msgcat::mc {Edit}] -menu $ds9(mb).edit
    $ds9(mb) add cascade -label [msgcat::mc {View}] -menu $ds9(mb).view
    $ds9(mb) add cascade -label [msgcat::mc {Frame}] -menu $ds9(mb).frame
    $ds9(mb) add cascade -label [msgcat::mc {Bin}] -menu $ds9(mb).bin
    $ds9(mb) add cascade -label [msgcat::mc {Zoom}] -menu $ds9(mb).zoom
    $ds9(mb) add cascade -label [msgcat::mc {Scale}] -menu $ds9(mb).scale
    $ds9(mb) add cascade -label [msgcat::mc {Color}] -menu $ds9(mb).color
    $ds9(mb) add cascade -label [msgcat::mc {Region}] -menu $ds9(mb).region
    $ds9(mb) add cascade -label [msgcat::mc {WCS}] -menu $ds9(mb).wcs
    $ds9(mb) add cascade -label [msgcat::mc {Analysis}] -menu $ds9(mb).analysis

    AppleMenu ds9(mb)
    FileMenu
    EditMenu
    ViewMenu
    FrameMenu
    BinMenu
    ZoomMenu
    ScaleMenu
    ColorMenu
    RegionMenu
    WCSMenu
    AnalysisMenu

    switch $ds9(wm) {
	x11 -
	win32 {HelpMenu ds9(mb)}
	aqua {}
    }
}

proc AppleMenu {varname} {
    upvar $varname var
    global ds9

    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {
	    # apple menu
	    $var add cascade -menu $var.apple
	    menu $var.apple
	    $var.apple add command \
		-label [msgcat::mc {About SAOImage DS9}] \
		-command AboutBox

	    proc ::tk::mac::ShowPreferences {} {
		PrefsDialog {}
	    }

	    # help menu
	    HelpMenu var
	}
    }
}

proc DefaultFontMenu {which varname1 varname2 varname3 cmd} {
    upvar $varname1 var1
    upvar $varname2 var2
    upvar $varname3 var3

    global ap
    global marker
    global prefs
    global grid

    menu $which
    $which add radiobutton -label {Times} -variable ${varname1} \
	-value times -command $cmd
    $which add radiobutton -label {Helvetica} -variable ${varname1} \
	-value helvetica -command $cmd
    $which add radiobutton -label {Symbol} -variable ${varname1} \
	-value symbol -command $cmd
    $which add radiobutton -label {Courier} -variable ${varname1} \
	-value courier -command $cmd
    $which add separator
    $which add radiobutton -label {9} -variable ${varname2} \
	-value 9 -command $cmd
    $which add radiobutton -label {10} -variable ${varname2} \
	-value 10 -command $cmd
    $which add radiobutton -label {12} -variable ${varname2} \
	-value 12 -command $cmd
    $which add radiobutton -label {14} -variable ${varname2} \
	-value 14 -command $cmd
    $which add radiobutton -label {16} -variable ${varname2} \
	-value 16 -command $cmd
    $which add radiobutton -label {18} -variable ${varname2} \
    -value 18 -command $cmd
    $which add radiobutton -label {24} -variable ${varname2} \
	-value 24 -command $cmd
    $which add radiobutton -label {30} -variable ${varname2} \
	-value 30 -command $cmd
    $which add radiobutton -label {36} -variable ${varname2} \
	-value 36 -command $cmd
    $which add radiobutton -label {72} -variable ${varname2} \
	-value 72 -command $cmd
    $which add radiobutton -label {96} -variable ${varname2} \
	-value 96 -command $cmd
    $which add separator
    $which add radiobutton -label [msgcat::mc {Plain}] \
	-variable ${varname3} -value normal -command $cmd
    $which add radiobutton -label [msgcat::mc {Bold}] \
	-variable ${varname3} -value bold -command $cmd
    $which add radiobutton -label [msgcat::mc {Italic}] \
	-variable ${varname3} -value italic -command $cmd
}

proc DefaultColorMenu {which varname cmd} {
    upvar $varname var

    global contour
    global mask
    global prefs
    global marker
    global ap

    menu $which
    $which add radiobutton -label [msgcat::mc {Black}] \
	-variable ${varname} -value black -command $cmd
    $which add radiobutton -label [msgcat::mc {White}] \
	-variable ${varname} -value white -command $cmd
    $which add radiobutton -label [msgcat::mc {Red}] \
	-variable ${varname} -value red -command $cmd
    $which add radiobutton -label [msgcat::mc {Green}] \
	-variable ${varname} -value green -command $cmd
    $which add radiobutton -label [msgcat::mc {Blue}] \
	-variable ${varname} -value blue -command $cmd
    $which add radiobutton -label [msgcat::mc {Cyan}] \
	-variable ${varname} -value cyan -command $cmd
    $which add radiobutton -label [msgcat::mc {Magenta}] \
	-variable ${varname} -value magenta -command $cmd
    $which add radiobutton -label [msgcat::mc {Yellow}] \
	-variable ${varname} -value yellow -command $cmd
    $which add separator
    $which add command -label "[msgcat::mc {Other}]..." \
	-command "[list EntryDialog [msgcat::mc {Color}] [msgcat::mc {Enter Color}] 20 ${varname}]; $cmd"
}

proc DefaultWidthMenu {which varname cmd} {
    upvar $varname var

    global contour
    global grid
    global prefs
    global marker
    global ap

    menu $which
    $which add radiobutton -label {1} -variable ${varname} \
	-value 1 -command $cmd
    $which add radiobutton -label {2} -variable ${varname} \
	-value 2 -command $cmd
    $which add radiobutton -label {3} -variable ${varname} \
	-value 3 -command $cmd
    $which add radiobutton -label {4} -variable ${varname} \
	-value 4 -command $cmd
}

proc DefaultCoordMenu {which varname1 varname2 varname3 cmd} {
    # system sky skyformat cmd
    upvar $varname1 var1
    upvar $varname2 var2
    upvar $varname3 var3

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] -menu $which.wcs
    $which add separator
    $which add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname1} -value image -command $cmd
    $which add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname1} -value physical -command $cmd
    $which add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname1} -value amplifier -command $cmd
    $which add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname1} -value detector -command $cmd
    $which add separator
    $which add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	-variable ${varname2} -value fk4 -command $cmd
    $which add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	-variable ${varname2} -value fk5 -command $cmd
    $which add radiobutton -label [msgcat::mc {ICRS}] \
	-variable ${varname2} -value icrs -command $cmd
    $which add radiobutton -label [msgcat::mc {Galactic}] \
	-variable ${varname2} -value galactic -command $cmd
    $which add radiobutton -label [msgcat::mc {Ecliptic}] \
	-variable ${varname2} -value ecliptic -command $cmd
    $which add separator
    $which add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname3} -value degrees -command $cmd
    $which add radiobutton -label {Sexagesimal} \
	-variable ${varname3} -value sexagesimal -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l" -command $cmd
    }
}

proc DefaultCoordMenu1 {which varname1 cmd} {
    # system cmd
    upvar $varname1 var1

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] -menu $which.wcs
    $which add separator
    $which add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname1} -value image -command $cmd
    $which add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname1} -value physical -command $cmd
    $which add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname1} -value amplifier -command $cmd
    $which add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname1} -value detector -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l"
    }
}

proc DefaultCoordMenu2 {which varname1 varname2 cmd} {
    # system sky cmd
    upvar $varname1 var1
    upvar $varname2 var2

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] -menu $which.wcs
    $which add separator
    $which add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname1} -value image -command $cmd
    $which add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname1} -value physical -command $cmd
    $which add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname1} -value amplifier -command $cmd
    $which add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname1} -value detector -command $cmd
    $which add separator
    $which add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	-variable ${varname2} -value fk4 -command $cmd
    $which add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	-variable ${varname2} -value fk5 -command $cmd
    $which add radiobutton -label [msgcat::mc {ICRS}] \
	-variable ${varname2} -value icrs -command $cmd
    $which add radiobutton -label [msgcat::mc {Galactic}] \
	-variable ${varname2} -value galactic -command $cmd
    $which add radiobutton -label [msgcat::mc {Ecliptic}] \
	-variable ${varname2} -value ecliptic -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l"
    }
}

proc DefaultWCSCoordMenu {which varname1 varname2 varname3 cmd} {
    upvar $varname1 var1
    upvar $varname2 var2
    upvar $varname3 var3

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] -menu $which.wcs
    $which add separator
    $which add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	-variable ${varname2} -value fk4 -command $cmd
    $which add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	-variable ${varname2} -value fk5 -command $cmd
    $which add radiobutton -label [msgcat::mc {ICRS}] \
	-variable ${varname2} -value icrs -command $cmd
    $which add radiobutton -label [msgcat::mc {Galactic}] \
	-variable ${varname2} -value galactic -command $cmd
    $which add radiobutton -label [msgcat::mc {Ecliptic}] \
	-variable ${varname2} -value ecliptic -command $cmd
    $which add separator
    $which add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname3} -value degrees -command $cmd
    $which add radiobutton -label {Sexagesimal} \
	-variable ${varname3} -value sexagesimal -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l" -command $cmd
    }
}

proc DefaultWCSCoordMenu2 {which varname1 varname2 cmd} {
    upvar $varname1 var1
    upvar $varname2 var2

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] -menu $which.wcs
    $which add separator
    $which add radiobutton -label "[msgcat::mc {Equatorial}] B1950" \
	-variable ${varname2} -value fk4 -command $cmd
    $which add radiobutton -label "[msgcat::mc {Equatorial}] J2000" \
	-variable ${varname2} -value fk5 -command $cmd
    $which add radiobutton -label [msgcat::mc {ICRS}] \
	-variable ${varname2} -value icrs -command $cmd
    $which add radiobutton -label [msgcat::mc {Galactic}] \
	-variable ${varname2} -value galactic -command $cmd
    $which add radiobutton -label [msgcat::mc {Ecliptic}] \
	-variable ${varname2} -value ecliptic -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l" -command $cmd
    }
}

proc DefaultDistMenu {which varname1 varname2 cmd} {
    upvar $varname1 var1
    upvar $varname2 var2

    menu $which 
    $which add radiobutton -label [msgcat::mc {WCS}] \
	-variable ${varname1} -value wcs -command $cmd
    $which add cascade -label [msgcat::mc {Multiple WCS}] \
	-menu $which.wcs
    $which add separator
    $which add radiobutton -label [msgcat::mc {Image}] \
	-variable ${varname1} -value image -command $cmd
    $which add radiobutton -label [msgcat::mc {Physical}] \
	-variable ${varname1} -value physical -command $cmd
    $which add radiobutton -label [msgcat::mc {Amplifier}] \
	-variable ${varname1} -value amplifier -command $cmd
    $which add radiobutton -label [msgcat::mc {Detector}] \
	-variable ${varname1} -value detector -command $cmd
    $which add separator
    $which add radiobutton -label [msgcat::mc {Degrees}] \
	-variable ${varname2} -value degrees -command $cmd
    $which add radiobutton -label [msgcat::mc {ArcMin}] \
	-variable ${varname2} -value arcmin -command $cmd
    $which add radiobutton -label [msgcat::mc {ArcSec}] \
	-variable ${varname2} -value arcsec -command $cmd

    menu $which.wcs 
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs add radiobutton -label "[msgcat::mc {WCS}] $l" \
	    -variable ${varname1} -value "wcs$l" -command $cmd
    }
}

# full coordinate menu
proc UpdateCoordMenu {frame which} {
    global current
    global ds9

    # no frame
    if {$frame == {}} {
	SetCoordMenu $which normal
	return
    }

    # unloaded frame
    if {![$frame has fits]} {
	SetCoordMenu $which normal
	return
    }

    # loaded frame
    if {[$frame has wcs WCS]} {
	$which entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$which entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $which entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$frame has wcs WCS$l]} {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }

    if {[$frame has wcs equatorial WCS]} {
	SetSkyFrameMenu $which normal
    } else {
	SetSkyFrameMenu $which disabled
    }

    if {[$frame has detector]} {
	$which entryconfig [msgcat::mc {Detector}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Detector}] -state disabled
    }
    
    if {[$frame has amplifier]} {
	$which entryconfig [msgcat::mc {Amplifier}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Amplifier}] -state disabled
    }

    if {[$frame has physical]} {
	$which entryconfig [msgcat::mc {Physical}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Physical}] -state disabled
    }

    $which entryconfig [msgcat::mc {Image}] -state normal
}

proc SetCoordMenu {which value} {
    $which entryconfig [msgcat::mc {WCS}] -state $value
    $which entryconfig [msgcat::mc {Multiple WCS}] -state $value

    $which entryconfig [msgcat::mc {Image}] -state $value
    $which entryconfig [msgcat::mc {Physical}] -state $value
    $which entryconfig [msgcat::mc {Amplifier}] -state $value
    $which entryconfig [msgcat::mc {Detector}] -state $value

    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig {Sexagesimal} -state $value

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state $value
    }
}

proc SetSkyFrameMenu {which value} {
    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig {Sexagesimal} -state $value
}

proc SetSkyFrameMenu2 {which value} {
    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value
}

# dist menu
proc UpdateDistMenu {frame which defnoframe defnofits} {
    global current

    # no frame
    if {$frame == {}} {
	if {$defnoframe} {
	    SetDistMenu $which normal
	} else {
	    SetDistMenu $which disabled
	}
	return
    }

    # unloaded frame
    if {![$frame has fits]} {
	if {$defnofits} {
	    SetDistMenu $which normal
	} else {
	    SetDistMenu $which disabled
	}
	return
    }

    # loaded frame
    if {[$frame has wcs WCS]} {
	$which entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$which entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $which entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$frame has wcs WCS$l]} {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }

    if {[$frame has wcs equatorial WCS]} {
	SetSkyFormatMenu $which normal
    } else {
	SetSkyFormatMenu $which disabled
    }

    if {[$frame has detector]} {
	$which entryconfig [msgcat::mc {Detector}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Detector}] -state disabled
    }

    if {[$frame has amplifier]} {
	$which entryconfig [msgcat::mc {Amplifier}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Amplifier}] -state disabled
    }

    if {[$frame has physical]} {
	$which entryconfig [msgcat::mc {Physical}] -state normal
    } else {
	$which entryconfig [msgcat::mc {Physical}] -state disabled
    }

    $which entryconfig [msgcat::mc {Image}] -state normal
}

proc SetDistMenu {which value} {
    $which entryconfig [msgcat::mc {WCS}] -state $value
    $which entryconfig [msgcat::mc {Multiple WCS}] -state $value

    $which entryconfig [msgcat::mc {Image}] -state $value
    $which entryconfig [msgcat::mc {Physical}] -state $value
    $which entryconfig [msgcat::mc {Amplifier}] -state $value
    $which entryconfig [msgcat::mc {Detector}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig [msgcat::mc {ArcMin}] -state $value
    $which entryconfig [msgcat::mc {ArcSec}] -state $value

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state $value
    }
}

proc SetSkyFormatMenu {which value} {
    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig [msgcat::mc {ArcMin}] -state $value
    $which entryconfig [msgcat::mc {ArcSec}] -state $value
}

# use sysname instead of sys, problems with macosx
proc AdjustCoord {frame sysname} {
    upvar $sysname sys

    if {$frame != {}} {
	if {[$frame has fits]} {
	    switch -- $sys {
		image {}
		physical {
		    if {![$frame has physical]} {
			set ${sysname} image
		    }
		}
		amplifier {
		    if {![$frame has amplifier]} {
			set ${sysname} image
		    }
		}
		detector {
		    if {![$frame has detector]} {
			set ${sysname} image
		    }
		}
		wcs {
		    if {![$frame has wcs $sys]} {
			set ${sysname} image
		    }
		}
		default {
		    if {![$frame has wcs $sys]} {
			if [$frame has wcs wcs] {
			    set ${sysname} wcs
			} else {
			    set ${sysname} image
			}
		    }
		}
	    }
	}
    }
}

proc AdjustDist {frame sysname} {
    upvar $sysname sys
    global current

    if {$frame != {}} {
	if {[$frame has fits]} {
	    switch -- $sys {
		image {}
		physical {
		    if {![$frame has physical]} {
			set ${sysname} image
		    }
		}
		amplifier {
		    if {![$frame has amplifier]} {
			set ${sysname} image
		    }
		}
		detector {
		    if {![$frame has detector]} {
			set ${sysname} image
		    }
		}
		default {
		    if {![$frame has wcs $sys]} {
			set ${sysname} wcs
		    }
		}
	    }
	}
    }
}


