#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Menus

proc FileMenu {} {
    global ds9

    menu $ds9(mb).file 
    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(mb).file add command \
		-label "[msgcat::mc {About SAOImage DS9}]..." \
		-command AboutBox
	    $ds9(mb).file add separator
	}
	aqua {}
    }
    $ds9(mb).file add command -label "[msgcat::mc {Open}]..." \
	-command {OpenFits {}} -accelerator "${ds9(ctrl)}O"
    $ds9(mb).file add cascade -label [msgcat::mc {Open Other}] \
	-menu $ds9(mb).file.open
    $ds9(mb).file add separator
    $ds9(mb).file add cascade -label [msgcat::mc {Preserve During Load}] \
	-menu $ds9(mb).file.preserve
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {Save Image}]..." \
	-command SaveImageDialog
    $ds9(mb).file add command -label "[msgcat::mc {Save Frame as Fits}]..." \
	-command SaveFitsDialog
    $ds9(mb).file add command -label "[msgcat::mc {Save Frames as MPEG}]..." \
	-command SaveMPEGDialog
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {Display Fits Header}]..." \
	-command DisplayHeaderMenu 
    $ds9(mb).file add separator
    $ds9(mb).file add command -label "[msgcat::mc {XPA Information}]..." \
	-command DisplayXPAInfo
    $ds9(mb).file add command -label "[msgcat::mc {Open TCL Console}]..." \
	-command OpenConsole
    $ds9(mb).file add command -label "[msgcat::mc {Source TCL}]..." \
	-command OpenSource
    $ds9(mb).file add separator
    switch $ds9(wm) {
	x11 {
	    $ds9(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-command PageSetup -accelerator "${ds9(shiftctrl)}P"
	    $ds9(mb).file add command -label "[msgcat::mc {Print}]..." \
		-command Print -accelerator "${ds9(ctrl)}P"
	}
	win32 -
	aqua {
	    $ds9(mb).file add command \
		-label "[msgcat::mc {Postscript Page Setup}]..." \
		-command PSPageSetup
	    $ds9(mb).file add command \
		-label "[msgcat::mc {Postscript Print}]..." \
		-command PSPrint
	    $ds9(mb).file add separator
	    $ds9(mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-command PageSetup -accelerator "${ds9(shiftctrl)}P"
	    $ds9(mb).file add command -label "[msgcat::mc {Print}]..." \
		-command Print -accelerator "${ds9(ctrl)}P"
	}
    }
    switch $ds9(wm) {
	x11 -
	win32 {
	    $ds9(mb).file add separator
	    $ds9(mb).file add command \
		-label [msgcat::mc {Exit}] -command QuitDS9
	}
	aqua {}
    }

    # File Open Menu
    menu $ds9(mb).file.open
    $ds9(mb).file.open add command -label "[msgcat::mc {Open Array}]..." \
	-command {OpenArray {}}
    $ds9(mb).file.open add command -label "[msgcat::mc {Open URL}]..." \
	-command OpenURL
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Fits Image}]..." \
	-command OpenRGBImageFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Fits Cube}]..."\
	-command OpenRGBCubeFits
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open RGB Array}]..." \
	-command OpenRGBArray
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Multi Ext as Data Cube}]..." \
	-command {OpenDataCubeFits {}}
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic IRAF}]..." \
	-command {OpenMosaicImageIRAFFits {}}
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic IRAF Segment}]..." \
	-command {OpenMosaicIRAFFits {}}
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WCS}]..." \
	-command {OpenMosaicImageWCSFits {}}
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WCS Segment}]..." \
	-command {OpenMosaicWCSFits {}}
    $ds9(mb).file.open add separator
    $ds9(mb).file.open add command \
	-label "[msgcat::mc {Open Mosaic WFPC2}]..." \
	-command OpenMosaicImageWFPC2Fits

    # File Preserve Menu
    menu $ds9(mb).file.preserve
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Scale}] \
	-variable scale(preserve) -command PreserveScale
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Pan}] \
	-variable panzoom(preserve) -command PreservePan
    $ds9(mb).file.preserve add checkbutton -label [msgcat::mc {Regions}] \
	-variable marker(preserve) -command MarkerPreserve

    # Bindings
    bind $ds9(top) <<Open>> {OpenFits {}}
    bind $ds9(top) <<PageSetup>> PageSetup
    bind $ds9(top) <<Print>> Print
}

# Buttons

proc ButtonsVarsFile {} {
    global buttons

    array set buttons {
	file,about 1
	file,open 1
	file,saveimage 1
	file,savefits 0
	file,savempeg 0
	file,header 1
	file,xpa 0
	file,console 0
	file,tcl 0
	file,pspage 0
	file,psprint 0
	file,page 1
	file,print 1
	file,exit 1
    }
}

proc CreateButtonsFile {} {
    global ds9
    global buttons

    frame $ds9(buttons).file

    ButtonButton $ds9(buttons).file.about \
	[string tolower [msgcat::mc {About}]] AboutBox
    ButtonButton $ds9(buttons).file.open \
	[string tolower [msgcat::mc {Open}]] {OpenFits {}}

    ButtonButton $ds9(buttons).file.saveimage \
	[string tolower [msgcat::mc {Save Image}]] SaveImageDialog
    ButtonButton $ds9(buttons).file.savefits \
	[string tolower [msgcat::mc {Save FITS}]] SaveFitsDialog
    ButtonButton $ds9(buttons).file.savempeg \
	[string tolower [msgcat::mc {Save MPEG}]] SaveMPEGDialog

    ButtonButton $ds9(buttons).file.header \
	[string tolower [msgcat::mc {Header}]] DisplayHeaderMenu

    ButtonButton $ds9(buttons).file.xpa \
	[string tolower {XPA}] DisplayXPAInfo
    ButtonButton $ds9(buttons).file.console \
	[string tolower [msgcat::mc {Console}]] OpenConsole
    ButtonButton $ds9(buttons).file.tcl \
	[string tolower {TCL}] OpenSource

    ButtonButton $ds9(buttons).file.pspage \
	[string tolower [msgcat::mc {PS Page Setup}]] PSPageSetup
    ButtonButton $ds9(buttons).file.psprint \
	[string tolower [msgcat::mc {PS Print}]] PSPrint

    ButtonButton $ds9(buttons).file.page \
	[string tolower [msgcat::mc {Page Setup}]] PageSetup
    ButtonButton $ds9(buttons).file.print \
	[string tolower [msgcat::mc {Print}]] Print

    ButtonButton $ds9(buttons).file.exit \
	[string tolower [msgcat::mc {Exit}]] QuitDS9

    set buttons(file) "
        $ds9(buttons).file.about buttons(file,about)
        $ds9(buttons).file.open buttons(file,open)
        $ds9(buttons).file.saveimage buttons(file,saveimage)
        $ds9(buttons).file.savefits buttons(file,savefits)
        $ds9(buttons).file.savempeg buttons(file,savempeg)
        $ds9(buttons).file.header buttons(file,header)
        $ds9(buttons).file.xpa buttons(file,xpa)
        $ds9(buttons).file.console buttons(file,console)
        $ds9(buttons).file.tcl buttons(file,tcl)
        $ds9(buttons).file.pspage buttons(file,pspage)
        $ds9(buttons).file.psprint buttons(file,psprint)
        $ds9(buttons).file.page buttons(file,page)
        $ds9(buttons).file.print buttons(file,print)
        $ds9(buttons).file.exit buttons(file,exit)
    "
}

proc PrefsDialogButtonbarFile {f} {
    global buttons

    menubutton $f -relief raised -text [msgcat::mc {File}] -menu $f.menu
    
    set m $f.menu
    menu $m
    $m add checkbutton -label "[msgcat::mc {About SAOImage DS9}]..." \
	-variable buttons(file,about) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Open}]..." \
	-variable buttons(file,open) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Save Image}]..." \
	-variable buttons(file,saveimage) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Save Frame as Fits}]..." \
	-variable buttons(file,savefits) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Save Frames as MPEG}]..." \
	-variable buttons(file,savempeg) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Display Fits Header}]..." \
	-variable buttons(file,header) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {XPA Information}]..." \
	-variable buttons(file,xpa) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Open TCL Console}]..." \
	-variable buttons(file,console) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Source TCL}]..." \
	-variable buttons(file,tcl) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Postscript Page Setup}]..." \
	-variable buttons(file,pspage) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Postscript Print}]..." \
	-variable buttons(file,psprint) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label "[msgcat::mc {Page Setup}]..." \
	-variable buttons(file,page) -command {UpdateButtons buttons(file)}
    $m add checkbutton -label "[msgcat::mc {Print}]..." \
	-variable buttons(file,print) -command {UpdateButtons buttons(file)}
    $m add separator
    $m add checkbutton -label [msgcat::mc {Exit}] \
	-variable buttons(file,exit) -command {UpdateButtons buttons(file)}
}

# Support

proc UpdateFileMenu {} {
    global ds9
    global current
    global crosshair
 
    global debug
    if {$debug(tcl,update)} {
	puts "UpdateFileMenu"
    }

    if {$current(frame) != {}} {
	$ds9(mb).file entryconfig "[msgcat::mc {Open}]..." \
	    -state normal
	$ds9(mb).file entryconfig [msgcat::mc {Open Other}] \
	    -state normal
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frame as Fits}]..." \
	    -state normal
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frames as MPEG}]..." \
	    -state normal

	if {[$current(frame) has fits]} {
	    $ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
		-state normal
	} else {
	    $ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
		-state disabled
	}

	if {[$current(frame) get type] == {rgb}} {
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Fits Image}]..." -state normal
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Fits Cube}]..." -state normal
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Array}]..." -state normal
	} else {
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Fits Image}]..." -state disabled
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Fits Cube}]..." -state disabled
	    $ds9(mb).file.open entryconfig \
		"[msgcat::mc {Open RGB Array}]..." -state disabled
	}

	$ds9(buttons).file.open configure -state normal
	$ds9(buttons).file.savefits configure -state normal
	$ds9(buttons).file.savempeg configure -state normal
	if {[$current(frame) has fits]} {
	    $ds9(buttons).file.header configure -state normal
	} else {
	    $ds9(buttons).file.header configure -state disabled
	}

    } else {
	$ds9(mb).file entryconfig "[msgcat::mc {Open}]..." \
	    -state disabled
	$ds9(mb).file entryconfig [msgcat::mc {Open Other}] \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frame as Fits}]..." \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Save Frames as MPEG}]..." \
	    -state disabled
	$ds9(mb).file entryconfig "[msgcat::mc {Display Fits Header}]..." \
	    -state disabled

	$ds9(buttons).file.open configure -state disabled
	$ds9(buttons).file.savefits configure -state disabled
	$ds9(buttons).file.savempeg configure -state disabled
	$ds9(buttons).file.header configure -state disabled
    }
}

