#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Menu

proc WCSMenu {} {
    global ds9
    global wcs

    DefaultWCSCoordMenu $ds9(mb).wcs \
	wcs(system) wcs(sky) wcs(skyformat) UpdateWCS
}

proc PrefsDialogWCS {} {
    global prefs

    set w $prefs(tabs)

    frame $w.wcs
    $prefs(tabs) tab configure wcs -window $w.wcs -fill both

    frame $w.wcs.menu -relief ridge -bd 1
    pack $w.wcs.menu -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.wcs

    # Default Menu
    set f $w.wcs.menu.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Default Menu}]
    menubutton $f.wcs -relief raised -text [msgcat::mc {WCS}] \
	-menu $f.wcs.menu

    grid $f.title $f.wcs -padx 4 -sticky w

    DefaultWCSCoordMenu $f.wcs.menu pwcs(system) pwcs(sky) pwcs(skyformat) {}
}

# Buttons

proc ButtonsVarsWCS {} {
    global buttons

    array set buttons {
	wcs,fk4 1
	wcs,fk5 1
	wcs,icrs 1
	wcs,galactic 1
	wcs,ecliptic 1
	wcs,degrees 1
	wcs,sexagesimal 1
    }
}

proc CreateButtonsWCS {} {
    global buttons
    global ds9
    global wcs

    frame $ds9(buttons).wcs

    RadioButton $ds9(buttons).wcs.fk4 {fk4} \
	wcs(sky) fk4 UpdateWCS
    RadioButton $ds9(buttons).wcs.fk5 {fk5} \
	wcs(sky) fk5 UpdateWCS
    RadioButton $ds9(buttons).wcs.icrs \
	[string tolower [msgcat::mc {ICRS}]] \
	wcs(sky) icrs UpdateWCS
    RadioButton $ds9(buttons).wcs.galactic \
	[string tolower [msgcat::mc {Galactic}]] \
	wcs(sky) galactic UpdateWCS
    RadioButton $ds9(buttons).wcs.ecliptic \
	[string tolower [msgcat::mc {Ecliptic}]] \
	wcs(sky) ecliptic UpdateWCS
    RadioButton $ds9(buttons).wcs.degrees \
	[string tolower [msgcat::mc {Degrees}]] \
	wcs(skyformat) degrees UpdateWCS
    RadioButton $ds9(buttons).wcs.sexagesimal {sexagesimal} \
	wcs(skyformat) sexagesimal UpdateWCS

    set buttons(wcs) "
        $ds9(buttons).wcs.fk4 buttons(wcs,fk4)
        $ds9(buttons).wcs.fk5 buttons(wcs,fk5)
        $ds9(buttons).wcs.icrs buttons(wcs,icrs)
        $ds9(buttons).wcs.galactic buttons(wcs,galactic)
        $ds9(buttons).wcs.ecliptic buttons(wcs,ecliptic)
        $ds9(buttons).wcs.degrees buttons(wcs,degrees)
        $ds9(buttons).wcs.sexagesimal buttons(wcs,sexagesimal)
    "
}

proc PrefsDialogButtonbarWCS {f} {
    global buttons

    menubutton $f -relief raised -text [msgcat::mc {WCS}] -menu $f.menu
    
    set m $f.menu
    menu $m
    $m add checkbutton -label "[msgcat::mc {Equatorial}] B1950" \
	-variable buttons(wcs,fk4) -command {UpdateButtons buttons(wcs)}
    $m add checkbutton -label "[msgcat::mc {Equatorial}] J2000" \
	-variable buttons(wcs,fk5) -command {UpdateButtons buttons(wcs)}
    $m add checkbutton -label [msgcat::mc {ICRS}] \
	-variable buttons(wcs,icrs) -command {UpdateButtons buttons(wcs)}
    $m add checkbutton -label [msgcat::mc {Galactic}] \
	-variable buttons(wcs,galactic) -command {UpdateButtons buttons(wcs)}
    $m add checkbutton -label [msgcat::mc {Ecliptic}] \
	-variable buttons(wcs,ecliptic) -command {UpdateButtons buttons(wcs)}
    $m add separator
    $m add checkbutton -label [msgcat::mc {Degrees}] \
	-variable buttons(wcs,degrees) -command {UpdateButtons buttons(wcs)}
    $m add checkbutton -label [msgcat::mc {Sexagesimal}] \
	-variable buttons(wcs,sexagesimal) -command {UpdateButtons buttons(wcs)}
}

# Support

proc UpdateWCSMenu {} {
    global ds9
    global current
    global wcs

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateWCSMenu"
    }

    # check for valid values 
    # bad, can't let wcs(system) change to non wcs value
    # AdjustCoord $current(frame) wcs(system)

    if {$current(frame) != {}} {
	$ds9(mb) entryconfig [msgcat::mc {WCS}] -state normal
	$ds9(buttons).major.wcs configure -state normal

	UpdateWCSCoordMenu "$current(frame)" $ds9(mb).wcs

    } else {
	$ds9(mb) entryconfig [msgcat::mc {WCS}] -state disabled
	$ds9(buttons).major.wcs configure -state disabled
    }
}

# full WCS coordinate menu
proc UpdateWCSCoordMenu {frame which} {
    global current
    global ds9

    # no frame
    if {$frame == {}} {
	SetWCSCoordMenu $which disabled
	return
    }

    # unloaded frame
    if {![$frame has fits]} {
	SetWCSCoordMenu $which normal
	return
    }

    # loaded frame
    if {[$frame has wcs WCS]} {
	$which entryconfig [msgcat::mc {WCS}] -state normal
    } else {
	$which entryconfig [msgcat::mc {WCS}] -state disabled
    }
    $which entryconfig [msgcat::mc {Multiple WCS}] -state normal
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {[$frame has wcs WCS$l]} {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state normal
	} else {
	    $which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state disabled
	}
    }

    if {[$frame has wcs equatorial WCS]} {
	SetSkyFrameMenu $which normal
    } else {
	SetSkyFrameMenu $which disable
    }
}

proc SetWCSCoordMenu {which value} {
    global ds9

    $which entryconfig [msgcat::mc {WCS}] -state $value
    $which entryconfig [msgcat::mc {Multiple WCS}] -state $value

    $which entryconfig "[msgcat::mc {Equatorial}] B1950" -state $value
    $which entryconfig "[msgcat::mc {Equatorial}] J2000" -state $value
    $which entryconfig [msgcat::mc {ICRS}] -state $value
    $which entryconfig [msgcat::mc {Galactic}] -state $value
    $which entryconfig [msgcat::mc {Ecliptic}] -state $value

    $which entryconfig [msgcat::mc {Degrees}] -state $value
    $which entryconfig {Sexagesimal} -state $value

    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$which.wcs entryconfig "[msgcat::mc {WCS}] $l" -state $value
    }
}

