#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc NRESApply {varname sync} {
    upvar #0 $varname var
    global $varname

    set var(sync) $sync

    ARApply $varname

    if {$var(name)!={}} {
	NSVRServer $varname
    } else {
	ARStatus $varname "Please specify an Object Name"
	ARReset $varname
    }
}

proc NRESDialog {} {
    global ds9
    global nres

    if [winfo exists $nres(top)] {
	raise $nres(top)
	return
    }

    set varname nresi
    upvar #0 $varname var
    global $varname

    # AR variables
    ARInit $varname {}

    # Variables
    set var(top) $nres(top)
    set var(mb) $nres(mb)
    set var(system) $nres(system)
    set var(sky) $nres(sky)
    set var(skyformat) $nres(skyformat)

    # top
    set w $var(top)
    set title [msgcat::mc {Name Resolution}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "ARDestroy $varname"
    wm minsize $w 375 100

    # menu
    $w configure -menu $var(mb)
    menu $var(mb)

    # file
    $var(mb) add cascade -label File -menu $var(mb).file
    menu $var(mb).file
    $var(mb).file add command -label [msgcat::mc {Retrieve}] \
	-command "NRESApply $varname 0"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "ARCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Pan To}] \
	-command "NRESPan $varname"
    $var(mb).file add command -label [msgcat::mc {Crosshair To}] \
	-command "NRESCrosshair $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "ARDestroy $varname"

    # edit
    AREditMenu $varname

    # name server
    NSVRServerMenu $varname
    ARCoordMenu $varname

    # dialog
    frame $w.param -relief ridge -bd 1
    frame $w.status -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.status $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    label $w.param.nametitle -text [msgcat::mc {Name}]
    entry $w.param.name -textvariable ${varname}(name) -width 48

    label $w.param.sky -textvariable ${varname}(sky) -anchor w
    set var(xname) [label $w.param.xtitle -text {} -font {symbol 12} -width 1]
    label $w.param.x -textvariable ${varname}(x) -width 14 -relief groove  
    set var(yname) [label $w.param.ytitle -text {} -font {symbol 12} -width 1]
    label $w.param.y -textvariable ${varname}(y) -width 14 -relief groove  
    label $w.param.skyformat -textvariable ${varname}(skyformat)

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4

    grid $w.param.nametitle x $w.param.name - - - - -padx 4 -pady 1 -sticky w
    grid $w.param.sky $w.param.xtitle $w.param.x $w.param.ytitle $w.param.y \
	$w.param.skyformat -padx 4 -pady 1 -sticky w

    label $w.status.title -text [msgcat::mc {Status}]
    label $w.status.item -textvariable ${varname}(status)
    pack $w.status.title $w.status.item -side left -anchor w -pady 4

    set var(apply) [button $w.buttons.apply -text [msgcat::mc {Retrieve}] \
			-command "NRESApply $varname 0"]
    set var(cancel) [button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
			 -command "ARCancel  $varname" -state disabled]

    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "ARDestroy $varname"
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.close \
	-side left -padx 10 -expand true

    ARCoord $varname
    ARStatus $varname {}
}

proc NRESPan {varname} {
    upvar #0 $varname var
    global $varname

    if {($var(x) != {}) && ($var(y) != {})} {
	PanTo $var(x) $var(y) $var(system) $var(sky)
    }
}

proc NRESCrosshair {varname} {
    upvar #0 $varname var
    global $varname

    global current
    global ds9

    if {($current(frame) != {})} {
	if {[$current(frame) has wcs equatorial $var(system)]} {
	    if {($var(x) != {}) && ($var(y) != {})} {
		set ds9(mode) crosshair
		ChangeMode
		$current(frame) crosshair \
		    $var(system) $var(sky) $var(x) $var(y)
	    }
	}
    }
}

# Process Cmds

proc ProcessNRESCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    set vvarname nresi
    upvar #0 $vvarname vvar
    global $vvarname

    global nres

    NRESDialog

    switch -- [string tolower [lindex $var $i]] {
	server {
	    incr i
	    set nres(server) [lindex $var $i]
	}
	format -
	skyformat {
	    incr i
	    set vvar(skyformat) [lindex $var $i]
	}
	name {
	    incr i
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
	default {
	    set vvar(name) [lindex $var $i]
	    NRESApply $vvarname 1
	}
    }
}
