#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CenterCurrentFrame {} {
    global current

    CenterFrame $current(frame)
}

proc CenterAllFrame {} {
    global ds9

    foreach f $ds9(frames) {
	CenterFrame $f
    }
}

proc CenterFrame {which} {
    if {$which != {}} {
	SetWatchCursor
	$which center
	UpdateGraphXAxis
	UnsetWatchCursor

        UpdateZoomMenu
	UpdateZoomDialog
    }
}

proc Pan {x y sys sky} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) pan $sys $sky $x $y
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc PanTo {x y sys sky} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) pan to $sys $sky $x $y
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonPan {which x y} {
    global panzoom

    switch -- $panzoom(mode) {
	click {}
	drag {
	    $which pan motion begin $x $y
	}
	panzoom {}
    }
}

proc MotionPan {which x y} {
    global panzoom

    switch -- $panzoom(mode) {
	click {}
	drag {
	    $which pan motion $x $y
	}
	panzoom {}
    }
}

proc ReleasePan {which x y} {
    global panzoom
    global current

    switch -- $panzoom(mode) {
	click {
	    SetWatchCursor
	    $which pan to canvas $x $y
	}
	drag {
	    $which pan motion end $x $y

	    # we need to delay setting the cursor here, since set cursor causes
	    # an update, so the display will flash

	    SetWatchCursor
	}
	panzoom {
	    if {$panzoom(last) != " $x $y "} {
		set panzoom(state) 1
	    }
	    SetWatchCursor
	    switch -- $panzoom(state) {
		1 {
		    $which pan to canvas $x $y
		    $which update
		    set c [$which get pan canvas]
		    $which warp to $c
		    set panzoom(last) $c

		    set z [$current(frame) get zoom]
		    if {$z < 2} {
			set panzoom(state) 2
		    } elseif {$z < 4} {
			set panzoom(state) 3
		    } elseif {$z < 8} {
			set panzoom(state) 4
		    } else {
			set panzoom(state) 5
		    }
		}
		2 {
		    $which zoom to 2 2 about canvas $x $y
		    set panzoom(state) 3
		}
		3 {
		    $which zoom to 4 4 about canvas $x $y
		    set panzoom(state) 4
		}
		4 {
		    $which zoom to 8 8 about canvas $x $y
		    set panzoom(state) 5
		}
		5 {
		    $which zoom to 1 1 about canvas $x $y
		    set panzoom(state) 2
		}
	    }
	    if {$which == $current(frame)} {
		set current(zoom) [$current(frame) get zoom]
	    }
	}
    }
    UpdateGraphXAxis
    UpdateZoomDialog
    UnsetWatchCursor
}

proc PreservePan {} {
    global current
    global panzoom

    if {$current(frame) != {}} {
	$current(frame) pan preserve $panzoom(preserve)
    }
}

proc ZoomToFit {} {
    global current
    global grid

    if {$current(frame) != {}} {
	SetWatchCursor

	# we need to update the grid because titles are zoom dependant
	if {$grid(view) && $grid(type) == "publication"} {
	    # recalculate to make room for labels
	    $current(frame) zoom to fit .8
	    set current(zoom) [$current(frame) get zoom]
	} else {
	    $current(frame) zoom to fit
	    set current(zoom) [$current(frame) get zoom]
	}
	GridUpdateZoom

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeZoom {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) zoom to $current(zoom)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc Zoom {x y} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) zoom $x $y
	set current(zoom) [$current(frame) get zoom]

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ButtonZoom {which x y} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$which zoom 2 2 about canvas $x $y
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ShiftZoom {which} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$which zoom .5 .5
	if {$which == $current(frame)} {
	    set current(zoom) [$current(frame) get zoom]
	}

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	GridUpdateZoom

	UnsetWatchCursor
    }
}

proc ChangeOrient {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) orient $current(orient)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc Rotate {value} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) rotate $value
	set current(rotate) [$current(frame) get rotate]

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ChangeRotate {} {
    global current

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) rotate to $current(rotate)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	UpdateZoomDialog
	UnsetWatchCursor
    }
}

proc ButtonRotate {which x y} {
    global panzoom

    $which rotate motion begin
    set panzoom(x) $x
}

proc MotionRotate {which x y} {
    global current
    global panzoom
    global cursor

    $which rotate motion [expr double($panzoom(x)-$x)/$panzoom(speed) * 180.]

    set current(rotate) [$which get rotate fixed]
    RefreshInfoBox $current(frame)
}

proc ReleaseRotate {which x y} {
    global current

    $which rotate motion end
    set current(rotate) [$which get rotate fixed]

    # we need to delay setting the cursor here, since set cursor causes
    # an update, so the display will flash

    SetWatchCursor

    RefreshInfoBox $which
    UpdateGraphXAxis
    UpdateZoomDialog

    UnsetWatchCursor
}

proc PanZoomDialog {} {
    global panzoom
    global panzoomdlg
    global ds9

    # see if we already have a window visible
    if [winfo exists $panzoom(top)] {
	raise $panzoom(top)
	return
    }

    set w $panzoom(top)
    set title [msgcat::mc {Pan Zoom Rotate Parameters}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PanZoomDestroyDialog
    wm minsize $w 375 100

    # init variables
    set panzoomdlg(clabel) $panzoom(system)

    $w configure -menu $panzoom(mb)

    menu $panzoom(mb)
    AppleMenu panzoom(mb)

    $panzoom(mb) add cascade -label [msgcat::mc {File}] \
	-menu $panzoom(mb).file
    $panzoom(mb) add cascade -label [msgcat::mc {Edit}] \
	-menu $panzoom(mb).edit
    $panzoom(mb) add cascade -label [msgcat::mc {Coordinate}] \
	-menu $panzoom(mb).coord
    $panzoom(mb) add cascade -label [msgcat::mc {Pan}] \
	-menu $panzoom(mb).pan
    $panzoom(mb) add cascade -label [msgcat::mc {Zoom}] \
	-menu $panzoom(mb).zoom
    $panzoom(mb) add cascade -label [msgcat::mc {Orientation}] \
	-menu $panzoom(mb).orient
    $panzoom(mb) add cascade -label [msgcat::mc {Rotate}] \
	-menu $panzoom(mb).rotate

    menu $panzoom(mb).file
    $panzoom(mb).file add command -label [msgcat::mc {Apply}] \
	-command PanZoomApplyDialog
    $panzoom(mb).file add separator
    $panzoom(mb).file add command -label [msgcat::mc {Close}] \
	-command PanZoomDestroyDialog

    menu $panzoom(mb).edit
    $panzoom(mb).edit add command -label [msgcat::mc {Cut}] \
	-command PanZoomCutDialog -accelerator "${ds9(ctrl)}X"
    $panzoom(mb).edit add command -label [msgcat::mc {Copy}] \
	-command PanZoomCopyDialog -accelerator "${ds9(ctrl)}C"
    $panzoom(mb).edit add command -label [msgcat::mc {Paste}] \
	-command PanZoomPasteDialog -accelerator "${ds9(ctrl)}X"

    DefaultCoordMenu $panzoom(mb).coord \
	panzoom(system) panzoom(sky) panzoom(skyformat) PanZoomDialogCoord

    menu $panzoom(mb).pan
    $panzoom(mb).pan add command -label [msgcat::mc {Center Image}] \
	-command CenterCurrentFrame

    menu $panzoom(mb).zoom
    $panzoom(mb).zoom add checkbutton -label [msgcat::mc {Align}] \
	-variable wcs(align) -command AlignWCSFrame
    $panzoom(mb).zoom add cascade -label [msgcat::mc {Align WCS}] \
	-menu $panzoom(mb).zoom.align
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add command -label [msgcat::mc {Zoom In}] \
	-command {Zoom 2 2}
    $panzoom(mb).zoom add command -label [msgcat::mc {Zoom Out}] \
	-command {Zoom .5 .5}
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add command -label [msgcat::mc {Zoom to Fit Frame}] \
	-command ZoomToFit
    $panzoom(mb).zoom add separator
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1/32" \
	-variable current(zoom) -value { 0.03125 0.03125 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1/16" \
	-variable current(zoom) -value { 0.0625 0.0625 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1/8" \
	-variable current(zoom) -value { 0.125 0.125 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1/4" \
	-variable current(zoom) -value { 0.25 0.25 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1/2" \
	-variable current(zoom) -value { 0.5 0.5 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 1" \
	-variable current(zoom) -value { 1 1 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 2" \
	-variable current(zoom) -value { 2 2 }  -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 4" \
	-variable current(zoom) -value { 4  4 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 8" \
	-variable current(zoom) -value { 8 8 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 16" \
	-variable current(zoom) -value { 16 16 } -command ChangeZoom
    $panzoom(mb).zoom add radiobutton -label "[msgcat::mc {Zoom}] 32" \
	-variable current(zoom) -value { 32 32 } -command ChangeZoom

    DefaultWCSCoordMenu2 $panzoom(mb).zoom.align \
	wcs(align,system) wcs(align,sky) AlignWCSFrame

    menu $panzoom(mb).orient
    $panzoom(mb).orient add radiobutton -label [msgcat::mc {None}] \
	-variable current(orient) -value none -command ChangeOrient
    $panzoom(mb).orient add radiobutton  -label "[msgcat::mc {Invert}] X" \
	-variable current(orient) -value x -command ChangeOrient
    $panzoom(mb).orient add radiobutton -label "[msgcat::mc {Invert}] Y" \
	-variable current(orient) -value y -command ChangeOrient
    $panzoom(mb).orient add radiobutton -label "[msgcat::mc {Invert}] XY" \
	-variable current(orient) -value xy -command ChangeOrient

    menu $panzoom(mb).rotate
    $panzoom(mb).rotate add radiobutton -label "0 [msgcat::mc {Degrees}]" \
	-variable current(rotate) -value 0 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label "90 [msgcat::mc {Degrees}]" \
	-variable current(rotate) -value 90 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label "180 [msgcat::mc {Degrees}]" \
	-variable current(rotate) -value 180 -command ChangeRotate
    $panzoom(mb).rotate add radiobutton -label "270 [msgcat::mc {Degrees}]" \
	-variable current(rotate) -value 270 -command ChangeRotate

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.zoomtitle -text [msgcat::mc {Zoom}]
    entry $w.param.zoomx -textvariable panzoomdlg(zoom,x) -width 14
    entry $w.param.zoomy -textvariable panzoomdlg(zoom,y) -width 14

    label $w.param.rottitle -text [msgcat::mc {Rotate}]
    entry $w.param.rotvalue -textvariable panzoomdlg(rotate) -width 14
    label $w.param.rottitle2 -text [msgcat::mc {Degrees}]

    label $w.param.pantitle -text [msgcat::mc {Pan}]
    entry $w.param.panx -textvariable panzoomdlg(x) -width 14
    entry $w.param.pany -textvariable panzoomdlg(y) -width 14
    label $w.param.pansystem -textvariable panzoomdlg(clabel) 
    
    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.zoomtitle $w.param.zoomx $w.param.zoomy \
	-padx 4 -sticky w
    grid $w.param.rottitle $w.param.rotvalue $w.param.rottitle2 \
	-padx 4 -sticky w
    grid $w.param.pantitle $w.param.panx $w.param.pany $w.param.pansystem \
	-padx 4 -sticky w

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command PanZoomApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command PanZoomDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    $w.param.zoomx select range 0 end

    UpdateZoomDialog
}

proc PanZoomApplyDialog {} {
    global panzoom
    global panzoomdlg
    global current

    if {$current(frame) != {}} {
	set current(zoom) "$panzoomdlg(zoom,x) $panzoomdlg(zoom,y)"
	set current(rotate) $panzoomdlg(rotate)

	SetWatchCursor
	$current(frame) zoom to $current(zoom) about \
	    $panzoom(system) $panzoom(sky) $panzoomdlg(x) $panzoomdlg(y) 
	$current(frame) rotate to $current(rotate)

	RefreshInfoBox $current(frame)
	UpdateGraphXAxis
	GridUpdateZoom
	UnsetWatchCursor

	UpdateZoomMenu
	UpdateZoomDialog
    }
}

proc PanZoomDestroyDialog {} {
    global panzoom
    global panzoomdlg

    if {[winfo exists $panzoom(top)]} {
	destroy $panzoom(top)
	destroy $panzoom(mb)
    }

    if {[info exists panzoomdlg]} {
	unset panzoomdlg
    }
}

proc PanZoomCutDialog {} {
    global panzoom

    tk_entryCut [focus -displayof $panzoom(top)]
}

proc PanZoomCopyDialog {} {
    global panzoom

    tk_entryCopy [focus -displayof $panzoom(top)]
}

proc PanZoomPasteDialog {} {
    global panzoom

    tk_entryPaste [focus -displayof $panzoom(top)]
}

proc PanZoomDialogCoord {} {
    global panzoom
    global panzoomdlg
    global current

    UpdateCoordMenu $current(frame) $panzoom(mb).coord
    AdjustCoord $current(frame) panzoom(system)

    if {$current(frame) != {}} {
	set coord [$current(frame) get pan $panzoom(system) $panzoom(sky) \
		       $panzoom(skyformat)]
	set panzoomdlg(x) [lindex $coord 0]
	set panzoomdlg(y) [lindex $coord 1]
    } else {
	set panzoomdlg(x) {}
	set panzoomdlg(y) {}
    }

    set panzoomdlg(clabel) $panzoom(system)
    switch -- $panzoom(system) {
	image -
	physical -
	amplifier -
	detector {}
	default {
	    if [$current(frame) has wcs $panzoom(system)] {
		if [$current(frame) has wcs equatorial $panzoom(system)] {
		    set panzoomdlg(clabel) $panzoom(sky)
		} else {
		    set name [$current(frame) get wcs name $panzoom(system)]
		    if {$name != {}} {
			set panzoomdlg(clabel) $name
		    }
		}
	    }
	}
    }
}

proc UpdateZoomDialog {} {
    global panzoom
    global panzoomdlg
    global current

    if {$current(frame) != {}} {
	global debug
	if {$debug(tcl,update)} {
	    puts "UpdateZoomDialog"
	}

	if {[winfo exists $panzoom(top)]} {
	    if {$current(frame) != {}} {
		set zz [$current(frame) get zoom]
		set panzoomdlg(zoom,x) [lindex $zz 0]
		set panzoomdlg(zoom,y) [lindex $zz 1]
		set panzoomdlg(rotate) [$current(frame) get rotate]
	    } else {
		set panzoomdlg(zoom,x) {}
		set panzoomdlg(zoom,y) {}
		set panzoomdlg(rotate) {}
	    }

	    PanZoomDialogCoord
	    UpdateAlignWCSMenu $panzoom(mb).zoom
	}
    }
}

# Process Cmds

proc ProcessPanCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    switch -- [string tolower [lindex $var $i]] {
	to {
	    set sys [lindex $var [expr $i+3]]
	    set sky [lindex $var [expr $i+4]]
	    switch -- $sys {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			wcs $sys
		    incr i 3
		}
		wcs -
		wcsa -
		wcsb -
		wcsc -
		wcsd -
		wcse -
		wcsf -
		wcsg -
		wcsh -
		wcsi -
		wcsj -
		wcsk -
		wcsl -
		wcsm -
		wcsn -
		wcso -
		wcsp -
		wcsq -
		wcsr -
		wcss -
		wcst -
		wcsu -
		wcsv -
		wcsw -
		wcsx -
		wcsy -
		wcsz {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			$sys $sky
		    incr i 4
		}
		image -
		physical -
		amplifier -
		detector {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			$sys {}
		    incr i 3
		}
		{} -
		default {
		    PanTo [lindex $var [expr $i+1]] [lindex $var [expr $i+2]] \
			physical {}
		    incr i 2
		}
	    }
	}
	default {
	    set sys [lindex $var [expr $i+2]]
	    set sky [lindex $var [expr $i+3]]
	    switch -- $sys {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			wcs $sys
		    incr i 2
		}
		wcs -
		wcsa -
		wcsb -
		wcsc -
		wcsd -
		wcse -
		wcsf -
		wcsg -
		wcsh -
		wcsi -
		wcsj -
		wcsk -
		wcsl -
		wcsm -
		wcsn -
		wcso -
		wcsp -
		wcsq -
		wcsr -
		wcss -
		wcst -
		wcsu -
		wcsv -
		wcsw -
		wcsx -
		wcsy -
		wcsz {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			$sys $sky
		    incr i 3
		}
		image -
		physical -
		amplifier -
		detector {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			$sys {}
		    incr i 2
		}
		{} -
		default {
		    Pan [lindex $var $i] [lindex $var [expr $i+1]] \
			physical {}
		    incr i 1
		}

	    }
	}
    }
}

proc ProcessZoomCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current

    switch -- [string tolower [lindex $var $i]] {
	to {
	    switch -- [string tolower [lindex $var [expr $i+1]]] {
		fit {
		    ZoomToFit
		    incr i
		}
		default {
		    set z1 [lindex $var [expr $i+1]]
		    set z2 [lindex $var [expr $i+2]]
		    if {[string is double $z2] && $z2 != {}} {
			set current(zoom) " $z1 $z2 "
			incr i 2
		    } else {
			set current(zoom) " $z1 $z1 "
			incr i
		    }
		    ChangeZoom
		}
	    }
	}
	default {
	    set z1 [lindex $var $i]
	    set z2 [lindex $var [expr $i+1]]
	    if {[string is double $z2] && $z2 != {}} {
		Zoom $z1 $z2
		incr i
	    } else {
		Zoom $z1 $z1
	    }
	}
    }
}

proc ProcessOrientCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    set current(orient) [string tolower [lindex $var $i]]
    ChangeOrient
}

proc ProcessRotateCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global current
    switch -- [string tolower [lindex $var $i]] {
	to {
	    set current(rotate) [lindex $var [expr $i+1]]
	    ChangeRotate
	    incr i
	}
	default {Rotate [lindex $var $i]}
    }
}
