#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc UpdatePixelTableDialog {which x y sys} {
    global pixel
    global pixelValues

    if {[winfo exists $pixel(top)]} {
	$which get pixel table $sys $x $y $pixel(size) pixelValues
    }
}

proc PixelTableDialog {} {
    global ds9
    global pixel

    if {[winfo exists $pixel(top)]} {
	raise $pixel(top)
	PixelTableConfigure
	return
    }
	
    # create the pixel table window

    set w $pixel(top)
    set title [msgcat::mc {Pixel Table}]

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PixelTableDestroyDialog

    $w configure -menu $pixel(mb)

    # simple menus

    menu $pixel(mb)
    AppleMenu pixel(mb)

    $pixel(mb) add cascade -label [msgcat::mc {File}] -menu $pixel(mb).file
    $pixel(mb) add cascade -label [msgcat::mc {Edit}] -menu $pixel(mb).edit
    $pixel(mb) add cascade -label [msgcat::mc {Size}] -menu $pixel(mb).size

    menu $pixel(mb).file
    $pixel(mb).file add command -label "[msgcat::mc {Save}]..." \
	-command PixelTableSaveDialog
    $pixel(mb).file add separator
    $pixel(mb).file add command -label [msgcat::mc {Close}] \
	-command PixelTableDestroyDialog

    menu $pixel(mb).edit
    $pixel(mb).edit add command -label [msgcat::mc {Cut}] \
	-state disabled -accelerator "${ds9(ctrl)}X"
    $pixel(mb).edit add command -label [msgcat::mc {Copy}] \
	-command PixelTableCopyDialog -accelerator "${ds9(ctrl)}C"
    $pixel(mb).edit add command -label [msgcat::mc {Paste}] \
	-state disabled -accelerator "${ds9(ctrl)}V"
    $pixel(mb).edit add command -label [msgcat::mc {Clear}] \
	-command PixelTableClearDialog

    menu $pixel(mb).size
    $pixel(mb).size add radiobutton -label {3x3} -variable pixel(size) \
	-value 3 -command PixelTableConfigure
    $pixel(mb).size add radiobutton -label {5x5} -variable pixel(size) \
	-value 5 -command PixelTableConfigure
    $pixel(mb).size add radiobutton -label {7x7} -variable pixel(size) \
	-value 7 -command PixelTableConfigure
    $pixel(mb).size add radiobutton -label {9x9} -variable pixel(size) \
	-value 9 -command PixelTableConfigure

    frame $w.table
    frame $w.buttons -relief ridge -bd 1
    pack $w.table
    pack $w.buttons -fill x -ipadx 4 -ipady 4

    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command PixelTableDestroyDialog
    pack $w.buttons.close -side left -padx 10 -expand true

    selection handle $pixel(top) PixelTableExportSelection

    PixelTableConfigure
}

proc PixelTableDestroyDialog {} {
    global pixel
    global pixelValues
    
    if {[winfo exists $pixel(top)]} {
	destroy $pixel(top)
	destroy $pixel(mb)
    }

    if {[info exists pixelValues]} {
	unset pixelValues
    }
}

proc PixelTableCopyDialog {} {
    global pixel

    set pixel(copy) [PixelTableRender]

    selection own -command PixelTableLostSelection $pixel(top)

    clipboard clear
    clipboard append $pixel(copy)
}

proc PixelTableClearDialog {} {
    global pixel
    global pixelValues

    if {[winfo exists $pixel(top)]} {
	for {set i 0} {$i<=$pixel(size)} {incr i} {
	    for {set j 0} {$j<=$pixel(size)} {incr j} {
		set pixelValues($i$j) {}
	    }
	}
    }
}

proc PixelTableSaveDialog {} {
    set filename [SaveFileDialog pixelfbox]

    if {$filename != {}} {
	set file [open $filename w]
	puts -nonewline $file [PixelTableRender]
	close $file
    }
}

# support

proc PixelTableConfigure {} {
    global pixel
    global pixelValues

    set w $pixel(top)

    # ungrid all
    for {set j 0} {$j<=$pixel(maxsize)} {incr j} {
	for {set i 0} {$i<=$pixel(maxsize)} {incr i} {
	    if {[winfo exists $w.table.b$i$j]} {
		grid forget $w.table.b$i$j
	    }
	}
    }
    if {[info exists pixelValues]} {
	unset pixelValues
    }

    # corner
    set i 0
    set j 0
    if {![winfo exists $w.table.b$i$j]} {
	label $w.table.b$i$j -width 12 -relief groove -bd 1
    }
    grid $w.table.b$i$j -column $i -row $j

    # x (column)
    set j 0
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		-textvariable pixelValues($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row $j
    }

    # y (row)
    set i 0
    for {set j 1} {$j<=$pixel(size)} {incr j} {
	if {![winfo exists $w.table.b$i$j]} {
	    label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		-textvariable pixelValues($i$j) -foreground blue
	}
	grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1] 
    }

    # body
    set half [expr ($pixel(size)+1)/2]
    set fg [$w.table.b00 cget -foreground]

    for {set i 1} {$i<=$pixel(size)} {incr i} {
	for {set j 1} {$j<=$pixel(size)} {incr j} {
	    if {![winfo exists $w.table.b$i$j]} {
		label $w.table.b$i$j -width 12 -relief groove -bd 1 \
		    -textvariable pixelValues($i$j)
	    }
	    if {($i==$half) && ($j==$half)} {
		$w.table.b$i$j configure -foreground red
	    } else {
		$w.table.b$i$j configure -foreground $fg
	    }
	    grid $w.table.b$i$j -column $i -row [expr $pixel(size)-$j+1]
	}
    }

    # show
    pack $w.table
}

proc PixelTableRender {} {
    global pixel
    global pixelValues

    set rr {}

    # col header
    append rr "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	set msg [format "%12s" $pixelValues(${i}0)]
	append rr "$msg"
    }
    append rr "\n"

    append rr "            "
    for {set i 1} {$i<=$pixel(size)} {incr i} {
	append rr " -----------"
    }
    append rr "\n"

    # body
    for {set j $pixel(size)} {$j>=1} {incr j -1} {
	set msg [format "%10s" $pixelValues(0${j})]
	append rr "$msg |"
	for {set i 1} {$i<=$pixel(size)} {incr i} {
	    set msg [format "%12.11s" $pixelValues($i$j)]
	    append rr "$msg"
	}
	append rr "\n"
    }

    return $rr
}

proc PixelTableExportSelection {offset bytes} {
    global pixel

    if {$pixel(copy) != {}} {
	return [string range $pixel(copy) $offset [expr $offset+$bytes]]
    }
}

proc PixelTableLostSelection {} {
    global pixel

    set pixel(copy) {}
}

# Process Cmds

proc ProcessPixelTableCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    switch -- [string tolower [lindex $var $i]] {
	yes -
	true -
	1 {PixelTableDialog}

	no -
	false -
	0 {PixelTableDestroyDialog}

	default {
	    PixelTableDialog
	    incr i -1
	}
    }
}
