#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc AnalysisPlot {tt wtt title xaxis yaxis dim data} {
    global ap

    # make the window name unique
    set i [lsearch $ap(xpa) $tt]
    if {$i>=0} {
	append tt $ap(unique,id)
	incr ap(unique,id)
    }

    # add it to our xpa list
    lappend ap(xpa) $tt

    # set the window title if none
    if {$wtt == {}} {
	set wtt $tt
    }

    # doit
    APDialog $tt $wtt $title $xaxis $yaxis
    APDataSet $tt $dim $data
}

proc APDialog {tt wtt title xaxis yaxis} {
    global ds9
    global ap
    global prefs

    if {[APPing $tt]} {
	APRaise $tt
	return
    }

    set ap($tt,top) ".${tt}"
    set ap($tt,mb) ".${tt}mb"

    set ap($tt,data,total) 0
    set ap($tt,data,current) 0

    set ap($tt,xdata) {}
    set ap($tt,ydata) {}
    set ap($tt,xedata) {}
    set ap($tt,yedata) {}

    set ap($tt,graph,title) "$title"
    set ap($tt,graph,xaxis) "$xaxis"
    set ap($tt,graph,yaxis) "$yaxis"

    set ap($tt,graph,x,auto) 1
    set ap($tt,graph,x,min) {}
    set ap($tt,graph,x,max) {}
    set ap($tt,graph,y,auto) 1
    set ap($tt,graph,y,min) {}
    set ap($tt,graph,y,max) {}

    set ap($tt,grid) $prefs(ap,grid)
    set ap($tt,grid,log) $prefs(ap,grid,log) 

    set ap($tt,discrete) $prefs(ap,discrete) 
    set ap($tt,discrete,symbol) $prefs(ap,discrete,symbol) 
    set ap($tt,discrete,color) $prefs(ap,discrete,color) 

    set ap($tt,linear) $prefs(ap,linear) 
    set ap($tt,linear,width) $prefs(ap,linear,width) 
    set ap($tt,linear,color) $prefs(ap,linear,color) 
    set ap($tt,linear,dash) $prefs(ap,linear,dash) 

    set ap($tt,step) $prefs(ap,step) 
    set ap($tt,step,width) $prefs(ap,step,width) 
    set ap($tt,step,color) $prefs(ap,step,color) 
    set ap($tt,step,dash) $prefs(ap,step,dash) 

    set ap($tt,quadratic) $prefs(ap,quadratic) 
    set ap($tt,quadratic,width) $prefs(ap,quadratic,width) 
    set ap($tt,quadratic,color) $prefs(ap,quadratic,color) 
    set ap($tt,quadratic,dash) $prefs(ap,quadratic,dash) 

    set ap($tt,error) $prefs(ap,error) 
    set ap($tt,error,width) $prefs(ap,error,width) 
    set ap($tt,error,color) $prefs(ap,error,color) 
    set ap($tt,error,style) $prefs(ap,error,style) 

    set ap($tt,titleFont) $prefs(ap,titleFont) 
    set ap($tt,titleSize) $prefs(ap,titleSize) 
    set ap($tt,titleStyle) $prefs(ap,titleStyle) 

    set ap($tt,textlabFont) $prefs(ap,textlabFont) 
    set ap($tt,textlabSize) $prefs(ap,textlabSize) 
    set ap($tt,textlabStyle) $prefs(ap,textlabStyle) 

    set ap($tt,numlabFont) $prefs(ap,numlabFont) 
    set ap($tt,numlabSize) $prefs(ap,numlabSize) 
    set ap($tt,numlabStyle) $prefs(ap,numlabStyle)

    toplevel $ap($tt,top)
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $ap($tt,top) document "closeBox fullZoom collapseBox resizable"}
    }

    wm title $ap($tt,top) $wtt
    wm iconname $ap($tt,top) $wtt
    wm group $ap($tt,top) $ds9(top)
    wm protocol $ap($tt,top) WM_DELETE_WINDOW "APDestroy $tt"

    $ap($tt,top) configure -menu $ap($tt,mb)

    menu $ap($tt,mb)
    AppleMenu ap(${tt},mb)

    $ap($tt,mb) add cascade -label [msgcat::mc {File}] -menu $ap($tt,mb).file
    $ap($tt,mb) add cascade -label [msgcat::mc {Edit}] -menu $ap($tt,mb).edit
    $ap($tt,mb) add cascade -label [msgcat::mc {Graph}] -menu $ap($tt,mb).graph
    $ap($tt,mb) add cascade -label [msgcat::mc {Font}] -menu $ap($tt,mb).font
    $ap($tt,mb) add cascade -label [msgcat::mc {DataSet}] \
	-menu $ap($tt,mb).dataset
    $ap($tt,mb) add cascade -label [msgcat::mc {View}] -menu $ap($tt,mb).view
    $ap($tt,mb) add cascade -label [msgcat::mc {Color}] -menu $ap($tt,mb).color
    $ap($tt,mb) add cascade -label [msgcat::mc {Line}] -menu $ap($tt,mb).line

    menu $ap($tt,mb).file
    $ap($tt,mb).file add command -label "[msgcat::mc {Load Data}]..." \
	-command "APLoadData $tt"
    $ap($tt,mb).file add command -label "[msgcat::mc {Save Data}]..." \
	-command "APSaveData $tt"
    $ap($tt,mb).file add command -label [msgcat::mc {Clear Data}] \
	-command "APClearData $tt"
    $ap($tt,mb).file add separator
    $ap($tt,mb).file add command \
	-label "[msgcat::mc {Load Configuration}]..." \
	-command "APLoadConfig $tt"
    $ap($tt,mb).file add command \
	-label "[msgcat::mc {Save Configuration}]..." \
	-command "APSaveConfig $tt"
    $ap($tt,mb).file add separator
    switch $ds9(wm) {
	x11 {
	    $ap($tt,mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-command PageSetup
	    $ap($tt,mb).file add command -label "[msgcat::mc {Print}]..." \
		-command "APPrint $tt"
	}
	win32 -
	aqua {
	    $ap($tt,mb).file add command \
		-label "[msgcat::mc {Postscript Page Setup}]..." \
		-command PSPageSetup
	    $ap($tt,mb).file add command \
		-label "[msgcat::mc {Postscript Print}]..." \
		-command "APPSPrint $tt"
	    $ap($tt,mb).file add separator
	    $ap($tt,mb).file add command -label "[msgcat::mc {Print}]..." \
		-state disabled
	    $ap($tt,mb).file add command -label "[msgcat::mc {Page Setup}]..." \
		-state disabled
	}
    }
    $ap($tt,mb).file add separator
    $ap($tt,mb).file add command -label [msgcat::mc {Close}] \
	-command "APDestroy $tt"

    menu $ap($tt,mb).edit
    $ap($tt,mb).edit add command -label [msgcat::mc {Cut}] \
	-state disabled -accelerator "${ds9(ctrl)}X"
    $ap($tt,mb).edit add command -label [msgcat::mc {Copy}] \
	-state disabled -accelerator "${ds9(ctrl)}C"
    $ap($tt,mb).edit add command -label [msgcat::mc {Paste}] \
	-state disabled -accelerator "${ds9(ctrl)}V"
    $ap($tt,mb).edit add command -label [msgcat::mc {Clear}] \
	-state disabled

    menu $ap($tt,mb).graph
    $ap($tt,mb).graph add checkbutton -label [msgcat::mc {Grid}] \
	-variable ap($tt,grid) -command "APUpdateGraph $tt"
    $ap($tt,mb).graph add separator
    $ap($tt,mb).graph add radiobutton -label [msgcat::mc {Linear-Linear}] \
	-variable ap($tt,grid,log) -value linearlinear \
	-command "APUpdateGraph $tt"
    $ap($tt,mb).graph add radiobutton -label [msgcat::mc {Linear-Log}] \
	-variable ap($tt,grid,log) -value linearlog \
	-command "APUpdateGraph $tt"
    $ap($tt,mb).graph add radiobutton -label [msgcat::mc {Log-Linear}] \
	-variable ap($tt,grid,log) -value loglinear \
	-command "APUpdateGraph $tt"
    $ap($tt,mb).graph add radiobutton -label [msgcat::mc {Log-Log}] \
	-variable ap($tt,grid,log) -value loglog \
	-command "APUpdateGraph $tt"
    $ap($tt,mb).graph add separator
    $ap($tt,mb).graph add command -label "[msgcat::mc {Axis Range}]..." \
	-command "APRange $tt"
    $ap($tt,mb).graph add command -label "[msgcat::mc {Labels}]..." \
	-command "APTitle $tt"

    menu $ap($tt,mb).font
    $ap($tt,mb).font add cascade -label [msgcat::mc {Axis Numbers}] \
	-menu $ap($tt,mb).font.numlab
    $ap($tt,mb).font add cascade -label [msgcat::mc {Axis Label}] \
	-menu $ap($tt,mb).font.textlab
    $ap($tt,mb).font add cascade -label [msgcat::mc {Title}] \
	-menu $ap($tt,mb).font.title

    DefaultFontMenu $ap($tt,mb).font.numlab \
	ap($tt,numlabFont) ap($tt,numlabSize) ap($tt,numlabStyle) \
	"APUpdateGraph $tt"
    DefaultFontMenu $ap($tt,mb).font.textlab \
	ap($tt,textlabFont) ap($tt,textlabSize) ap($tt,textlabStyle) \
	"APUpdateGraph $tt"
    DefaultFontMenu $ap($tt,mb).font.title \
	ap($tt,titleFont) ap($tt,titleSize) ap($tt,titleStyle) \
	"APUpdateGraph $tt"

    menu $ap($tt,mb).dataset

    menu $ap($tt,mb).view
    $ap($tt,mb).view add checkbutton -label [msgcat::mc {Discrete}] \
	-variable ap($tt,discrete) -command "APUpdateState $tt"
    $ap($tt,mb).view add checkbutton -label [msgcat::mc {Linear}] \
	-variable ap($tt,linear) -command "APUpdateState $tt"
    $ap($tt,mb).view add checkbutton -label [msgcat::mc {Step}] \
	-variable ap($tt,step) -command "APUpdateState $tt"
    $ap($tt,mb).view add checkbutton -label [msgcat::mc {Quadratic}] \
	-variable ap($tt,quadratic) -command "APUpdateState $tt"
    $ap($tt,mb).view add checkbutton -label [msgcat::mc {Error}] \
	-variable ap($tt,error) -command "APUpdateState $tt"

    menu $ap($tt,mb).color
    $ap($tt,mb).color add cascade -label [msgcat::mc {Discrete}] \
	-menu $ap($tt,mb).color.discrete
    $ap($tt,mb).color add cascade -label [msgcat::mc {Linear}] \
	-menu $ap($tt,mb).color.linear
    $ap($tt,mb).color add cascade -label [msgcat::mc {Step}] \
	-menu $ap($tt,mb).color.step
    $ap($tt,mb).color add cascade -label [msgcat::mc {Quadratic}] \
	-menu $ap($tt,mb).color.quadratic
    $ap($tt,mb).color add cascade -label [msgcat::mc {Error}] \
	-menu $ap($tt,mb).color.error

    DefaultColorMenu $ap($tt,mb).color.discrete \
	ap($tt,discrete,color) "APUpdateState $tt"
    DefaultColorMenu $ap($tt,mb).color.linear \
	ap($tt,linear,color) "APUpdateState $tt"
    DefaultColorMenu $ap($tt,mb).color.step \
	ap($tt,step,color) "APUpdateState $tt"
    DefaultColorMenu $ap($tt,mb).color.quadratic \
	ap($tt,quadratic,color) "APUpdateState $tt"
    DefaultColorMenu $ap($tt,mb).color.error \
	ap($tt,error,color) "APUpdateState $tt"

    menu $ap($tt,mb).line
    $ap($tt,mb).line add cascade -label [msgcat::mc {Discrete}] \
	-menu $ap($tt,mb).line.discrete
    $ap($tt,mb).line add cascade -label [msgcat::mc {Linear}] \
	-menu $ap($tt,mb).line.linear
    $ap($tt,mb).line add cascade -label [msgcat::mc {Step}] \
	-menu $ap($tt,mb).line.step
    $ap($tt,mb).line add cascade -label [msgcat::mc {Quadratic}] \
	-menu $ap($tt,mb).line.quadratic
    $ap($tt,mb).line add cascade -label [msgcat::mc {Error}] \
	-menu $ap($tt,mb).line.error

    menu $ap($tt,mb).line.discrete
    $ap($tt,mb).line.discrete add radiobutton -label [msgcat::mc {Circle}] \
	-variable ap($tt,discrete,symbol) -value circle \
	-command "APUpdateState $tt"
    $ap($tt,mb).line.discrete add radiobutton -label [msgcat::mc {Diamond}] \
	-variable ap($tt,discrete,symbol) -value diamond \
	-command "APUpdateState $tt"
    $ap($tt,mb).line.discrete add radiobutton -label [msgcat::mc {Plus}] \
	-variable ap($tt,discrete,symbol) -value plus \
	-command "APUpdateState $tt"
    $ap($tt,mb).line.discrete add radiobutton -label [msgcat::mc {Cross}] \
	-variable ap($tt,discrete,symbol) -value cross \
	-command "APUpdateState $tt"

    APCreateLineMenu $ap($tt,mb).line.linear \
	ap($tt,linear,width) ap($tt,linear,dash) "APUpdateState $tt"
    APCreateLineMenu $ap($tt,mb).line.step \
	ap($tt,step,width) ap($tt,step,dash) "APUpdateState $tt"
    APCreateLineMenu $ap($tt,mb).line.quadratic \
	ap($tt,quadratic,width) ap($tt,quadratic,dash) "APUpdateState $tt"

    DefaultWidthMenu $ap($tt,mb).line.error ap($tt,error,width) \
	"APUpdateState $tt"
    $ap($tt,mb).line.error add separator
    $ap($tt,mb).line.error add radiobutton -label "[msgcat::mc {Style}] 1" \
	-variable ap($tt,error,style) -value 1 -command "APUpdateState $tt"
    $ap($tt,mb).line.error add radiobutton -label "[msgcat::mc {Style}] 2" \
	-variable ap($tt,error,style) -value 2 -command "APUpdateState $tt"

    set ap($tt,graph) [blt::graph $ap($tt,top).g -plotrelief groove -bd 2 \
			   -width 500 -height 400 \
			   -plotbackground [$ds9(main) cget -background]]
    $ap($tt,graph) legend configure -hide yes

    switch $ds9(wm) {
	x11 -
	win32 {blt::ZoomStack $ap($tt,graph) "ButtonPress-1" "ButtonPress-3"} 
	aqua {blt::ZoomStack $ap($tt,graph) "ButtonPress-1" "ButtonPress-2"}
    }

    pack $ap($tt,graph) -expand yes -fill both

    APUpdateGraph $tt
}

proc APDestroy {tt} {
    global ap

    # see if it still is around
    if {![APPing $tt]} {
	return
    }

    for {set nn 1} {$nn<=$ap($tt,data,total)} {incr nn} {
	if {$ap($tt,$nn,manage)} {
	    blt::vector destroy \
		$ap($tt,$nn,xdata) $ap($tt,$nn,ydata) \
		$ap($tt,$nn,xedata) $ap($tt,$nn,yedata)
	}
    }

    destroy $ap($tt,top)
    destroy $ap($tt,mb)

    # unset all ap($tt,*)
    foreach x [array names ap] {
	set f [split $x ,]
	if {[lindex $f 0] == $tt} {
	    unset ap($x)
	}
    }

    # delete it from the xpa list
    set i [lsearch $ap(xpa) $tt]
    if {$i>=0} {
	set ap(xpa) [lreplace $ap(xpa) $i $i]
    }
}

proc APCurrentData {tt} {
    global ap

    if {$ap($tt,data,total) > 0} {
	set nn $ap($tt,data,current)

	set ap($tt,xdata) $ap($tt,$nn,xdata)
	set ap($tt,ydata) $ap($tt,$nn,ydata)
	set ap($tt,xedata) $ap($tt,$nn,xedata)
	set ap($tt,yedata) $ap($tt,$nn,yedata)

	set ap($tt,discrete) $ap($tt,$nn,discrete) 
	set ap($tt,discrete,symbol) $ap($tt,$nn,discrete,symbol) 
	set ap($tt,discrete,color) $ap($tt,$nn,discrete,color) 

	set ap($tt,linear) $ap($tt,$nn,linear) 
	set ap($tt,linear,width) $ap($tt,$nn,linear,width) 
	set ap($tt,linear,color) $ap($tt,$nn,linear,color) 
	set ap($tt,linear,dash) $ap($tt,$nn,linear,dash) 

	set ap($tt,step) $ap($tt,$nn,step) 
	set ap($tt,step,width) $ap($tt,$nn,step,width) 
	set ap($tt,step,color) $ap($tt,$nn,step,color) 
	set ap($tt,step,dash) $ap($tt,$nn,step,dash) 

	set ap($tt,quadratic) $ap($tt,$nn,quadratic) 
	set ap($tt,quadratic,width) $ap($tt,$nn,quadratic,width) 
	set ap($tt,quadratic,color) $ap($tt,$nn,quadratic,color) 
	set ap($tt,quadratic,dash) $ap($tt,$nn,quadratic,dash) 

	set ap($tt,error) $ap($tt,$nn,error) 
	set ap($tt,error,width) $ap($tt,$nn,error,width) 
	set ap($tt,error,color) $ap($tt,$nn,error,color) 
	set ap($tt,error,style) $ap($tt,$nn,error,style) 
    }
}

proc APDataFormatDialog {} {
    global ed

    set w {.apdata}

    set ed(ok) 0
    set ed(dim) xy

    DialogCreate $w [msgcat::mc {Data Format}]

    frame $w.dim -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.dim $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.dim.title -text [msgcat::mc {Data Format}]
    radiobutton $w.dim.xy -text {X Y} -variable ed(dim) -value xy
    radiobutton $w.dim.xyex -text {X Y XErr} -variable ed(dim) -value xyex
    radiobutton $w.dim.xyey -text {X Y YErr} -variable ed(dim) -value xyey
    radiobutton $w.dim.xyexey -text {X Y XErr YErr} -variable ed(dim) \
	-value xyexey

    grid $w.dim.title -padx 4 -sticky w
    grid $w.dim.xy -padx 4 -sticky w
    grid $w.dim.xyex -padx 4 -sticky w
    grid $w.dim.xyey -padx 4 -sticky w
    grid $w.dim.xyexey -padx 4 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok) $w.dim.xy
    DialogDismiss $w
}

proc APLoadData {tt} {
    global ds9
    global ap
    global ed

    set filename [OpenFileDialog apdatafbox]
    if {$filename != {}} {
	APDataFormatDialog

	if {$ed(ok)} {
	    APPLoadData $tt $filename $ed(dim)
	}
	unset ed
    }
}

proc APPLoadData {tt filename dim} {
    set ch [open $filename]
    set data [read $ch]
    close $ch

    APRaise $tt
    APDataSet $tt $dim $data
}

proc APSaveData {tt} {
    APPSaveData $tt [SaveFileDialog apdatafbox]
}

proc APPSaveData {tt filename} {
    global ap
    global ds9

    if {$filename != {}} {
	set ch [open $filename w]

	global $ap($tt,xdata) $ap($tt,ydata) $ap($tt,xedata) $ap($tt,yedata)
	set l [$ap($tt,xdata) length]

	if {([$ap($tt,xedata) length] == 0) && 
	    ([$ap($tt,yedata) length] == 0)} {
	    for {set i 0} {$i<$l} {incr i} {
		set x [expr "$$ap($tt,xdata)\($i\)"]
		set y [expr "$$ap($tt,ydata)\($i\)"]
		puts $ch "$x $y"
	    }
	} elseif {([$ap($tt,xedata) length] == 0) &&
		  ([$ap($tt,yedata) length] != 0)} {
	    for {set i 0} {$i<$l} {incr i} {
		set x [expr "$$ap($tt,xdata)\($i\)"]
		set y [expr "$$ap($tt,ydata)\($i\)"]
		set ye [expr "$$ap($tt,yedata)\($i\)"]
		puts $ch "$x $y $ye"
	    }
	} elseif {([$ap($tt,xedata) length] != 0) &&
		  ([$ap($tt,yedata) length] == 0)} {
	    for {set i 0} {$i<$l} {incr i} {
		set x [expr "$$ap($tt,xdata)\($i\)"]
		set y [expr "$$ap($tt,ydata)\($i\)"]
		set xe [expr "$$ap($tt,xedata)\($i\)"]
		puts $ch "$x $y $xe"
	    }
	} else {
	    for {set i 0} {$i<$l} {incr i} {
		set x [expr "$$ap($tt,xdata)\($i\)"]
		set y [expr "$$ap($tt,ydata)\($i\)"]
		set xe [expr "$$ap($tt,xedata)\($i\)"]
		set ye [expr "$$ap($tt,yedata)\($i\)"]
		puts $ch "$x $y $xe $ye"
	    }
	}

	close $ch
    }
    APRaise $tt
}

proc APClearData {tt} {
    global ap

    # this is hard coded to ignore an external first dataset

    if {$ap($tt,1,manage)} {
	set clear 1
	set nn 1
    } else {
	set clear 0
	set nn 2
    }

    for {} {$nn<=$ap($tt,data,total)} {incr nn} {
	# delete elements
	foreach el [$ap($tt,graph) element names] {
	    set f [split $el -]
	    if {[lindex $f 1] == $nn} {
		$ap($tt,graph) element delete $el
	    }
	}

	blt::vector destroy $ap($tt,$nn,xdata) $ap($tt,$nn,ydata) \
	    $ap($tt,$nn,xedata) $ap($tt,$nn,yedata)

	# unset all ap($tt,$nn,*)
	foreach x [array names ap] {
	    set f [split $x ,]
	    if {([lindex $f 0] == $tt) && ([lindex $f 1] == $nn)} {
		unset ap($x)
	    }
	}
    }

    if {$clear} {
	set ap($tt,data,total) 0
	set ap($tt,data,current) 0

	# reset other variables
	set ap($tt,xdata) {}
	set ap($tt,ydata) {}
	set ap($tt,xedata) {}
	set ap($tt,yedata) {}

	set ap($tt,graph,title) {}
	set ap($tt,graph,xaxis) {}
	set ap($tt,graph,yaxis) {}

	set ap($tt,graph,x,auto) 1
	set ap($tt,graph,x,min) {}
	set ap($tt,graph,x,max) {}
	set ap($tt,graph,y,auto) 1
	set ap($tt,graph,y,min) {}
	set ap($tt,graph,y,max) {}
	
	$ap($tt,mb).dataset delete 1 end
    } else {
	if {$ap($tt,data,total) > 1} {
	    $ap($tt,mb).dataset delete 2 end
	}

	set ap($tt,data,total) 1
	set ap($tt,data,current) 1
    }

    APUpdateGraph $tt
}

proc APLoadConfig {tt} {
    APPLoadConfig $tt [OpenFileDialog apconfigfbox]
}

proc APPLoadConfig {tt filename} {
    global ds9
    global ap

    if {$filename != {}} {
	source $filename

        set ap($tt,discrete) $analysisplot(discrete)
        set ap($tt,discrete,symbol) $analysisplot(discrete,symbol)
        set ap($tt,discrete,color) $analysisplot(discrete,color)

        set ap($tt,linear) $analysisplot(linear)
        set ap($tt,linear,width) $analysisplot(linear,width)
        set ap($tt,linear,color) $analysisplot(linear,color)
        set ap($tt,linear,dash) $analysisplot(linear,dash)

        set ap($tt,step) $analysisplot(step)
        set ap($tt,step,width) $analysisplot(step,width)
        set ap($tt,step,color) $analysisplot(step,color)
        set ap($tt,step,dash) $analysisplot(step,dash)

        set ap($tt,quadratic) $analysisplot(quadratic)
        set ap($tt,quadratic,width) $analysisplot(quadratic,width)
        set ap($tt,quadratic,color) $analysisplot(quadratic,color)
        set ap($tt,quadratic,dash) $analysisplot(quadratic,dash)

        set ap($tt,error) $analysisplot(error)
        set ap($tt,error,width) $analysisplot(error,width)
        set ap($tt,error,color) $analysisplot(error,color)
        set ap($tt,error,style) $analysisplot(error,style)

	APUpdateState $tt
    }
}

proc APSaveConfig {tt} {
    APPSaveConfig $tt [SaveFileDialog apconfigfbox]
}

proc APPSaveConfig {tt filename} {
    global ds9
    global ap

    if {$filename != {}} {
	set file [open $filename w]

        puts $file "set analysisplot(discrete) $ap($tt,discrete)"
        puts $file "set analysisplot(discrete,symbol) $ap($tt,discrete,symbol)"
        puts $file "set analysisplot(discrete,color) $ap($tt,discrete,color)"

        puts $file "set analysisplot(linear) $ap($tt,linear)"
        puts $file "set analysisplot(linear,width) $ap($tt,linear,width)"
        puts $file "set analysisplot(linear,color) $ap($tt,linear,color)"
        puts $file "set analysisplot(linear,dash) \"$ap($tt,linear,dash)\""

        puts $file "set analysisplot(step) $ap($tt,step)"
        puts $file "set analysisplot(step,width) $ap($tt,step,width)"
        puts $file "set analysisplot(step,color) $ap($tt,step,color)"
        puts $file "set analysisplot(step,dash) \"$ap($tt,step,dash)\""

        puts $file "set analysisplot(quadratic) $ap($tt,quadratic)"
        puts $file "set analysisplot(quadratic,width) $ap($tt,quadratic,width)"
        puts $file "set analysisplot(quadratic,color) $ap($tt,quadratic,color)"
        puts $file \
	    "set analysisplot(quadratic,dash) \"$ap($tt,quadratic,dash)\""

        puts $file "set analysisplot(error) $ap($tt,error)"
        puts $file "set analysisplot(error,width) $ap($tt,error,width)"
        puts $file "set analysisplot(error,color) $ap($tt,error,color)"
        puts $file "set analysisplot(error,style) \"$ap($tt,error,style)\""

	close $file
    }
}

proc APRange {tt} {
    global ap
    global ed

    set w {.aptitle}

    set ed(ok) 0
    set ed(xauto) $ap($tt,graph,x,auto)
    set ed(xmin) $ap($tt,graph,x,min)
    set ed(xmax) $ap($tt,graph,x,max)
    set ed(yauto) $ap($tt,graph,y,auto)
    set ed(ymin) $ap($tt,graph,y,min)
    set ed(ymax) $ap($tt,graph,y,max)

    DialogCreate $w [msgcat::mc {Range}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.xlabel -text [msgcat::mc {X Axis}]
    entry $w.param.xmin -textvariable ed(xmin) -width 12
    label $w.param.xlabel2 -text [msgcat::mc {To}]
    entry $w.param.xmax -textvariable ed(xmax) -width 12
    checkbutton $w.param.xauto -text [msgcat::mc {Automatic}] \
	-variable ed(xauto)

    label $w.param.ylabel -text [msgcat::mc {Y Axis}]
    entry $w.param.ymin -textvariable ed(ymin) -width 12
    label $w.param.ylabel2 -text [msgcat::mc {To}]
    entry $w.param.ymax -textvariable ed(ymax) -width 12
    checkbutton $w.param.yauto -text [msgcat::mc {Automatic}] \
	-variable ed(yauto)

    grid $w.param.xlabel $w.param.xmin $w.param.xlabel2 $w.param.xmax \
	$w.param.xauto  -padx 4 -pady 1 -sticky w
    grid $w.param.ylabel $w.param.ymin $w.param.ylabel2 $w.param.ymax \
	$w.param.yauto  -padx 4 -pady 1 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok) $w.param.xmin
    DialogDismiss $w

    if {$ed(ok)} {
	set ap($tt,graph,x,auto) $ed(xauto)
	set ap($tt,graph,x,min) $ed(xmin) 
	set ap($tt,graph,x,max) $ed(xmax) 
	set ap($tt,graph,y,auto) $ed(yauto) 
	set ap($tt,graph,y,min) $ed(ymin) 
	set ap($tt,graph,y,max) $ed(ymax) 

	APUpdateGraph $tt
    }
    
    unset ed
}

proc APTitle {tt} {
    global ap
    global ed

    set w {.aptitle}

    set ed(ok) 0
    set ed(title) $ap($tt,graph,title)
    set ed(xaxis) $ap($tt,graph,xaxis)
    set ed(yaxis) $ap($tt,graph,yaxis)

    DialogCreate $w [msgcat::mc {Labels}]

    frame $w.param -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.param.label -text [msgcat::mc {Plot Title}]
    entry $w.param.title -textvariable ed(title) -width 23
    label $w.param.xlabel -text [msgcat::mc {X Axis Label}]
    entry $w.param.xtitle -textvariable ed(xaxis) -width 23
    label $w.param.ylabel -text [msgcat::mc {Y Axis Label}]
    entry $w.param.ytitle -textvariable ed(yaxis) -width 23

    grid $w.param.label $w.param.title -padx 4 -pady 1 -sticky w
    grid $w.param.xlabel $w.param.xtitle -padx 4 -pady 1 -sticky w
    grid $w.param.ylabel $w.param.ytitle -padx 4 -pady 1 -sticky w

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok) $w.param.title
    DialogDismiss $w

    if {$ed(ok)} {
	set ap($tt,graph,title) $ed(title)
	set ap($tt,graph,xaxis) $ed(xaxis)
	set ap($tt,graph,yaxis) $ed(yaxis)
	APUpdateGraph $tt
    }
    
    unset ed
}

proc APCreateLineMenu {which var1 var2 cmd} {
    global ap

    menu $which
    $which add radiobutton -label 1 -variable $var1 -value 1 -command $cmd
    $which add radiobutton -label 2 -variable $var1 -value 2 -command $cmd
    $which add radiobutton -label 3 -variable $var1 -value 3 -command $cmd
    $which add radiobutton -label 4 -variable $var1 -value 4 -command $cmd
    $which add separator
    $which add radiobutton -label Solid -variable $var2 -value {no} \
	-command $cmd
    $which add radiobutton -label Dash  -variable $var2 -value {yes} \
	-command $cmd
}

proc APLineSymbolMenu {which var} {
    global prefs

    menu $which
    $which add radiobutton -label [msgcat::mc {Circle}] \
	-variable $var -value circle
    $which add radiobutton -label [msgcat::mc {Diamond}]  \
	-variable $var -value diamond
    $which add radiobutton -label [msgcat::mc {Plus}]  \
	-variable $var -value plus
    $which add radiobutton -label [msgcat::mc {Cross}]  \
	-variable $var -value cross
}

proc APLineWidthMenu {which var} {
    global prefs

    menu $which
    $which add radiobutton -label 0 -variable $var -value 0
    $which add radiobutton -label 1 -variable $var -value 1
    $which add radiobutton -label 2 -variable $var -value 2
    $which add radiobutton -label 3 -variable $var -value 3
}

proc APLineDashMenu {which var} {
    global prefs

    menu $which
    $which add radiobutton -label [msgcat::mc {No}] -variable $var \
	-value no
    $which add radiobutton -label [msgcat::mc {Yes}]  -variable $var \
	-value yes
}

# Print

proc APPrint {tt} {
    global ds9

    switch $ds9(wm) {
	x11 {APPSPrint $tt}
	win32 {}
	aqua {}
    }
}

proc APPSPrint {tt} {
    if {[PSPrintDialog ap]} {
	if [catch {APPostScript $tt} printError] {
	    Error "[msgcat::mc {An error has occurred while printing}] $printError"
	}
    }
}

proc APPostScript {tt} {
    global ps
    global ap

    global debug
    if {$debug(tcl,idletasks)} {
	puts "APPostScript"
    }
    update idletasks

    set options "-decorations false -colormode $ps(color2)"

    # Orientation

    switch -- $ps(orient) {
	portrait {set options "$options -landscape false"}
	landscape {set options "$options -landscape true"}
    }

    # Page size

    switch -- $ps(size) {
	letter {APPostScripPageSize 8.5i 11.i options $tt}
	legal {APPostScripPageSize 8.5i 14.i options $tt}
	tabloid {APPostScripPageSize 11.i 17.i options $tt}
	poster {APPostScripPageSize 36.i 48.i options $tt}
	a4 {APPostScripPageSize 195m 282m options $tt}
	other {
	    if {$ps(width) != {} && $ps(height) != {}} {
		set w [expr $ps(width)]
		set h [expr $ps(height)]
		APPostScripPageSize [append w i] [append h i] options $tt
	    }
	}
	othermm {
	    if {$ps(width) != {} && $ps(height) != {}} {
		set w [expr $ps(width)]
		set h [expr $ps(height)]
		APPostScripPageSize [append w m] [append h m] options $tt
	    }
	}
    }

    if {$ps(dest) == "file" && $ps(filename) != {}} {
	eval $ap($tt,graph) postscript output $ps(filename) $options
    } else {
	set ch [open "| $ps(cmd)" w]
	puts $ch [eval $ap($tt,graph) postscript output $options]
	close $ch
    }
}

proc APPostScripPageSize {w h optname tt} {
    upvar $optname options
    global ps
    global ap

    set options "$options -paperwidth $w -paperheight $h"

    set width [winfo width $ap($tt,graph)]
    set height [winfo height $ap($tt,graph)]
    set aspect [expr double($height)/$width]

    if {$ps(scale) == "scaled"} {
	regexp {([0-9.]*)([icmp])?} $w foo a b
	set options "$options -width $w -height [expr $a*$aspect]${b}"
    } else {
	set options "$options -width 0 -height 0"
    }
}

# Prefs

proc PrefsDialogPlot {} {
    global prefs

    set w $prefs(tabs)

    frame $w.plot
    $prefs(tabs) tab configure plot -window $w.plot -fill both

    frame $w.plot.grid -relief ridge -bd 1
    frame $w.plot.font -relief ridge -bd 1
    frame $w.plot.discrete -relief ridge -bd 1
    frame $w.plot.linear -relief ridge -bd 1
    frame $w.plot.step -relief ridge -bd 1
    frame $w.plot.quadratic -relief ridge -bd 1
    frame $w.plot.error -relief ridge -bd 1
    pack $w.plot.grid $w.plot.font $w.plot.discrete $w.plot.linear \
	$w.plot.step $w.plot.quadratic $w.plot.error \
	-side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.plot

    # Graph
    set f $w.plot.grid.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Grid}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,grid)
    radiobutton $f.linearlinear -text [msgcat::mc {Linear-Linear}] \
	-variable prefs(ap,grid,log) -value linearlinear
    radiobutton $f.linearlog -text [msgcat::mc {Linear-Log}] \
	-variable prefs(ap,grid,log) -value linearlog
    radiobutton $f.loglinear -text [msgcat::mc {Log-Linear}] \
	-variable prefs(ap,grid,log) -value loglinear
    radiobutton $f.loglog -text [msgcat::mc {Log-Log}] \
	-variable prefs(ap,grid,log) -value loglog

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.linearlinear $f.linearlog $f.loglinear $f.loglog \
	-padx 4 -sticky w

    # Font
    set f $w.plot.font.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.ftitle -text [msgcat::mc {Font}]
    label $f.tnumlab -text [msgcat::mc {Axis Numbers}]
    menubutton $f.numlab -relief raised \
	-textvariable prefs(ap,numlabFont) -menu $f.numlab.menu
    label $f.ttextlab -text [msgcat::mc {Axis Label}]
    menubutton $f.textlab -relief raised \
	-textvariable prefs(ap,textlabFont) -menu $f.textlab.menu
    label $f.ttitle -text [msgcat::mc {Title}]
    menubutton $f.title -relief raised \
	-textvariable prefs(ap,titleFont) -menu $f.title.menu

    grid $f.ftitle -padx 4 -sticky w
    grid $f.tnumlab $f.numlab $f.ttextlab $f.textlab \
	$f.ttitle $f.title -padx 4 -sticky w

    DefaultFontMenu $f.numlab.menu \
	prefs(ap,numlabFont) prefs(ap,numlabSize) prefs(ap,numlabStyle) {}
    DefaultFontMenu $f.textlab.menu \
	prefs(ap,textlabFont) prefs(ap,textlabSize) prefs(ap,textlabStyle) {}
    DefaultFontMenu $f.title.menu \
	prefs(ap,titleFont) prefs(ap,titleSize) prefs(ap,titleStyle) {}

    # Discrete
    set f $w.plot.discrete.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Discrete}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,discrete)
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable prefs(ap,discrete,color) -menu $f.color.menu
    label $f.stitle -text [msgcat::mc {Symbol}]
    menubutton $f.symbol -relief raised \
	-textvariable prefs(ap,discrete,symbol) -menu $f.symbol.menu

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.ctitle $f.color $f.stitle $f.symbol \
	-padx 4 -sticky w

    DefaultColorMenu $f.color.menu prefs(ap,discrete,color) {}
    APLineSymbolMenu $f.symbol.menu prefs(ap,discrete,symbol)

    # Line
    set f $w.plot.linear.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Linear}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,linear)
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable prefs(ap,linear,color) -menu $f.color.menu
    label $f.wtitle -text [msgcat::mc {Width}]
    menubutton $f.width -relief raised \
	-textvariable prefs(ap,linear,width) -menu $f.width.menu
    label $f.dtitle -text [msgcat::mc {Dash}]
    menubutton $f.dash -relief raised \
	-textvariable prefs(ap,linear,dash) -menu $f.dash.menu

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.ctitle $f.color $f.wtitle $f.width 	$f.dtitle $f.dash \
	-padx 4 -sticky w

    DefaultColorMenu $f.color.menu prefs(ap,linear,color) {}
    APLineWidthMenu $f.width.menu prefs(ap,linear,width)
    APLineDashMenu $f.dash.menu prefs(ap,linear,dash)

    # Step
    set f $w.plot.step.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Step}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,step)
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable prefs(ap,step,color) -menu $f.color.menu
    label $f.wtitle -text [msgcat::mc {Width}]
    menubutton $f.width -relief raised \
	-textvariable prefs(ap,step,width) -menu $f.width.menu
    label $f.dtitle -text [msgcat::mc {Dash}]
    menubutton $f.dash -relief raised \
	-textvariable prefs(ap,step,dash) -menu $f.dash.menu

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.ctitle $f.color $f.wtitle $f.width $f.dtitle $f.dash \
	-padx 4 -sticky w

    DefaultColorMenu $f.color.menu prefs(ap,step,color) {}
    APLineWidthMenu $f.width.menu prefs(ap,step,width)
    APLineDashMenu $f.dash.menu prefs(ap,step,dash)

    # Quadratic
    set f $w.plot.quadratic.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Quadratic}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,quadratic)
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable prefs(ap,quadratic,color) -menu $f.color.menu
    label $f.wtitle -text [msgcat::mc {Width}]
    menubutton $f.width -relief raised \
	-textvariable prefs(ap,quadratic,width) -menu $f.width.menu
    label $f.dtitle -text [msgcat::mc {Dash}]
    menubutton $f.dash -relief raised \
	-textvariable prefs(ap,quadratic,dash) -menu $f.dash.menu

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.ctitle $f.color $f.wtitle $f.width $f.dtitle $f.dash \
	-padx 4 -sticky w

    DefaultColorMenu $f.color.menu prefs(ap,quadratic,color) {}
    APLineWidthMenu $f.width.menu prefs(ap,quadratic,width)
    APLineDashMenu $f.dash.menu prefs(ap,quadratic,dash)

    # Error
    set f $w.plot.error.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Error}]
    checkbutton $f.show -text [msgcat::mc {Show}] \
	-variable prefs(ap,error)
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable prefs(ap,error,color) -menu $f.color.menu
    label $f.wtitle -text [msgcat::mc {Width}]
    menubutton $f.width -relief raised \
	-textvariable prefs(ap,error,width) -menu $f.width.menu
    label $f.stitle -text [msgcat::mc {Style}]
    menubutton $f.style -relief raised \
	-textvariable prefs(ap,error,style) -menu $f.style.menu

    grid $f.title $f.show -padx 4 -sticky w
    grid $f.ctitle $f.color $f.wtitle $f.width $f.stitle $f.style \
	-padx 4 -sticky w

    DefaultColorMenu $f.color.menu prefs(ap,error,color) {}
    APLineWidthMenu $f.width.menu prefs(ap,error,width)

    set m $f.style.menu 
    menu $m
    $m add radiobutton -label "[msgcat::mc {Style}] 1" \
	-variable prefs(ap,error,style) -value 1
    $m add radiobutton -label "[msgcat::mc {Style}] 2" \
	-variable prefs(ap,error,style) -value 2
}

# Process Cmds

proc ProcessPlotCmd {varname iname bufname} {
    upvar $varname var
    upvar $iname i
    upvar $bufname buf

    global ap

    set tt $ap(tt)
    set itt 0

    # determine which plot
    switch -- [string tolower [lindex $var $i]] {
	{} -
	new {}

	data -
	load -
	save -
	clear -
	loadconfig -
	saveconfig -
	print -
	page -
	close -

	graph -
	font -
	dataset -
	view -
	color -
	line {
	    set tt [lindex $ap(xpa) end]
	    set itt [lsearch $ap(xpa) $tt]
	}

	default {
	    set tt [lindex $var $i]
	    set itt [lsearch $ap(xpa) $tt]
	    incr i
	}
    }

    # we better have a tt by now
    if {$itt == -1} {
	Error "[msgcat::mc {Unable to find plot window}] $tt"
	return
    }

    # now, process plot command
    switch -- [string tolower [lindex $var $i]] {
	{} {APProcessNew $tt $varname $iname $bufname}
	new {
	    incr i
	    switch -- [lindex $var $i] {
		name {
		    set tt [lindex $var [expr $i+1]]
		    incr i 2
		}
	    }
	    APProcessNew $tt $varname $iname $bufname
	}
	data {incr i; APProcessData $tt $varname $iname $bufname}
	load {
	    set fn [lindex $var [expr $i+1]]
	    set dim [lindex $var [expr $i+2]]
	    APPLoadData $tt $fn $dim
	    FileLastFull apdatafbox $fn
	    incr i 2
	}
	save {
	    incr i
	    set fn [lindex $var $i]
	    APPSaveData $tt $fn
	    FileLastFull apdatafbox $fn
	}
	clear {APClearData $tt}
	loadconfig {
	    incr i
	    set fn [lindex $var $i]
	    APPLoadConfig $tt $fn
	    FileLastFull apconfigfbox $fn
	}
	saveconfig {
	    incr i
	    set fn [lindex $var $i]
	    APPSaveConfig $tt $fn
	    FileLastFull apconfigfbox $fn
	}
	print {incr i; APProcessPrint $tt $varname $iname}
	page {incr i; APProcessPage $tt $varname $iname}
	close {APDestroy $tt}

	graph {incr i; APProcessGraph $tt $varname $iname}
	font {incr i; APProcessFont $tt $varname $iname}
	dataset {incr i; APProcessDataset $tt $varname $iname}
	view {incr i; APProcessView $tt $varname $iname}
	color {incr i; APProcessColor $tt $varname $iname}
	line {incr i; APProcessLine $tt $varname $iname}
    }
}

proc APProcessNew {tt varname iname bufname} {
    upvar 2 $varname var
    upvar 2 $iname i
    upvar 2 $bufname buf

    global ap

    switch -- [string tolower [lindex $var $i]] {
	stdin {
	    AnalysisPlotStdin $tt {} buf
	}
	{} {
	    AnalysisPlot $tt {} {} {} {} xy $buf
	    incr i -1
	}
	default {
	    AnalysisPlot $tt {} \
		[lindex $var $i] \
		[lindex $var [expr $i+1]] \
		[lindex $var [expr $i+2]] \
		[lindex $var [expr $i+3]] \
		$buf
	    incr i 3
	}
    }
}

proc APProcessData {tt varname iname bufname} {
    upvar 2 $varname var
    upvar 2 $iname i
    upvar 2 $bufname buf

    global ap

    APRaise $tt
    APDataSet $tt [lindex $var $i] $buf
}

proc APProcessPrint {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ps
    global current

    switch -- [string tolower [lindex $var $i]] {
	destination {incr i; set ps(dest) [lindex $var $i]}
	command {incr i; set ps(cmd) [lindex $var $i]}
	filename {incr i; set ps(filename) [lindex $var $i] }
	palette {incr i; set ps(color2) [lindex $var $i] }

	{} {APPostScript $tt}
	default {incr i -1; APPostScript $tt}
    }
}

proc APProcessPage {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ps

    switch -- [string tolower [lindex $var $i]] {
	orientation {incr i; set ps(orient) [lindex $var $i]}
	pagescale {incr i; set ps(scale) [lindex $var $i]}
	pagesize {incr i; set ps(size) [lindex $var $i] }
    }
}

proc APProcessGraph  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    switch -- [string tolower [lindex $var $i]] {
	grid {incr i; set ap($tt,grid) [FromYesNo [lindex $var $i]]}
	scale {incr i; set ap($tt,grid,log) [lindex $var $i]}
	range {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		x {
		    incr i
		    switch -- [string tolower [lindex $var $i]] {
			auto {incr i; set ap($tt,graph,x,auto) \
				  [FromYesNo [lindex $var $i]]}
			min {incr i; set ap($tt,graph,x,min) [lindex $var $i]}
			max {incr i; set ap($tt,graph,x,max) [lindex $var $i]}
		    }
		}
		y {
		    incr i
		    switch -- [string tolower [lindex $var $i]] {
			auto {incr i; set ap($tt,graph,y,auto) \
				  [FromYesNo [lindex $var $i]]}
			min {incr i; set ap($tt,graph,y,min) [lindex $var $i]}
			max {incr i; set ap($tt,graph,y,max) [lindex $var $i]}
		    }
		}
	    }
	}
	labels {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		title {incr i; set ap($tt,graph,title) [lindex $var $i]}
		xaxis {incr i; set ap($tt,graph,xaxis) [lindex $var $i]}
		yaxis {incr i; set ap($tt,graph,yaxis) [lindex $var $i]}
	    }
	}
    }
    APUpdateGraph $tt
}

proc APProcessFont  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    switch -- [string tolower [lindex $var $i]] {
	numbers {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		font {incr i; set ap($tt,numlabFont) [lindex $var $i]}
		size {incr i; set ap($tt,numlabSize) [lindex $var $i]}
		style {incr i; set ap($tt,numlabStyle) [lindex $var $i]}
	    }
	}
	labels {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		font {incr i; set ap($tt,textlabFont) [lindex $var $i]}
		size {incr i; set ap($tt,textlabSize) [lindex $var $i]}
		style {incr i; set ap($tt,textlabStyle) [lindex $var $i]}
	    }
	}
	title {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		font {incr i; set ap($tt,titleFont) [lindex $var $i]}
		size {incr i; set ap($tt,titleSize) [lindex $var $i]}
		style {incr i; set ap($tt,titleStyle) [lindex $var $i]}
	    }
	}
    }
    APUpdateGraph $tt
}

proc APProcessDataset  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    set  ap($tt,data,current) [lindex $var $i]
    APCurrentData $tt
}

proc APProcessView  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    switch -- [string tolower [lindex $var $i]] {
	discrete {incr i; set ap($tt,discrete) [FromYesNo [lindex $var $i]]}
	line -
	linear {incr i; set ap($tt,linear) [FromYesNo [lindex $var $i]]}
	step {incr i; set ap($tt,step) [FromYesNo [lindex $var $i]]}
	quadratic {incr i; set ap($tt,quadratic) [FromYesNo [lindex $var $i]]}
	errorbar {incr i; set ap($tt,error) [FromYesNo [lindex $var $i]]}
    }

    APUpdateState $tt
}

proc APProcessColor  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    switch -- [string tolower [lindex $var $i]] {
	discrete {incr i; set ap($tt,discrete,color) [lindex $var $i]}
	line -
	linear {incr i; set ap($tt,linear,color) [lindex $var $i]}
	step {incr i; set ap($tt,step,color) [lindex $var $i]}
	quadratic {incr i; set ap($tt,quadratic,color) [lindex $var $i]}
	errorbar {incr i; set ap($tt,error,color) [lindex $var $i]}
    }

    APUpdateState $tt
}

proc APProcessLine  {tt varname iname} {
    upvar 2 $varname var
    upvar 2 $iname i

    global ap

    switch -- [string tolower [lindex $var $i]] {
	discrete {incr i; set ap($tt,discrete,symbol) [lindex $var $i]}
	line -
	linear {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		width {incr i; set ap($tt,linear,width) [lindex $var $i]}
		dash {
		    incr i
		    set ap($tt,linear,dash) [lindex $var $i]}
	    }
	}
	step {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		width {incr i; set ap($tt,step,width) [lindex $var $i]}
		dash {
		    incr i
		    set ap($tt,step,dash) [lindex $var $i]}
	    }
	}
	quadratic {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		width {incr i; set ap($tt,quadratic,width) [lindex $var $i]}
		dash {
		    incr i
		    set ap($tt,quadratic,dash) [lindex $var $i]}
	    }
	}
	error -
	errorbar {
	    incr i
	    switch -- [string tolower [lindex $var $i]] {
		width {incr i; set ap($tt,error,width) [lindex $var $i]}
		style {
		    incr i;
		    set ap($tt,error,style) [lindex $var $i]}
	    }
	}
    }

    APUpdateState $tt
}
