#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc LoadPrefs {} {
    # fixes an issue with prefs files version b9
    global prefs
    global ds9
    set prefs(font) $ds9(font)

    SourceInitFile {.prf}
    FixPrefs
}

proc CheckPrefs {} {
    global ds9
    global prefs

    if {[string compare $prefs(version) [lindex $ds9(version) 0]] == -1} {
	if {[tk_messageBox -type yesno -icon question \
		 -message [msgcat::mc {DS9 has detected an older preferences file, do you wish to update?}]] \
		== {yes}} {
	    SavePrefs
	}
    }
}

proc ClearPrefs {} {
    DeleteInitFile {.prf}
    Info [msgcat::mc {Preferences have been reset to the default values. Please restart DS9 for these changes to take effect}]
}

proc SavePrefs {} {
    global ds9
    global prefs
    global env

    # make sure old prefs files are removed
    DeleteInitFile {.prf}

    # open prefs file
    set file [open [file join [GetEnvHome] ".$ds9(app).prf"] w]
    if {$file == {}} {
	Error [msgcat::mc {An error has occurred while saving}]
	return
    }

    puts $file "global ds9"
    puts $file "global prefs"
    puts $file "set prefs(version) [lindex $ds9(version) 0]"

    # check for wrong prefs
    puts $file "\# this is a check for to ensure a match between the"
    puts $file "\# current ds9 version matches the prefs version"
    puts $file "if {\[string compare \$prefs(version) \[lindex \$ds9(version)\ 0\]] == 1} {"
    puts $file "    tk_messageBox -type ok -icon warning -message \"\[msgcat::mc {DS9 has detected a newer version of a preferences file and therefore will not process this file.}\]\""
    puts $file "    return"
    puts $file "}"

    # prefs
    puts $file ""
    puts $file "\# Bin prefs"
    global bin
    global pbin
    puts $file "global bin"
    puts $file "global pbin"
    puts $file "array set pbin \{ [array get pbin] \}"
    puts $file {array set bin [array get pbin]}

    puts $file ""
    puts $file "\# Contour prefs"
    global contour
    global pcontour
    puts $file "global contour"
    puts $file "global pcontour"
    puts $file "array set pcontour \{ [array get pcontour] \}"
    puts $file {array set contour [array get pcontour]}

    puts $file ""
    puts $file "\# Mask prefs"
    global mask
    global pmask
    puts $file "global mask"
    puts $file "global pmask"
    puts $file "array set pmask \{ [array get pmask] \}"
    puts $file {array set mask [array get pmask]}

    puts $file ""
    puts $file "\# MinMax prefs"
    global minmax
    global pminmax
    puts $file "global minmax"
    puts $file "global pminmax"
    puts $file "array set pminmax \{ [array get pminmax] \}"
    puts $file {array set minmax [array get pminmax]}

    puts $file ""
    puts $file "\# Postscript prefs"
    global ps
    global pps
    puts $file "global ps"
    puts $file "global pps"
    puts $file "array set pps \{ [array get pps] \}"
    puts $file {array set ps [array get pps]}

    puts $file ""
    puts $file "\# Scale prefs"
    global scale
    global pscale
    puts $file "global scale"
    puts $file "global pscale"
    puts $file "array set pscale \{ [array get pscale] \}"
    puts $file {array set scale [array get pscale]}

    puts $file ""
    puts $file "\# Smooth prefs"
    global smooth
    global psmooth
    puts $file "global smooth"
    puts $file "global psmooth"
    puts $file "array set psmooth \{ [array get psmooth] \}"
    puts $file {array set smooth [array get psmooth]}

    puts $file ""
    puts $file "\# View prefs"
    global view
    global pview
    puts $file "global view"
    puts $file "global pview"
    puts $file "array set pview \{ [array get pview] \}"
    puts $file {array set view [array get pview]}

    puts $file ""
    puts $file "\# Buttons prefs"
    global buttons
    puts $file "global buttons"
    puts $file "array set buttons \{ [array get buttons] \}"

    puts $file ""
    puts $file "\# WCS prefs"
    global wcs
    global pwcs
    puts $file "global wcs"
    puts $file "global pwcs"
    puts $file "array set pwcs \{ [array get pwcs] \}"
    puts $file {array set wcs [array get pwcs]}

    puts $file ""
    puts $file "\# ZScale prefs"
    global zscale
    global pzscale
    puts $file "global zscale"
    puts $file "global pzscale"
    puts $file "array set pzscale \{ [array get pzscale] \}"
    puts $file {array set zscale [array get pzscale]}

    # Edit prefs
    puts $file ""
    puts $file "\# Edit prefs"
    puts $file "set prefs(ds9,mode) $prefs(ds9,mode)"
    puts $file "set ds9(mode) $prefs(ds9,mode)"

    # Frame prefs
    puts $file ""
    puts $file "\# Frame prefs"
    global tile
    puts $file "global tile"
    puts $file "set prefs(tile,mode) $prefs(tile,mode)"
    puts $file "set tile(mode) $prefs(tile,mode)"
    global blink
    puts $file "global blink"
    puts $file "set prefs(blink,interval) $prefs(blink,interval)"
    puts $file "set blink(interval) $prefs(blink,interval)"

    # Zoom prefs
    puts $file ""
    puts $file "\# Zoom prefs"
    global current
    puts $file "global current"
    global wcs
    puts $file "global wcs"
    puts $file "set prefs(zoom) \{$prefs(zoom)\}"
    puts $file "set current(zoom) \{$prefs(zoom)\}"
    puts $file "set prefs(orient) $prefs(orient)"
    puts $file "set current(orient) $prefs(orient)"
    puts $file "set prefs(rotate) $prefs(rotate)"
    puts $file "set current(rotate) $prefs(rotate)"

    # Color prefs
    puts $file ""
    puts $file "\# Color prefs"
    global colorbar
    puts $file "global colorbar"
    puts $file "set prefs(colorbar,map) $prefs(colorbar,map)"
    puts $file "set colorbar(map) $prefs(colorbar,map)"
    puts $file "set prefs(colorbar,invert) $prefs(colorbar,invert)"
    puts $file "set colorbar(invert) $prefs(colorbar,invert)"

    # Region prefs
    puts $file ""
    puts $file "\# Region prefs"
    global marker
    puts $file "global marker"
    puts $file "set prefs(marker,shape) \{$prefs(marker,shape)\}"
    puts $file "set marker(shape) \{$prefs(marker,shape)\}"
    puts $file "set prefs(marker,color) $prefs(marker,color)"
    puts $file "set marker(color) $prefs(marker,color)"
    puts $file "set prefs(marker,width) $prefs(marker,width)"
    puts $file "set marker(width) $prefs(marker,width)"

    puts $file "set prefs(marker,edit) $prefs(marker,edit)"
    puts $file "set marker(edit) $prefs(marker,edit)"
    puts $file "set prefs(marker,move) $prefs(marker,move)"
    puts $file "set marker(move) $prefs(marker,move)"
    puts $file "set prefs(marker,rotate) $prefs(marker,rotate)"
    puts $file "set marker(rotate) $prefs(marker,rotate)"
    puts $file "set prefs(marker,delete) $prefs(marker,delete)"
    puts $file "set marker(delete) $prefs(marker,delete)"
    puts $file "set prefs(marker,fixed) $prefs(marker,fixed)"
    puts $file "set marker(fixed) $prefs(marker,fixed)"
    puts $file "set prefs(marker,include) $prefs(marker,include)"
    puts $file "set marker(include) $prefs(marker,include)"
    puts $file "set prefs(marker,source) $prefs(marker,source)"
    puts $file "set marker(source) $prefs(marker,source)"
    puts $file "set prefs(marker,font) $prefs(marker,font)"
    puts $file "set marker(font) $prefs(marker,font)"
    puts $file "set prefs(marker,font,size) $prefs(marker,font,size)"
    puts $file "set marker(font,size) $prefs(marker,font,size)"
    puts $file "set prefs(marker,font,style) $prefs(marker,font,style)"
    puts $file "set marker(font,style) $prefs(marker,font,style)"

    puts $file "set prefs(marker,format) $prefs(marker,format)"
    puts $file "set marker(format) $prefs(marker,format)"
    puts $file "set prefs(marker,strip) $prefs(marker,strip)"
    puts $file "set marker(strip) $prefs(marker,strip)"
    puts $file "set prefs(marker,wcs) $prefs(marker,wcs)"
    puts $file "set marker(wcs) $prefs(marker,wcs)"
    puts $file "set prefs(marker,system) $prefs(marker,system)"
    puts $file "set marker(system) $prefs(marker,system)"
    puts $file "set prefs(marker,sky) $prefs(marker,sky)"
    puts $file "set marker(sky) $prefs(marker,sky)"
    puts $file "set prefs(marker,skyformat) $prefs(marker,skyformat)"
    puts $file "set marker(skyformat) $prefs(marker,skyformat)"

    puts $file "set marker(dialog,system) $marker(dialog,system)"
    puts $file "set marker(dialog,sky) $marker(dialog,sky)"
    puts $file "set marker(dialog,skyformat) $marker(dialog,skyformat)"
    puts $file "set marker(dialog,dist,system) $marker(dialog,dist,system)"
    puts $file "set marker(dialog,dist,format) $marker(dialog,dist,format)"

    puts $file "set marker(circle,radius) $marker(circle,radius)"
    puts $file "set marker(ellipse,radius1) $marker(ellipse,radius1)"
    puts $file "set marker(ellipse,radius2) $marker(ellipse,radius2)"
    puts $file "set marker(box,radius1) $marker(box,radius1)"
    puts $file "set marker(box,radius2) $marker(box,radius2)"
    puts $file "set marker(polygon,width) $marker(polygon,width)"
    puts $file "set marker(polygon,height) $marker(polygon,height)"
    puts $file "set marker(projection,thick) $marker(projection,thick)"
    puts $file "set marker(annulus,inner) $marker(annulus,inner)"
    puts $file "set marker(annulus,outer) $marker(annulus,outer)"
    puts $file "set marker(annulus,annuli) $marker(annulus,annuli)"
    puts $file "set marker(ellipseannulus,radius1) $marker(ellipseannulus,radius1)"
    puts $file "set marker(ellipseannulus,radius2) $marker(ellipseannulus,radius2)"
    puts $file "set marker(ellipseannulus,radius3) $marker(ellipseannulus,radius3)"
    puts $file "set marker(ellipseannulus,annuli) $marker(ellipseannulus,annuli)"
    puts $file "set marker(boxannulus,radius1) $marker(boxannulus,radius1)"
    puts $file "set marker(boxannulus,radius2) $marker(boxannulus,radius2)"
    puts $file "set marker(boxannulus,radius3) $marker(boxannulus,radius3)"
    puts $file "set marker(boxannulus,annuli) $marker(boxannulus,annuli)"
    puts $file "set marker(panda,inner) $marker(panda,inner)"
    puts $file "set marker(panda,outer) $marker(panda,outer)"
    puts $file "set marker(panda,annuli) $marker(panda,annuli)"
    puts $file "set marker(panda,ang1) $marker(panda,ang1)"
    puts $file "set marker(panda,ang2) $marker(panda,ang2)"
    puts $file "set marker(panda,angnum) $marker(panda,angnum)"
    puts $file "set marker(epanda,radius1) $marker(epanda,radius1)"
    puts $file "set marker(epanda,radius2) $marker(epanda,radius2)"
    puts $file "set marker(epanda,radius3) $marker(epanda,radius3)"
    puts $file "set marker(epanda,annuli) $marker(epanda,annuli)"
    puts $file "set marker(epanda,ang1) $marker(epanda,ang1)"
    puts $file "set marker(epanda,ang2) $marker(epanda,ang2)"
    puts $file "set marker(epanda,angnum) $marker(epanda,angnum)"
    puts $file "set marker(bpanda,radius1) $marker(bpanda,radius1)"
    puts $file "set marker(bpanda,radius2) $marker(bpanda,radius2)"
    puts $file "set marker(bpanda,radius3) $marker(bpanda,radius3)"
    puts $file "set marker(bpanda,annuli) $marker(bpanda,annuli)"
    puts $file "set marker(bpanda,ang1) $marker(bpanda,ang1)"
    puts $file "set marker(bpanda,ang2) $marker(bpanda,ang2)"
    puts $file "set marker(bpanda,angnum) $marker(bpanda,angnum)"
    puts $file "set marker(point,size) $marker(point,size)"

    global graph
    puts $file "global graph"
    puts $file "set graph(horz,grid) $graph(horz,grid)"
    puts $file "set graph(horz,log) $graph(horz,log)"
    puts $file "set graph(vert,grid) $graph(vert,grid)"
    puts $file "set graph(vert,log) $graph(vert,log)"
    global nres
    puts $file "global nres"
    puts $file "set nres(server) \{$nres(server)\}"
    global cat
    puts $file "global cat"
    puts $file "set cat(server) \{$cat(server)\}"
    puts $file "set cat(sym,shape) \{$cat(sym,shape)\}"
    puts $file "set cat(sym,color) $cat(sym,color)"

    global hv
    puts $file "global hv"
    foreach n {1 2 3 4} {
	puts $file "set hv(archive,menu,$n) \{$hv(archive,menu,$n)\}"
	puts $file "set hv(archive,url,$n) \{$hv(archive,url,$n)\}"
    }
    global vo
    puts $file "global vo"
    puts $file "set vo(server) \{$vo(server)\}"
    puts $file "set vo(hv) $vo(hv)"
    puts $file "set vo(method) $vo(method)"

    puts $file "set ds9(analysis,user)  \{$ds9(analysis,user)\}"
    puts $file "set ds9(analysis,user2) \{$ds9(analysis,user2)\}"
    puts $file "set ds9(analysis,user3) \{$ds9(analysis,user3)\}"
    puts $file "set ds9(analysis,user4) \{$ds9(analysis,user4)\}"

    global analysis
    puts $file "global analysis"
    puts $file "set prefs(analysis,log) $prefs(analysis,log)"
    puts $file "set analysis(log) $prefs(analysis,log)"

    puts $file ""
    puts $file "\# Analysis Plot prefs"
    puts $file "set prefs(ap,grid) $prefs(ap,grid)"
    puts $file "set prefs(ap,grid,log) $prefs(ap,grid,log)"

    puts $file "set prefs(ap,discrete) $prefs(ap,discrete)"
    puts $file "set prefs(ap,discrete,symbol) $prefs(ap,discrete,symbol)"
    puts $file "set prefs(ap,discrete,color) $prefs(ap,discrete,color)"

    puts $file "set prefs(ap,linear) $prefs(ap,linear)"
    puts $file "set prefs(ap,linear,width) $prefs(ap,linear,width)"
    puts $file "set prefs(ap,linear,color) $prefs(ap,linear,color)"
    puts $file "set prefs(ap,linear,dash) $prefs(ap,linear,dash)"

    puts $file "set prefs(ap,step) $prefs(ap,step)"
    puts $file "set prefs(ap,step,width) $prefs(ap,step,width)"
    puts $file "set prefs(ap,step,color) $prefs(ap,step,color)"
    puts $file "set prefs(ap,step,dash) $prefs(ap,step,dash)"

    puts $file "set prefs(ap,quadratic) $prefs(ap,quadratic)"
    puts $file "set prefs(ap,quadratic,width) $prefs(ap,quadratic,width)"
    puts $file "set prefs(ap,quadratic,color) $prefs(ap,quadratic,color)"
    puts $file "set prefs(ap,quadratic,dash) $prefs(ap,quadratic,dash)"

    puts $file "set prefs(ap,error,width) $prefs(ap,error,width)"
    puts $file "set prefs(ap,error,color) $prefs(ap,error,color)"
    puts $file "set prefs(ap,error,style) $prefs(ap,error,style)"

    puts $file "set prefs(ap,titleFont) $prefs(ap,titleFont)"
    puts $file "set prefs(ap,titleSize) $prefs(ap,titleSize)"
    puts $file "set prefs(ap,titleStyle) $prefs(ap,titleStyle)"

    puts $file "set prefs(ap,textlabFont) $prefs(ap,textlabFont)"
    puts $file "set prefs(ap,textlabSize) $prefs(ap,textlabSize)"
    puts $file "set prefs(ap,textlabStyle) $prefs(ap,textlabStyle)"

    puts $file "set prefs(ap,numlabFont) $prefs(ap,numlabFont)"
    puts $file "set prefs(ap,numlabSize) $prefs(ap,numlabSize)"
    puts $file "set prefs(ap,numlabStyle) $prefs(ap,numlabStyle)"

    # Language
    puts $file ""
    puts $file "\# Language"
    puts $file "set ds9(language) $ds9(language)"
    puts $file "set ds9(language,name) $ds9(language,name)"

    # Background Color prefs
    puts $file ""
    puts $file "\# Background Color prefs"
    puts $file "set ds9(bg,color) $ds9(bg,color)"

    # Blank/Inf/NaN Color prefs
    puts $file ""
    puts $file "\# Blank/Inf/NaN Color prefs"
    puts $file "set ds9(nan,color) $ds9(nan,color)"

    # Default Font prefs
    puts $file ""
    puts $file "\# Font prefs"
    puts $file "set ds9(font) $ds9(font)"
    puts $file "set ds9(font,size) $ds9(font,size)"
    puts $file "set ds9(font,style) $ds9(font,style)"
    puts $file "if \{\$ds9\(font\) \!\= \"default\"\} \{"
    puts $file "option add *font \{$ds9(font) $ds9(font,size) $ds9(font,style)\}"
    puts $file "\}"

    # Panner prefs
    puts $file ""
    puts $file "\# Panner prefs"
    global panner
    puts $file "global panner"
    puts $file "set panner(compass,image) $panner(compass,image)"
    puts $file "set panner(compass,wcs) $panner(compass,wcs)"
    puts $file "set panner(compass,wcs,system) $panner(compass,wcs,system)"
    puts $file "set panner(compass,wcs,sky) $panner(compass,wcs,sky)"

    # Magnifier prefs
    puts $file ""
    puts $file "\# Magnifier prefs"
    global magnifier
    puts $file "global magnifier"
    puts $file "set magnifier(region) $magnifier(region)"
    puts $file "set magnifier(cursor) $magnifier(cursor)"
    puts $file "set magnifier(zoom) $magnifier(zoom)"

    # Truecolor Colorbar prefs
    puts $file ""
    puts $file "\# Truecolor Colorbar prefs"
    global colorbar
    puts $file "global colorbar"
    puts $file "set colorbar(area) $colorbar(area)"
    puts $file "set colorbar(mode) $colorbar(mode)"

    # Dialog prefs
    puts $file ""
    puts $file "\# Dialog prefs"
    puts $file "global ds9"
    puts $file "set ds9(dialog) $ds9(dialog)"

    # PanZoom prefs
    puts $file ""
    puts $file "\# PanZoom prefs"
    global panzoom
    puts $file "global panzoom"
    puts $file "set panzoom(mode) $panzoom(mode)"

    # Print Coordinates prefs
    puts $file ""
    puts $file "\# Print Coordinates prefs"
    global coord
    puts $file "global coord"
    puts $file "set coord(value) $coord(value)"
    puts $file "set coord(image) $coord(image)"
    puts $file "set coord(physical) $coord(physical)"
    puts $file "set coord(amplifier) $coord(amplifier)"
    puts $file "set coord(detector) $coord(detector)"
    foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	puts $file "set coord(wcs$l) $coord(wcs$l)"
    }
    puts $file "set coord(sky) $coord(sky)"
    puts $file "set coord(skyformat) $coord(skyformat)"

    # Examine prefs
    puts $file ""
    puts $file "\# Examine prefs"
    global examine
    puts $file "global examine"
    puts $file "set examine(mode) $examine(mode)"
    puts $file "set examine(zoom) $examine(zoom)"

    # Misc prefs
    puts $file ""
    puts $file "\# Misc prefs"
    global ds9
    puts $file "global ds9"
    puts $file "set ds9(tmpdir) \{$ds9(tmpdir)\}"
    puts $file "set ds9(automarker) $ds9(automarker)"
    puts $file "set ds9(xpa) $ds9(xpa)"
    puts $file "set ds9(confirm) $ds9(confirm)"

    # HTTP prefs
    puts $file ""
    puts $file "\# HTTP prefs"
    global http
    puts $file "global http"
    puts $file "set http(proxy) $http(proxy)"
    puts $file "set http(proxy,host) \{$http(proxy,host)\}"
    puts $file "set http(proxy,port) \{$http(proxy,port)\}"
    puts $file "set http(auth) $http(auth)"
    puts $file "set http(auth,user) \{$http(auth,user)\}"
    puts $file "set http(auth,passwd) \{$http(auth,passwd)\}"

    # and close
    close $file
}

# Process Cmds

proc ProcessPrefsCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global prefs
    global ds9
    global wcs

    switch -- [string tolower [lindex $var $i]] {
	clear {ClearPrefs}
	bgcolor {
	    incr i
	    set ds9(bg,color) [lindex $var $i]
	    PrefsBkgrColor
	}
	nancolor {
	    incr i
	    set ds9(nan,color) [lindex $var $i]
	    PrefsNaNColor
	}
    }
}

# Backward Compatibility
proc FixPrefs {} {
    global prefs

    # bin
    global pbin
    if [info exists prefs(bin,factor)] {
	set pbin(function) $prefs(bin,function)
	set pbin(factor) "$prefs(bin,factor)"
	set pbin(depth) $prefs(bin,depth)
	set pbin(buffersize) $prefs(bin,buffersize)
    }

    # contour
    global pscontour
    if [info exists prefs(contour,method)] {
	set pcontour(method) $prefs(contour,method)
	set pcontour(color) $prefs(contour,color)
	set pcontour(width) $prefs(contour,width)
    }

    # ps
    global pps
    if [info exists prefs(ps,dest)] {
	set pps(dest) $prefs(ps,dest)
	set pps(cmd) "$prefs(ps,cmd)"
	set pps(filename) "$prefs(ps,filename)"
	set pps(color) $prefs(ps,color)
	set pps(level) $prefs(ps,level)
	set pps(resolution) $prefs(ps,resolution)
	set pps(orient) $prefs(ps,orient)
	set pps(scale) $prefs(ps,scale)
	set pps(size) $prefs(ps,size)
	set pps(width) $prefs(ps,width)
	set pps(height) $prefs(ps,height)
    }

    # scale
    global pscale
    if [info exists prefs(scale)] {
	set pscale(type) $prefs(scale)
	set pscale(mode) $prefs(scale,mode)
	set pscale(scope) $prefs(scale,scope)
	set pscale(datasec) $prefs(scale,datasec)
    }
    global pminmax
    if [info exists prefs(minmax,mode)] {
	set pminmax(mode) $prefs(minmax,mode)
	set pminmax(sample) $prefs(minmax,sample)
   }

    # smooth
    global psmooth
    if [info exists prefs(smooth,function)] {
	set psmooth(function) $prefs(smooth,function)
	set psmooth(radius) $prefs(smooth,radius)
    }

    # wcs
    global pwcs
    if [info exists prefs(wcs,system)] {
	set pwcs(system) $prefs(wcs,system)
	set pwcs(sky) $prefs(wcs,sky)
	set pwcs(skyformat) $prefs(wcs,skyformat)
	set pwcs(align) $prefs(wcs,align)
	set pwcs(align,system) $prefs(wcs,align,system)
	set pwcs(align,sky) $prefs(wcs,align,sky)
    }

    # view
    global pview
    if [info exists prefs(view,layout)] {
	set pview(layout) $prefs(view,layout)
	set pview(info) $prefs(view,info)
	set pview(view) $prefs(view,panner)
	set pview(magnifier) $prefs(view,magnifier)
	set pview(buttons) $prefs(view,buttons)
	set pview(colorbar) $prefs(view,colorbar)
	set pview(graph,horz) $prefs(view,graph,horz)
	set pview(graph,vert) $prefs(view,graph,vert)
	set pview(info,filename) $prefs(view,info,filename)
	set pview(info,object) $prefs(view,info,object)
	set pview(info,minmax) $prefs(view,info,minmax)
	set pview(info,detector) $prefs(view,info,detector)
	set pview(info,amplifier) $prefs(view,info,amplifier)
	set pview(info,physical) $prefs(view,info,physical)
	set pview(info,image) $prefs(view,info,image)
	set pview(info,frame) $prefs(view,info,frame)
    }
    if [info exists prefs(view,colorbar,numerics)] {
	set pview(color,numerics) $prefs(view,colorbar,numerics)
    }
    if [info exists prefs(view,info,lowhigh)] {
	set pview(info,lowhigh) $prefs(view,info,lowhigh)
    }
    if [info exists prefs(view,info,wcsa)] {
	foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    set pview(info,wcs$l) $prefs(view,info,wcs$l)
	}
    }

    # Other
    global view
    global info
    if [info exists info(view)] {
	set view(info) $info(view)
    }
    global panner
    if [info exists panner(view)] {
	set view(panner) $panner(view)
    }
    global magnifier
    if [info exists magnifier(view)] {
	set view(magnifier) $magnifier(view)
    }
    global colorbar
    if [info exists colorbar(view)] {
	set view(colorbar) $colorbar(view)
    }
    if [info exists info(filename)] {
	set view(info,filename) $info(filename)
    }
    if [info exists info(object)] {
	set view(info,object) $info(object)
    }
    if [info exists info(minmax)] {
	set view(info,minmax) $info(minmax)
	set view(info,lowhigh) $info(minmax)
    }
    if [info exists info(wcs)] {
	set view(info,wcs) $info(wcs)
    }
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if [info exists info(wcs$l)] {
	    set view(info,wcs$l) $info(wcs$l)
	}
    }
    if [info exists info(detector)] {
	set view(info,detector) $info(detector)
    }
    if [info exists info(amplifier)] {
	set view(info,amplifier) $info(amplifier)
    }
    if [info exists info(physical)] {
	set view(info,physical) $info(physical)
    }
    if [info exists info(image)] {
	set view(info,image) $info(image)
    }
    if [info exists info(frame)] {
	set view(info,frame) $info(frame)
    }
}
