#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc PrefsDialog {which} {
    global prefs
    global ds9
    global canvas

    # see if we already have a window visible

    if [winfo exists $prefs(top)] {
	raise $prefs(top)
	return
    }

    set w $prefs(top)
    set title [msgcat::mc {Preferences}]

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PrefsDialogOK
    wm minsize $w 200 100

    set prefs(tabs) [blt::tabset $w.tabs \
			 -tearoff 0 \
			 -relief ridge \
			 -borderwidth 1 \
			 -tiers 4 \
			 -font $canvas(font) \
			 -selectcommand PrefsDialogSize]
    pack $w.tabs -fill both -expand true
    
    $w.tabs insert end \
	general -text [msgcat::mc {General}] \
	\
	view -text [msgcat::mc {View}] \
	frame -text [msgcat::mc {Frame}] \
	bin -text [msgcat::mc {Bin}] \
	zoom -text [msgcat::mc {Zoom}] \
	scale -text [msgcat::mc {Scale}] \
	color -text [msgcat::mc {Color}] \
	region -text [msgcat::mc {Region}] \
	annulus -text [msgcat::mc {Annulus}] \
	panda -text [msgcat::mc {Panda}] \
	wcs -text [msgcat::mc {WCS}] \
	analysis -text [msgcat::mc {Analysis}] \
	\
	buttonbar -text [msgcat::mc {Buttons}] \
	plot -text [msgcat::mc {Plot}] \
	vo -text [msgcat::mc {VO}] \
	http -text [msgcat::mc {HTTP}]

    switch $ds9(wm) {
	x11 {$w.tabs insert end print -text [msgcat::mc {Print}]}
	win32 -
	aqua {$w.tabs insert end print -text [msgcat::mc {Postscript}]}
    }

    PrefsDialogGeneral

    PrefsDialogView
    PrefsDialogFrame
    PrefsDialogBin
    PrefsDialogZoom
    PrefsDialogScale
    PrefsDialogColor
    PrefsDialogRegion
    PrefsDialogAnnulus
    PrefsDialogPanda
    PrefsDialogWCS
    PrefsDialogAnalysis

    PrefsDialogButtonbar
    PrefsDialogPlot
    PrefsDialogVO
    PrefsDialogHTTP
    PrefsDialogPrint

    if {$which != {}} {
	$prefs(tabs) select [$prefs(tabs) index -name $which]
    }

    # we need update idletasks (and not update, cause slower ppcs)
    update idletasks
    PrefsDialogSize
}

proc PrefsDialogButtons {w} {
    frame $w.buttons -relief ridge -bd 1

    button $w.buttons.ok \
	-text [msgcat::mc {OK}] -command PrefsDialogOK
    button $w.buttons.cancel \
	-text [msgcat::mc {Cancel}] -command PrefsDialogCancel
    button $w.buttons.clear \
	-text [msgcat::mc {Clear Preferences}] -command PrefsDialogClear
    pack $w.buttons.clear $w.buttons.cancel \
	$w.buttons.ok -side left -padx 10 -expand true

    pack $w.buttons -side top -fill both -expand true
}

proc PrefsDialogOK {} {
    global prefs

    if {[winfo exists $prefs(top)]} {
	destroy $prefs(top)
    }

    SavePrefs
}

proc PrefsDialogCancel {} {
    global prefs

    if {[winfo exists $prefs(top)]} {
	destroy $prefs(top)
    }
}

proc PrefsDialogClear {} {
    global ds9
    global prefs

    if {$ds9(confirm)} {
	if {[tk_messageBox -type okcancel -message [msgcat::mc {Clear Preferences?}]] != {ok}} {
	    return
	}
    }
    
    if {[winfo exists $prefs(top)]} {
	destroy $prefs(top)
    }

    ClearPrefs
}

proc PrefsDialogSize {} {
    global prefs

    set tab [$prefs(tabs) get select]
    set ww [$prefs(tabs) tab cget $tab -window]
    set hh [winfo reqheight $ww]
    $prefs(tabs) configure -height [expr $hh+20]
}

proc PrefsDialogGeneral {} {
    global prefs

    set w $prefs(tabs)

    frame $w.general
    $prefs(tabs) tab configure general -window $w.general -fill both

    frame $w.general.mode -relief ridge -bd 1
    frame $w.general.examine -relief ridge -bd 1
    frame $w.general.coord -relief ridge -bd 1
    frame $w.general.tmp -relief ridge -bd 1
    frame $w.general.misc -relief ridge -bd 1
    pack $w.general.mode $w.general.examine $w.general.coord $w.general.tmp \
	$w.general.misc -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.general

    # Mode
    set f $w.general.mode.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Mouse Mode}]
    menubutton $f.mode -relief raised \
	-textvariable prefs(ds9,mode) -menu $f.mode.menu

    grid $f.title $f.mode -padx 4 -sticky w

    set m $f.mode.menu
    menu $m
    $m add radiobutton -label [msgcat::mc {None}] \
	-variable prefs(ds9,mode) -value none
    $m add radiobutton -label [msgcat::mc {Pointer}] \
	-variable prefs(ds9,mode) -value pointer
    $m add radiobutton -label [msgcat::mc {Crosshair}] \
	-variable prefs(ds9,mode) -value crosshair
    $m add radiobutton -label [msgcat::mc {Colorbar}] \
	-variable prefs(ds9,mode) -value colorbar
    $m add radiobutton -label [msgcat::mc {Pan}] \
	-variable prefs(ds9,mode) -value pan
    $m add radiobutton -label [msgcat::mc {Zoom}] \
	-variable prefs(ds9,mode) -value zoom
    $m add radiobutton -label [msgcat::mc {Rotate}] \
	-variable prefs(ds9,mode) -value rotate
    $m add radiobutton -label [msgcat::mc {Catalog}] \
	-variable prefs(ds9,mode) -value catalog
    $m add radiobutton -label [msgcat::mc {Examine}] \
	-variable prefs(ds9,mode) -value examine

    # Tmp Dir
    set f $w.general.tmp.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title \
	-text [msgcat::mc {Select Directory to be used for temporary files}]

    frame $f.dir
    entry $f.dir.value -textvariable ds9(tmpdir) -width 60
    button $f.dir.browse -text [msgcat::mc {Browse}] \
	-command "TmpDirOpen ds9(tmpdir)"
    pack $f.dir.value -ipadx 2 -ipady 2
    pack $f.dir.value $f.dir.browse -side left -padx 4 -pady 4

    grid $f.title -padx 4 -sticky w
    grid $f.dir -padx 4 -sticky w

    # Examine
    set f $w.general.examine.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Examine Mode}]
    radiobutton $f.new -text [msgcat::mc {New Frame each Time}] \
	-variable examine(mode) -value new
    radiobutton $f.one -text [msgcat::mc {Examine Frame}] \
	-variable examine(mode) -value one
    radiobutton $f.x1 -text {1x} -variable examine(zoom) -value 1
    radiobutton $f.x2 -text {2x} -variable examine(zoom) -value 2
    radiobutton $f.x4 -text {4x} -variable examine(zoom) -value 4
    radiobutton $f.x8 -text {8x} -variable examine(zoom) -value 8
    radiobutton $f.x16 -text {16x} -variable examine(zoom) -value 16

    grid $f.title -padx 4 -sticky w
    grid $f.new - - $f.one - -padx 4 -sticky w
    grid $f.x1 $f.x2 $f.x4 $f.x8 $f.x16 -padx 4 -sticky w

    # Coord
    set f $w.general.coord.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Print Coordinates}]
    checkbutton $f.value -text [msgcat::mc {Value}] -variable coord(value)
    checkbutton $f.wcs -text [msgcat::mc {WCS}] -variable coord(wcs)
    menubutton $f.mwcs  -relief raised \
	-text [msgcat::mc {Multiple WCS}] -menu $f.mwcs.menu

    checkbutton $f.image -text [msgcat::mc {Image}] \
	-variable coord(image)
    checkbutton $f.physical -text [msgcat::mc {Physical}] \
	-variable coord(physical)
    checkbutton $f.amplifier -text [msgcat::mc {Amplifier}] \
	-variable coord(amplifier)
    checkbutton $f.detector -text [msgcat::mc {Detector}] \
	-variable coord(detector)

    radiobutton $f.fk4 -text "[msgcat::mc {Equatorial}] B1950" \
	-variable coord(sky) -value fk4
    radiobutton $f.fk5 -text "[msgcat::mc {Equatorial}] J2000" \
	-variable coord(sky) -value fk5
    radiobutton $f.icrs -text [msgcat::mc {ICRS}] \
	-variable coord(sky) -value icrs
    radiobutton $f.galactic -text [msgcat::mc {Galactic}] \
	-variable coord(sky) -value galactic
    radiobutton $f.ecliptic -text [msgcat::mc {Ecliptic}] \
	-variable coord(sky) -value ecliptic

    radiobutton $f.degrees -text [msgcat::mc {Degrees}] \
	-variable coord(skyformat) -value degrees
    radiobutton $f.sex -text {Sexagesimal} \
	-variable coord(skyformat) -value sexagesimal

    grid $f.title -padx 4 -sticky w
    grid $f.value -padx 4 -sticky w
    grid $f.wcs $f.mwcs -padx 4 -sticky w
    grid $f.image $f.physical $f.amplifier $f.detector  -padx 4 -sticky w
    grid $f.fk4 $f.fk5 $f.icrs $f.galactic $f.ecliptic -padx 4 -sticky w
    grid $f.degrees $f.sex -padx 4 -sticky w

    set m $f.mwcs.menu
    menu $m
    foreach l {a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	$m add checkbutton -label "[msgcat::mc {WCS}] $l" \
	    -variable "coord(wcs$l)"
    }

    # Misc
    set f $w.general.misc.f
    frame $f
    pack $f -anchor w -fill y -expand true

    checkbutton $f.auto -text [msgcat::mc {Autoload FITS Regions}] \
	-variable ds9(automarker)
    checkbutton $f.xpa -text [msgcat::mc {Initialize XPA}] \
	-variable ds9(xpa)
    checkbutton $f.confirm -text [msgcat::mc {Enable Confirmation Dialogs}] \
	-variable ds9(confirm)

    grid $f.auto -padx 4 -sticky w
    grid $f.xpa -padx 4 -sticky w
    grid $f.confirm -padx 4 -sticky w

}

proc PrefsDialogAnalysis {} {
    global prefs

    set w $prefs(tabs)

    frame $w.analysis
    $prefs(tabs) tab configure analysis -window $w.analysis -fill both

    frame $w.analysis.general -relief ridge -bd 1
    frame $w.analysis.mask -relief ridge -bd 1
    frame $w.analysis.contour -relief ridge -bd 1
    frame $w.analysis.smooth -relief ridge -bd 1
    frame $w.analysis.namesvr -relief ridge -bd 1
    frame $w.analysis.cat -relief ridge -bd 1
    pack $w.analysis.general $w.analysis.mask $w.analysis.contour \
	$w.analysis.smooth $w.analysis.namesvr $w.analysis.cat \
	-side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.analysis

    # General
    set f $w.analysis.general.f
    frame $f
    pack $f -anchor w -fill y -expand true

    checkbutton $f.log -text [msgcat::mc {Analysis Command Log}] \
	-variable prefs(analysis,log)
    label $f.title -text [msgcat::mc {Preload Analysis File}]
    entry $f.pre -textvariable ds9(analysis,user) -width 60
    button $f.browse -text [msgcat::mc {Browse}] \
	-command "AnalysisPrefOpen ds9(analysis,user)"
    entry $f.pre2 -textvariable ds9(analysis,user2) -width 60
    button $f.browse2 -text [msgcat::mc {Browse}] \
	-command "AnalysisPrefOpen ds9(analysis,user2)"
    entry $f.pre3 -textvariable ds9(analysis,user3) -width 60
    button $f.browse3 -text [msgcat::mc {Browse}] \
	-command "AnalysisPrefOpen ds9(analysis,user3)"
    entry $f.pre4 -textvariable ds9(analysis,user4) -width 60
    button $f.browse4 -text [msgcat::mc {Browse}] \
	-command "AnalysisPrefOpen ds9(analysis,user4)"

    grid $f.title -padx 4 -sticky w
    grid $f.pre $f.browse -padx 4 -sticky w
    grid $f.pre2 $f.browse2 -padx 4 -sticky w
    grid $f.pre3 $f.browse3 -padx 4 -sticky w
    grid $f.pre4 $f.browse4 -padx 4 -sticky w
    grid $f.log -padx 4 -sticky w

    # Mask
    set f $w.analysis.mask.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Mask}]
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable pmask(color) -menu $f.color.menu
    checkbutton $f.show -text [msgcat::mc {Show}] -variable pmask(view)

    grid $f.title -padx 4 -sticky w
    grid $f.ctitle $f.color $f.show -padx 4 -sticky w

    DefaultColorMenu $f.color.menu pmask(color) {}

    # Contour
    set f $w.analysis.contour.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Contour}]
    label $f.mtitle -text [msgcat::mc {Method}]
    menubutton $f.method -relief raised \
	-textvariable pcontour(method) -menu $f.method.menu
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable pcontour(color) -menu $f.color.menu
    label $f.wtitle -text [msgcat::mc {Width}]
    menubutton $f.width -relief raised \
	-textvariable pcontour(width) -menu $f.width.menu

    grid $f.title -padx 4 -sticky w
    grid $f.mtitle $f.method $f.ctitle $f.color $f.wtitle $f.width \
	-padx 4 -sticky w

    set m $f.method.menu 
    menu $m
    $m add radiobutton -label [msgcat::mc {Block}] \
	-variable pcontour(method) -value block
    $m add radiobutton -label [msgcat::mc {Smooth}] \
	-variable pcontour(method) -value smooth

    DefaultColorMenu $f.color.menu pcontour(color) {}
    DefaultWidthMenu $f.width.menu pcontour(width) {}

    # Smooth
    set f $w.analysis.smooth.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Smooth Function}]
    menubutton $f.function -relief raised \
	-textvariable psmooth(function) -menu $f.function.menu

    menu $f.function.menu
    $f.function.menu add radiobutton -label [msgcat::mc {Boxcar}] \
	-variable psmooth(function) -value boxcar
    $f.function.menu add radiobutton -label [msgcat::mc {Tophat}] \
	-variable psmooth(function) -value tophat
    $f.function.menu add radiobutton -label [msgcat::mc {Gaussian}] \
	-variable psmooth(function) -value gaussian

    grid $f.title $f.function -padx 4 -sticky w

    # Name Server
    set f $w.analysis.namesvr.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Name Server}]
    menubutton $f.svr -relief raised \
	-textvariable nres(server) -menu $f.svr.menu

    menu $f.svr.menu
    $f.svr.menu add radiobutton -label {NED@SAO} \
	-variable nres(server) -value ned-sao
    $f.svr.menu add radiobutton -label {NED@ESO} \
	-variable nres(server) -value ned-eso
    $f.svr.menu add radiobutton -label {SIMBAD@SAO} \
	-variable nres(server) -value simbad-sao
    $f.svr.menu add radiobutton -label {SIMBAD@ESO} \
	-variable nres(server) -value simbad-eso

    grid $f.title $f.svr -padx 4 -sticky w

    # Catalog
    set f $w.analysis.cat.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title -text [msgcat::mc {Catalogs}]
    label $f.stitle -text [msgcat::mc {Server}]
    menubutton $f.svr -relief raised \
	-textvariable cat(server) -menu $f.svr.menu
    label $f.shtitle -text [msgcat::mc {Shape}]
    menubutton $f.shape -relief raised \
	-textvariable cat(sym,shape) -menu $f.shape.menu
    label $f.ctitle -text [msgcat::mc {Color}]
    menubutton $f.color -relief raised \
	-textvariable cat(sym,color) -menu $f.color.menu

    menu $f.svr.menu
    $f.svr.menu add radiobutton -label {CDS} \
	-variable cat(server) -value cds
    $f.svr.menu add radiobutton -label {SAO} \
	-variable cat(server) -value sao
    $f.svr.menu add radiobutton -label {CADC} \
	-variable cat(server) -value cadc
    $f.svr.menu add radiobutton -label {ADAC} \
	-variable cat(server) -value adac
    $f.svr.menu add radiobutton -label {IUCAA} \
	-variable cat(server) -value iucaa
    $f.svr.menu add radiobutton -label {BEJING} \
	-variable cat(server) -value bejing
    $f.svr.menu add radiobutton -label {CAMBRIDGE UK} \
	-variable cat(server) -value cambridge
    $f.svr.menu add radiobutton -label {UKIRT HAWAII} \
	-variable cat(server) -value ukirt

    menu $f.shape.menu
    $f.shape.menu add radiobutton -label [msgcat::mc {Circle Point}] \
	-variable cat(sym,shape) -value {circle point}
    $f.shape.menu add radiobutton -label [msgcat::mc {Box Point}] \
	-variable cat(sym,shape) -value {box point}
    $f.shape.menu add radiobutton -label [msgcat::mc {Diamond Point}] \
	-variable cat(sym,shape) -value {diamond point}
    $f.shape.menu add radiobutton -label [msgcat::mc {Cross Point}] \
	-variable cat(sym,shape) -value {cross point}
    $f.shape.menu add radiobutton -label [msgcat::mc {X Point}] \
	-variable cat(sym,shape) -value {x point}
    $f.shape.menu add radiobutton -label [msgcat::mc {Arrow Point}] \
	-variable cat(sym,shape) -value {arrow point}
    $f.shape.menu add radiobutton -label [msgcat::mc {BoxCircle Point}] \
	-variable cat(sym,shape) -value {boxcircle point}
    $f.shape.menu add separator
    $f.shape.menu add radiobutton -label [msgcat::mc {Circle}] \
	-variable cat(sym,shape) -value circle
    $f.shape.menu add radiobutton -label [msgcat::mc {Ellipse}] \
	-variable cat(sym,shape) -value ellipse
    $f.shape.menu add radiobutton -label [msgcat::mc {Box}] \
	-variable cat(sym,shape) -value box
    $f.shape.menu add radiobutton -label [msgcat::mc {Text}] \
	-variable cat(sym,shape) -value text

    DefaultColorMenu $f.color.menu cat(sym,color) {}

    grid $f.title -padx 4 -sticky w
    grid $f.stitle $f.svr $f.shtitle $f.shape $f.ctitle $f.color \
	-padx 4 -sticky w
}

proc PrefsDialogHTTP {} {
    global prefs

    set w $prefs(tabs)

    frame $w.http
    $prefs(tabs) tab configure http -window $w.http -fill both

    frame $w.http.a -relief ridge -bd 1
    pack $w.http.a -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.http

    # HTTP
    set f $w.http.a.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.tproxy -text [msgcat::mc {Use Proxy}]
    checkbutton $f.proxy -variable http(proxy)
    label $f.thost -text [msgcat::mc {Proxy Host}]
    entry $f.host -textvariable http(proxy,host) -width 50
    label $f.tport -text [msgcat::mc {Proxy Port}]
    entry $f.port -textvariable http(proxy,port) -width 10
    label $f.tauth -text [msgcat::mc {Use Authentication}]
    checkbutton $f.auth -variable http(auth)
    label $f.tuser -text [msgcat::mc {Username}]
    entry $f.user -textvariable http(auth,user) -width 30
    label $f.tpasswd -text [msgcat::mc {Password}]
    entry $f.passwd -textvariable http(auth,passwd) -show "*" -width 10

    grid $f.tproxy  $f.proxy -padx 4 -sticky w
    grid $f.thost $f.host -padx 4 -sticky w
    grid $f.tport $f.port -padx 4 -sticky w
    grid $f.tauth  $f.auth  -padx 4 -sticky w
    grid $f.tuser $f.user -padx 4 -sticky w
    grid $f.tpasswd $f.passwd -padx 4 -sticky w
}

