#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

# Print procs

proc Print {} {
    global ds9

    RealizeDS9

    switch $ds9(wm) {
	x11 {PSPrint}
	win32 {Win32Print}
	aqua {MacOSXPrint}
    }
}

proc PrintPre {} {
    global ds9

    RealizeDS9

    switch $ds9(wm) {
	x11 {PostScript}
	win32 {Win32PrintPre}
	aqua {MacOSXPrintPre}
    }
}

proc PSPrint {} {
    if {[PSPrintDialog ps]} {
	if [catch {PostScript} printError] {
	    Error "[msgcat::mc {An error has occurred while printing}] $printError"
	}
    }
}

proc PostScript {} {
    global ds9
    global ps
    global graph
    global view
    global canvas

    set options " "

    # Orientation

    switch -- $ps(orient) {
	portrait {append options " -rotate false"}
	landscape {append options " -rotate true"}
    }

    # Page size

    if {$ps(scale) == {scaled}} {
	switch -- $ps(size) {
	    letter {PostScriptPageSize 4.25i 5.5i 7.5i 10.i options}
	    legal {PostScriptPageSize 4.25i 7.i 7.5i 13.i options}
	    tabloid {PostScriptPageSize 5.5i 8.5i 10.i 16.i options}
	    poster {PostScriptPageSize 18.i 24.i 35.i 47.i options}
	    a4 {PostScriptPageSize 105m 148.5m 185m 272m options}
	    other {
		if {$ps(width) != {} && $ps(height) != {}} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    PostScriptPageSize [append x i] [append y i] \
			[append w i] [append h i] options
		}
	    }
	    othermm {
		if {$ps(width) != {} && $ps(height) != {}} {
		    set x [expr double($ps(width))/2.]
		    set y [expr double($ps(height))/2.]
		    set w [expr $ps(width)-1.]
		    set h [expr $ps(height)-1.]
		    PostScriptPageSize [append x m] [append y m] \
			[append w m] [append h m] options
		}
	    }
	}
    }

    # Printer vs File
    set channel {}
    switch -- $ps(dest) {
	"file" {
	    append options " -file \{$ps(filename)\}"
	}
	"printer" {
	    set channel [open "| $ps(cmd)" w]
	    append options " -channel $channel"
	}
    }

    # set color specific postscript options
    switch -- $ds9(visual) {
	pseudocolor {
	    colorbar postscript level $ps(level)
	    colorbar postscript colorspace $ps(color)
	    colorbar postscript resolution $ps(resolution)
	}
	truecolor {
	    colorbar postscript level $ps(level)
	    colorbar postscript colorspace $ps(color)
	    colorbar postscript resolution $ps(resolution)

	    colorbarrgb postscript level $ps(level)
	    colorbarrgb postscript colorspace $ps(color)
	    colorbarrgb postscript resolution $ps(resolution)
	}
    }

    # set frame specific postscript options
    foreach f $ds9(frames) {
	$f postscript level $ps(level)
	$f postscript colorspace $ps(color)
	$f postscript resolution $ps(resolution)
    }

    # graphs
    if {$view(graph,vert)} {
	$ds9(graph,vert) configure -plotbackground white -bg white
    }
    if {$view(graph,horz)} {
	$ds9(graph,horz) configure -plotbackground white -bg white
    }

    # now invoke canvas postscript command
    if [catch {eval $ds9(canvas) postscript $options} rr] {
	Error "[msgcat::mc {A postscript generation error has occurred}] $rr"
    }

    switch -- $ps(dest) {
	"file" {}
	"printer" {
	    if {$channel != {}} {
		close $channel
	    }
	}
    }

    # reset graphs
    if {$view(graph,vert)} {
	$ds9(graph,vert) configure -plotbackground $canvas(bg) -bg $canvas(bg)
    }
    if {$view(graph,horz)} {
	$ds9(graph,horz) configure -plotbackground $canvas(bg) -bg $canvas(bg)
    }
}

proc PostScriptPageSize {x y w h optname} {
    upvar $optname options

    global ds9
    global ps

    set width [winfo width $ds9(canvas)]
    set height [winfo height $ds9(canvas)]
    set wo [string trim $w "icmp"]
    set ho [string trim $h "icmp"]

    switch -- $ps(orient) {
	portrait {
	    if {[expr double($wo)/$width] < [expr double($ho)/$height]} {
		append options " -pagewidth $w"
	    } else {
		append options " -pageheight $h"
	    }
	}
	landscape {
	    if {[expr double($wo)/$width] > [expr double($ho)/$height]} {
		append options " -pageheight $w"
	    } else {
		append options " -pagewidth $h"
	    }
	}
    }

    append options " -pagex $x -pagey $y"
}

# Print Dialog procs

proc PSPrintDialog {which} {
    global ps
    global ed

    set ed(ok) 0
    array set ed [array get ps]

    set w {.print}

    DialogCreate $w [msgcat::mc {Print}]

    frame $w.print -relief ridge -bd 1
    frame $w.options -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1

    switch -- $which {
	ap -
	ps {pack $w.print $w.options $w.buttons -fill x -ipadx 4 -ipady 4}
	txt {pack $w.print $w.buttons -fill x -ipadx 4 -ipady 4}
    }

    # Print
    label $w.print.title1 -text [msgcat::mc {Print To}]
    radiobutton $w.print.printer -text [msgcat::mc {Printer}] \
	-variable ed(dest) -value printer
    radiobutton $w.print.file -text [msgcat::mc {File}] \
	-variable ed(dest) -value file

    label $w.print.title2 -text [msgcat::mc {Print Command}]
    entry $w.print.cmd -textvariable ed(cmd) -width 30

    label $w.print.title3 -text [msgcat::mc {File Name}]
    entry $w.print.name -textvariable ed(filename) -width 30

    button $w.print.browse -text [msgcat::mc {Browse}] \
	-command "PSPrintBrowse ed(filename)"

    grid rowconfigure $w.print 0 -pad 4
    grid rowconfigure $w.print 1 -pad 4
    grid rowconfigure $w.print 2 -pad 4

    grid $w.print.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $w.print.printer -row 0 -column 1 -padx 4 -sticky w
    grid $w.print.file -row 0 -column 2 -padx 4 -sticky w
	    
    grid $w.print.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $w.print.cmd -row 1 -column 1 -columnspan 2 -padx 4 -sticky w

    grid $w.print.title3 -row 2 -column 0 -padx 4 -sticky w
    grid $w.print.name -row 2 -column 1 -columnspan 2 -padx 4 -sticky w

    grid $w.print.browse -row 2 -column 3 -padx 4

    # Options

    switch -- $which {
	txt {}
	ps {
	    label $w.options.title2 -text [msgcat::mc {Print}]
	    radiobutton $w.options.rgb -text [msgcat::mc {RGB Color}] \
		-variable ed(color) -value rgb 
	    radiobutton $w.options.cmyk -text [msgcat::mc {CMYK Color}] \
		-variable ed(color) -value cmyk 
	    radiobutton $w.options.gray -text [msgcat::mc {Grayscale}] \
		-variable ed(color) -value gray 

	    label $w.options.title4 -text [msgcat::mc {Postscript}]
	    radiobutton $w.options.level1 -text "[msgcat::mc {Level}] 1" \
		-variable ed(level) -value 1 -command "PSLevel $w"
	    radiobutton $w.options.level2 -text "[msgcat::mc {Level}] 2" \
		-variable ed(level) -value 2 -command "PSLevel $w"

	    label $w.options.title5 -text [msgcat::mc {Resolution}]

	    frame $w.options.resolution
	    label $w.options.resolution.title -text [msgcat::mc {DPI}]

	    tk_optionMenu $w.options.resolution.button ed(resolution) \
		53 72 75 150 300 600
	    pack $w.options.resolution.button $w.options.resolution.title \
		-side left

	    grid rowconfigure $w.options 0 -pad 4
	    grid rowconfigure $w.options 1 -pad 4
	    grid rowconfigure $w.options 2 -pad 4

	    grid $w.options.title2 $w.options.rgb $w.options.cmyk \
		$w.options.gray -padx 4 -sticky w
	    grid $w.options.title4 $w.options.level2 $w.options.level1 \
		-padx 4 -sticky w
	    grid $w.options.title5 $w.options.resolution -padx 4 -sticky w

	    PSLevel $w
	}
	ap {
	    label $w.options.title2 -text [msgcat::mc {Print}]
	    radiobutton $w.options.color -text [msgcat::mc {Color}] \
		-variable ed(color2) -value color 
	    radiobutton $w.options.gray -text [msgcat::mc {Grayscale}] \
		-variable ed(color2) -value gray 
	    radiobutton $w.options.mono -text [msgcat::mc {Black and White}] \
		-variable ed(color2) -value mono 

	    grid rowconfigure $w.options 0 -pad 4

	    grid $w.options.title2 -row 0 -column 0 -padx 4 -sticky w
	    grid $w.options.color -row 0 -column 1 -padx 4 -sticky w
	    grid $w.options.gray -row 0 -column 2 -padx 4 -sticky w
	    grid $w.options.mono -row 0 -column 3 -padx 4 -sticky w
	}
    }


    # Buttons
    button $w.buttons.ok -text [msgcat::mc {OK}] -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok
    DialogDismiss $w

    if {$ed(ok)} {
	array set ps [array get ed]
    }

    set rr $ed(ok)

    # we want to destroy this window
    destroy $w 
    unset ed

    return $rr
}

proc PSLevel {w} {
    global ed
    switch -- $ed(level) {
	1 {
	    $w.options.cmyk configure -state disabled
	    if {$ed(color)=={cmyk}} {
		set ed(color) rgb
	    }
	}
	2 {$w.options.cmyk configure -state normal}
    }
}

proc PSPrintBrowse {varname} {
    upvar $varname var

    FileLast pssavfbox $var
    set var [SaveFileDialog pssavfbox]
}

# Page Setup Dialog procs

proc PageSetup {} {
    global ds9

    switch $ds9(wm) {
	x11 {PSPageSetup}
	win32 {Win32PageSetup}
	aqua {MacOSXPageSetup}
    }
}

proc PSPageSetup {} {
    global ps
    global ed

    set ed(ok) 0
    array set ed [array get ps]

    set w {.page}

    DialogCreate $w [msgcat::mc {Page Setup}]

    frame $w.options -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.options $w.buttons -fill x -ipadx 4 -ipady 4

    # Options

    label $w.options.title1 -text [msgcat::mc {Orientation}]
    radiobutton $w.options.portrait -text [msgcat::mc {Portrait}] \
	-variable ed(orient) -value portrait
    radiobutton $w.options.landscape -text [msgcat::mc {Landscape}] \
	-variable ed(orient) -value landscape

    label $w.options.title2 -text [msgcat::mc {Page Size}]
    radiobutton $w.options.fixed -text [msgcat::mc {Fixed in Size}] \
	-variable ed(scale) -value fixed -command "PSPageSetupSize $w"
    radiobutton $w.options.scaled -text [msgcat::mc {Scale to fit Page}] \
	-variable ed(scale) -value scaled -command "PSPageSetupSize $w"

    frame $w.options.s -relief ridge -bd 1
    radiobutton $w.options.s.letter -text "[msgcat::mc {Letter}](8.5 x 11 in)"\
	-variable ed(size) -value letter 
    radiobutton $w.options.s.legal -text "[msgcat::mc {Legal}](8.5 x 14 in)"\
	-variable ed(size) -value legal 
    radiobutton $w.options.s.tabloid -text "[msgcat::mc {Tabloid}](11 x 17 in)"\
	-variable ed(size) -value tabloid 
    radiobutton $w.options.s.poster -text "[msgcat::mc {Poster}](36 x 48 in)"\
	-variable ed(size) -value poster 
    radiobutton $w.options.s.a4 -text {A4(210 x 297 mm)} \
	-variable ed(size) -value a4 
    radiobutton $w.options.s.other -text "[msgcat::mc {Other}] (inches)" \
	-variable ed(size) -value other
    radiobutton $w.options.s.othermm -text "[msgcat::mc {Other}] (mm)" \
	-variable ed(size) -value othermm

    frame $w.options.s.wh
    label $w.options.s.wh.title3 -text [msgcat::mc {Width}]
    entry $w.options.s.wh.width -textvariable ed(width) -width 10

    label $w.options.s.wh.title4 -text [msgcat::mc {Height}]
    entry $w.options.s.wh.height -textvariable ed(height) -width 10

    grid rowconfigure $w.options 0 -pad 4
    grid rowconfigure $w.options 1 -pad 4
    grid rowconfigure $w.options 2 -pad 4

    grid $w.options.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.portrait -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.landscape -row 0 -column 2 -padx 4 -sticky w

    grid $w.options.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.scaled -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.fixed -row 1 -column 2 -padx 4 -sticky w

    grid $w.options.s -row 2 -column 1 -padx 4 -sticky w -columnspan 2

    grid rowconfigure $w.options.s 0 -pad 4
    grid rowconfigure $w.options.s 1 -pad 4
    grid rowconfigure $w.options.s 2 -pad 4
    grid rowconfigure $w.options.s 3 -pad 4
    grid rowconfigure $w.options.s 4 -pad 4

    grid $w.options.s.letter -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.legal -row 1 -column 1 -padx 4 -sticky w
    grid $w.options.s.tabloid -row 2 -column 1 -padx 4 -sticky w
    grid $w.options.s.poster -row 3 -column 1 -padx 4 -sticky w
    grid $w.options.s.a4 -row 4 -column 1 -padx 4 -sticky w
    grid $w.options.s.other -row 5 -column 1 -padx 4 -sticky w
    grid $w.options.s.othermm -row 6 -column 1 -padx 4 -sticky w

    grid $w.options.s.wh -row 5 -column 2 -padx 4 -sticky w -rowspan 2

    grid rowconfigure $w.options.s.wh 0 -pad 4
    grid rowconfigure $w.options.s.wh 1 -pad 4

    grid $w.options.s.wh.title3 -row 0 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.width -row 0 -column 1 -padx 4 -sticky w
    grid $w.options.s.wh.title4 -row 1 -column 0 -padx 4 -sticky w
    grid $w.options.s.wh.height -row 1 -column 1 -padx 4 -sticky w

    # Buttons

    button $w.buttons.ok -text [msgcat::mc {OK}] -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    PSPageSetupSize $w
    DialogCenter $w
    DialogWait $w ed(ok) $w.buttons.ok 
    DialogDismiss $w

    if {$ed(ok)} {
	array set ps [array get ed]
    }

    set rr $ed(ok)

    # we want to destroy this window
    destroy $w 
    unset ed

    return $rr
}

proc PSPageSetupSize {w} {
    global ed

    if {$ed(scale) == {scaled}} {
	$w.options.s.letter configure -state normal
	$w.options.s.legal configure -state normal
	$w.options.s.tabloid configure -state normal
	$w.options.s.poster configure -state normal
	$w.options.s.a4 configure -state normal
	$w.options.s.other configure -state normal
	$w.options.s.othermm configure -state normal
    } else {
	$w.options.s.letter configure -state disabled
	$w.options.s.legal configure -state disabled
	$w.options.s.tabloid configure -state disabled
	$w.options.s.poster configure -state disabled
	$w.options.s.a4 configure -state disabled
	$w.options.s.other configure -state disabled
	$w.options.s.othermm configure -state disabled
    }
}

# Prefs

proc PrefsDialogPrint {} {
    global prefs

    set w $prefs(tabs)

    frame $w.print
    $prefs(tabs) tab configure print -window $w.print -fill both

    frame $w.print.print -relief ridge -bd 1
    frame $w.print.options -relief ridge -bd 1
    frame $w.print.pagesetup -relief ridge -bd 1
    pack $w.print.print $w.print.options $w.print.pagesetup \
	-side top -fill both -expand true

    # Print
    set f $w.print.print.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title1 -text [msgcat::mc {Print To}]
    radiobutton $f.printer -text [msgcat::mc {Printer}] \
	-variable pps(dest) -value printer
    radiobutton $f.file -text [msgcat::mc {File}] \
	-variable pps(dest) -value file

    label $f.title2 -text [msgcat::mc {Print Command}]
    entry $f.cmd -textvariable pps(cmd) -width 30

    label $f.title3 -text [msgcat::mc {File Name}]
    entry $f.name -textvariable pps(filename) -width 30

    button $f.browse -text [msgcat::mc {Browse}] \
	-command "PSPrintBrowse pps(filename)"

    grid $f.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $f.printer -row 0 -column 1 -padx 4 -sticky w
    grid $f.file -row 0 -column 2 -padx 4 -sticky w
    grid $f.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $f.cmd -row 1 -column 1 -columnspan 2 -padx 4 -sticky w
    grid $f.title3 -row 2 -column 0 -padx 4 -sticky w
    grid $f.name -row 2 -column 1 -columnspan 2 -padx 4 -sticky w
    grid $f.browse -row 2 -column 3 -padx 4

    # Options
    set f $w.print.options.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title2 -text [msgcat::mc {Print}]
    radiobutton $f.rgb -text [msgcat::mc {RGB Color}] \
	-variable pps(color) -value rgb 
    radiobutton $f.cmyk -text [msgcat::mc {CMYK Color}] \
	-variable pps(color) -value cmyk 
    radiobutton $f.gray -text [msgcat::mc {Grayscale}] \
	-variable pps(color) -value gray 

    label $f.title4 -text [msgcat::mc {Postscript}]
    radiobutton $f.level1 -text "[msgcat::mc {Level}] 1" \
	-variable pps(level) -value 1
    radiobutton $f.level2 -text "[msgcat::mc {Level}] 2" \
	-variable pps(level) -value 2

    label $f.title5 -text [msgcat::mc {Resolution}]

    frame $f.resolution
    label $f.resolution.title -text [msgcat::mc {DPI}]

    tk_optionMenu $f.resolution.button pps(resolution) \
	53 72 75 150 300 600
    pack $f.resolution.button $f.resolution.title \
	-side left

    grid $f.title2 $f.rgb $f.cmyk $f.gray -padx 4 -sticky w
    grid $f.title4 $f.level2 $f.level1 -padx 4 -sticky w
    grid $f.title5 $f.resolution -padx 4 -sticky w

    # Page Setup
    set f $w.print.pagesetup.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.title1 -text [msgcat::mc {Orientation}]
    radiobutton $f.portrait -text [msgcat::mc {Portrait}] \
	-variable pps(orient) -value portrait
    radiobutton $f.landscape -text [msgcat::mc {Landscape}] \
	-variable pps(orient) -value landscape

    label $f.title2 -text [msgcat::mc {Page Size}]
    radiobutton $f.fixed -text [msgcat::mc {Fixed in Size}] \
	-variable pps(scale) -value fixed -command "PSPageSetupSize $w"
    radiobutton $f.scaled -text [msgcat::mc {Scale to fit Page}] \
	-variable pps(scale) -value scaled -command "PSPageSetupSize $w"

    frame $f.s -relief ridge -bd 1
    radiobutton $f.s.letter -text "[msgcat::mc {Letter}](8.5 x 11 in)"\
	-variable pps(size) -value letter 
    radiobutton $f.s.legal -text "[msgcat::mc {Legal}](8.5 x 14 in)"\
	-variable pps(size) -value legal 
    radiobutton $f.s.tabloid -text "[msgcat::mc {Tabloid}](11 x 17 in)"\
	-variable pps(size) -value tabloid 
    radiobutton $f.s.poster -text "[msgcat::mc {Poster}](36 x 48 in)"\
	-variable pps(size) -value poster 
    radiobutton $f.s.a4 -text {A4(210 x 297 mm)} \
	-variable pps(size) -value a4 
    radiobutton $f.s.other -text "[msgcat::mc {Other}] (inches)" \
	-variable pps(size) -value other
    radiobutton $f.s.othermm -text "[msgcat::mc {Other}] (mm)" \
	-variable pps(size) -value othermm

    frame $f.s.wh
    label $f.s.wh.title3 -text [msgcat::mc {Width}]
    entry $f.s.wh.width -textvariable pps(width) -width 10

    label $f.s.wh.title4 -text [msgcat::mc {Height}]
    entry $f.s.wh.height -textvariable pps(height) -width 10

    grid $f.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $f.portrait -row 0 -column 1 -padx 4 -sticky w
    grid $f.landscape -row 0 -column 2 -padx 4 -sticky w

    grid $f.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $f.scaled -row 1 -column 1 -padx 4 -sticky w
    grid $f.fixed -row 1 -column 2 -padx 4 -sticky w

    grid $f.s -row 2 -column 1 -padx 4 -sticky w -columnspan 2

    grid $f.s.letter -row 0 -column 1 -padx 4 -sticky w
    grid $f.s.legal -row 1 -column 1 -padx 4 -sticky w
    grid $f.s.tabloid -row 2 -column 1 -padx 4 -sticky w
    grid $f.s.poster -row 3 -column 1 -padx 4 -sticky w
    grid $f.s.a4 -row 4 -column 1 -padx 4 -sticky w
    grid $f.s.other -row 5 -column 1 -padx 4 -sticky w
    grid $f.s.othermm -row 6 -column 1 -padx 4 -sticky w

    grid $f.s.wh -row 5 -column 2 -padx 4 -sticky w -rowspan 2

    grid $f.s.wh.title3 -row 0 -column 0 -padx 4 -sticky w
    grid $f.s.wh.width -row 0 -column 1 -padx 4 -sticky w
    grid $f.s.wh.title4 -row 1 -column 0 -padx 4 -sticky w
    grid $f.s.wh.height -row 1 -column 1 -padx 4 -sticky w
}

# Process Cmds

proc ProcessPrintCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ps
    global current

    switch -- [string tolower [lindex $var $i]] {
	destination {incr i; set ps(dest) [lindex $var $i]}
	command {incr i; set ps(cmd) [lindex $var $i]}
	filename {incr i; set ps(filename) [lindex $var $i]}
	palette {incr i; set ps(color) [lindex $var $i]}
	level {incr i; set ps(level) [lindex $var $i]}
	interpolate {incr i}
	resolution {incr i; set ps(resolution) [lindex $var $i]}

	{} {PrintPre}
	default {incr i -1; PrintPre}
    }
}

proc ProcessPageSetupCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ps

    switch -- [string tolower [lindex $var $i]] {
	orientation {incr i; set ps(orient) [string tolower [lindex $var $i]]}
	pagescale {incr i; set ps(scale) [string tolower [lindex $var $i]]}
	pagesize {incr i; set ps(size) [string tolower [lindex $var $i]] }
    }
}
