#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc UpdateScale {} {
    global debug
    if {$debug(tcl,update)} {
	puts "UpdateScale"
    }

    UpdateColormapLevel
    UpdateScaleMenu
    UpdateScaleDialog
    UpdateGraphYAxis
    UpdateInfoBoxCurrent
    UnsetWatchCursor
}

proc ChangeDATASEC {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) datasec $scale(datasec)"
	UpdateScale
    }
}

proc ChangeScale {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) colorscale $scale(type)"
	UpdateScale
    }
}

proc ChangeScaleMode {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) clip mode $scale(mode)"
	UpdateScale
    }
}

proc ChangeScaleLimit {} {
    global current
    global scale

    if {$current(frame) != {}} {
	set scale(mode) user
	$current(frame) clip user $scale(min) $scale(max)
	$current(frame) clip mode $scale(mode)
	UpdateScale
    }
}

proc ChangeScaleScope {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) clip scope $scale(scope)"
	UpdateScale
    }
}

proc PreserveScale {} {
    global current
    global scale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) \
	    "$current(frame) clip preserve $scale(preserve)"
	UpdateScale
    }
}

proc ChangeMinMax {} {
    global current
    global minmax
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) clip minmax mode $minmax(mode)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip minmax sample $minmax(sample)"
	UpdateScale
    }
}

proc ChangeZScale {} {
    global current
    global zscale
    global rgb

    if {$current(frame) != {}} {
	SetWatchCursor
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale contrast $zscale(contrast)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale sample $zscale(sample)"
	RGBEvalLock rgb(lock,scale) "$current(frame) clip zscale line $zscale(line)"
	UpdateScale
    }
}

proc ScaleDialog {} {
    global scale
    global scaledlg
    global current
    global graph
    global ds9
    global minmax
    global canvas

    # see if we already have a window visible

    if [winfo exists $scale(top)] {
	raise $scale(top)
	return
    }

    set w $scale(top)
    set title [msgcat::mc {Scale Parameters}]

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW ScaleDestroyDialog
    wm minsize $w 425 200

    $w configure -menu $scale(mb)

    menu $scale(mb)
    AppleMenu scale(mb)

    $scale(mb) add cascade -label [msgcat::mc {File}] -menu $scale(mb).file
    $scale(mb) add cascade -label [msgcat::mc {Scale}] -menu $scale(mb).scale
    $scale(mb) add cascade -label [msgcat::mc {Limits}] -menu $scale(mb).limit
    $scale(mb) add cascade -label [msgcat::mc {Scope}] -menu $scale(mb).scope
    $scale(mb) add cascade -label [msgcat::mc {Min Max}] \
	-menu $scale(mb).minmax
    $scale(mb) add cascade -label [msgcat::mc {Parameters}] \
	-menu $scale(mb).param
    $scale(mb) add cascade -label [msgcat::mc {Graph}] -menu $scale(mb).graph

    menu $scale(mb).file
    $scale(mb).file add command -label [msgcat::mc {Apply}] \
	-command ScaleApplyDialog
    $scale(mb).file add separator
    $scale(mb).file add command -label [msgcat::mc {Close}] \
	-command ScaleDestroyDialog

    menu $scale(mb).scale
    $scale(mb).scale add radiobutton -label [msgcat::mc {Linear}] \
	-variable scale(type) -command ChangeScale -value linear
    $scale(mb).scale add radiobutton -label [msgcat::mc {Log}] \
	-variable scale(type) -command ChangeScale -value log
    $scale(mb).scale add radiobutton -label [msgcat::mc {Power}] \
	-variable scale(type) -command ChangeScale -value pow
    $scale(mb).scale add radiobutton -label [msgcat::mc {Square Root}] \
	-variable scale(type) -command ChangeScale -value sqrt
    $scale(mb).scale add radiobutton -label [msgcat::mc {Squared}] \
	-variable scale(type) -command ChangeScale -value squared
    $scale(mb).scale add radiobutton \
	-label [msgcat::mc {Histogram Equalization}] \
	-variable scale(type) -command ChangeScale -value histequ

    menu $scale(mb).limit
    $scale(mb).limit add radiobutton -label [msgcat::mc {Min Max}] \
	-variable scale(mode) -command ChangeScaleMode -value minmax
    $scale(mb).limit add separator
    $scale(mb).limit add radiobutton -label {99.5%} \
	-variable scale(mode) -command ChangeScaleMode -value 99.5
    $scale(mb).limit add radiobutton -label {99%} \
	-variable scale(mode) -command ChangeScaleMode -value 99
    $scale(mb).limit add radiobutton -label  {98%} \
	-variable scale(mode) -command ChangeScaleMode -value 98
    $scale(mb).limit add radiobutton -label  {97%} \
	-variable scale(mode) -command ChangeScaleMode -value 97
    $scale(mb).limit add radiobutton -label  {96%} \
	-variable scale(mode) -command ChangeScaleMode -value 96
    $scale(mb).limit add radiobutton -label {95%} \
	-variable scale(mode) -command ChangeScaleMode -value 95
    $scale(mb).limit add radiobutton -label {92.5%} \
	-variable scale(mode) -command ChangeScaleMode -value 92.5
    $scale(mb).limit add radiobutton -label {90%} \
	-variable scale(mode) -command ChangeScaleMode -value 90
    $scale(mb).limit add separator
    $scale(mb).limit add radiobutton -label {ZScale} \
	-variable scale(mode) -command ChangeScaleMode -value zscale
    $scale(mb).limit add radiobutton -label {ZMax} \
	-variable scale(mode) -command ChangeScaleMode -value zmax
    $scale(mb).limit add radiobutton -label [msgcat::mc {User}] \
	-variable scale(mode) -command ChangeScaleMode -value user

    menu $scale(mb).scope 
    $scale(mb).scope add radiobutton -label [msgcat::mc {Global}] \
	-variable scale(scope) -command ChangeScaleScope -value global
    $scale(mb).scope add radiobutton -label [msgcat::mc {Local}] \
	-variable scale(scope) -command ChangeScaleScope -value local

    menu $scale(mb).minmax 
    $scale(mb).minmax add radiobutton -label [msgcat::mc {Automatic}] \
	-variable minmax(mode) -value auto -command ChangeMinMax
    $scale(mb).minmax add radiobutton -label [msgcat::mc {Scan}] \
	-variable minmax(mode) -value scan -command ChangeMinMax
    $scale(mb).minmax add radiobutton -label [msgcat::mc {Sample}] \
	-variable minmax(mode) -value sample -command ChangeMinMax
    $scale(mb).minmax add radiobutton -label {DATAMIN DATAMAX} \
	-variable minmax(mode) -value datamin -command ChangeMinMax
    $scale(mb).minmax add radiobutton -label {IRAF-MIN IRAF-MAX} \
	-variable minmax(mode) -value irafmin -command ChangeMinMax
    $scale(mb).minmax add separator
    $scale(mb).minmax add command \
	-label "[msgcat::mc {Sample Parameters}]..." -command MinMaxDialog

    menu $scale(mb).param 
    $scale(mb).param add checkbutton -label "[msgcat::mc {Use}] DATASEC" \
	-variable scale(datasec) -command ChangeDATASEC
    $scale(mb).param add separator
    $scale(mb).param add command -label {ZScale...} -command ZScaleDialog

    menu $scale(mb).graph 
    $scale(mb).graph add radiobutton -label [msgcat::mc {Linear}] \
	-value linear -variable scale(yaxis) -command ScaleYAxisDialog
    $scale(mb).graph add radiobutton -label [msgcat::mc {Log}] \
	-value log -variable scale(yaxis) -command ScaleYAxisDialog
    $scale(mb).graph add separator
    $scale(mb).graph add radiobutton -label [msgcat::mc {Full Range}] \
	-value full -variable scale(xaxis) -command ScaleXAxisDialog
    $scale(mb).graph add radiobutton -label [msgcat::mc {Current Range}] \
	-value current -variable scale(xaxis) -command ScaleXAxisDialog

    frame $w.top -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.top $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    # graph
    
    set bgcolor [$w.top cget -background]
    set scaledlg(hist) [blt::barchart $w.top.chart \
			    -width 400 -height 200 \
			    -title [msgcat::mc {Pixel Distribution}] \
			    -plotrelief groove \
			    -plotbackground $bgcolor \
			    -plotpadx 0 -plotpady 0]

    set fgcolor [$scaledlg(hist) cget -foreground]
    $scaledlg(hist) legend configure -hide yes
    $scaledlg(hist) element create bar1 -hide yes \
	-foreground $fgcolor -background $fgcolor -relief flat
    $scaledlg(hist) xaxis configure -title {} -hide yes \
	-color $fgcolor -tickfont $canvas(font) -ticklength 4 
    $scaledlg(hist) yaxis configure -title {} -hide yes

    pack $scaledlg(hist)

    # Cut Lines

    $scaledlg(hist) marker bind min <B1-Motion> \
	"ScaleMotionDialog %x %y scaledlg(min)"
    $scaledlg(hist) marker bind max <B1-Motion> \
	"ScaleMotionDialog %x %y scaledlg(max)"
    $scaledlg(hist) marker bind up <ButtonRelease-1> ScaleReleaseDialog

    set scaledlg(histmin) [$scaledlg(hist) marker create line -element bar1 \
			    -outline red -bindtags [list min up]]
    set scaledlg(histmax) [$scaledlg(hist) marker create line -element bar1 \
			    -outline green -bindtags [list max up]]

    # Cut Levels

    frame $w.top.low
    frame $w.top.high
    pack $w.top.low $w.top.high -side left -padx 10

    label $w.top.low.title -text [msgcat::mc {Low}]
    entry $w.top.low.value -textvariable scaledlg(min) -width 9
    pack $w.top.low.title $w.top.low.value -side left -padx 4

    label $w.top.high.title -text [msgcat::mc {High}]
    entry $w.top.high.value -textvariable scaledlg(max) -width 9
    pack $w.top.high.title $w.top.high.value -side left -padx 4

    # Buttons

    button $w.buttons.apply -text [msgcat::mc {Apply}] \
	-command ScaleApplyDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command ScaleDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    UpdateScaleDialog
}

proc ScaleApplyDialog {} {
    global scale
    global scaledlg
    global current

    if {$current(frame) != {} && 
	$scaledlg(min) != {} && 
	$scaledlg(max) != {}} {

	SetWatchCursor

	$scaledlg(hist) marker configure $scaledlg(histmin) \
	    -coords "$scaledlg(min) -Inf $scaledlg(min) Inf"
	$scaledlg(hist) marker configure $scaledlg(histmax) \
	    -coords "$scaledlg(max) -Inf $scaledlg(max) Inf"

	set scale(min) $scaledlg(min)
	set scale(max) $scaledlg(max)

	set scale(mode) user
	$current(frame) clip user $scale(min) $scale(max)
	$current(frame) clip mode $scale(mode)

	UpdateScale
    }
}

proc ScaleDestroyDialog {} {
    global scale
    global scaledlg

    if {[winfo exists $scale(top)]} {
	destroy $scale(top)
	destroy $scale(mb)
    }

    if {[info exists scaledlg]} {
	unset scaledlg
    }
}

proc ScaleMotionDialog {x y varname} {
    upvar $varname var
    global scale
    global scaledlg

    set var [lindex [$scaledlg(hist) invtransform $x $y] 0]
    if {$scaledlg(min) < $scaledlg(minmin)} {
	set scaledlg(min) $scaledlg(minmin)
    }
    if {$scaledlg(max) > $scaledlg(minmax)} {
	set scaledlg(max) $scaledlg(minmax)
    }

    if {$scaledlg(min) > $scaledlg(max)} {
	if {$varname == "scaledlg(min)"} {
	    set var $scaledlg(max)
	} else {
	    set var $scaledlg(min)
	}
    }

    $scaledlg(hist) marker configure $scaledlg(histmin) \
	-coords "$scaledlg(min) -Inf $scaledlg(min) Inf"
    $scaledlg(hist) marker configure $scaledlg(histmax) \
	-coords "$scaledlg(max) -Inf $scaledlg(max) Inf"
}

proc ScaleReleaseDialog {} {
    global scale
    global scaledlg
    global current

    SetWatchCursor

    set scale(min) $scaledlg(min)
    set scale(max) $scaledlg(max)

    set scale(mode) user
    $current(frame) clip user $scale(min) $scale(max)
    $current(frame) clip mode $scale(mode)

    UpdateScale

    ScaleXAxisDialog
}

proc UpdateScaleDialog {} {
    global current
    global scale
    global scaledlg
    global histX histY

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateScaleDialog"
    }

    if {[winfo exists $scale(top)]} {
	if {$current(frame) != {}} {
	    set limits [$current(frame) get clip]
	    set scaledlg(min) [lindex $limits 0]
	    set scaledlg(max) [lindex $limits 1]

	    if {[$current(frame) has fits] & ![$current(frame) has iis]} {

		set limits [$current(frame) get minmax]
		set scaledlg(minmin) [lindex $limits 0]
		set scaledlg(minmax) [lindex $limits 1]

		set width [$current(frame) get histogram histX histY]

		# add an offset for log scaling if y min is zero
		
		if {$histY(min) == 0} {
		    histY set [histY + .001]
		}

		set stepsize [format "%.3f" [expr ($histX(max)-$histX(min))/5]]

		$scaledlg(hist) element configure bar1 -xdata histX \
		    -ydata histY -hide no -barwidth $width
		$scaledlg(hist) xaxis configure -hide no -stepsize $stepsize \
		    -min [expr $histX(min)-$width] \
		    -max [expr $histX(max)+$width]
		$scaledlg(hist) yaxis configure -min 1

		$scaledlg(hist) marker configure $scaledlg(histmin) \
		    -coords "$scaledlg(min) -Inf $scaledlg(min) Inf"
		$scaledlg(hist) marker configure $scaledlg(histmax) \
		    -coords "$scaledlg(max) -Inf $scaledlg(max) Inf"

		if {[$current(frame) has fits mosaic] || \
			[$current(frame) has fits cube]} {
		    $scale(mb) entryconfig [msgcat::mc {Scope}] -state normal
		} else {
		    $scale(mb) entryconfig [msgcat::mc {Scope}] -state disabled
		}

		if [$current(frame) has datamin] {
		    $scale(mb).minmax \
			entryconfig {DATAMIN DATAMAX} -state normal
		} else {
		    $scale(mb).minmax \
			entryconfig {DATAMIN DATAMAX} -state disabled
		}
		if [$current(frame) has irafmin] {
		    $scale(mb).minmax \
			entryconfig {IRAF-MIN IRAF-MAX} -state normal
		} else {
		    $scale(mb).minmax \
			entryconfig {IRAF-MIN IRAF-MAX} -state disabled
		}

		if [$current(frame) has datasec] {
		    $scale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" \
			-state normal
		} else {
		    $scale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" \
			-state disabled
		}

		ScaleYAxisDialog
		ScaleXAxisDialog

		return
	    }

	    $scaledlg(hist) element configure bar1 -hide yes
	    $scaledlg(hist) xaxis configure -hide yes

	    $scale(mb) entryconfig [msgcat::mc {Scope}] -state normal
	    $scale(mb).minmax entryconfig {DATAMIN DATAMAX} -state normal
	    $scale(mb).minmax entryconfig {IRAF-MIN IRAF-MAX} -state normal

	    return
	}

	set scaledlg(min) {}
	set scaledlg(max) {}
	$scaledlg(hist) element configure bar1 -hide yes
	$scaledlg(hist) xaxis configure -hide yes
	$scale(mb) entryconfig [msgcat::mc {Scope}] -state normal
	$scale(mb).minmax entryconfig {DATAMIN DATAMAX} -state normal
	$scale(mb).minmax entryconfig {IRAF-MIN IRAF-MAX} -state normal
	$scale(mb).param entryconfig "[msgcat::mc {Use}] DATASEC" -state normal
    }
}

proc ScaleYAxisDialog {} {
    global scale
    global scaledlg

    switch -- $scale(yaxis) {
	linear {$scaledlg(hist) yaxis configure -logscale 0}
	log {$scaledlg(hist) yaxis configure -logscale 1 -min 1}
    }
}

proc ScaleXAxisDialog {} {
    global scale
    global scaledlg

    switch -- $scale(xaxis) {
	full {$scaledlg(hist) xaxis configure -min {} -max {}}
	current {
	    set diff [expr $scaledlg(max)-$scaledlg(min)]
	    set per .25
	    set a [expr $scaledlg(min)-($diff*$per)]
	    if {$a < $scaledlg(minmin)} {
		set a {}
	    }
	    set b [expr $scaledlg(max)+($diff*$per)]
	    if {$b > $scaledlg(minmax)} {
		set b {}
	    }
	    $scaledlg(hist) xaxis configure -min $a -max $b
	}
    }
}

proc MinMaxDialog {} {
    global minmax

    if {[MinMaxTemplate minmax]} {
	ChangeMinMax
    }
}

proc MinMaxTemplate {varname} {
    upvar $varname var
    global $varname
    global ed

    set w {.sample}
    set length 300

    set ed(ok) 0
    array set ed [array get ${varname}]

    DialogCreate $w [msgcat::mc {Sample Parameters}]

    frame $w.sample -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.sample $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.sample.incr -from 0 -to 1000 -length $length \
	-variable ed(sample) -orient horizontal \
	-label [msgcat::mc {Sample Increment}] \
	-tickinterval 200 -showvalue true
    pack $w.sample.incr

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	if {$ed(sample) == 0} {
	    set ed(sample) 1
	}
	array set ${varname} [array get ed]
    }

    set rr $ed(ok)

    # we want to destroy this window
    destroy $w 
    unset ed

    return $rr
}

proc ZScaleDialog {} {
    global zscale

    if {[ZScaleTemplate zscale]} {
	ChangeZScale
    }
}

proc ZScaleTemplate {varname} {
    upvar $varname var
    global $varname
    global ed

    set w {.zscale}
    set length 300

    set ed(ok) 0
    array set ed [array get ${varname}]

    DialogCreate $w "ZScale [msgcat::mc {Parameters}]"

    frame $w.zscale -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.zscale $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.zscale.contrast -from 0 -to 1 -length $length \
	-variable ed(contrast) -orient horizontal \
	-label [msgcat::mc {Contrast}] \
	-tickinterval .2 -showvalue true -resolution .01
    scale $w.zscale.size -from 0 -to 1000 -length $length \
	-variable ed(sample) -orient horizontal \
	-label [msgcat::mc {Number of Samples}] \
	-tickinterval 200 -showvalue true -resolution 10
    scale $w.zscale.line -from 0 -to 500 -length $length \
	-variable ed(line) -orient horizontal \
	-label [msgcat::mc {Samples per Line}] \
	-tickinterval 100 -showvalue true -resolution 5
    pack $w.zscale.contrast $w.zscale.size $w.zscale.line -side top

    button $w.buttons.ok -text [msgcat::mc {OK}] \
	-default active -command {set ed(ok) 1}
    button $w.buttons.cancel -text [msgcat::mc {Cancel}] \
	-command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w ed(ok)
    DialogDismiss $w

    if {$ed(ok)} {
	if {$ed(line) == 0} {
	    set ed(line) 1
	}
	array set ${varname} [array get ed]
    }

    set rr $ed(ok)

    # we want to destroy this window
    destroy $w 
    unset ed

    return $rr
}

proc MatchScales {} {
    global current

    if {[$current(frame) get type] == {rgb}} {
	MatchScalesRGB
    } else {
	MatchScalesBase
    }
}

proc MatchScalesRGB {} {
    global ds9
    global current

    set ch [$current(frame) get rgb channel]
    foreach c {red green blue} {
	$current(frame) rgb channel $c

	set type [$current(frame) get colorscale]
	set mode [$current(frame) get clip mode]
	set scope [$current(frame) get clip scope]
	set mmmode [$current(frame) get clip minmax mode]
	set mmsample [$current(frame) get clip minmax sample]
	set zscontrast [$current(frame) get clip zscale contrast]
	set zssample [$current(frame) get clip zscale sample]
	set zsline [$current(frame) get clip zscale line]

	set limits [$current(frame) get clip]
	set min [lindex $limits 0]
	set max [lindex $limits 1]

	foreach f $ds9(frames) {
	    if {$f != $current(frame) && [$f get type] == {rgb}} {
		set chh [$f get rgb channel]
		$f rgb channel $c

		$f colorscale $type
		$f clip mode $mode
		$f clip scope $scope
		$f clip minmax mode $mmmode
		$f clip minmax sample $mmsample
		$f clip zscale contrast $zscontrast
		$f clip zscale sample $zssample
		$f clip zscale line $zsline
		$f clip user $min $max

		$f rgb channel $chh
	    }
	}
    }

    $current(frame) rgb channel $ch
}

proc MatchScalesBase {} {
    global ds9
    global current

    set type [$current(frame) get colorscale]
    set mode [$current(frame) get clip mode]
    set scope [$current(frame) get clip scope]
    set mmmode [$current(frame) get clip minmax mode]
    set mmsample [$current(frame) get clip minmax sample]
    set zscontrast [$current(frame) get clip zscale contrast]
    set zssample [$current(frame) get clip zscale sample]
    set zsline [$current(frame) get clip zscale line]

    set limits [$current(frame) get clip]
    set min [lindex $limits 0]
    set max [lindex $limits 1]

    foreach f $ds9(frames) {
	if {$f != $current(frame) && [$f get type] == "base"} {
	    $f colorscale $type
	    $f clip mode $mode
	    $f clip scope $scope
	    $f clip minmax mode $mmmode
	    $f clip minmax sample $mmsample
	    $f clip zscale contrast $zscontrast
	    $f clip zscale sample $zssample
	    $f clip zscale line $zsline
	    $f clip user $min $max
	}
    }
}

# Process Cmds

proc ProcessScaleCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global scale

    switch -- [string tolower [lindex $var $i]] {
	linear -
	log -
	pow -
	sqrt -
	squared -
	histequ {
	    set scale(type) [string tolower [lindex $var $i]]
	    ChangeScale
	}
	datasec {
	    incr i
	    set scale(datasec) [FromYesNo [lindex $var $i]]
	    ChangeDATASEC
	}
	limits {
	    incr i
	    set scale(min) [lindex $var $i]
	    incr i
	    set scale(max) [lindex $var $i]
	    ChangeScaleLimit
	}
	minmax -
	zscale -
	zmax -
	user {
	    set scale(mode) [string tolower [lindex $var $i]]
	    ChangeScaleMode
	}
	mode {
	    incr i
	    set scale(mode) [string tolower [lindex $var $i]]
	    ChangeScaleMode
	}
	local -
	global {
	    set scale(scope) [string tolower [lindex $var $i]]
	    ChangeScaleScope
	}
	scope {
	    incr i
	    set scale(scope) [string tolower [lindex $var $i]]
	    ChangeScaleScope
	}
    }
}

proc ProcessMinMaxCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global minmax
    global scale

    switch -- [string tolower [lindex $var $i]] {
	scan -
	sample -
	datamin -
	irafmin {
	    set minmax(mode) [string tolower [lindex $var $i]]
	    ChangeMinMax
	}
	mode {
	    incr i
	    set minmax(mode) [string tolower [lindex $var $i]]
	    ChangeMinMax
	}
	interval {
	    incr i
	    set minmax(sample) [lindex $var $i]
	    ChangeMinMax
	}
	default {
	    # for backward compatibility
	    set scale(mode) minmax
	    ChangeScaleMode
	    incr i -1
	}
    }
}

proc ProcessZScaleCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global zscale
    global scale

    switch -- [string tolower [lindex $var $i]] {
	contrast {
	    incr i
	    set zscale(contrast) [lindex $var $i]
	    ChangeZScale
	}
	sample {
	    incr i
	    set zscale(sample) [lindex $var $i]
	    ChangeZScale
	}
	line {
	    incr i
	    set zscale(line) [lindex $var $i]
	    ChangeZScale
	}
	default {
	    # for backward compatibility
	    set scale(mode) zscale
	    ChangeScaleMode
	    incr i -1
	}
    }
}
