#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc SmoothUpdate {} {
    global smooth
    global current
    global rgb

    SetWatchCursor
    if {$current(frame) != {}} {
	if {$smooth(view) && [$current(frame) has fits]} {
	    RGBEvalLock rgb(lock,smooth) "$current(frame) smooth $smooth(function) $smooth(radius)"
	} else {
	    RGBEvalLock rgb(lock,smooth) "$current(frame) smooth delete"
	}
    }

    UpdateContourMenu
    UpdateContourDialog
    UpdateColormapLevel
    UpdateGraphXAxis
    UpdateScaleDialog
    UpdateInfoBoxCurrent
    UnsetWatchCursor
}

proc SmoothDialog {} {
    global ds9
    global smooth

    # see if we already have a window visible

    if [winfo exists $smooth(top)] {
	raise $smooth(top)
	return
    }

    set w $smooth(top)
    set title [msgcat::mc {Smooth Parameters}]

    # create the window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW SmoothDestroyDialog
    wm minsize $w 375 150

    $w configure -menu $smooth(mb)

    menu $smooth(mb)
    AppleMenu smooth(mb)

    $smooth(mb) add cascade -label [msgcat::mc {File}] -menu $smooth(mb).file
    $smooth(mb) add cascade -label [msgcat::mc {Edit}] -menu $smooth(mb).edit

    menu $smooth(mb).file
    $smooth(mb).file add command -label [msgcat::mc {Apply}] \
	-command SmoothApplyDialog
    $smooth(mb).file add command -label [msgcat::mc {Clear}] \
	-command SmoothOffDialog
    $smooth(mb).file add separator
    $smooth(mb).file add command -label [msgcat::mc {Close}] \
	-command SmoothDestroyDialog

    menu $smooth(mb).edit
    $smooth(mb).edit add command -label [msgcat::mc {Cut}] \
	-command SmoothCutDialog -accelerator "${ds9(ctrl)}X"
    $smooth(mb).edit add command -label [msgcat::mc {Copy}] \
	-command SmoothCopyDialog -accelerator "${ds9(ctrl)}C"
    $smooth(mb).edit add command -label [msgcat::mc {Paste}] \
	-command SmoothPasteDialog -accelerator "${ds9(ctrl)}V"

    frame $w.params
    frame $w.buttons -relief ridge -bd 1
    pack $w.params $w.buttons -fill x -ipadx 4 -ipady 4

    # Params
    frame $w.params.a -relief ridge -bd 1
    frame $w.params.b -relief ridge -bd 1
    pack $w.params.a $w.params.b -side top \
	-fill both -expand true -ipadx 4

    frame $w.params.a.a
    pack $w.params.a.a -anchor w -ipadx 4 -ipady 4

    label $w.params.a.a.title -text [msgcat::mc {Function}]
    radiobutton $w.params.a.a.boxcar -text [msgcat::mc {Boxcar}] \
	-variable smooth(function) -value boxcar 
    radiobutton $w.params.a.a.tophat -text [msgcat::mc {Tophat}] \
	-variable smooth(function) -value tophat 
    radiobutton $w.params.a.a.gaussian -text [msgcat::mc {Gaussian}] \
	-variable smooth(function) -value gaussian 
    grid $w.params.a.a.title $w.params.a.a.boxcar -sticky w
    grid x $w.params.a.a.tophat -sticky w
    grid x $w.params.a.a.gaussian -sticky w
    
    scale $w.params.b.slider  -from 1 -to 20 -length 300 \
	-label [msgcat::mc {Kernel Radius}] \
	-variable smooth(radius) \
	-orient horizontal \
	-tickinterval 2 -showvalue false -resolution 1
    entry $w.params.b.value -textvariable smooth(radius) -width 7 
    pack $w.params.b.slider $w.params.b.value -side left -padx 4

    # buttons
    button $w.buttons.apply -text [msgcat::mc {Apply}] -command SmoothApplyDialog
    button $w.buttons.clear -text [msgcat::mc {Clear}] \
	-command SmoothOffDialog
    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command SmoothDestroyDialog
    pack $w.buttons.apply $w.buttons.clear $w.buttons.close \
	-side left -padx 10 -expand true

    UpdateSmoothDialog
}

proc SmoothApplyDialog {} {
    global smooth

    set smooth(view) 1
    SmoothUpdate
}

proc SmoothDestroyDialog {} {
    global smooth

    if {[winfo exists $smooth(top)]} {
	destroy $smooth(top)
	destroy $smooth(mb)
    }
}

proc SmoothCutDialog {} {
    global smooth

    tk_entryCut [focus -displayof $smooth(top)]
}

proc SmoothCopyDialog {} {
    global smooth

    tk_entryCopy [focus -displayof $smooth(top)]
}

proc SmoothPasteDialog {} {
    global smooth

    tk_entryPaste [focus -displayof $smooth(top)]
}

proc SmoothOffDialog {} {
    global smooth

    set smooth(view) 0
    SmoothUpdate
}

proc UpdateSmoothMenu {} {
    global smooth
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateSmoothMenu"
    }

    if {($current(frame) != {}) && [$current(frame) has fits]} {
	set smooth(view) [$current(frame) has smooth]
	set smooth(function) [$current(frame) get smooth function]
	set smooth(radius) [$current(frame) get smooth radius]
    } else {
	set smooth(view) 0
	set smooth(function) gaussian
	set smooth(radius) 3
    }
}

proc UpdateSmoothDialog {} {
    global smooth
    global current

    global debug
    if {$debug(tcl,update)} {
	puts "UpdateSmoothDialog"
    }

    if {[winfo exists $smooth(top)] && $current(frame) != {}} {
    }
}

# Process Cmds

proc ProcessSmoothCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global smooth

    switch -- [string tolower [lindex $var $i]] {
	radius {
	    incr i
	    set smooth(radius) [lindex $var $i]
	    SmoothUpdate
	}
	function {
	    incr i
	    set smooth(function) [lindex $var $i]
	    SmoothUpdate
	}
	yes -
	true -
	1 -
	no -
	false -
	0 {
	    set smooth(view) [FromYesNo [lindex $var $i]]
	    SmoothUpdate
	}

	default {
	    set smooth(view) 1
	    SmoothUpdate
	    incr i -1
	}
    }
}
