#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc VOCancel {varname} {
    upvar #0 $varname var
    global $varname

    # set state to 0 so that we don't process the finish proc
    set var(active) 0

    if [info exists var(token)] {
	http::reset $var(token)
    }
}

proc VODestroy {varname} {
    upvar #0 $varname var
    global $varname

    VOCancel $varname

    if {[winfo exists $var(top)]} {
	destroy $var(top)
	destroy $var(mb)
    }

    unset $varname
}

proc VOReset {varname} {
    upvar #0 $varname var
    global $varname

    set var(active) 0

    if {[info exists var(token)]} {
	http::cleanup $var(token)
	unset var(token)
    }
}

proc VODone {varname} {
    upvar #0 $varname var
    global $varname

    VOReset $varname
}

proc VOCancelled {varname} {
    upvar #0 $varname var
    global $varname

    VOReset $varname
}

proc VOError {varname message} {
    upvar #0 $varname var
    global $varname

    Error $message
    VOReset $varname
}

proc VODialog {sync} {
    global ds9
    global vo
    global current

    if [winfo exists $vo(top)] {
	raise $vo(top)
	return
    }

    set varname voi
    upvar #0 $varname var
    global $varname

    # variables
    set var(top) $vo(top)
    set var(mb) $vo(mb)
    set var(sync) $sync

    set w $var(top)
    set title [msgcat::mc {Virtual Observatory}]

    # create the window
    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "VODestroy $varname"

    $w configure -menu $var(mb)

    # menu
    menu $var(mb)
    AppleMenu ${varname}(mb)

    $var(mb) add cascade -label [msgcat::mc {File}] -menu $var(mb).file

    menu $var(mb).file
    $var(mb).file add command -label [msgcat::mc {Apply}] \
	-command "VOApply $varname"
    $var(mb).file add command -label [msgcat::mc {Cancel}] \
	-command "VOCancel $varname"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Help Me Choose}] \
	-command HelpVO
    $var(mb).file add command -label [msgcat::mc {Configure}] \
	-command "PrefsDialog http"
    $var(mb).file add separator
    $var(mb).file add command -label [msgcat::mc {Close}] \
	-command "VODestroy $varname"

    frame $w.param -relief ridge -bd 1
    frame $w.opt -relief ridge -bd 1
    frame $w.buttons -relief ridge -bd 1
    pack $w.param $w.opt $w.buttons \
	-fill x -expand true -ipadx 4 -ipady 4

    frame $w.opt.o
    pack $w.opt.o -side left
    label $w.opt.o.t1  -text [msgcat::mc {Use Internal Web Browser}]
    set var(hv,button) [checkbutton $w.opt.o.hv -text {} \
			    -variable vo(hv) \
			    -command SavePrefs]
    label $w.opt.o.t2  -text [msgcat::mc {Connect to VO Servers}]
    radiobutton $w.opt.o.xpa -text [msgcat::mc {Directly}] \
	-variable vo(method) -value xpa -command PrefsVOMethod
    radiobutton $w.opt.o.http -text [msgcat::mc {Through Remote Web Server}] \
	-variable vo(method) -value mime -command PrefsVOMethod
    button $w.opt.o.help -text [msgcat::mc {Help Me Choose}] \
	-command HelpVO
    button $w.opt.o.proxy -text [msgcat::mc {Configure}] \
	-command "PrefsDialog http"

    grid $w.opt.o.t1 $w.opt.o.hv -padx 4 -sticky w
    grid $w.opt.o.t2 $w.opt.o.xpa $w.opt.o.http -padx 4 -sticky w
    grid x $w.opt.o.help $w.opt.o.proxy -padx 4 -sticky w

    button $w.buttons.close -text [msgcat::mc {Close}] \
	-command "VODestroy $varname"
    pack $w.buttons.close -side left -padx 10 -expand true

    if {[string length $vo(server,host)] == 0} {
	VOApply $varname
    } else {
	set l [llength $vo(server,host)]
	for {set ii 0} {$ii<$l} {incr ii} {
	    set b [lindex $vo(server,button) $ii]
	    checkbutton $b -text "[lindex $vo(server,title) $ii] ([lindex $vo(server,url) $ii])" -variable vo(b$ii) -command "VOCheck $varname $ii"
	    pack $b -anchor w -padx 4

	    # update color
	    if {$vo(b$ii)} {
		$b configure -selectcolor yellow

		global debug
		if {$debug(tcl,idletasks)} {
		    puts "VODialog"
		}
		update idletasks

		$b configure -selectcolor green
	    } else {
		$b configure -selectcolor yellow
		$b select

		global debug
		if {$debug(tcl,idletasks)} {
		    puts "VODialog"
		}
		update idletasks

		$b configure -selectcolor {}
		$b deselect
	    }
	}
    }
}

proc VOApply {varname} {
    upvar #0 $varname var
    global $varname
    global vo

    global xpa
    global http

    set w $var(top)

    # delete old servers
    for {set ii 0} {$ii < [llength $vo(server,button)]} {incr ii} {
	catch {xparemote $xpa [lindex $vo(server,host) $ii] - -proxy}
	catch {destroy [lindex $vo(server,button) $ii]}
	catch {unset vo(b$ii)}
    }
    set vo(server,host) {}
    set vo(server,title) {}
    set vo(server,url) {}
    set vo(server,button) {}

    set var(url) $vo(server)
    VOLoad $varname 
}

proc VOLoad {varname} {
    upvar #0 $varname var
    global $varname

    global http

    if {$var(sync)} {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -timeout $http(timeout) \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	    VOFinish $varname $var(token)
	} else {
	    VOError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    } else {
	if {![catch {set var(token) [http::geturl $var(url) \
					 -timeout $http(timeout) \
					 -command [list VOFinish $varname] \
					 -headers "[ProxyHTTP]"]
	}]} {
	    set var(active) 1
	} else {
	    VOError $varname "[msgcat::mc {Unable to locate URL}] $var(url)"
	}
    }
}

proc VOFinish {varname token} {
    upvar #0 $varname var
    global $varname

    if {!($var(active))} {
	VOCancelled $varname
	return
    }

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Log it
    HTTPLog $token

    # Result?
    switch -- $code {
	200 -
	203 -
	503 {
	    VOParse $varname
	    VODone $varname
	}

	201 -
	300 -
	301 -
	302 -
	303 -
	305 -
	307 {
	    foreach {name value} $meta {
		if {[regexp -nocase ^location$ $name]} {
		    global debug
		    if {$debug(tcl,hv)} {
			puts "VOFinish redirect $code to $value"
		    }
		    # clean up and resubmit
		    http::cleanup $token
		    unset var(token)

		    set var(url) $value
		    VOLoad $varname
		}
	    }
	}

	default {VOError $varname [msgcat::mc {An error has occured while updating VO server list}]}
    }
}

proc VOParse {varname} {
    upvar #0 $varname var
    global $varname
    global vo

    set w $var(top)

    set data [string trimright [http::data $var(token)] \n]

    set lines [split $data \n]
    set l [llength $lines]
    for {set ii 0} {$ii<$l} {incr ii} {
	set line [lindex $lines $ii]
	set b "$w.param.b$ii"

	set ll [split $line \t]
	lappend vo(server,host) [lindex $ll 0]
	lappend vo(server,title) [lindex $ll 1]
	lappend vo(server,url) [lindex $ll 2]
	lappend vo(server,button) $b
	checkbutton $b -text "[lindex $vo(server,title) $ii] ([lindex $vo(server,url) $ii])" -variable vo(b$ii) -command "VOCheck $varname $ii"
	pack $b -anchor w -padx 4
    }
}

proc VOCheck {varname ii} {
    upvar #0 $varname var
    global $varname
    global vo

    global xpa

    set w $var(top)
    set b "$w.param.b$ii"

    if {$vo(b$ii)} {
	$b configure -selectcolor yellow

	global debug
	if {$debug(tcl,idletasks)} {
	    puts "VOCheck"
	}
	update idletasks

	switch $vo(method) {
	    mime {}
	    xpa {
		if {[catch {xparemote $xpa \
				[lindex $vo(server,host) $ii] + -proxy}]} {
		    $b configure -selectcolor {}
		    $b deselect
		    set vo(b$ii) 0
		    Error [msgcat::mc {Unable to connect to VO server}]
		    return
		}
	    }
	}

	$b configure -selectcolor green

	if {$vo(hv)} {
	    set url [lindex $vo(server,url) $ii]
	    ParseURL $url r
	    HV "vo$ii" "$r(authority)" $url 4 {}
	}
    } else {
	$b configure -selectcolor yellow
	$b select

	global debug
	if {$debug(tcl,idletasks)} {
	    puts "VOCheck"
	}
	update idletasks

	switch $vo(method) {
	    mime {}
	    xpa {
		catch {xparemote $xpa [lindex $var(server,host) $ii] - -proxy}
	    }
	}

	$b configure -selectcolor {}
	$b deselect
    }
}

# Prefs Cmds

proc PrefsDialogVO {} {
    global prefs

    set w $prefs(tabs)

    frame $w.vo
    $prefs(tabs) tab configure vo -window $w.vo -fill both

    frame $w.vo.a -relief ridge -bd 1
    frame $w.vo.b -relief ridge -bd 1
    pack $w.vo.a $w.vo.b -side top -fill both -expand true

    # Buttons
    PrefsDialogButtons $w.vo

    set f $w.vo.a.f
    frame $f
    pack $f -anchor w -fill y -expand true

    checkbutton $f.web -text [msgcat::mc {Use Internal Web Browser}] \
	-variable vo(hv)
    radiobutton $f.mime -text [msgcat::mc {Connect Use Web Proxy}] \
	-variable vo(method) -value mime -command PrefsVOMethod
    radiobutton $f.xpa -text [msgcat::mc {Connect Directly}] \
	-variable vo(method) -value xpa -command PrefsVOMethod

    grid $f.web -padx 4 -sticky w
    grid $f.mime $f.xpa -padx 4 -sticky w

    set f $w.vo.b.f
    frame $f
    pack $f -anchor w -fill y -expand true

    label $f.stitle -text [msgcat::mc {Select VO Server}]
    entry $f.server -textvariable vo(server) -width 50

    grid $f.stitle $f.server -padx 4 -sticky w
}

proc PrefsVOMethod {} {
    global vo

    switch $vo(method) {
	mime {set vo(hv) 1}
	xpa {}
    }
}

# Process Cmds

proc ProcessVOCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global vo

    set vvarname voi
    upvar #0 $vvarname vvar
    global $vvarname

    VODialog 1

    # find best match
    set ii [lsearch $vo(server,url) "*[lindex $var $i]*"]
    if {$ii>=0} {
	set vo(b$ii) 1
	VOCheck $vvarname $ii
    }
}
