#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc UpdateWCS {} {
    global wcs

    # infobox
    # if one wcs coord system is visiable, change it
    global view
    set cnt 0
    foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$view(info,wcs$l)} {
	    incr cnt
	}
    }
    if {$cnt == 1} {
	foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    set view(info,wcs$l) 0
	}
	set view(info,$wcs(system)) 1
	LayoutInfoPanel
    }

    # align
    set wcs(align,system) $wcs(system)
    set wcs(align,sky) $wcs(sky)
    AlignWCSFrame

    # compass
    global panner
    global ds9
    set panner(compass,wcs,system) $wcs(system)
    set panner(compass,wcs,sky) $wcs(sky)
    foreach f $ds9(frames) {
	$f panner wcs $panner(compass,wcs,system) $panner(compass,wcs,sky)
    }

    # grid
    global grid
    set grid(system) $wcs(system)
    set grid(sky) $wcs(sky)
    set grid(skyformat) $wcs(skyformat)
    GridUpdate

    # regions
    global marker
    set marker(system) $wcs(system)
    set marker(sky) $wcs(sky)
    set marker(skyformat) $wcs(skyformat)
    UpdateMarkerFormatMenu

    set marker(dialog,system) $wcs(system)
    set marker(dialog,sky) $wcs(sky)
    set marker(dialog,skyformat) $wcs(skyformat)

    # print coords
    global coord
    set cnt 0
    foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	if {$coord(wcs$l)} {incr cnt}
    }
    # if one one displayed, replace
    if {$cnt == 1} {
	foreach l {{} a b c d e f g h i j k l m n o p q r s t u v w x y z} {
	    set coord(wcs$l) 0
	}
    }
    set coord($wcs(system)) 1
    set coord(sky) $wcs(sky)
    set coord(skyformat) $wcs(skyformat)

    # panzoom dialog
    global panzoom
    set panzoom(system) $wcs(system)
    set panzoom(sky) $wcs(sky)
    set panzoom(skyformat) $wcs(skyformat)
    UpdateZoomDialog
}

proc AlignWCSFrame {} {
    global current
    global wcs
    global ds9

    if {$current(frame) != {}} {
	SetWatchCursor
	$current(frame) wcs align $wcs(align) $wcs(align,system) \
	    $wcs(align,sky)
	UnsetWatchCursor
	UpdateAlignWCSMenu $ds9(mb).zoom
    }
}

# Process Cmds

proc ProcessWCSCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global wcs
    global current

    switch -- [string tolower [lindex $var $i]] {
	system {
	    incr i
	    set wcs(system) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	sky {
	    incr i
	    set wcs(sky) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	format -
	skyformat {
	    incr i
	    set wcs(skyformat) [string tolower [lindex $var $i]]
	    UpdateWCS
	}
	align {
	    incr i
	    set wcs(align) [FromYesNo [lindex $var $i]]
	    AlignWCSFrame
	}
	reset {
	    $current(frame) wcs reset
	    UpdateWCSMenu
	    UpdateWCS
	}
	replace {
	    incr i
	    if {[lindex $var $i] == "file"} {
		incr i
	    }
	    set fn [lindex $var $i]
	    if {$fn != {}} {
		$current(frame) wcs replace "\{$fn\}"
	    }
	    UpdateWCSMenu
	    UpdateWCS
	}
	append {
	    incr i
	    if {[lindex $var $i] == "file"} {
		incr i
	    }
	    set fn [lindex $var $i]
	    if {$fn != {}} {
		$current(frame) wcs append "\{$fn\}"
	    }
	    UpdateWCSMenu
	    UpdateWCS
	}
	default {
	    set wcs(system) [string tolower [lindex $var $i]]
	    switch -- $wcs(system) {
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {
		    set wcs(sky) $wcs(system)
		    set wcs(system) wcs
		}
	    }
	    UpdateWCS
	}
    }
}




