#  Copyright (C) 1999-2008
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc InitXPA {} {
    global ds9
    global env

    if {$ds9(xpa)} {
	if {[info exists env(XPA_METHOD)]} {
	    if {$env(XPA_METHOD) == {local}} {
		catch {CreateXPA}
	    } else {
		if {![checkdns {} 5]} {
		    catch {CreateXPA}
		} else {
		    Error [msgcat::mc {XPA unable to verify hostname}]
		}
	    }
	} else {
	    if {![checkdns {} 5]} {
		catch {CreateXPA}
	    } else {
		Error [msgcat::mc {XPA unable to verify hostname}]
	    }
	}
    }
}

proc CreateXPA {} {
    global xpa
    global ds9

    set xpa [xpacmdnew "DS9" $ds9(title)]

    xpacmdadd $xpa 2mass \
	{} \
	XPASend2MASS {} {} \
	XPARcvd2MASS {} "fillbuf=false"

    xpacmdadd $xpa about \
	{} \
	XPASendAbout {} {} \
	{} {} {}

    xpacmdadd $xpa analysis \
	{} \
	XPASendAnalysis {} {} \
	XPARcvdAnalysis {} {}

    xpacmdadd $xpa array \
	{} \
	XPASendArray {} {} \
	XPARcvdArray {} "fillbuf=false"

    xpacmdadd $xpa blink \
	{} \
	XPASendBlink {} {} \
	XPARcvdBlink {} "fillbuf=false"

    xpacmdadd $xpa bin \
	{} \
	XPASendBin {} {} \
	XPARcvdBin {} "fillbuf=false"

    xpacmdadd $xpa catalog \
	{} \
	XPASendCAT {} {} \
	XPARcvdCAT {} "fillbuf=false"

    xpacmdadd $xpa cd \
	{} \
	XPASendCD {} {} \
	XPARcvdCD {} "fillbuf=false"

    xpacmdadd $xpa cmap \
	{} \
	XPASendColormap {} {} \
	XPARcvdColormap {} "fillbuf=false"

    xpacmdadd $xpa console \
	{} \
	{} {} {} \
	XPARcvdConsole {} "fillbuf=false"

    xpacmdadd $xpa contour \
	{} \
	XPASendContour {} {} \
	XPARcvdContour {} "fillbuf=false"

    xpacmdadd $xpa crosshair \
	{} \
	XPASendCrosshair {} {} \
	XPARcvdCrosshair {} "fillbuf=false"

    xpacmdadd $xpa cursor \
	{} \
	{} {} {} \
	XPARcvdCursor {} "fillbuf=false"

    xpacmdadd $xpa data \
	{} \
	XPASendData {} {} \
	{} {} {}

    xpacmdadd $xpa datacube \
	{} \
	XPASendDataCube {} {} \
	XPARcvdDataCube {} "fillbuf=false"

    xpacmdadd $xpa dss \
	{} \
	XPASendSAO {} {} \
	XPARcvdSAO {} "fillbuf=false"

    xpacmdadd $xpa dsssao \
	{} \
	XPASendSAO {} {} \
	XPARcvdSAO {} "fillbuf=false"

    xpacmdadd $xpa dsseso \
	{} \
	XPASendESO {} {} \
	XPARcvdESO {} "fillbuf=false"

    xpacmdadd $xpa dssstsci \
	{} \
	XPASendSTSCI {} {} \
	XPARcvdSTSCI {} "fillbuf=false"

    xpacmdadd $xpa exit \
	{} \
	{} {} {} \
	XPARcvdExit {} "fillbuf=false"

    xpacmdadd $xpa file \
	{} \
	XPASendFile {} {} \
	XPARcvdFile {} "fillbuf=false"

    xpacmdadd $xpa first \
	{} \
	XPASendFIRST {} {} \
	XPARcvdFIRST {} "fillbuf=false"

    xpacmdadd $xpa fits \
	{} \
	XPASendFits {} "fillbuf=false" \
	XPARcvdFits {} "fillbuf=false"

    xpacmdadd $xpa frame \
	{} \
	XPASendFrame {} {} \
	XPARcvdFrame {} "fillbuf=false"

    xpacmdadd $xpa grid \
	{} \
	XPASendGrid {} {} \
	XPARcvdGrid {} "fillbuf=false"

    xpacmdadd $xpa header \
	{} \
	{} {} {} \
	XPARcvdHeader {} "fillbuf=false"

    xpacmdadd $xpa height \
	{} \
	XPASendHeight {} {} \
	XPARcvdHeight {} "fillbuf=false"

    xpacmdadd $xpa iconify \
	{} \
	XPASendIconify {} {} \
	XPARcvdIconify {} "fillbuf=false"

    xpacmdadd $xpa iis \
	{} \
	XPASendIIS {} {} \
	XPARcvdIIS {} "fillbuf=false"

    xpacmdadd $xpa imexam \
	{} \
	XPASendImexam {} {} \
	{} {} {}

    xpacmdadd $xpa lock \
	{} \
	{} {} {} \
	XPARcvdLock {} "fillbuf=false"

    xpacmdadd $xpa lower \
	{} \
	{} {} {} \
	XPARcvdLower {} "fillbuf=false"

    xpacmdadd $xpa mask \
	{} \
	XPASendMask {} {} \
	XPARcvdMask {} "fillbuf=false"

    xpacmdadd $xpa match \
	{} \
	{} {} {} \
	XPARcvdMatch {} "fillbuf=false"

    xpacmdadd $xpa minmax \
	{} \
	XPASendMinMax {} {} \
	XPARcvdMinMax {} "fillbuf=false"

    xpacmdadd $xpa mode \
	{} \
	XPASendMode {} {} \
	XPARcvdMode {} "fillbuf=false"

    xpacmdadd $xpa nameserver \
	{} \
	XPASendNRES {} {} \
	XPARcvdNRES {} "fillbuf=false"

    xpacmdadd $xpa orient \
	{} \
	XPASendOrient {} {} \
	XPARcvdOrient {} "fillbuf=false"

    xpacmdadd $xpa {page setup} \
	{} \
	XPASendPageSetup {} {} \
	XPARcvdPageSetup {} "fillbuf=false"

    xpacmdadd $xpa pan \
	{} \
	XPASendPan {} {} \
	XPARcvdPan {} "fillbuf=false"

    xpacmdadd $xpa pixeltable \
	{} \
	XPASendPixelTable {} {} \
	XPARcvdPixelTable {} "fillbuf=false"

    xpacmdadd $xpa prefs \
	{} \
	XPASendPrefs {} {} \
	XPARcvdPrefs {} "fillbuf=false"

    xpacmdadd $xpa preserve \
	{} \
	XPASendPreserve {} {} \
	XPARcvdPreserve {} "fillbuf=false"

    xpacmdadd $xpa print \
	{} \
	XPASendPrint {} {} \
	XPARcvdPrint {} "fillbuf=false"

    xpacmdadd $xpa plot \
	{} \
	XPASendPlot {} {} \
	XPARcvdPlot {} {}

    xpacmdadd $xpa quit \
	{} \
	{} {} {} \
	XPARcvdExit {} "fillbuf=false"

    xpacmdadd $xpa raise \
	{} \
	{} {} {} \
	XPARcvdRaise {} "fillbuf=false"

    xpacmdadd $xpa regions \
	{} \
	XPASendRegions {} {} \
	XPARcvdRegions {} "fillbuf=false"

    xpacmdadd $xpa rgb \
	{} \
	XPASendRGB {} {} \
	XPARcvdRGB {} "fillbuf=false"

    xpacmdadd $xpa rotate \
	{} \
	XPASendRotate {} {} \
	XPARcvdRotate {} "fillbuf=false"

    xpacmdadd $xpa saveimage \
	{} \
	{} {} {} \
	XPARcvdSaveImage {} "fillbuf=false"

    xpacmdadd $xpa savefits \
	{} \
	{} {} {} \
	XPARcvdSaveFits {} "fillbuf=false"

    xpacmdadd $xpa savempeg \
	{} \
	{} {} {} \
	XPARcvdSaveMPEG {} "fillbuf=false"

    xpacmdadd $xpa scale \
	{} \
	XPASendScale {} {} \
	XPARcvdScale {} "fillbuf=false"

    xpacmdadd $xpa single \
	{} \
	XPASendSingle {} {} \
	XPARcvdSingle {} "fillbuf=false"

    xpacmdadd $xpa shm \
	{} \
	XPASendShm {} {} \
	XPARcvdShm {} "fillbuf=false"

    xpacmdadd $xpa skyview \
	{} \
	XPASendSkyView {} {} \
	XPARcvdSkyView {} "fillbuf=false"

    xpacmdadd $xpa smooth \
	{} \
	XPASendSmooth {} {} \
	XPARcvdSmooth {} "fillbuf=false"

    xpacmdadd $xpa source \
	{} \
	{} {} {} \
	XPARcvdSource {} "fillbuf=false"

    xpacmdadd $xpa tcl \
	{} \
	{} {} {} \
	XPARcvdTcl {} {}

    xpacmdadd $xpa tile \
	{} \
	XPASendTile {} {} \
	XPARcvdTile {} "fillbuf=false"

    xpacmdadd $xpa update \
	{} \
	{} {} {} \
	XPARcvdUpdate {} "fillbuf=false"

    xpacmdadd $xpa version \
	{} \
	XPASendVersion {} {} \
	{} {} {}

    xpacmdadd $xpa view \
	{} \
	XPASendView {} {} \
	XPARcvdView {} "fillbuf=false"

    xpacmdadd $xpa vo \
	{} \
	XPASendVO {} {} \
	XPARcvdVO {} "fillbuf=false"

    xpacmdadd $xpa wcs \
	{} \
	XPASendWCS {} {} \
	XPARcvdWCS {} "fillbuf=false"

    xpacmdadd $xpa web \
	{} \
	XPASendWeb {} {} \
	XPARcvdWeb {} "fillbuf=false"

    xpacmdadd $xpa width \
	{} \
	XPASendWidth {} {} \
	XPARcvdWidth {} "fillbuf=false"

    xpacmdadd $xpa zscale \
	{} \
	XPASendZScale {} {} \
	XPARcvdZScale {} "fillbuf=false"

    xpacmdadd $xpa zoom \
	{} \
	XPASendZoom {} {} \
	XPARcvdZoom {} "fillbuf=false"
}

proc HVXPASet {fn paramlist} {
    global debug
    if {$debug(tcl,hv)} {
	puts "HVXPASet:$fn:$paramlist"
    }

    set cmd [lindex $paramlist 0]
    set param [lrange $paramlist 1 end]
    set buf {}
    set len 0

    switch -- $cmd {
    	2mass {XPARcvd2MASS {} {} $param $buf $len}
    	about {}
    	analysis {XPAHVRcvdAnalysis {} $fn $param $buf $len}
    	array {XPAHVRcvdArray {} $fn $param $buf $len}
    	blink {XPARcvdBlink {} {} $param $buf $len}
    	bin {XPARcvdBin {} {} $param $buf $len}
    	catalog {XPARcvdCAT {} {} $param $buf $len}
    	cd {XPARcvdCD {} {} $param $buf $len}
    	cmap {XPARcvdColormap {} {} $param $buf $len}
	console {XPARcvdConsole {} {} $param $buf $len}
    	contour {XPARcvdContour {} {} $param $buf $len}
    	crosshair {XPARcvdCrosshair {} {} $param $buf $len}
    	cursor {XPARcvdCursor {} {} $param $buf $len}
    	data {}
    	datacube {XPARcvdDataCube {} {} $param $buf $len}
    	dss -
	dsssao {XPARcvdSAO {} {} $param $buf $len}
	dsseso {XPARcvdESO {} {} $param $buf $len}
	dssstsci {XPARcvdSTSCI {} {} $param $buf $len}
    	exit {XPARcvdExit {} {} $param $buf $len}
    	file {XPARcvdFile {} {} $param $buf $len}
    	first {XPARcvdFIRST {} {} $param $buf $len}
    	fits {XPAHVRcvdFits {} $fn $param $buf $len}
    	frame {XPARcvdFrame {} {} $param $buf $len}
    	grid {XPARcvdGrid {} {} $param $buf $len}
    	header {XPARcvdHeader {} {} $param $buf $len}
    	height {XPARcvdHeight {} {} $param $buf $len}
    	iconify {XPARcvdIconify {} {} $param $buf $len}
    	iis {XPARcvdIIS {} {} $param $buf $len}
    	imexam {}
    	lock {XPARcvdLock {} {} $param $buf $len}
    	lower {XPARcvdLower {} {} $param $buf $len}
    	mask {XPARcvdMask {} {} $param $buf $len}
    	match {XPARcvdMatch {} {} $param $buf $len}
    	minmax {XPARcvdMinMax {} {} $param $buf $len}
    	mode {XPARcvdMode {} {} $param $buf $len}
    	nameserver {XPARcvdNRES {} {} $param $buf $len}
    	orient {XPARcvdOrient {} {} $param $buf $len}
    	{page setup} {XPARcvdPageSetup {} {} $param $buf $len}
    	pan {XPARcvdPan {} {} $param $buf $len}
    	pixeltable {XPARcvdPixelTable {} {} $param $buf $len}
    	prefs {XPARcvdPrefs {} {} $param $buf $len}
    	preserve {XPARcvdPreserve {} {} $param $buf $len}
    	print {XPARcvdPrint {} {} $param $buf $len}
    	plot {XPARcvdPlot {} {} $param $buf $len}
    	quit {XPARcvdExit {} {} $param $buf $len}
    	raise {XPARcvdRaise {} {} $param $buf $len}
    	regions {XPAHVRcvdRegions {} $fn $param $buf $len}
    	rgb {XPARcvdRGB {} {} $param $buf $len}
    	rotate {XPARcvdRotate {} {} $param $buf $len}
    	saveimage {XPARcvdSaveImage {} {} $param $buf $len}
    	savefits {XPARcvdSaveFits {} {} $param $buf $len}
    	savempeg {XPARcvdSaveMPEG {} {} $param $buf $len}
    	scale {XPARcvdScale {} {} $param $buf $len}
    	single {XPARcvdSingle {} {} $param $buf $len}
    	shm {XPARcvdShm {} {} $param $buf $len}
	skyview {XPARcvdSkyView {} {} $param $buf $len}
    	smooth {XPARcvdSmooth {} {} $param $buf $len}
    	source {XPARcvdSource {} {} $param $buf $len}
    	tcl {XPARcvdTcl {} {} $param $buf $len}
    	tile {XPARcvdTile {} {} $param $buf $len}
    	update {XPARcvdUpdate {} {} $param $buf $len}
	version {}
    	view {XPARcvdView {} {} $param $buf $len}
    	vo {XPARcvdVO {} {} $param $buf $len}
    	wcs {XPAHVRcvdWCS {} {} $param $buf $len}
    	web {XPARcvdWeb {} {} $param $buf $len}
    	width {XPARcvdWidth {} {} $param $buf $len}
    	zscale {XPARcvdZScale {} {} $param $buf $len}
    	zoom {XPARcvdZoom {} {} $param $buf $len} 
    }
}

proc XPASend2MASS {xpa cdata param} {
    global twomassi

    InitXPAError $xpa
    2MASSDialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    survey {xpasetbuf $xpa "$twomassi(survey)\n"}
	    coord {xpasetbuf $xpa "$twomassi(x) $twomassi(y)\n"}
	    x {xpasetbuf $xpa "$twomassi(x)\n"}
	    y {xpasetbuf $xpa "$twomassi(y)\n"}
	    size {xpasetbuf $xpa "$twomassi(width) $twomassi(height)\n"}
	    width {xpasetbuf $xpa "$twomassi(width)\n"}
	    height {xpasetbuf $xpa "$twomassi(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$twomassi(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvd2MASS {xpa cdata param buf len} {
    set i 0
    catch {Process2MASSCmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendAbout {xpa cdata param} {
    global help

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$help(about)\n"}
    CatchXPAError $xpa
}

proc XPASendAnalysis {xpa cdata param} {
    global analysis

    InitXPAError $xpa
    catch {
	set result {}
	switch -- [string tolower [lindex $param 0]] {
	    entry {
		AnalysisEntry [lrange $param 1 end] result
		append result "\n"
	    }
	    default {
		for {set i 0} {$i<$analysis(menu,count)} {incr i} {
		    set result "$result\#$i menu"
		    set result "$result\n$analysis(menu,$i,item)"
		    set result "$result\n$analysis(menu,$i,template)"
		    set result "$result\n$analysis(menu,$i,cmd)"
		    set result "$result\n\n"
		}
		for {set i 0} {$i<$analysis(bind,count)} {incr i} {
		    set result "$result\#$i bind"
		    set result "$result\n$analysis(bind,$i,item)"
		    set result "$result\n$analysis(bind,$i,template)"
		    set result "$result\n$analysis(bind,$i,cmd)"
		    set result "$result\n\n"
		}
	    }
	}
	xpasetbuf $xpa $result
    }
    CatchXPAError $xpa
}

proc XPARcvdAnalysis {xpa cdata param buf len} {
    global ap

    InitXPAError $xpa
    catch {
	switch -- [string tolower [lindex $param 0]] {
	    message {
		switch -- [llength $param] {
		    0 -
		    1 {}
		    2 {AnalysisMessage ok [lindex $param 1]}
		    3 {AnalysisMessage [lindex $param 1] [lindex $param 2]}
		}
	    }
	    text {AnalysisText apXPA Analysis $buf append}
	    plot {
		# for backward compatibility
		# use xpa plot instead

		set tt $ap(tt)
		if {[string tolower [lindex $param 1]] == "stdin"} {
		    AnalysisPlotStdin  $tt {} buf
		} elseif {[string tolower [lindex $param 1]] == "close"} {
		    foreach f $ap(xpa) {
			APDestroy $f
		    }
		    set ap(xpa) {}
		} elseif {[llength $param] == 1} {
		    AnalysisPlot $tt Plot {} {} {} xy $buf
		} elseif {[llength $param] == 5}  {
		    AnalysisPlot $tt Plot \
			[lindex $param 1] \
			[lindex $param 2] \
			[lindex $param 3] \
			[lindex $param 4] \
			$buf
		}
	    }
	    load {XPASendAnalysisLoad $xpa buf [lindex $param 1]}
	    clear {
		ClearAnalysis
		switch -- [lindex $param 1] {
		    load {XPASendAnalysisLoad $xpa buf [lindex $param 2]}
		}
	    }
	    default {
		AnalysisTask [expr $param-1] menu {} 0 0
		# don't know why
		global errorInfo
		set errorInfo {}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPAHVRcvdAnalysis {xpa cdata param buf len} {
    if {$cdata != {}} {
	if [catch {open "$cdata" r} ch] {
	    Error "[msgcat::mc {Unable to open file}] $cdata"
	    return
	}
	set buf [read $ch]
	set len [string length $buf]
	catch {close $ch}
	XPARcvdAnalysis $xpa {} $param $buf $len
    }
}

proc XPASendAnalysisLoad {xpa bufname fn} {
    upvar $bufname buf

    InitXPAError $xpa
    catch {
	if {$fn != {}} {
	    ProcessAnalysisFile $fn
	} else {
	    ProcessAnalysis buf
	}

	UnBindEventsCanvas
	BindEventsCanvas
    }
    CatchXPAError $xpa
}

proc XPASendArray {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    if {[$current(frame) has fits]} {
		$current(frame) save array socket [xparec $xpa datafd]
	    }
	} else {
	    Error [msgcat::mc {No file loaded}]
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdArray {xpa cdata param buf len} {
    global current
    global loadParam

    InitXPAError $xpa
    catch {
	set param [XPARcvdArrayPre $param]

	if {$current(frame) != {}} {
	    set loadParam(load,type) socketgz
	    # set loadParam(load,layer) $file(layer)
	    set loadParam(socket,id) [xparec $xpa datafd]
	    set loadParam(file,type) array
	    set loadParam(file,mode) {}
	    set loadParam(file,name) [lindex $param 0]
	    if {$loadParam(file,name) == {}} {
		set loadParam(file,name) stdin
	    }

	    XPARcvdArrayPost $param
	}
    }
    CatchXPAError $xpa
}

proc XPAHVRcvdArray {xpa cdata param buf len} {
    global current
    global loadParam

    catch {
	set param [XPARcvdArrayPre $param]

	if {$current(frame) != {}} {
	    set loadParam(load,type) allocgz
	    # set loadParam(load,layer) $file(layer)
	    set loadParam(file,type) array
	    set loadParam(file,mode) {}
	    set loadParam(file,name) [lindex $param 0]
	    set loadParam(file,fn) $cdata

	    XPARcvdArrayPost $param
	}
    }
}

proc XPARcvdArrayPre {param} {
    global loadParam

    set loadParam(load,layer) {}
    switch -- [string tolower [lindex $param 0]] {
	new {
	    switch -- [string tolower [lindex $param 1]] {
		rgb {CreateRGBFrame}
		default {CreateFrame}
	    }
	    set param [lreplace $param 0 0]
	}
	mask {
	    set loadParam(load,layer) mask
	    set param [lreplace $param 0 0]
	}
    }

    return $param
}

proc XPARcvdArrayPost {param} {
    global current
    global loadParam

    StartLoad
    switch -- [lindex $param 0] {
	rgb {
	    if {[$current(frame) get type] == {rgb}} {
		set loadParam(file,mode) {rgb cube}
		set loadParam(file,name) [lindex $param 1]
		ProcessLoad
	    } else {
		Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
	    }
	}
	default {ProcessLoad}
    }
    FinishLoad
}

proc XPASendBlink {xpa cdata param} {
    global ds9

    InitXPAError $xpa
    catch {
	if {$ds9(display,user) == {blink}} {
	    xpasetbuf $xpa [ToYesNo 1]
	} else {
	    xpasetbuf $xpa [ToYesNo 0]
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdBlink {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0;ProcessBlinkCmd param i}
    CatchXPAError $xpa
}

proc XPASendBin {xpa cdata param} {
    global bin
    global current

    InitXPAError $xpa
    catch {
	switch -- [string tolower [lindex $param 0]] {
	    about {
		if {$current(frame) != {}} {
		    xpasetbuf $xpa "[$current(frame) get bin cursor]\n"
		}
	    }
	    buffersize {xpasetbuf $xpa "$bin(buffersize)\n"}
	    cols {
		if {$current(frame) != {}} {
		    xpasetbuf $xpa "[$current(frame) get bin cols]\n"
		}
	    }
	    factor {xpasetbuf $xpa "$bin(factor)\n"}
	    depth {xpasetbuf $xpa "$bin(depth)\n"}
	    filter {
		if {$current(frame) != {}} {
		    xpasetbuf $xpa "[$current(frame) get bin filter]\n"
		}
	    }
	    function {xpasetbuf $xpa "$bin(function)\n"}
	    smooth {
		# back compatibility
		global smooth
		switch -- [string tolower [lindex $param 1]] {
		    function {xpasetbuf $xpa "$smooth(function)\n"}
		    radius {xpasetbuf $xpa "$smooth(radius)\n"}
		    default {xpasetbuf $xpa "$smooth(view)\n"}
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdBin {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessBinCmd param i}
    CatchXPAError $xpa
}

proc XPASendCAT {xpa cdata param} {
    global current
    global cat

    InitXPAError $xpa
    catch {
	switch -- [string tolower [lindex $param 0]] {
	    {} {xpasetbuf $xpa "$cat(cats)\n"}
	    header {
		set cc [lindex $cat(cats) end]
		xpasetbuf $xpa "[CATGetHeader $cc]\n"
	    }
	    default {
		set cc [lindex $param 0]
		xpasetbuf $xpa "[CATGetHeader $cc]\n"
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdCAT {xpa cdata param buf len} {
    catch {set i 0; ProcessCatalogCmd param i}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendCD {xpa cdata param} {
    InitXPAError $xpa
    catch {xpasetbuf $xpa "[pwd]\n"}
    CatchXPAError $xpa
}

proc XPARcvdCD {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {cd $param}
    CatchXPAError $xpa
}

proc XPARcvdConsole {xpa cdata param buf len} {
    global current
    global ds9

    InitXPAError $xpa
    catch {OpenConsole}
    # ignore the invalid event error
    InitXPAError $xpa
}

proc XPASendContour {xpa cdata param} {
    global contour
    global current

    InitXPAError $xpa
    catch {
	if {$param != {}} {
	    if {$current(frame) != {}} {
		switch [lindex $param 0] {
		    color {xpasetbuf $xpa "$contour(color)\n"}
		    width {xpasetbuf $xpa "$contour(width)\n"}
		    smooth {xpasetbuf $xpa "$contour(smooth)\n"}
		    method {xpasetbuf $xpa "$contour(method)\n"}
		    nlevels {xpasetbuf $xpa "$contour(levels)\n"}
		    scale {xpasetbuf $xpa "$contour(scale)\n"}
		    mode {xpasetbuf $xpa "$contour(mode)\n"}
		    limits {xpasetbuf $xpa "$contour(min) $contour(max)\n"}
		    levels {
			global contourdlg
			ContourDialog
			xpasetbuf $xpa "[$contourdlg(txt) get 1.0 end]"
		    }
		    default {
			xpasetbuf $xpa [$current(frame) get contour [lindex $param 0] [lindex $param 1]]
		    }
		}
	    }
	} else {
	    xpasetbuf $xpa [ToYesNo $contour(view)]
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdContour {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessContourCmd param i}
    CatchXPAError $xpa
}

proc XPASendColormap {xpa cdata param} {
    global current
    global colorbar

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    file {xpasetbuf $xpa "[$current(colorbar) get file name]\n"}
	    invert {xpasetbuf $xpa [ToYesNo $colorbar(invert)]}
	    value {xpasetbuf $xpa "[$current(colorbar) get contrast] [$current(colorbar) get bias]\n"}
	    {} {xpasetbuf $xpa "[$current(colorbar) get name]\n"}
	    default {xpaerror $xpa "Illegal option: $param"}
	}
	CatchXPAError $xpa
    }
}

proc XPARcvdColormap {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessCmapCmd param i}
    CatchXPAError $xpa
}

proc XPASendCrosshair {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	set sys [lindex $param 0]
	set sky [lindex $param 1]
	set skyformat [lindex $param 2]
	if {$current(frame) != {}} {
	    switch -- $sys {
		{} {set sys physical}
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {set skyformat $sky; set sky $sys; set sys wcs}
	    }

	    xpasetbuf $xpa \
		"[$current(frame) get crosshair $sys $sky $skyformat]\n"
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdCrosshair {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessCrosshairCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdCursor {xpa cdata param buf len} {
    global current
    global ds9

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    set x [lindex $param 0]
	    set y [lindex $param 1]

	    switch -- $ds9(mode) {
		none {$current(frame) warp $x $y}
		pointer {MarkerArrowKey $current(frame) $x $y}
		catalog {MarkerArrowKey $current(frame) $x $y}
		crosshair {ArrowKeyCrosshair $current(frame) $x $y}
		colorbar {}
		pan {Pan $x $y canvas {}}
		zoom {}
		rotate {}
		examine {}
		illustrate {}
		imexam {}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPASendSAO {xpa cdata param} {
    global saoi

    InitXPAError $xpa
    SAODialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    coord {xpasetbuf $xpa "$saoi(x) $saoi(y)\n"}
	    x {xpasetbuf $xpa "$saoi(x)\n"}
	    y {xpasetbuf $xpa "$saoi(y)\n"}
	    size {xpasetbuf $xpa "$saoi(width) $saoi(height)\n"}
	    width {xpasetbuf $xpa "$saoi(width)\n"}
	    height {xpasetbuf $xpa "$saoi(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$saoi(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdSAO {xpa cdata param buf len} {
    catch {set i 0; ProcessSAOCmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendESO {xpa cdata param} {
    global esoi

    InitXPAError $xpa
    ESODialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    survey {xpasetbuf $xpa "$esoi(survey)\n"}
	    coord {xpasetbuf $xpa "$esoi(x) $esoi(y)\n"}
	    x {xpasetbuf $xpa "$esoi(x)\n"}
	    y {xpasetbuf $xpa "$esoi(y)\n"}
	    size {xpasetbuf $xpa "$esoi(width) $esoi(height)\n"}
	    width {xpasetbuf $xpa "$esoi(width)\n"}
	    height {xpasetbuf $xpa "$esoi(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$esoi(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdESO {xpa cdata param buf len} {
    catch {set i 0; ProcessESOCmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendSTSCI {xpa cdata param} {
    global stscii

    InitXPAError $xpa
    STSCIDialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    survey {xpasetbuf $xpa "$stscii(survey)\n"}
	    coord {xpasetbuf $xpa "$stscii(x) $stscii(y)\n"}
	    x {xpasetbuf $xpa "$stscii(x)\n"}
	    y {xpasetbuf $xpa "$stscii(y)\n"}
	    size {xpasetbuf $xpa "$stscii(width) $stscii(height)\n"}
	    width {xpasetbuf $xpa "$stscii(width)\n"}
	    height {xpasetbuf $xpa "$stscii(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$stscii(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdSTSCI {xpa cdata param buf len} {
    catch {set i 0; ProcessSTSCICmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendData {xpa cdata param} {
    global cube
    global blink
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    set sys [lindex $param 0]
	    set sky [lindex $param 1]
	    set x [lindex $param 2]
	    set y [lindex $param 3]
	    set w [lindex $param 4]
	    set h [lindex $param 5]
	    set strip [lindex $param 6]
	    switch -- $sys {
		image -
		physical -
		detector -
		amplifier {
		    set strip $h
		    set h $w
		    set w $y
		    set y $x
		    set x $sky
		    set sky fk5
		}
		fk5 -
		fk4 -
		icrs -
		galactic -
		ecliptic {
		    set strip $h
		    set h $w
		    set w $y
		    set y $x
		    set x $sky
		    set sky $sys
		    set sys wcs
		}
	    }
	    set strip [FromYesNo $strip]

	    $current(frame) get data $sys $sky $x $y $w $h rr
	    set ss {}
	    foreach id [array names rr] {
		if {$strip} {
		    append ss "$rr($id)\n"
		} else {
		    append ss "$id = $rr($id)\n"
		}
	    }
	    xpasetbuf $xpa "$ss"
	}
    }
    CatchXPAError $xpa
}

proc XPASendDataCube {xpa cdata param} {
    global cube
    global blink

    InitXPAError $xpa
    catch {
	switch -- [string tolower [lindex $param 0]] {
	    interval {xpasetbuf $xpa "$blink(interval)\n"}
	    default {xpasetbuf $xpa "$cube(slice)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdDataCube {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessDataCubeCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdExit {xpa cdata param buf len} {
    QuitDS9
}

proc XPASendFile {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    xpasetbuf $xpa "[$current(frame) get fits file name full]\n"
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdFile {xpa cdata param buf len} {
    global current

    InitXPAError $xpa

    set file(layer) {}
    catch {
	switch -- [string tolower [lindex $param 0]] {
	    new {
		switch -- [string tolower [lindex $param 1]] {
		    rgbimage -
		    rgbcube -
		    rgbarray {CreateRGBFrame}
		    default {CreateFrame}
		}
		set param [lreplace $param 0 0]
	    }
	    mask {
		set file(layer) mask
		set param [lreplace $param 0 0]
	    }
	}

	set fn  [lindex $param 1]
	set fn2 [lindex $param 2]
	set fn3 [lindex $param 3]

	switch -- [lindex $param 0] {
	    save {
		switch -- [lindex $param 1] {
		    gz {
			$current(frame) save fits image file $fn2 gz
		    }
		    resample {
			switch -- [lindex $param 2] {
			    gz {$current(frame) save fits resample file \
				    $fn3 gz}
			    default {$current(frame) save fits resample file \
					 $fn2}
			}
		    }
		    default {$current(frame) save fits image file $fn}
		}
	    }
	    default {
		StartLoad
		switch -- [lindex $param 0] {
		    fits {LoadFits $fn $file(layer)}
		    sfits {LoadSFits $fn $fn2 $file(layer)}

		    datacube -
		    medatacube {LoadDataCubeFits $fn $file(layer)}

		    mosaicimage {
			if {[lindex $param 2] == {}} {
			    LoadMosaicImageIRAFFits $fn $file(layer)
			} else {
			    switch -- [lindex $param 1] {
				iraf {LoadMosaicImageIRAFFits $fn2 $file(layer)}
				wfpc2 {LoadMosaicImageWFPC2Fits $fn2}
				default {LoadMosaicImageWCSFits \
					     [lindex $param 1] $fn2 $file(layer)}
			    }
			}
		    }
		    mosaic {
			if {[lindex $param 2] == {}} {
			    LoadMosaicIRAFFits $fn $file(layer)
			} else {
			    switch -- [lindex $param 1] {
				iraf {LoadMosaicIRAFFits $fn2 $file(layer)}
				default {LoadMosaicWCSFits \
					     [lindex $param 1] $fn2 $file(layer)}
			    }
			}
		    }
		    smosaic {
			switch -- [lindex $param 1] {
			    iraf {LoadMosaicIRAFSFits $fn2 $fn3 $file(layer)}
			    default {LoadMosaicWCSSFits [lindex $param 1] \
					 $fn2 $fn3 $file(layer)}
			}
		    }

		    mosaicimageiraf {
			# backward compatibility
			LoadMosaicImageIRAFFits $fn $file(layer)
		    }
		    mosaiciraf {
			# backward compatibility
			LoadMosaicIRAFFits $fn $file(layer)
		    }
		    mosaicimagewcs {
			# backward compatibility
			LoadMosaicImageWCSFits wcs $fn $file(layer)
		    }
		    mosaicwcs {
			# backward compatibility
			LoadMosaicWCSFits wcs $fn $file(layer)
		    }
		    mosaicwfpc2 -
		    mosaicimagewfpc2 {
			# backward compatibility
			LoadMosaicImageWFPC2Fits $fn
		    }

		    rgbcube {LoadRGBCubeFits $fn}
		    srgbcube {LoadRGBCubeSFits $fn $fn2}
		    rgbimage {LoadRGBImageFits $fn}
		    rgbarray {LoadRGBArray $fn}

		    array {LoadArray $fn $file(layer)}

		    url {LoadURL [lindex $param 1]}
		    default {LoadFits [lindex $param 0] $file(layer)}
		}
		FinishLoad
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPASendFIRST {xpa cdata param} {
    global firsti

    InitXPAError $xpa
    FIRSTDialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    coord {xpasetbuf $xpa "$firsti(x) $firsti(y)\n"}
	    x {xpasetbuf $xpa "$firsti(x)\n"}
	    y {xpasetbuf $xpa "$firsti(y)\n"}
	    size {xpasetbuf $xpa "$firsti(width) $firsti(height)\n"}
	    width {xpasetbuf $xpa "$firsti(width)\n"}
	    height {xpasetbuf $xpa "$firsti(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$firsti(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdFIRST {xpa cdata param buf len} {
    set i 0
    catch {ProcessFIRSTCmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendFits {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    if {[$current(frame) has fits]} {
		switch -- [string tolower [lindex $param 0]] {
		    size {
			xpasetbuf $xpa "[$current(frame) get fits size [lindex $param 1] [lindex $param 2] [lindex $param 3]]\n"
		    }
		    width {
			xpasetbuf $xpa "[$current(frame) get fits width]\n"
		    }
		    height {
			xpasetbuf $xpa "[$current(frame) get fits height]\n"
		    }
		    bitpix {
			xpasetbuf $xpa "[$current(frame) get fits bitpix]\n"
		    }
		    depth {
			xpasetbuf $xpa "[$current(frame) get fits depth]\n"
		    }
		    header {
			switch [llength $param] {
			    1 {xpasetbuf $xpa "[$current(frame) get fits header 1]\n"}
			    2 {xpasetbuf $xpa "[$current(frame) get fits header [lindex $param 1]]\n"}
			    3 {xpasetbuf $xpa "[$current(frame) get fits header 1 keyword [lindex $param 2]]\n"}
			    4 {xpasetbuf $xpa "[$current(frame) get fits header [lindex $param 1] keyword [lindex $param 3]]\n"}
			}
		    }
		    type {
			if {[$current(frame) has fits bin]} {
			    xpasetbuf $xpa "table\n"
			} else {
			    xpasetbuf $xpa "image\n"
			}
		    }
		    table {
			switch -- [lindex $param 1] {
			    gz {
				$current(frame) save fits table socket \
				    [xparec $xpa datafd] gz
			    }
			    default {
				$current(frame) save fits table socket \
				    [xparec $xpa datafd]
			    }
			}
		    }
		    image {
			switch -- [lindex $param 1] {
			    gz {
				$current(frame) save fits image socket \
				    [xparec $xpa datafd] gz
			    }
			    default {
				$current(frame) save fits image socket \
				    [xparec $xpa datafd]
			    }
			}
		    }
		    resample {
			switch -- [lindex $param 1] {
			    gz {
				$current(frame) save fits resample socket \
				    [xparec $xpa datafd] gz
			    }
			    default {
				$current(frame) save fits resample socket \
				    [xparec $xpa datafd]
			    }
			}
		    }
		    gz {
			$current(frame) save fits image socket \
			    [xparec $xpa datafd] gz
		    }
		    default {
			$current(frame) save fits image socket \
			    [xparec $xpa datafd]
		    }
		}
	    } else {
		Error [msgcat::mc {No file loaded}]
	    }
	} else {
	    Error [msgcat::mc {No file loaded}]
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdFits {xpa cdata param buf len} {
    global current
    global loadParam

    InitXPAError $xpa
    catch {
	set param [XPARcvdFitsPre $param]

	if {$current(frame) != {}} {
	    set loadParam(load,type) socketgz
	    # set loadParam(load,layer) $file(layer)
	    set loadParam(socket,id) [xparec $xpa datafd]
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {}
	    set loadParam(file,name) [lindex $param 1]
	    if {$loadParam(file,name) == {}} {
		set loadParam(file,name) stdin
	    }

	    XPARcvdFitsPost $param
	}
    }
    CatchXPAError $xpa
}

proc XPAHVRcvdFits {xpa cdata param buf len} {
    global current
    global loadParam

    catch {
	set param [XPARcvdFitsPre $param]

	if {$current(frame) != {}} {
	    set loadParam(load,type) allocgz
	    # set loadParam(load,layer) $file(layer)
	    set loadParam(file,type) fits
	    set loadParam(file,mode) {}
	    set loadParam(file,name) [lindex $param 1]
	    set loadParam(file,fn) $cdata

	    XPARcvdFitsPost $param
	}
    }
}

proc XPARcvdFitsPre {param} {
    global loadParam

    set loadParam(load,layer) {}
    switch -- [string tolower [lindex $param 0]] {
	new {
	    CreateFrame
	    set param [lreplace $param 0 0]
	}
	mask {
	    set loadParam(load,layer) mask
	    set param [lreplace $param 0 0]
	}
    }
    return $param
}

proc XPARcvdFitsPost {param} {
    global current
    global loadParam

    switch -- [lindex $param 0] {
	datacube -
	medatacube {
	    set loadParam(file,mode) {data cube}
	    ProcessLoad
	}

	mosaicimage {
	    if {[lindex $param 2] == {}} {
		set loadParam(file,mode) [list mosaic image iraf]
	    } else {
		set loadParam(file,mode) \
		    [list mosaic image [lindex $param 1]]
		set loadParam(file,name) [lindex $param 2]
	    }
	    ProcessLoad
	}
	mosaic {
	    if {[lindex $param 2] == {}} {
		set loadParam(file,mode) [list mosaic iraf]
	    } else {
		set loadParam(file,mode) \
		    [list mosaic [lindex $param 1]]
		set loadParam(file,name) [lindex $param 2]
	    }
	    ProcessLoad
	}

	mosaicimageiraf {
	    # backward compatibility
	    set loadParam(file,mode) {mosaic image iraf}
	    ProcessLoad
	}
	mosaiciraf {
	    # backward compatibility
	    set loadParam(file,mode) {mosaic iraf}
	    ProcessLoad
	}
	mosaicimagewcs {
	    # backward compatibility
	    set loadParam(file,mode) {mosaic image wcs}
	    ProcessLoad
	}
	mosaicwcs {
	    # backward compatibility
	    set loadParam(file,mode) {mosaic wcs}
	    ProcessLoad
	}
	mosaicwfpc2 -
	mosaicimagewfpc2 {
	    # backward compatibility
	    set loadParam(file,mode) {mosaic image wfpc2}
	    ProcessLoad
	}

	rgbimage {
	    if {[$current(frame) get type] == {rgb}} {
		set loadParam(file,mode) {rgb image}
		ProcessLoad
	    } else {
		Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
	    }
	}
	rgbcube {
	    if {[$current(frame) get type] == {rgb}} {
		set loadParam(file,mode) {rgb cube}
		ProcessLoad
	    } else {
		Error [msgcat::mc {Unable to load RGB image into a non-rgb frame}]
	    }
	}
	default {
	    set loadParam(file,name) [lindex $param 0] 
	    if {$loadParam(file,name) == {}} {
		set loadParam(file,name) stdin
	    }
	    ProcessLoad
	}
    }
    FinishLoad
}

proc XPASendFrame {xpa cdata param} {
    global ds9
    global current
    global rgb

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    active {
		set r {}
		foreach f $ds9(active) {
		    append r "[string range $f 5 end] "
		}
		xpasetbuf $xpa "$r\n"
	    }
	    all {
		set r {}
		foreach f $ds9(frames) {
		    append r "[string range $f 5 end] "
		}
		xpasetbuf $xpa "$r\n"
	    }
	    default {xpasetbuf $xpa "[string range $current(frame) 5 end]\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdFrame {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessFrameCmd param i}
    CatchXPAError $xpa
}

proc XPASendGrid {xpa cdata param} {
    global grid

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    system {xpasetbuf $xpa "$grid(system)\n"}
	    sky {xpasetbuf $xpa "$grid(sky)\n"}
	    skyformat {xpasetbuf $xpa "$grid(skyformat)\n"}
	    type {
		switch -- [lindex $param 1] {
		    axes {xpasetbuf $xpa "$grid(labelling)\n"}
		    numerics {xpasetbuf $xpa "$grid(type,numlab)\n"}
		    default {xpasetbuf $xpa "$grid(type)\n"}
		}
	    }
	    view {
		switch -- [lindex $param 1] {
		    grid {xpasetbuf $xpa [ToYesNo $grid(grid)]}
		    axes {
			switch -- [lindex $param 2] {
			    numbers {xpasetbuf $xpa [ToYesNo $grid(numlab)]}
			    tickmarks {xpasetbuf $xpa [ToYesNo $grid(tick)]}
			    label {xpasetbuf $xpa [ToYesNo $grid(textlab)]}
			    default {xpasetbuf $xpa [ToYesNo $grid(axes)]}
			}
		    }
		    title {xpasetbuf $xpa [ToYesNo $grid(title)]}
		    border {xpasetbuf $xpa [ToYesNo $grid(border)]}
		    vertical {xpasetbuf $xpa [ToYesNo $grid(labelup)]}
		}
	    }
	    default {xpasetbuf $xpa [ToYesNo $grid(view)]}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdGrid {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessGridCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdHeader {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessHeaderCmd param i}
    CatchXPAError $xpa
}

proc XPASendHeight {xpa cdata param} {
    global canvas

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$canvas(height)\n"}
    CatchXPAError $xpa
}

proc XPARcvdHeight {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessHeightCmd param i}
    CatchXPAError $xpa
}

proc XPASendIconify {xpa cdata param} {
    global ds9

    InitXPAError $xpa
    catch {
	if {[wm state $ds9(top)] == "normal"} {
	    xpasetbuf $xpa "no\n"
	} else {
	    xpasetbuf $xpa "yes\n"
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdIconify {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessIconifyCmd param i}
    CatchXPAError $xpa
}

proc XPASendIIS {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    switch -- [string tolower [lindex $param 0]] {
		filename {xpasetbuf $xpa "[$current(frame) get iis file name [lindex $param 1]]\n"}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdIIS {xpa cdata param buf len} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    switch -- [string tolower [lindex $param 0]] {
		filename {
		    $current(frame) iis set file name \
			[lindex $param 1] [lindex $param 2]
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPASendImexam {xpa cdata param} {
    global ds9
    global cursor

    InitXPAError $xpa
    catch {
	set ds9(imexam,frame) {}
	set ds9(imexam,x) {}
	set ds9(imexam,y) {}
	set ds9(imexam,event) {}
	set ds9(imexam,mode) $ds9(mode)

	set ds9(mode) imexam

	set ds9(imexam) 0
	set ds9(imexam,key) 0

	# turn on blinking cursor
	set cursor(timer) 1
	CursorTimer

	switch -- [string tolower [lindex $param 0]] {
	    key {
		set ds9(imexam,key) 1
		set varname {ds9(imexam,key)}
		set param [lreplace $param 0 0]
	    }
	    default {
		set ds9(imexam) 1
		set varname {ds9(imexam)}
	    }
	}

	switch -- [string tolower [lindex $param 0]] {
	    value -
	    data {
		vwait $varname
		set w [lindex $param 1]
		set h [lindex $param 2]
		if {$w == {}} {
		    set w 1
		}
		if {$h == {}} {
		    set h 1
		}
		xpasetbuf $xpa "$ds9(imexam,event) [$ds9(imexam,frame) get data canvas $ds9(imexam,x) $ds9(imexam,y) $w $h]\n"
	    }
	    coordinate {
		set sys [lindex $param 1]
		set sky [lindex $param 2]
		set skyformat [lindex $param 3]
		switch -- $skyformat {
		    {} {set skyformat degrees}
		}
		switch -- $sky {
		    {} {set sky fk5}
		}
		switch -- $sys {
		    {} {set sys physical}
		    fk4 -
		    fk5 -
		    icrs -
		    galactic -
		    ecliptic {set sky $sys; set sys wcs}
		}

		vwait $varname
		xpasetbuf $xpa "$ds9(imexam,event) [$ds9(imexam,frame) get coordinates canvas $ds9(imexam,x) $ds9(imexam,y) $sys $sky $skyformat]\n"
	    }
	}
	# turn off blinking cursor
	set cursor(timer) 0

	set ds9(mode) $ds9(imexam,mode)

	set ds9(imexam) 0
	set ds9(imexam,key) 0

	set ds9(imexam,frame) {}
	set ds9(imexam,x) {}
	set ds9(imexam,y) {}
	set ds9(imexam,event) {}
	set ds9(imexam,mode) {}
    }
    CatchXPAError $xpa
}

proc XPARcvdLock {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessLockCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdLower {xpa cdata param buf len} {
    global ds9

    InitXPAError $xpa
    catch {set i 0; ProcessLowerCmd param i}
    CatchXPAError $xpa
}

proc XPASendMask {xpa cdata param} {
    global mask

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    color {xpasetbuf $xpa "$mask(color)\n"}
	    default {xpasetbuf $xpa [ToYesNo $mask(view)]}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdMask {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessMaskCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdMatch {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessMatchCmd param i}
    CatchXPAError $xpa
}

proc XPASendMinMax {xpa cdata param} {
    global current
    global minmax

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    mode {xpasetbuf $xpa "$minmax(mode)\n"}
	    interval {xpasetbuf $xpa "$minmax(sample)\n"}
	    default {xpasetbuf $xpa "$minmax(mode)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdMinMax {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessMinMaxCmd param i}
    CatchXPAError $xpa
}

proc XPASendMode {xpa cdata param} {
    global ds9

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$ds9(mode)\n"}
    CatchXPAError $xpa
}

proc XPARcvdMode {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessModeCmd param i}
    CatchXPAError $xpa
}

proc XPASendNRES {xpa cdata param} {
    global nres

    InitXPAError $xpa
    NRESDialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    server {xpasetbuf $xpa "$nres(server)\n"}
	    sky {xpasetbuf $xpa "$nres(sky)\n"}
	    format -
	    skyformat {xpasetbuf $xpa "$nres(skyformat)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdNRES {xpa cdata param buf len} {
    catch {set i 0; ProcessNRESCmd param i}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendOrient {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$current(orient)\n"}
    CatchXPAError $xpa
}

proc XPARcvdOrient {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessOrientCmd param i}
    CatchXPAError $xpa
}

proc XPASendPageSetup {xpa cdata param} {
    global ps

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    orientation {xpasetbuf $xpa "$ps(orient)\n"}
	    pagescale {xpasetbuf $xpa "$ps(scale)\n"}
	    pagesize {xpasetbuf $xpa "$ps(size)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdPageSetup {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPageSetupCmd param i}
    CatchXPAError $xpa
}

proc XPASendPan {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	set sys [lindex $param 0]
	set sky [lindex $param 1]
	set skyformat [lindex $param 2]
	if {$current(frame) != {}} {
	    switch -- $sys {
		{} {set sys physical}
		fk4 -
		fk5 -
		icrs -
		galactic -
		ecliptic {set skyformat $sky; set sky $sys; set sys wcs}
	    }

	    xpasetbuf $xpa "[$current(frame) get pan $sys $sky $skyformat]\n"
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdPan {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPanCmd param i}
    CatchXPAError $xpa
}

proc XPASendPixelTable {xpa cdata param} {
    global pixel

    InitXPAError $xpa
    PixelTableDialog
    catch {xpasetbuf $xpa [PixelTableRender]}
    CatchXPAError $xpa
}

proc XPARcvdPixelTable {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPixelTableCmd param i}
    CatchXPAError $xpa
}

proc XPASendPrefs {xpa cdata param} {
    global prefs
    global ds9

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    bgcolor {xpasetbuf $xpa "$ds9(bg,color)\n"}
	    nancolor {xpasetbuf $xpa "$ds9(nan,color)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdPrefs {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPrefsCmd param i}
    CatchXPAError $xpa
}

proc XPASendPreserve {xpa cdata param} {
    global ds9
    global scale
    global panzoom
    global marker

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    scale {xpasetbuf $xpa [ToYesNo $scale(preserve)]}
	    pan {xpasetbuf $xpa [ToYesNo $panzoom(preserve)]}
	    regions {xpasetbuf $xpa [ToYesNo $marker(preserve)]}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdPreserve {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPreserveCmd param i}
    CatchXPAError $xpa
}

proc XPASendPlot {xpa cdata param} {
    global current
    global ap

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$ap(xpa)\n"}
    CatchXPAError $xpa
}

proc XPARcvdPlot {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPlotCmd param i buf}
    CatchXPAError $xpa
}

proc XPASendPrint {xpa cdata param} {
    global ps

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    destination {xpasetbuf $xpa "$ps(dest)\n"}
	    command {xpasetbuf $xpa "$ps(cmd)\n"}
	    filename {xpasetbuf $xpa "$ps(filename)\n"}
	    palette {xpasetbuf $xpa "$ps(color)\n"}
	    level {xpasetbuf $xpa "$ps(level)\n"}
	    interpolate {xpasetbuf $xpa "0\n"}
	    resolution {xpasetbuf $xpa "$ps(resolution)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdPrint {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessPrintCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdRaise {xpa cdata param buf len} {
    global ds9

    InitXPAError $xpa
    catch {set i 0; ProcessRaiseCmd param i}
    CatchXPAError $xpa
}

proc XPASendRegions {xpa cdata param} {
    global current
    global marker

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    set item [lindex $param 0]
	    switch -- $item {
		show {xpasetbuf $xpa [ToYesNo $marker(show)]}
		showtext {xpasetbuf $xpa [ToYesNo $marker(show,text)]}
		format {xpasetbuf $xpa "$marker(format)\n"}
		coord -
		system {xpasetbuf $xpa "$marker(system)\n"}
		sky  {xpasetbuf $xpa "$marker(sky)\n"}
		coordformat -
		skyformat {xpasetbuf $xpa "$marker(skyformat)\n"}
		strip {xpasetbuf $xpa [ToYesNo $marker(strip)]}
		wcs {xpasetbuf $xpa [ToYesNo $marker(wcs)]}
		delim {
		    if {$marker(strip)} {
			xpasetbuf $xpa "semicolon\n"
		    } else {
			xpasetbuf $xpa "nl\n"
		    }
		}
		shape {xpasetbuf $xpa "$marker(shape)\n"}
		color {xpasetbuf $xpa "$marker(color)\n"}
		width {xpasetbuf $xpa "$marker(width)\n"}

		tag -
		tags -
		group -
		groups {xpasetbuf $xpa \
		"[lsort [$current(frame) get marker tag all]]\n"}

		default {
		    set format $marker(format)
		    set sys $marker(system)
		    set sky $marker(sky)
		    set skyformat $marker(skyformat)
		    set strip $marker(strip)
		    set wcs $marker(wcs)
		    set select {}
		    set props {}
		    set tags {}

		    set i 0
		    set l [llength $param]
		    while {$i < $l} {
			switch -- [lindex $param $i] {
			    -format {incr i; set format [lindex $param $i]}
			    -coord -
			    -system {
				incr i
				# for backward compatibility
				switch -- [lindex $param $i] {
				    fk4 -
				    fk5 -
				    icrs -
				    galactic -
				    ecliptic {
					set sys wcs
					set sky [lindex $param $i]
				    }
				    
				    default {set sys [lindex $param $i]}
				}
			    }
			    -sky {incr i; set sky [lindex $param $i]}
			    -coordformat -
			    -skyformat {
				incr i;
				set skyformat [lindex $param $i]
			    }
			    -strip {
				incr i; 
				set strip [FromYesNo [lindex $param $i]]
			    }
			    -wcs {
				incr i; 
				set wcs [FromYesNo [lindex $param $i]]
			    }
			    -delim  {
				incr i;
				if {[lindex $param $i] != "nl"} {
				    set strip 1
				} else {
				    set strip 0
				}
			    }

			    include {append props " include = 1"}
			    exclude {append props " include = 0"}
			    source {append props " source = 1"}
			    background {append props " source = 0"}
			    selected {set select "select"}
			    -prop {
				append props \
				    " [lindex $param [expr $i+1]] = [lindex $param [expr $i+2]]"
				incr i 2
			    }
			    -tag -
			    -group {
				incr i
				append tags "tag = \{[lindex $param $i]\}"
			    }
			    default {
				xpaerror $xpa \
				    "Illegal option: [lindex $param $i]"
			    }
			}
			incr i
		    }

		    xpasetbuf $xpa [$current(frame) marker list \
					$select \
					$format $sys $sky $skyformat \
					$strip $wcs $props $tags]
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdRegions {xpa cdata param buf len} {
    global marker
    global current

    InitXPAError $xpa
    if {$param != {}} {
	catch {set i 0; ProcessMarkerCmd param i}
    } else {
	if {[$current(frame) has fits]} {
	    switch -- $marker(format) {
		xy {
		    $current(frame) marker load $marker(format) \
			[xparec $xpa datafd] $marker(system) $marker(sky)
		    UpdateGroupDialog
		}
		default {
		    $current(frame) marker load $marker(format) \
			[xparec $xpa datafd]
		    UpdateGroupDialog
		}
	    } 
	} else {
	    Error [msgcat::mc {Unable to load regions without valid image loaded}]
	}
    }
    CatchXPAError $xpa
}

proc XPAHVRcvdRegions {xpa cdata param buf len} {
    if {$param != {}} {
	catch {set i 0; ProcessMarkerCmd param i}
    } else {
	if {$cdata != {}} {
	    MarkerLoad $cdata
	}
    }
}

proc XPASendRGB {xpa cdata param} {
    global current
    global rgb

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    channel {xpasetbuf $xpa "$current(rgb)\n"}
	    lock {
		switch -- [string tolower [lindex $param 1]] {
		    bin {xpasetbuf $xpa "$rgb(lock,bin)\n"}
		    scale {xpasetbuf $xpa "$rgb(lock,scale)\n"}
		    colorbar {xpasetbuf $xpa "$rgb(lock,colorbar)\n"}
		}
	    }
	    system {xpasetbuf $xpa "$rgb(system)\n"}
	    view {
		switch -- [lindex $param 1] {
		    red {xpasetbuf $xpa [ToYesNo $rgb(red)]}
		    green {xpasetbuf $xpa [ToYesNo $rgb(green)]}
		    blue {xpasetbuf $xpa [ToYesNo $rgb(blue)]}
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdRGB {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessRGBCmd param i}
    CatchXPAError $xpa
}

proc XPASendRotate {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$current(rotate)\n"}
    CatchXPAError $xpa
}

proc XPARcvdRotate {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessRotateCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdSaveImage {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSaveImageCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdSaveFits {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSaveFitsCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdSaveMPEG {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSaveMPEGCmd param i}
    CatchXPAError $xpa
}

proc XPASendScale {xpa cdata param} {
    global current
    global scale

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    datasec {xpasetbuf $xpa "$scale(datasec)\n"}
	    limits {
		if {$current(frame) != {}} {
		    set lims [$current(frame) get clip]
		    xpasetbuf $xpa "[lindex $lims 0] [lindex $lims 1]\n"
		}
	    }
	    mode {xpasetbuf $xpa "$scale(mode)\n"}
	    scope {xpasetbuf $xpa "$scale(scope)\n"}
	    default {xpasetbuf $xpa "$scale(type)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdScale {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessScaleCmd param i}
    CatchXPAError $xpa
}

proc XPASendSingle {xpa cdata param} {
    global ds9

    InitXPAError $xpa
    catch {
	if {$ds9(display,user) == "single"} {
	    xpasetbuf $xpa [ToYesNo 1]
	} else {
	    xpasetbuf $xpa [ToYesNo 0]
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdSingle {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSingleCmd param i}
    CatchXPAError $xpa
}

proc XPASendShm {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    xpasetbuf $xpa "[$current(frame) get fits file name full]\n"
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdShm {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessShmCmd param i 0 foobar}
    CatchXPAError $xpa
}

proc XPASendSkyView {xpa cdata param} {
    global skyviewi

    InitXPAError $xpa
    SkyViewDialog

    catch {
	switch -- [string tolower [lindex $param 0]] {
	    survey {xpasetbuf $xpa "$skyviewi(survey)\n"}
	    coord {xpasetbuf $xpa "$skyviewi(x) $skyviewi(y)\n"}
	    x {xpasetbuf $xpa "$skyviewi(x)\n"}
	    y {xpasetbuf $xpa "$skyviewi(y)\n"}
	    size {xpasetbuf $xpa "$skyviewi(width) $skyviewi(height)\n"}
	    width {xpasetbuf $xpa "$skyviewi(width)\n"}
	    height {xpasetbuf $xpa "$skyviewi(height)\n"}
	    name -
	    default {xpasetbuf $xpa "$skyviewi(name)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdSkyView {xpa cdata param buf len} {
    catch {set i 0; ProcessSkyViewCmd param i 0 foobar}
    # http has an error, so just clear
    InitXPAError $xpa
}

proc XPASendSmooth {xpa cdata param} {
    global current
    global smooth

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    function {xpasetbuf $xpa "$smooth(function)\n"}
	    radius {xpasetbuf $xpa "$smooth(radius)\n"}
	    default {xpasetbuf $xpa [ToYesNo $smooth(view)]}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdSmooth {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSmoothCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdSource {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessSourceCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdTcl {xpa cdata param buf len} {
    global ds9

    InitXPAError $xpa
    if {$ds9(xpa,tcl)} {
	catch {
	    if [string compare $param {}] {
		uplevel #0 $param
	    } else {
		uplevel #0 $buf
	    }
	}
    } else {
	Error [msgcat::mc {Executing TCL code from XPA is not enabled}]
    }
    CatchXPAError $xpa
}

proc XPASendTile {xpa cdata param} {
    global ds9
    global tile

    InitXPAError $xpa
    catch {
	switch -- [lindex $param 0] {
	    mode {xpasetbuf $xpa "$tile(mode)\n"}
	    grid {
		switch -- [lindex $param 1] {
		    mode {xpasetbuf $xpa "$tile(grid,mode)\n"}
		    layout {xpasetbuf $xpa "$tile(grid,row) $tile(grid,col)\n"}
		    gap {xpasetbuf $xpa "$tile(grid,gap)\n"}
		}
	    }
	    default {
		if {$ds9(display,user)=="tile"} {
		    xpasetbuf $xpa [ToYesNo 1]
		} else {
		    xpasetbuf $xpa [ToYesNo 0]
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdTile {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessTileCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdUpdate {xpa cdata param buf len} {
    global current
    global ds9

    InitXPAError $xpa
    catch {
	if {$current(frame) != {}} {
	    switch -- [string tolower [lindex $param 0]] {
		now {
		    if {[lindex $param 1] != {}} {
			$current(frame) update now \
			    [lindex $param 1] \
			    [lindex $param 2] [lindex $param 3] \
			    [lindex $param 4] [lindex $param 5]
		    } else {
			$current(frame) update now
		    }
		}

		on -
		yes {set ds9(xpa,idletasks) 1}
		no -
		off {set ds9(xpa,idletasks) 0}

		{} {$current(frame) update}
		default {
		    $current(frame) update \
			[lindex $param 0] \
			[lindex $param 1] [lindex $param 2] \
			[lindex $param 3] [lindex $param 4]
		}
	    }
	}
    }
    CatchXPAError $xpa
}

proc XPASendVersion {xpa cdata param} {
    global ds9

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$ds9(title) [lindex $ds9(version) 0]\n"}
    CatchXPAError $xpa
}

proc XPASendView {xpa cdata param} {
    global view

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    info {xpasetbuf $xpa [ToYesNo $view(info)]}
	    panner {xpasetbuf $xpa [ToYesNo $view(panner)]}
	    magnifier {xpasetbuf $xpa [ToYesNo $view(magnifier)]}
	    buttons {xpasetbuf $xpa [ToYesNo $view(buttons)]}
	    colorbar {xpasetbuf $xpa [ToYesNo $view(colorbar)]}
	    colorbarnumerics {xpasetbuf $xpa [ToYesNo $view(colorbar,numerics)]}
	    horzgraph {xpasetbuf $xpa [ToYesNo $view(graph,horz)]}
	    vertgraph {xpasetbuf $xpa [ToYesNo $view(graph,vert)]}

	    filename {xpasetbuf $xpa [ToYesNo $view(info,filename)]}
	    object {xpasetbuf $xpa [ToYesNo $view(info,object)]}
	    minmax {xpasetbuf $xpa [ToYesNo $view(info,minmax)]}
	    lowhigh {xpasetbuf $xpa [ToYesNo $view(info,lowhigh)]}
	    frame {xpasetbuf $xpa [ToYesNo $view(info,frame)]}

	    detector {xpasetbuf $xpa [ToYesNo $view(info,detector)]}
	    amplifier {xpasetbuf $xpa [ToYesNo $view(info,amplifier)]}
	    physical {xpasetbuf $xpa [ToYesNo $view(info,physical)]}
	    image {xpasetbuf $xpa [ToYesNo $view(info,image)]}
	    wcs {xpasetbuf $xpa [ToYesNo $view(info,wcs)]}
	    wcsa {xpasetbuf $xpa [ToYesNo $view(info,wcsa)]}
	    wcsb {xpasetbuf $xpa [ToYesNo $view(info,wcsb)]}
	    wcsc {xpasetbuf $xpa [ToYesNo $view(info,wcsc)]}
	    wcsd {xpasetbuf $xpa [ToYesNo $view(info,wcsd)]}
	    wcse {xpasetbuf $xpa [ToYesNo $view(info,wcse)]}
	    wcsf {xpasetbuf $xpa [ToYesNo $view(info,wcsf)]}
	    wcsg {xpasetbuf $xpa [ToYesNo $view(info,wcsg)]}
	    wcsh {xpasetbuf $xpa [ToYesNo $view(info,wcsh)]}
	    wcsi {xpasetbuf $xpa [ToYesNo $view(info,wcsi)]}
	    wcsj {xpasetbuf $xpa [ToYesNo $view(info,wcsj)]}
	    wcsk {xpasetbuf $xpa [ToYesNo $view(info,wcsk)]}
	    wcsl {xpasetbuf $xpa [ToYesNo $view(info,wcsl)]}
	    wcsm {xpasetbuf $xpa [ToYesNo $view(info,wcsm)]}
	    wcsn {xpasetbuf $xpa [ToYesNo $view(info,wcsn)]}
	    wcso {xpasetbuf $xpa [ToYesNo $view(info,wcso)]}
	    wcsp {xpasetbuf $xpa [ToYesNo $view(info,wcsp)]}
	    wcsq {xpasetbuf $xpa [ToYesNo $view(info,wcsq)]}
	    wcsr {xpasetbuf $xpa [ToYesNo $view(info,wcsr)]}
	    wcss {xpasetbuf $xpa [ToYesNo $view(info,wcss)]}
	    wcst {xpasetbuf $xpa [ToYesNo $view(info,wcst)]}
	    wcsu {xpasetbuf $xpa [ToYesNo $view(info,wcsu)]}
	    wcsv {xpasetbuf $xpa [ToYesNo $view(info,wcsv)]}
	    wcsw {xpasetbuf $xpa [ToYesNo $view(info,wcsw)]}
	    wcsx {xpasetbuf $xpa [ToYesNo $view(info,wcsx)]}
	    wcsy {xpasetbuf $xpa [ToYesNo $view(info,wcsy)]}
	    wcsz {xpasetbuf $xpa [ToYesNo $view(info,wcsz)]}
	    default {xpasetbuf $xpa "$view(layout)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdView {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessViewCmd param i}
    CatchXPAError $xpa
}

proc XPASendVO {xpa cdata param} {
    global vo

    InitXPAError $xpa
    set l [llength $vo(server,host)]
    for {set ii 0} {$ii<$l} {incr ii} {
	xpasetbuf $xpa \
	    "$vo(server,host) $vo(server,title) $vo(server,url) $vo(b$ii)\n"
    }
    CatchXPAError $xpa
}

proc XPARcvdVO {xpa cdata param buf len} {
    catch {set i 0; ProcessVOCmd param i}
    # someone is setting the error state
    InitXPAError $xpa
}

proc XPASendWCS {xpa cdata param} {
    global wcs

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    align {xpasetbuf $xpa [ToYesNo $wcs(align)]}
	    system {xpasetbuf $xpa "$wcs(system)\n"}
	    sky  {xpasetbuf $xpa "$wcs(sky)\n"}
	    format -
	    skyformat {xpasetbuf $xpa "$wcs(skyformat)\n"}
	    default {xpasetbuf $xpa "$wcs(system)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdWCS {xpa cdata param buf len} {
    global wcs
    global current

    InitXPAError $xpa
    set which [lindex $param 0]
    if {($which=="replace" || $which=="append") && [lindex $param 1]=={}} {
	$current(frame) wcs $which [xparec $xpa datafd]
	UpdateWCSMenu
	UpdateWCS
    } else {
	set i 0
	ProcessWCSCmd param i
    }
    CatchXPAError $xpa
}

proc XPAHVRcvdWCS {xpa cdata param buf len} {
    if {$cdata != {}} {
	if [catch {open "$cdata" r} ch] {
	    Error "[msgcat::mc {Unable to open file}] $cdata"
	    return
	}
	set buf [read $ch]
	set len [string length $buf]
	catch {close $ch}

	XPARcvdWCS $xpa {} $param $buf $len
    }
}

proc XPASendWeb {xpa cdata param} {
    global hv

    InitXPAError $xpa
    if {[info exists hv(web,url)]} {
	xpasetbuf $xpa "$hv(web,url)\n"
    }
    CatchXPAError $xpa
}

proc XPARcvdWeb {xpa cdata param buf len} {
    catch {set i 0; ProcessWebCmd param i}
    # someone is setting an error state
    InitXPAError $xpa
}

proc XPASendWidth {xpa cdata param} {
    global canvas

    InitXPAError $xpa
    catch {xpasetbuf $xpa "$canvas(width)\n"}
    CatchXPAError $xpa
}

proc XPARcvdWidth {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessWidthCmd param i}
    CatchXPAError $xpa
}

proc XPASendZoom {xpa cdata param} {
    global current

    InitXPAError $xpa
    catch {
	set z1 [lindex $current(zoom) 0]
	set z2 [lindex $current(zoom) 1]
	if {$z1 != $z2} {
	    xpasetbuf $xpa "$current(zoom)\n"
	} else {
	    xpasetbuf $xpa "$z1\n"
	}
    }
    CatchXPAError $xpa
}

proc XPASendZScale {xpa cdata param} {
    global current
    global zscale
    global scale

    InitXPAError $xpa
    catch {
	switch -- [string tolower $param] {
	    contrast {xpasetbuf $xpa "$zscale(contrast)\n"}
	    sample {xpasetbuf $xpa "$zscale(sample)\n"}
	    line {xpasetbuf $xpa "$zscale(line)\n"}
	}
    }
    CatchXPAError $xpa
}

proc XPARcvdZScale {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessZScaleCmd param i}
    CatchXPAError $xpa
}

proc XPARcvdZoom {xpa cdata param buf len} {
    InitXPAError $xpa
    catch {set i 0; ProcessZoomCmd param i}
    CatchXPAError $xpa
}

proc DisplayXPAInfo {} {
    global xpa
    global message

    set r {}
    if { ([info exists xpa] && ($xpa != {})) } {
	append r "[format "XPA_VERSION:\t%s" [xparec $xpa version]]\n"
	append r "[format "XPA_CLASS:\t%s"   [xparec $xpa class]]\n"
	append r "[format "XPA_NAME:\t%s"    [xparec $xpa name]]\n"
	append r "[format "XPA_METHOD:\t%s"  [xparec $xpa method]]\n"
	SimpleTextDialog xpatxt "[msgcat::mc {XPA Information}]" \
	    80 20 append bottom $r
    } else {
	Error "[msgcat::mc {XPA not initialized}]"
    }
}

# unwind xpa errors
# requires catch {} to allow a check to take place
proc InitXPAError {xpa} {
    global ds9
    global errorInfo

    if {$xpa != {}} {
	set ds9(xpa,ptr) $xpa
	set errorInfo {}
    }
}

proc CatchXPAError {xpa} {
    global ds9
    global errorInfo

    if {$xpa != {}} {
	set ds9(xpa,ptr) 0
	if {$errorInfo != {}} {
	    # reset cursor
	    set cursor(count) 0
	    UnsetWatchCursor

	    xpaerror $xpa "$errorInfo"
	    set errorInfo {}
	}
    }
}

proc XPAMethod {} {
    global xpa

    if {[info exists xpa]} {
	return [xparec $xpa method]
    } else {
	return {}
    }
}

# Process Cmds

proc ProcessXPACmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global env

    switch -- [string tolower [lindex $var $i]] {
	unix -
	inet -
	local -
	localhost {set env(XPA_METHOD) [lindex $var $i]}
	noxpans {set env(XPA_NSREGISTER) false}

	tcl {incr i; set ds9(xpa,tcl) [FromYesNo [lindex $var $i]]}

	yes -
	true -
	1 -
	no -
	false -
	0 {set ds9(xpa) [FromYesNo [lindex $var $i]]}
    }
}

